/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgebal.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgebal.java,v 1.8 2012/12/24 14:49:09 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dgebal {

// *> \brief \b DGEBAL
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DGEBAL + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEBAL( JOB, N, A, LDA, ILO, IHI, SCALE, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOB
// *       INTEGER            IHI, ILO, INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), SCALE( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEBAL balances a general real matrix A.  This involves, first,
// *> permuting A by a similarity transformation to isolate eigenvalues
// *> in the first 1 to ILO-1 and last IHI+1 to N elements on the
// *> diagonal; and second, applying a diagonal similarity transformation
// *> to rows and columns ILO to IHI to make the rows and columns as
// *> close in norm as possible.  Both steps are optional.
// *>
// *> Balancing may reduce the 1-norm of the matrix, and improve the
// *> accuracy of the computed eigenvalues and/or eigenvectors.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOB
// *> \verbatim
// *>          JOB is CHARACTER*1
// *>          Specifies the operations to be performed on A:
// *>          = 'N':  none:  simply set ILO = 1, IHI = N, SCALE(I) = 1.0
// *>                  for i = 1,...,N;
// *>          = 'P':  permute only;
// *>          = 'S':  scale only;
// *>          = 'B':  both permute and scale.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE array, dimension (LDA,N)
// *>          On entry, the input matrix A.
// *>          On exit,  A is overwritten by the balanced matrix.
// *>          If JOB = 'N', A is not referenced.
// *>          See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *> \param[out] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>          ILO and IHI are set to integers such that on exit
// *>          A(i,j) = 0 if i > j and j = 1,...,ILO-1 or I = IHI+1,...,N.

// *>          If JOB = 'N' or 'S', ILO = 1 and IHI = N.
// *> \endverbatim
// *>
// *> \param[out] SCALE
// *> \verbatim
// *>          SCALE is DOUBLE array, dimension (N)
// *>          Details of the permutations and scaling factors applied to
// *>          A.  If P(j) is the index of the row and column interchanged

// *>          with row and column j and D(j) is the scaling factor
// *>          applied to row and column j, then
// *>          SCALE(j) = P(j)    for j = 1,...,ILO-1
// *>                   = D(j)    for j = ILO,...,IHI
// *>                   = P(j)    for j = IHI+1,...,N.
// *>          The order in which the interchanges are made is N to IHI+1,

// *>          then 1 to ILO-1.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit.
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The permutations consist of row and column interchanges which put
// *>  the matrix in the form
// *>
// *>             ( T1   X   Y  )
// *>     P A P = (  0   B   Z  )
// *>             (  0   0   T2 )
// *>
// *>  where T1 and T2 are upper triangular matrices whose eigenvalues lie

// *>  along the diagonal.  The column indices ILO and IHI mark the startin
// *>  and ending columns of the submatrix B. Balancing consists of applyin
// *>  a diagonal similarity transformation inv(D) * B * D to make the
// *>  1-norms of each row of B and its corresponding column nearly equal.

// *>  The output matrix is
// *>
// *>     ( T1     X*D          Y    )
// *>     (  0  inv(D)*B*D  inv(D)*Z ).
// *>     (  0      0           T2   )
// *>
// *>  Information about the permutations P and the diagonal matrix D is
// *>  returned in the vector SCALE.
// *>
// *>  This subroutine is based on the EISPACK routine BALANC.
// *>
// *>  Modified by Tzu-Yi Chen, Computer Science Division, University of
// *>    California at Berkeley, USA
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dgebal (final char job,
int n,
double [] a, int _a_offset,
int lda,
intW ilo,
intW ihi,
double [] scale, int _scale_offset,
intW info)  {

boolean noconv= false;
int i= 0;
int ica= 0;
int iexc= 0;
int ira= 0;
int j= 0;
int k= 0;
int l= 0;
int m= 0;
double c= 0.0d;
double ca= 0.0d;
double f= 0.0d;
double g= 0.0d;
double r= 0.0d;
double ra= 0.0d;
double s= 0.0d;
double sfmax1= 0.0d;
double sfmax2= 0.0d;
double sfmin1= 0.0d;
double sfmin2= 0.0d;
//job = job.substring((1)-1,1);
final int NLOOPMAX=2*n+Util.ITMAX;
final double FACTOR = 0.95;
info.val = 0;
if (((((!com.polytechnik.lapack.Lsame.lsame(job,'N')) && (!com.polytechnik.lapack.Lsame.lsame(job,'P'))) && (!com.polytechnik.lapack.Lsame.lsame(job,'S'))) && (!com.polytechnik.lapack.Lsame.lsame(job,'B'))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -4;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGEBAL",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dgebal",999999);
// *
}
k = 1;
// *
l = n;
// *
label210:{
if ((n == 0)) {
    break label210;//Dummy.go_to("com/polytechnik/lapack/Dgebal",210);
}
    if (com.polytechnik.lapack.Lsame.lsame(job,'N'))  {
    {
for (i = 1; i <= n; i++) {
scale[(i-(1))+ _scale_offset] = 1.0;
//Dummy.label("com/polytechnik/lapack/Dgebal",10);
}              //  Close for() loop. 
}
break label210;//Dummy.go_to("com/polytechnik/lapack/Dgebal",210);
// *
}
/***
*
*     Permutation to isolate eigenvalues if possible
*
***/
if (com.polytechnik.lapack.Lsame.lsame(job,'S')) {
    //Dummy.go_to("com/polytechnik/lapack/Dgebal",120);
}
else /* label120 */ {
    /***
*
*     Row and column exchange.
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgebal",50);
label20:for(int jj20=0;jj20<NLOOPMAX;jj20++){ // Dummy.label("com/polytechnik/lapack/Dgebal",20);
label90:{
label50:if(jj20>0){
scale[(m-(1))+ _scale_offset] = (double)(j);
// *
if ((j == m)) {
    //Dummy.go_to("com/polytechnik/lapack/Dgebal",30);
}
else /* label30 */ {
    com.polytechnik.lapack.Dswap.dswap(l,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1,a,(1-(1))+(m-(1)) * (lda)+ _a_offset,1);
// *
com.polytechnik.lapack.Dswap.dswap(((n-k)+1),a,(j-(1))+(k-(1)) * (lda)+ _a_offset,lda,a,(m-(1))+(k-(1)) * (lda)+ _a_offset,lda);
} //label30: Dummy.label("com/polytechnik/lapack/Dgebal",30);
/***
*
*     Search for rows isolating an eigenvalue and push them down.
*
***/
{
  int _cg_tmp = iexc;
if (_cg_tmp == 1) {
  //Dummy.go_to("com/polytechnik/lapack/Dgebal",40);
}
else if (_cg_tmp == 2) {
  k = (k+1);
  break label90;//Dummy.go_to("com/polytechnik/lapack/Dgebal",80);
}
}
//label40:
//   Dummy.label("com/polytechnik/lapack/Dgebal",40);
if ((l == 1)) {
    break label210;//Dummy.go_to("com/polytechnik/lapack/Dgebal",210);
}
    // *
l = (l-1);
} //label50: Dummy.label("com/polytechnik/lapack/Dgebal",50);

// *
{
int _j_inc = -1;
label70:for (j = l; j >= 1; j += _j_inc) {
{
label60:for (i = 1; i <= l; i++) {
if ((i == j)) {
    continue label60;//Dummy.go_to("com/polytechnik/lapack/Dgebal",60);
}
    if ((a[(j-(1))+(i-(1)) * (lda)+ _a_offset] != 0.0)) {
	continue label70;//Dummy.go_to("com/polytechnik/lapack/Dgebal",70);
}
    // *
//Dummy.label("com/polytechnik/lapack/Dgebal",60);
}              //  Close for() loop. 
}
m = l;
iexc = 1;
continue label20;//Dummy.go_to("com/polytechnik/lapack/Dgebal",20);
// *
//Dummy.label("com/polytechnik/lapack/Dgebal",70);
}              //  Close for() loop. 
}
/***
*
*     Search for columns isolating an eigenvalue and push them left.
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgebal",90);
//label80:
//   Dummy.label("com/polytechnik/lapack/Dgebal",80);
// *
//k = (k+1);
} //label90:Dummy.label("com/polytechnik/lapack/Dgebal",90);

// *
{
label110:for (j = k; j <= l; j++) {
{
label100:for (i = k; i <= l; i++) {
if ((i == j)) {
    continue label100;//Dummy.go_to("com/polytechnik/lapack/Dgebal",100);
}
    if ((a[(i-(1))+(j-(1)) * (lda)+ _a_offset] != 0.0)) {
	continue label110;//Dummy.go_to("com/polytechnik/lapack/Dgebal",110);
}
    // *
//Dummy.label("com/polytechnik/lapack/Dgebal",100);
}              //  Close for() loop. 
}
m = k;
iexc = 2;
continue label20;//Dummy.go_to("com/polytechnik/lapack/Dgebal",20);
// *
//Dummy.label("com/polytechnik/lapack/Dgebal",110);
}              //  Close for() loop. 
}
break label20;}//label20
} //label120: Dummy.label("com/polytechnik/lapack/Dgebal",120);

{
for (i = k; i <= l; i++) {
scale[(i-(1))+ _scale_offset] = 1.0;
// *
//Dummy.label("com/polytechnik/lapack/Dgebal",130);
}              //  Close for() loop. 
}
/***
*
*     Balance the submatrix in rows K to L.
*
*     Iterative loop for norm reduction
*
***/
if (com.polytechnik.lapack.Lsame.lsame(job,'P')) {
    break label210;//Dummy.go_to("com/polytechnik/lapack/Dgebal",210);
}
    sfmin1 = (com.polytechnik.lapack.Dlamch.dlamch('S')/com.polytechnik.lapack.Dlamch.dlamch('P'));
sfmax1 = (1.0/sfmin1);
sfmin2 = (sfmin1*2.0);
sfmax2 = (1.0/sfmin2);
label140:for(int jj140=0;jj140<NLOOPMAX;jj140++){//Dummy.label("com/polytechnik/lapack/Dgebal",140);
// *
noconv = false;
{
label200:for (i = k; i <= l; i++) {
c = 0.0;
// *
r = 0.0;
{
label150:for (j = k; j <= l; j++) {
if ((j == i)) {
    continue label150;//Dummy.go_to("com/polytechnik/lapack/Dgebal",150);
}
    c = (c+Math.abs(a[(j-(1))+(i-(1)) * (lda)+ _a_offset]));
r = (r+Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
continue label150;//Dummy.label("com/polytechnik/lapack/Dgebal",150);
}              //  Close for() loop. 
}
ica = com.polytechnik.lapack.Idamax.idamax(l,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1);
ca = Math.abs(a[(ica-(1))+(i-(1)) * (lda)+ _a_offset]);
ira = com.polytechnik.lapack.Idamax.idamax(((n-k)+1),a,(i-(1))+(k-(1)) * (lda)+ _a_offset,lda);
/***
*
*        Guard against zero C or R due to underflow.
*
***/
ra = Math.abs(a[(i-(1))+(((ira+k)-1)-(1)) * (lda)+ _a_offset]);
if (((c == 0.0) || (r == 0.0))) {
    continue label200;//Dummy.go_to("com/polytechnik/lapack/Dgebal",200);
}
    g = (r/2.0);
f = 1.0;
s = (c+r);
label160:for(int jj160=0;jj160<NLOOPMAX;jj160++){//Dummy.label("com/polytechnik/lapack/Dgebal",160);
if (((!(c < g) || !(Util.max(f, c, ca) < sfmax2)) || !(Util.min(r, g, ra) > sfmin2))) {
    break label160;//Dummy.go_to("com/polytechnik/lapack/Dgebal",170);
}
    /***
*
*           Exit if NaN to avoid infinite loop
*
***/
if (jj160>=(NLOOPMAX-1)||com.polytechnik.lapack.Disnan.disnan((((((c+f)+ca)+r)+g)+ra)))  {
    info.val = -3;
//com.polytechnik.lapack.Xerbla.xerbla("DGEBAL",(-(info.val)));
return;//Dummy.go_to("com/polytechnik/lapack/Dgebal",999999);
}
f = (f*2.0);
c = (c*2.0);
ca = (ca*2.0);
r = (r/2.0);
g = (g/2.0);
ra = (ra/2.0);
// *
} //Dummy.go_to("com/polytechnik/lapack/Dgebal",160);
//label170:
//   Dummy.label("com/polytechnik/lapack/Dgebal",170);
g = (c/2.0);
label180:for(int jj180=0;jj180<NLOOPMAX;jj180++){//Dummy.label("com/polytechnik/lapack/Dgebal",180);
if (((!(g >= r) || !(Math.max(r, ra) < sfmax2)) || !(Math.min(Util.min(f, c, g), ca)  > sfmin2))) {
    break label180;//Dummy.go_to("com/polytechnik/lapack/Dgebal",190);
}
    f = (f/2.0);
c = (c/2.0);
g = (g/2.0);
ca = (ca/2.0);
r = (r*2.0);
ra = (ra*2.0);
/***
*
*        Now balance.
*
***/
} //Dummy.go_to("com/polytechnik/lapack/Dgebal",180);
//label190:
//   Dummy.label("com/polytechnik/lapack/Dgebal",190);
if ((((c+r)) >= (FACTOR*s))) {
    continue label200;//Dummy.go_to("com/polytechnik/lapack/Dgebal",200);
}
    if (((f < 1.0) && (scale[(i-(1))+ _scale_offset] < 1.0)))  {
    if (((f*scale[(i-(1))+ _scale_offset]) <= sfmin1)) {
	continue label200;//Dummy.go_to("com/polytechnik/lapack/Dgebal",200);
}
    }
if (((f > 1.0) && (scale[(i-(1))+ _scale_offset] > 1.0)))  {
    if ((scale[(i-(1))+ _scale_offset] >= (sfmax1/f))) {
	continue label200;//Dummy.go_to("com/polytechnik/lapack/Dgebal",200);
}
    }
g = (1.0/f);
scale[(i-(1))+ _scale_offset] = (scale[(i-(1))+ _scale_offset]*f);
// *
noconv = true;
com.polytechnik.lapack.Dscal.dscal(((n-k)+1),g,a,(i-(1))+(k-(1)) * (lda)+ _a_offset,lda);
// *
com.polytechnik.lapack.Dscal.dscal(l,f,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1);
// *
//Dummy.label("com/polytechnik/lapack/Dgebal",200);
}              //  Close for() loop. 
}
// *
if (noconv) {
    continue label140;//Dummy.go_to("com/polytechnik/lapack/Dgebal",140);
}
break label140;}
} // label210: Dummy.label("com/polytechnik/lapack/Dgebal",210);
ilo.val = k;
// *
ihi.val = l;
/***
*
*     End of DGEBAL
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgebal",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dgebal",999999);
return;
   }
} // End class.
