/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgees.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgees.java,v 1.8 2012/12/22 18:11:08 mal Exp $
 *
 */

package com.polytechnik.lapack;


public class Dgees {

// *> \brief <b> DGEES computes the eigenvalues, the Schur form, and, optio
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DGEES + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEES( JOBVS, SORT, SELECT, N, A, LDA, SDIM, WR, WI,

// *                         VS, LDVS, WORK, LWORK, BWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBVS, SORT
// *       INTEGER            INFO, LDA, LDVS, LWORK, N, SDIM
// *       ..
// *       .. Array Arguments ..
// *       LOGICAL            BWORK( * )
// *       DOUBLE PRECISION   A( LDA, * ), VS( LDVS, * ), WI( * ), WORK( * 
// *      $                   WR( * )
// *       ..
// *       .. Function Arguments ..
// *       LOGICAL            SELECT
// *       EXTERNAL           SELECT
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEES computes for an N-by-N real nonsymmetric matrix A, the
// *> eigenvalues, the real Schur form T, and, optionally, the matrix of
// *> Schur vectors Z.  This gives the Schur factorization A = Z*T*(Z**T).

// *>
// *> Optionally, it also orders the eigenvalues on the diagonal of the
// *> real Schur form so that selected eigenvalues are at the top left.
// *> The leading columns of Z then form an orthonormal basis for the
// *> invariant subspace corresponding to the selected eigenvalues.
// *>
// *> A matrix is in real Schur form if it is upper quasi-triangular with
// *> 1-by-1 and 2-by-2 blocks. 2-by-2 blocks will be standardized in the
// *> form
// *>         [  a  b  ]
// *>         [  c  a  ]
// *>
// *> where b*c < 0. The eigenvalues of such a block are a +- sqrt(bc).
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBVS
// *> \verbatim
// *>          JOBVS is CHARACTER*1
// *>          = 'N': Schur vectors are not computed;
// *>          = 'V': Schur vectors are computed.
// *> \endverbatim
// *>
// *> \param[in] SORT
// *> \verbatim
// *>          SORT is CHARACTER*1
// *>          Specifies whether or not to order the eigenvalues on the
// *>          diagonal of the Schur form.
// *>          = 'N': Eigenvalues are not ordered;
// *>          = 'S': Eigenvalues are ordered (see SELECT).
// *> \endverbatim
// *>
// *> \param[in] SELECT
// *> \verbatim
// *>          SELECT is a LOGICAL FUNCTION of two DOUBLE PRECISION argumen
// *>          SELECT must be declared EXTERNAL in the calling subroutine.

// *>          If SORT = 'S', SELECT is used to select eigenvalues to sort

// *>          to the top left of the Schur form.
// *>          If SORT = 'N', SELECT is not referenced.
// *>          An eigenvalue WR(j)+sqrt(-1)*WI(j) is selected if
// *>          SELECT(WR(j),WI(j)) is true; i.e., if either one of a comple
// *>          conjugate pair of eigenvalues is selected, then both complex
// *>          eigenvalues are selected.
// *>          Note that a selected complex eigenvalue may no longer
// *>          satisfy SELECT(WR(j),WI(j)) = .TRUE. after ordering, since
// *>          ordering may change the value of complex eigenvalues
// *>          (especially if the eigenvalue is ill-conditioned); in this
// *>          case INFO is set to N+2 (see INFO below).
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A. N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the N-by-N matrix A.
// *>          On exit, A has been overwritten by its real Schur form T.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] SDIM
// *> \verbatim
// *>          SDIM is INTEGER
// *>          If SORT = 'N', SDIM = 0.
// *>          If SORT = 'S', SDIM = number of eigenvalues (after sorting)

// *>                         for which SELECT is true. (Complex conjugate

// *>                         pairs for which SELECT is true for either
// *>                         eigenvalue count as 2.)
// *> \endverbatim
// *>
// *> \param[out] WR
// *> \verbatim
// *>          WR is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] WI
// *> \verbatim
// *>          WI is DOUBLE PRECISION array, dimension (N)
// *>          WR and WI contain the real and imaginary parts,
// *>          respectively, of the computed eigenvalues in the same order

// *>          that they appear on the diagonal of the output Schur form T.
// *>          Complex conjugate pairs of eigenvalues will appear
// *>          consecutively with the eigenvalue having the positive
// *>          imaginary part first.
// *> \endverbatim
// *>
// *> \param[out] VS
// *> \verbatim
// *>          VS is DOUBLE PRECISION array, dimension (LDVS,N)
// *>          If JOBVS = 'V', VS contains the orthogonal matrix Z of Schur
// *>          vectors.
// *>          If JOBVS = 'N', VS is not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDVS
// *> \verbatim
// *>          LDVS is INTEGER
// *>          The leading dimension of the array VS.  LDVS >= 1; if
// *>          JOBVS = 'V', LDVS >= N.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) contains the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.  LWORK >= max(1,3*N).
// *>          For good performance, LWORK must generally be larger.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] BWORK
// *> \verbatim
// *>          BWORK is LOGICAL array, dimension (N)
// *>          Not referenced if SORT = 'N'.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -i, the i-th argument had an illegal value.
// *>          > 0: if INFO = i, and i is
// *>             <= N: the QR algorithm failed to compute all the
// *>                   eigenvalues; elements 1:ILO-1 and i+1:N of WR and W
// *>                   contain those eigenvalues which have converged; if

// *>                   JOBVS = 'V', VS contains the matrix which reduces A
// *>                   to its partially converged Schur form.
// *>             = N+1: the eigenvalues could not be reordered because som
// *>                   eigenvalues were too close to separate (the problem
// *>                   is very ill-conditioned);
// *>             = N+2: after reordering, roundoff changed values of some

// *>                   complex eigenvalues so that leading eigenvalues in

// *>                   the Schur form no longer satisfy SELECT=.TRUE.  Thi
// *>                   could also be caused by underflow due to scaling.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleGEeigen
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*     .. Function Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dgees (final char jobvs,
//final char sort,
//Object select,
int n,
double [] a, int _a_offset,
int lda,
//intW sdim,
double [] wr, int _wr_offset,
double [] wi, int _wi_offset,
double [] vs, int _vs_offset,
int ldvs,
double [] work, int _work_offset,
int lwork,
intW info)  {

boolean cursl= false;
boolean lastsl= false;
boolean lquery= false;
boolean lst2sl= false;
boolean scalea= false;
boolean wantst= false;
boolean wantvs= false;
int hswork= 0;
int i= 0;
int i1= 0;
int i2= 0;
int ibal= 0;
int icond= 0;
intW ierr= new intW(0);
intW ieval= new intW(0);
intW ihi= new intW(0);
intW ilo= new intW(0);
int inxt= 0;
int ip= 0;
int itau= 0;
int iwrk= 0;
int maxwrk= 0;
int minwrk= 0;
double anrm= 0.0d;
doubleW bignum= new doubleW(0.0d);
double cscale= 0.0d;
double eps= 0.0d;
double s= 0.0d;
double sep= 0.0d;
doubleW smlnum= new doubleW(0.0d);
//int [] idum= new int[(1)];
//double [] dum= new double[(1)];
//jobvs = jobvs.substring((1)-1,1);
//sort = sort.substring((1)-1,1);
//  java.lang.reflect.Method _select_meth  = select.getClass().getDeclaredMethods()[0];
//try {
info.val = 0;
lquery = ((lwork == -1));
wantvs = com.polytechnik.lapack.Lsame.lsame(jobvs,'V');
final char sort='N';
wantst = com.polytechnik.lapack.Lsame.lsame(sort,'S');
if ((((!wantvs)) && ((!com.polytechnik.lapack.Lsame.lsame(jobvs,'N')))))  {
    info.val = -1;
}
else if ((((!wantst)) && ((!com.polytechnik.lapack.Lsame.lsame(sort,'N')))))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -6;
}              // Close else if()
else if (((ldvs < 1) || ((wantvs && (ldvs < n)))))  {
    info.val = -11;
/***
*
*     Compute workspace
*      (Note: Comments in the code beginning "Workspace:" describe the
*       minimal amount of workspace needed at that point in the code,
*       as well as the preferred amount for good performance.
*       NB refers to the optimal block size for the immediately
*       following subroutine, as returned by ILAENV.
*       HSWORK refers to the workspace preferred by DHSEQR, as
*       calculated below. HSWORK is computed assuming ILO=1 and IHI=N,
*       the worst case.)
*
***/
}              // Close else if()
if ((info.val == 0))  {
    if ((n == 0))  {
    minwrk = 1;
maxwrk = 1;
}
else  {
//  maxwrk = ((2*n)+(n*com.polytechnik.lapack.Ilaenv.ilaenv(1,"DGEHRD"," ",n,1,n,0)));
// *
minwrk = (3*n);
maxwrk = minwrk;
}              //  Close else.
// *
work[(1-(1))+ _work_offset] = (double)(maxwrk);
if (((lwork < minwrk) && (!lquery)))  {
    info.val = -13;
}
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGEES ",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dgees",999999);
}
else if (lquery)  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dgees",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
if ((n == 0))  {
    //sdim.val = 0;
    return;//Dummy.go_to("com/polytechnik/lapack/Dgees",999999);
/***
*
*     Get machine constants
*
***/
}
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
smlnum.val = com.polytechnik.lapack.Dlamch.dlamch('S');
bignum.val = (1.0/smlnum.val);
com.polytechnik.lapack.Dlabad.dlabad(smlnum,bignum);
smlnum.val = (Math.sqrt(smlnum.val)/eps);
/***
*
*     Scale A if max element outside range [SMLNUM,BIGNUM]
*
***/
bignum.val = (1.0/smlnum.val);
anrm = com.polytechnik.lapack.Dlange.dlange('M',n,n,a,_a_offset,lda,null,0);
scalea = false;
if (((anrm > 0.0) && (anrm < smlnum.val)))  {
    scalea = true;
cscale = smlnum.val;
}
else if ((anrm > bignum.val))  {
    scalea = true;
cscale = bignum.val;
}              // Close else if()
/***
*
*     Permute the matrix to make it more nearly triangular
*     (Workspace: need N)
*
***/
if (scalea) {
    com.polytechnik.lapack.Dlascl.dlascl('G',0,0,anrm,cscale,n,n,a,_a_offset,lda,ierr);
}
    ibal = 1;
/***
*
*     Reduce to upper Hessenberg form
*     (Workspace: need 3*N, prefer 2*N+N*NB)
*
***/
com.polytechnik.lapack.Dgebal.dgebal('P',n,a,_a_offset,lda,ilo,ihi,work,(ibal-(1))+ _work_offset,ierr);
itau = (n+ibal);
iwrk = (n+itau);
// *
com.polytechnik.lapack.Dgehrd.dgehrd(n,ilo.val,ihi.val,a,_a_offset,lda,work,(itau-(1))+ _work_offset,work,(iwrk-(1))+ _work_offset,((lwork-iwrk)+1),ierr);
/***
*
*        Copy Householder vectors to VS
*
***/
if (wantvs)  {
    /***
*
*        Generate orthogonal matrix in VS
*        (Workspace: need 3*N-1, prefer 2*N+(N-1)*NB)
*
***/
com.polytechnik.lapack.Dlacpy.dlacpy('L',n,n,a,_a_offset,lda,vs,_vs_offset,ldvs);
com.polytechnik.lapack.Dorghr.dorghr(n,ilo.val,ihi.val,vs,_vs_offset,ldvs,work,(itau-(1))+ _work_offset,work,(iwrk-(1))+ _work_offset,((lwork-iwrk)+1),ierr);
// *
}
/***
*
*     Perform QR iteration, accumulating Schur vectors in VS if desired
*     (Workspace: need N+1, prefer N+HSWORK (see comments) )
*
***/
//sdim.val = 0;
iwrk = itau;
com.polytechnik.lapack.Dhseqr.dhseqr('S',jobvs,n,ilo.val,ihi.val,a,_a_offset,lda,wr,_wr_offset,wi,_wi_offset,vs,_vs_offset,ldvs,work,(iwrk-(1))+ _work_offset,((lwork-iwrk)+1),ieval);
/***
*
*     Sort eigenvalues if desired
*
***/
if ((ieval.val > 0)) {
    info.val = ieval.val;
}

/***
*
*        Undo balancing
*        (Workspace: need N)
*
***/
if (wantvs)  {
    com.polytechnik.lapack.Dgebak.dgebak('P','R',n,ilo.val,ihi.val,work,(ibal-(1))+ _work_offset,n,vs,_vs_offset,ldvs,ierr);
// *
}
/***
*
*        Undo scaling for the Schur form of A
*
***/
if (scalea)  {
    com.polytechnik.lapack.Dlascl.dlascl('H',0,0,cscale,anrm,n,n,a,_a_offset,lda,ierr);
com.polytechnik.lapack.Dcopy.dcopy(n,a,_a_offset,(lda+1),wr,_wr_offset,1);
/***
*
*           If scaling back towards underflow, adjust WI if an
*           offdiagonal element of a 2-by-2 block in the Schur form
*           underflows.
*
***/
if ((cscale == smlnum.val))  {
    if ((ieval.val > 0))  {
    i1 = (ieval.val+1);
i2 = (ihi.val-1);
com.polytechnik.lapack.Dlascl.dlascl('G',0,0,cscale,anrm,(ilo.val-1),1,wi,_wi_offset,Math.max((ilo.val-1), 1),ierr);
}
else if (wantst)  {
    i1 = 1;
i2 = (n-1);
}              // Close else if()
else  {
  i1 = ilo.val;
i2 = (ihi.val-1);
}              //  Close else.
inxt = (i1-1);
{
label20:for (i = i1; i <= i2; i++) {
if ((i < inxt)) {
    continue label20;//Dummy.go_to("com/polytechnik/lapack/Dgees",20);
}
    if ((wi[(i-(1))+ _wi_offset] == 0.0))  {
    inxt = (i+1);
}
else  {
  if ((a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] == 0.0))  {
    wi[(i-(1))+ _wi_offset] = 0.0;
wi[((i+1)-(1))+ _wi_offset] = 0.0;
}
else if (((a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] != 0.0) && (a[(i-(1))+((i+1)-(1)) * (lda)+ _a_offset] == 0.0)))  {
    wi[(i-(1))+ _wi_offset] = 0.0;
wi[((i+1)-(1))+ _wi_offset] = 0.0;
if ((i > 1)) {
    com.polytechnik.lapack.Dswap.dswap((i-1),a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,1);
}
    if ((n > (i+1))) {
    com.polytechnik.lapack.Dswap.dswap(((n-i)-1),a,(i-(1))+((i+2)-(1)) * (lda)+ _a_offset,lda,a,((i+1)-(1))+((i+2)-(1)) * (lda)+ _a_offset,lda);
}
    if (wantvs)  {
    com.polytechnik.lapack.Dswap.dswap(n,vs,(1-(1))+(i-(1)) * (ldvs)+ _vs_offset,1,vs,(1-(1))+((i+1)-(1)) * (ldvs)+ _vs_offset,1);
}
a[(i-(1))+((i+1)-(1)) * (lda)+ _a_offset] = a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset];
a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] = 0.0;
}              // Close else if()
inxt = (i+2);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dgees",20);
}              //  Close for() loop. 
}
/***
*
*        Undo scaling for the imaginary part of the eigenvalues
*
***/
}
com.polytechnik.lapack.Dlascl.dlascl('G',0,0,cscale,anrm,(n-ieval.val),1,wi,((ieval.val+1)-(1))+ _wi_offset,Math.max((n-ieval.val), 1),ierr);
// *
}

work[(1-(1))+ _work_offset] = (double)(maxwrk);
/***
*
*     End of DGEES
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgees",999999);
//// dummy stmt
//} catch (java.lang.reflect.InvocationTargetException _e) {
//   System.err.println("Error calling method.  "+ _e.getMessage());
//} catch (java.lang.IllegalAccessException _e2) {
//   System.err.println("Error calling method.  "+ _e2.getMessage());
//}
//Dummy.label("com/polytechnik/lapack/Dgees",999999);
return;
   }

} // End class.
