/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgehrd.f
 *  f2java version: 0.8.1
 *
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgehrd.java,v 1.4 2012/12/21 16:32:23 mal Exp $
 */

package com.polytechnik.lapack;


public class Dgehrd {

// *> \brief \b DGEHRD
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DGEHRD + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEHRD( N, ILO, IHI, A, LDA, TAU, WORK, LWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IHI, ILO, INFO, LDA, LWORK, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION  A( LDA, * ), TAU( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEHRD reduces a real general matrix A to upper Hessenberg form H by

// *> an orthogonal similarity transformation:  Q**T * A * Q = H .
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>
// *>          It is assumed that A is already upper triangular in rows
// *>          and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
// *>          set by a previous call to DGEBAL; otherwise they should be
// *>          set to 1 and N respectively. See Further Details.
// *>          1 <= ILO <= IHI <= N, if N > 0; ILO=1 and IHI=0, if N=0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the N-by-N general matrix to be reduced.
// *>          On exit, the upper triangle and the first subdiagonal of A
// *>          are overwritten with the upper Hessenberg matrix H, and the

// *>          elements below the first subdiagonal, with the array TAU,
// *>          represent the orthogonal matrix Q as a product of elementary
// *>          reflectors. See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          The scalar factors of the elementary reflectors (see Further
// *>          Details). Elements 1:ILO-1 and IHI:N-1 of TAU are set to
// *>          zero.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (LWORK)
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The length of the array WORK.  LWORK >= max(1,N).
// *>          For optimum performance LWORK >= N*NB, where NB is the
// *>          optimal blocksize.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleGEcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  The matrix Q is represented as a product of (ihi-ilo) elementary
// *>  reflectors
// *>
// *>     Q = H(ilo) H(ilo+1) . . . H(ihi-1).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i) = 0, v(i+1) = 1 and v(ihi+1:n) = 0; v(i+2:ihi) is stored on
// *>  exit in A(i+2:ihi,i), and tau in TAU(i).
// *>
// *>  The contents of A are illustrated by the following example, with
// *>  n = 7, ilo = 2 and ihi = 6:
// *>
// *>  on entry,                        on exit,
// *>
// *>  ( a   a   a   a   a   a   a )    (  a   a   h   h   h   h   a )
// *>  (     a   a   a   a   a   a )    (      a   h   h   h   h   a )
// *>  (     a   a   a   a   a   a )    (      h   h   h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  h   h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  v3  h   h   h   h )
// *>  (     a   a   a   a   a   a )    (      v2  v3  v4  h   h   h )
// *>  (                         a )    (                          a )
// *>
// *>  where a denotes an element of the original matrix A, h denotes a
// *>  modified element of the upper Hessenberg matrix H, and vi denotes an
// *>  element of the vector defining H(i).
// *>
// *>  This file is a slight modification of LAPACK-3.0's DGEHRD
// *>  subroutine incorporating improvements proposed by Quintana-Orti and

// *>  Van de Geijn (2006). (See DLAHR2.)
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dgehrd (int n,
int ilo,
int ihi,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] work, int _work_offset,
int lwork,
intW info)  {

int iws= 0;


/***
*
*     Use unblocked code to reduce the rest of the matrix
*
***/
com.polytechnik.lapack.Dgehd2.dgehd2(n,ilo,ihi,a,_a_offset,lda,tau,_tau_offset,work,_work_offset,info);
// *
//work[(2-(1))+ _work_offset] = (double)(iws);
/***
*
*     End of DGEHRD
*
***/

return;
   }
} // End class.
