/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dgemv.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dgemv.java,v 1.6 2012/12/20 18:04:34 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dgemv {

// *> \brief \b DGEMV
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DGEMV(TRANS,M,N,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA,BETA
// *       INTEGER INCX,INCY,LDA,M,N
// *       CHARACTER TRANS
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),X(*),Y(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DGEMV  performs one of the matrix-vector operations
// *>
// *>    y := alpha*A*x + beta*y,   or   y := alpha*A**T*x + beta*y,
// *>
// *> where alpha and beta are scalars, x and y are vectors and A is an
// *> m by n matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>           On entry, TRANS specifies the operation to be performed as

// *>           follows:
// *>
// *>              TRANS = 'N' or 'n'   y := alpha*A*x + beta*y.
// *>
// *>              TRANS = 'T' or 't'   y := alpha*A**T*x + beta*y.
// *>
// *>              TRANS = 'C' or 'c'   y := alpha*A**T*x + beta*y.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>           On entry, M specifies the number of rows of the matrix A.
// *>           M must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry, N specifies the number of columns of the matrix A
// *>           N must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *>           Before entry, the leading m by n part of the array A must
// *>           contain the matrix of coefficients.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program. LDA must be at least
// *>           max( 1, m ).
// *> \endverbatim
// *>
// *> \param[in] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array of DIMENSION at least
// *>           ( 1 + ( n - 1 )*abs( INCX ) ) when TRANS = 'N' or 'n'
// *>           and at least
// *>           ( 1 + ( m - 1 )*abs( INCX ) ) otherwise.
// *>           Before entry, the incremented array X must contain the
// *>           vector x.
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>           On entry, INCX specifies the increment for the elements of

// *>           X. INCX must not be zero.
// *> \endverbatim
// *>
// *> \param[in] BETA
// *> \verbatim
// *>          BETA is DOUBLE PRECISION.
// *>           On entry, BETA specifies the scalar beta. When BETA is
// *>           supplied as zero then Y need not be set on input.
// *> \endverbatim
// *>
// *> \param[in,out] Y
// *> \verbatim
// *>          Y is DOUBLE PRECISION array of DIMENSION at least
// *>           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n'
// *>           and at least
// *>           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise.
// *>           Before entry with BETA non-zero, the incremented array Y
// *>           must contain the vector y. On exit, Y is overwritten by the
// *>           updated vector y.
// *> \endverbatim
// *>
// *> \param[in] INCY
// *> \verbatim
// *>          INCY is INTEGER
// *>           On entry, INCY specifies the increment for the elements of

// *>           Y. INCY must not be zero.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level2
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 2 Blas routine.
// *>  The vector and matrix arguments are not referenced when N = 0, or M 
// *>
// *>  -- Written on 22-October-1986.
// *>     Jack Dongarra, Argonne National Lab.
// *>     Jeremy Du Croz, Nag Central Office.
// *>     Sven Hammarling, Nag Central Office.
// *>     Richard Hanson, Sandia National Labs.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dgemv (final char trans,
int m,
int n,
double alpha,
double [] a, int _a_offset,
int lda,
double [] x, int _x_offset,
int incx,
double beta,
double [] y, int _y_offset,
int incy)  {

double temp= 0.0d;
int i= 0;
int info= 0;
int ix= 0;
int iy= 0;
int j= 0;
int jx= 0;
int jy= 0;
int kx= 0;
int ky= 0;
int lenx= 0;
int leny= 0;
//trans = trans.substring((1)-1,1);
info = 0;
if ((((!com.polytechnik.lapack.Lsame.lsame(trans,'N')) && (!com.polytechnik.lapack.Lsame.lsame(trans,'T'))) && (!com.polytechnik.lapack.Lsame.lsame(trans,'C'))))  {
    info = 1;
}
else if ((m < 0))  {
    info = 2;
}              // Close else if()
else if ((n < 0))  {
    info = 3;
}              // Close else if()
else if ((lda < Math.max(1, m)))  {
    info = 6;
}              // Close else if()
else if ((incx == 0))  {
    info = 8;
}              // Close else if()
else if ((incy == 0))  {
    info = 11;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DGEMV ",info);
    return;//Dummy.go_to("com/polytechnik/lapack/Dgemv",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     Set  LENX  and  LENY, the lengths of the vectors x and y, and set
*     up the start points in  X  and  Y.
*
***/
if (((((m == 0)) || ((n == 0))) || ((((alpha == 0.0)) && ((beta == 1.0)))))) {
    //Dummy.go_to("com/polytechnik/lapack/Dgemv",999999);
    return;
}
    if (com.polytechnik.lapack.Lsame.lsame(trans,'N'))  {
    lenx = n;
leny = m;
}
else  {
  lenx = m;
leny = n;
}              //  Close else.
if ((incx > 0))  {
    kx = 1;
}
else  {
  kx = (1-(((lenx-1))*incx));
}              //  Close else.
if ((incy > 0))  {
    ky = 1;
}
else  {
  ky = (1-(((leny-1))*incy));
/***
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
*     First form  y := beta*y.
*
***/
}              //  Close else.
if ((beta != 1.0))  {
    if ((incy == 1))  {
    if ((beta == 0.0))  {
    {
for (i = 1; i <= leny; i++) {
y[(i-(1))+ _y_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dgemv",10);
}              //  Close for() loop. 
}
}
else  {
  {
for (i = 1; i <= leny; i++) {
y[(i-(1))+ _y_offset] = (beta*y[(i-(1))+ _y_offset]);
//Dummy.label("com/polytechnik/lapack/Dgemv",20);
}              //  Close for() loop. 
}
}              //  Close else.
}
else  {
  iy = ky;
if ((beta == 0.0))  {
    {
for (i = 1; i <= leny; i++) {
y[(iy-(1))+ _y_offset] = 0.0;
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack/Dgemv",30);
}              //  Close for() loop. 
}
}
else  {
  {
for (i = 1; i <= leny; i++) {
y[(iy-(1))+ _y_offset] = (beta*y[(iy-(1))+ _y_offset]);
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack/Dgemv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
}
if ((alpha == 0.0)) {
    //Dummy.go_to("com/polytechnik/lapack/Dgemv",999999);
    return;
}
    /***
*
*        Form  y := alpha*A*x + y.
*
***/
if (com.polytechnik.lapack.Lsame.lsame(trans,'N'))  {
    jx = kx;
if ((incy == 1))  {
    {
for (j = 1; j <= n; j++) {
if ((x[(jx-(1))+ _x_offset] != 0.0))  {
    temp = (alpha*x[(jx-(1))+ _x_offset]);
{
for (i = 1; i <= m; i++) {
y[(i-(1))+ _y_offset] = (y[(i-(1))+ _y_offset]+(temp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dgemv",50);
}              //  Close for() loop. 
}
}
jx = (jx+incx);
//Dummy.label("com/polytechnik/lapack/Dgemv",60);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
if ((x[(jx-(1))+ _x_offset] != 0.0))  {
    temp = (alpha*x[(jx-(1))+ _x_offset]);
iy = ky;
{
for (i = 1; i <= m; i++) {
y[(iy-(1))+ _y_offset] = (y[(iy-(1))+ _y_offset]+(temp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack/Dgemv",70);
}              //  Close for() loop. 
}
}
jx = (jx+incx);
//Dummy.label("com/polytechnik/lapack/Dgemv",80);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Form  y := alpha*A**T*x + y.
*
***/
}
else  {
  jy = ky;
if ((incx == 1))  {
    {
for (j = 1; j <= n; j++) {
temp = 0.0;
{
for (i = 1; i <= m; i++) {
temp = (temp+(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*x[(i-(1))+ _x_offset]));
//Dummy.label("com/polytechnik/lapack/Dgemv",90);
}              //  Close for() loop. 
}
y[(jy-(1))+ _y_offset] = (y[(jy-(1))+ _y_offset]+(alpha*temp));
jy = (jy+incy);
//Dummy.label("com/polytechnik/lapack/Dgemv",100);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
temp = 0.0;
ix = kx;
{
for (i = 1; i <= m; i++) {
temp = (temp+(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*x[(ix-(1))+ _x_offset]));
ix = (ix+incx);
//Dummy.label("com/polytechnik/lapack/Dgemv",110);
}              //  Close for() loop. 
}
y[(jy-(1))+ _y_offset] = (y[(jy-(1))+ _y_offset]+(alpha*temp));
jy = (jy+incy);
//Dummy.label("com/polytechnik/lapack/Dgemv",120);
}              //  Close for() loop. 
}
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DGEMV .
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dgemv",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dgemv",999999);
return;
   }
} // End class.
