/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dhseqr.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dhseqr.java,v 1.6 2012/12/22 11:13:24 mal Exp $
 */

package com.polytechnik.lapack;



public class Dhseqr {

// *> \brief \b DHSEQR
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DHSEQR + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DHSEQR( JOB, COMPZ, N, ILO, IHI, H, LDH, WR, WI, Z,
// *                          LDZ, WORK, LWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IHI, ILO, INFO, LDH, LDZ, LWORK, N
// *       CHARACTER          COMPZ, JOB
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   H( LDH, * ), WI( * ), WORK( * ), WR( * ),
// *      $                   Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *>    DHSEQR computes the eigenvalues of a Hessenberg matrix H
// *>    and, optionally, the matrices T and Z from the Schur decomposition
// *>    H = Z T Z**T, where T is an upper quasi-triangular matrix (the
// *>    Schur form), and Z is the orthogonal matrix of Schur vectors.
// *>
// *>    Optionally Z may be postmultiplied into an input orthogonal
// *>    matrix Q so that this routine can give the Schur factorization
// *>    of a matrix A which has been reduced to the Hessenberg form H
// *>    by the orthogonal matrix Q:  A = Q*H*Q**T = (QZ)*T*(QZ)**T.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOB
// *> \verbatim
// *>          JOB is CHARACTER*1
// *>           = 'E':  compute eigenvalues only;
// *>           = 'S':  compute eigenvalues and the Schur form T.
// *> \endverbatim
// *>
// *> \param[in] COMPZ
// *> \verbatim
// *>          COMPZ is CHARACTER*1
// *>           = 'N':  no Schur vectors are computed;
// *>           = 'I':  Z is initialized to the unit matrix and the matrix 
// *>                   of Schur vectors of H is returned;
// *>           = 'V':  Z must contain an orthogonal matrix Q on entry, and
// *>                   the product Q*Z is returned.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           The order of the matrix H.  N .GE. 0.
// *> \endverbatim
// *>
// *> \param[in] ILO
// *> \verbatim
// *>          ILO is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IHI
// *> \verbatim
// *>          IHI is INTEGER
// *>
// *>           It is assumed that H is already upper triangular in rows
// *>           and columns 1:ILO-1 and IHI+1:N. ILO and IHI are normally
// *>           set by a previous call to DGEBAL, and then passed to ZGEHRD
// *>           when the matrix output by DGEBAL is reduced to Hessenberg
// *>           form. Otherwise ILO and IHI should be set to 1 and N
// *>           respectively.  If N.GT.0, then 1.LE.ILO.LE.IHI.LE.N.
// *>           If N = 0, then ILO = 1 and IHI = 0.
// *> \endverbatim
// *>
// *> \param[in,out] H
// *> \verbatim
// *>          H is DOUBLE PRECISION array, dimension (LDH,N)
// *>           On entry, the upper Hessenberg matrix H.
// *>           On exit, if INFO = 0 and JOB = 'S', then H contains the
// *>           upper quasi-triangular matrix T from the Schur decompositio
// *>           (the Schur form); 2-by-2 diagonal blocks (corresponding to

// *>           complex conjugate pairs of eigenvalues) are returned in
// *>           standard form, with H(i,i) = H(i+1,i+1) and
// *>           H(i+1,i)*H(i,i+1).LT.0. If INFO = 0 and JOB = 'E', the
// *>           contents of H are unspecified on exit.  (The output value o
// *>           H when INFO.GT.0 is given under the description of INFO
// *>           below.)
// *>
// *>           Unlike earlier versions of DHSEQR, this subroutine may
// *>           explicitly H(i,j) = 0 for i.GT.j and j = 1, 2, ... ILO-1
// *>           or j = IHI+1, IHI+2, ... N.
// *> \endverbatim
// *>
// *> \param[in] LDH
// *> \verbatim
// *>          LDH is INTEGER
// *>           The leading dimension of the array H. LDH .GE. max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WR
// *> \verbatim
// *>          WR is DOUBLE PRECISION array, dimension (N)
// *> \endverbatim
// *>
// *> \param[out] WI
// *> \verbatim
// *>          WI is DOUBLE PRECISION array, dimension (N)
// *>
// *>           The real and imaginary parts, respectively, of the computed
// *>           eigenvalues. If two eigenvalues are computed as a complex
// *>           conjugate pair, they are stored in consecutive elements of

// *>           WR and WI, say the i-th and (i+1)th, with WI(i) .GT. 0 and

// *>           WI(i+1) .LT. 0. If JOB = 'S', the eigenvalues are stored in
// *>           the same order as on the diagonal of the Schur form returne
// *>           in H, with WR(i) = H(i,i) and, if H(i:i+1,i:i+1) is a 2-by-
// *>           diagonal block, WI(i) = sqrt(-H(i+1,i)*H(i,i+1)) and
// *>           WI(i+1) = -WI(i).
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ,N)
// *>           If COMPZ = 'N', Z is not referenced.
// *>           If COMPZ = 'I', on entry Z need not be set and on exit,
// *>           if INFO = 0, Z contains the orthogonal matrix Z of the Schu
// *>           vectors of H.  If COMPZ = 'V', on entry Z must contain an
// *>           N-by-N matrix Q, which is assumed to be equal to the unit
// *>           matrix except for the submatrix Z(ILO:IHI,ILO:IHI). On exit
// *>           if INFO = 0, Z contains Q*Z.
// *>           Normally Q is the orthogonal matrix generated by DORGHR
// *>           after the call to DGEHRD which formed the Hessenberg matrix
// *>           H. (The output value of Z when INFO.GT.0 is given under
// *>           the description of INFO below.)
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>           The leading dimension of the array Z.  if COMPZ = 'I' or
// *>           COMPZ = 'V', then LDZ.GE.MAX(1,N).  Otherwize, LDZ.GE.1.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (LWORK)
// *>           On exit, if INFO = 0, WORK(1) returns an estimate of
// *>           the optimal value for LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>           The dimension of the array WORK.  LWORK .GE. max(1,N)
// *>           is sufficient and delivers very good and sometimes
// *>           optimal performance.  However, LWORK as large as 11*N
// *>           may be required for optimal performance.  A workspace
// *>           query is recommended to determine the optimal workspace
// *>           size.
// *>
// *>           If LWORK = -1, then DHSEQR does a workspace query.
// *>           In this case, DHSEQR checks the input parameters and
// *>           estimates the optimal workspace size for the given
// *>           values of N, ILO and IHI.  The estimate is returned
// *>           in WORK(1).  No error message related to LWORK is
// *>           issued by XERBLA.  Neither H nor Z are accessed.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>             =  0:  successful exit
// *>           .LT. 0:  if INFO = -i, the i-th argument had an illegal
// *>                    value
// *>           .GT. 0:  if INFO = i, DHSEQR failed to compute all of
// *>                the eigenvalues.  Elements 1:ilo-1 and i+1:n of WR
// *>                and WI contain those eigenvalues which have been
// *>                successfully computed.  (Failures are rare.)
// *>
// *>                If INFO .GT. 0 and JOB = 'E', then on exit, the
// *>                remaining unconverged eigenvalues are the eigen-
// *>                values of the upper Hessenberg matrix rows and
// *>                columns ILO through INFO of the final, output
// *>                value of H.
// *>
// *>                If INFO .GT. 0 and JOB   = 'S', then on exit
// *>
// *>           (*)  (initial value of H)*U  = U*(final value of H)
// *>
// *>                where U is an orthogonal matrix.  The final
// *>                value of H is upper Hessenberg and quasi-triangular
// *>                in rows and columns INFO+1 through IHI.
// *>
// *>                If INFO .GT. 0 and COMPZ = 'V', then on exit
// *>
// *>                  (final value of Z)  =  (initial value of Z)*U
// *>
// *>                where U is the orthogonal matrix in (*) (regard-
// *>                less of the value of JOB.)
// *>
// *>                If INFO .GT. 0 and COMPZ = 'I', then on exit
// *>                      (final value of Z)  = U
// *>                where U is the orthogonal matrix in (*) (regard-
// *>                less of the value of JOB.)
// *>
// *>                If INFO .GT. 0 and COMPZ = 'N', then Z is not
// *>                accessed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleOTHERcomputational
// *
// *> \par Contributors:
// *  ==================
// *>
// *>       Karen Braman and Ralph Byers, Department of Mathematics,
// *>       University of Kansas, USA
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>             Default values supplied by
// *>             ILAENV(ISPEC,'DHSEQR',JOB(:1)//COMPZ(:1),N,ILO,IHI,LWORK)
// *>             It is suggested that these defaults be adjusted in order

// *>             to attain best performance in each particular
// *>             computational environment.
// *>
// *>            ISPEC=12: The DLAHQR vs DLAQR0 crossover point.
// *>                      Default: 75. (Must be at least 11.)
// *>
// *>            ISPEC=13: Recommended deflation window size.
// *>                      This depends on ILO, IHI and NS.  NS is the
// *>                      number of simultaneous shifts returned
// *>                      by ILAENV(ISPEC=15).  (See ISPEC=15 below.)
// *>                      The default for (IHI-ILO+1).LE.500 is NS.
// *>                      The default for (IHI-ILO+1).GT.500 is 3*NS/2.
// *>
// *>            ISPEC=14: Nibble crossover point. (See IPARMQ for
// *>                      details.)  Default: 14% of deflation window
// *>                      size.
// *>
// *>            ISPEC=15: Number of simultaneous shifts in a multishift
// *>                      QR iteration.
// *>
// *>                      If IHI-ILO+1 is ...
// *>
// *>                      greater than      ...but less    ... the
// *>                      or equal to ...      than        default is
// *>
// *>                           1               30          NS =   2(+)
// *>                          30               60          NS =   4(+)
// *>                          60              150          NS =  10(+)
// *>                         150              590          NS =  **
// *>                         590             3000          NS =  64
// *>                        3000             6000          NS = 128
// *>                        6000             infinity      NS = 256
// *>
// *>                  (+)  By default some or all matrices of this order
// *>                       are passed to the implicit double shift routine
// *>                       DLAHQR and this parameter is ignored.  See
// *>                       ISPEC=12 above and comments in IPARMQ for
// *>                       details.
// *>
// *>                 (**)  The asterisks (**) indicate an ad-hoc
// *>                       function of N increasing from 10 to 64.
// *>
// *>            ISPEC=16: Select structured matrix multiply.
// *>                      If the number of simultaneous shifts (specified

// *>                      by ISPEC=15) is less than 14, then the default
// *>                      for ISPEC=16 is 0.  Otherwise the default for
// *>                      ISPEC=16 is 2.
// *> \endverbatim
// *
// *> \par References:
// *  ================
// *>
// *>       K. Braman, R. Byers and R. Mathias, The Multi-Shift QR
// *>       Algorithm Part I: Maintaining Well Focused Shifts, and Level 3

// *>       Performance, SIAM Journal of Matrix Analysis, volume 23, pages

// *>       929--947, 2002.
// *> \n
// *>       K. Braman, R. Byers and R. Mathias, The Multi-Shift QR
// *>       Algorithm Part II: Aggressive Early Deflation, SIAM Journal
// *>       of Matrix Analysis, volume 23, pages 948--973, 2002.
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
*
*     ==== Matrices of order NTINY or smaller must be processed by
*     .    DLAHQR because of insufficient subdiagonal scratch space.
*     .    (This is a hard limit.) ====
***/
/***
*
*     ==== NL allocates some local workspace to help small matrices
*     .    through a rare DLAHQR failure.  NL .GT. NTINY = 11 is
*     .    required and NL .LE. NMIN = ILAENV(ISPEC=12,...) is recom-
*     .    mended.  (The default value of NMIN is 75.)  Using NL = 49
*     .    allows up to six simultaneous shifts and a 16-by-16
*     .    deflation window.  ====
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     ==== Decode and check the input parameters. ====
*
***/

public static void dhseqr (final char job,
final char compz,
int n,
int ilo,
int ihi,
double [] h, int _h_offset,
int ldh,
double [] wr, int _wr_offset,
double [] wi, int _wi_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
intW info)  {

double [] hl= new double[(49) * (49)];
double [] workl= new double[(49)];
int i= 0;
int kbot= 0;
int nmin= 0;
boolean initz= false;
boolean lquery= false;
boolean wantt= false;
boolean wantz= false;
//job = job.substring((1)-1,1);
//compz = compz.substring((1)-1,1);
wantt = com.polytechnik.lapack.Lsame.lsame(job,'S');
initz = com.polytechnik.lapack.Lsame.lsame(compz,'I');
wantz = (initz || com.polytechnik.lapack.Lsame.lsame(compz,'V'));
work[(1-(1))+ _work_offset] = (double)(Math.max(1, n));
// *
lquery = (lwork == -1);
info.val = 0;
if (((!com.polytechnik.lapack.Lsame.lsame(job,'E')) && (!wantt)))  {
    info.val = -1;
}
else if (((!com.polytechnik.lapack.Lsame.lsame(compz,'N')) && (!wantz)))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if (((ilo < 1) || (ilo > Math.max(1, n))))  {
    info.val = -4;
}              // Close else if()
else if (((ihi < Math.min(ilo, n)) || (ihi > n)))  {
    info.val = -5;
}              // Close else if()
else if ((ldh < Math.max(1, n)))  {
    info.val = -7;
}              // Close else if()
else if (((ldz < 1) || ((wantz && (ldz < Math.max(1, n))))))  {
    info.val = -11;
}              // Close else if()
else if (((lwork < Math.max(1, n)) && (!lquery)))  {
    info.val = -13;
// *
}              // Close else if()
/***
*
*        ==== Quick return in case of invalid argument. ====
*
***/
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DHSEQR",(-(info.val)));
// *
    return;//Dummy.go_to("com/polytechnik/lapack/Dhseqr",999999);
/***
*
*        ==== Quick return in case N = 0; nothing to do. ====
*
***/
}
else if ((n == 0))  {
    // *
    return;//Dummy.go_to("com/polytechnik/lapack/Dhseqr",999999);
/***
*
*        ==== Quick return in case of a workspace query ====
*
***/
}              // Close else if()
else if (lquery)  {
    /***
*        ==== Ensure reported workspace size is backward-compatible with
*        .    previous LAPACK versions. ====
***/
/*
com.polytechnik.lapack.Dlaqr0.dlaqr0(wantt,wantz,n,ilo,ihi,h,_h_offset,ldh,wr,_wr_offset,wi,_wi_offset,ilo,ihi,z,_z_offset,ldz,work,_work_offset,lwork,info.val);
work[(1-(1))+ _work_offset] = Math.max((double)(Math.max(1, n)), work[(1-(1))+ _work_offset]);
*/
// *
return;//Dummy.go_to("com/polytechnik/lapack/Dhseqr",999999);
/***
*
*        ==== copy eigenvalues isolated by DGEBAL ====
*
***/
}              // Close else if()
else  {
  {
for (i = 1; i <= (ilo-1); i++) {
wr[(i-(1))+ _wr_offset] = h[(i-(1))+(i-(1)) * (ldh)+ _h_offset];
wi[(i-(1))+ _wi_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dhseqr",10);
}              //  Close for() loop. 
}
{
for (i = (ihi+1); i <= n; i++) {
wr[(i-(1))+ _wr_offset] = h[(i-(1))+(i-(1)) * (ldh)+ _h_offset];
wi[(i-(1))+ _wi_offset] = 0.0;
/***
*
*        ==== Initialize Z, if requested ====
*
***/
//Dummy.label("com/polytechnik/lapack/Dhseqr",20);
}              //  Close for() loop. 
}
/***
*
*        ==== Quick return if possible ====
*
***/
if (initz) {
    com.polytechnik.lapack.Dlaset.dlaset('A',n,n,0.0,1.0,z,_z_offset,ldz);
}
    if ((ilo == ihi))  {
    wr[(ilo-(1))+ _wr_offset] = h[(ilo-(1))+(ilo-(1)) * (ldh)+ _h_offset];
wi[(ilo-(1))+ _wi_offset] = 0.0;
return;//Dummy.go_to("com/polytechnik/lapack/Dhseqr",999999);
/***
*
*        ==== DLAHQR/DLAQR0 crossover point ====
*
***/
}
//nmin = com.polytechnik.lapack.Ilaenv.ilaenv(12,"DHSEQR",(job.substring((1)-1,1)+compz.substring((1)-1,1)),n,ilo,ihi,lwork);
/***
*
*        ==== DLAQR0 for big matrices; DLAHQR for small ones ====
*
***/
nmin = Math.max(11, nmin);
if (false && (n > nmin))  {
//    com.polytechnik.lapack.Dlaqr0.dlaqr0(wantt,wantz,n,ilo,ihi,h,_h_offset,ldh,wr,_wr_offset,wi,_wi_offset,ilo,ihi,z,_z_offset,ldz,work,_work_offset,lwork,info.val);
/***
*
*           ==== Small matrix ====
*
***/
}
else  {
  // *
com.polytechnik.lapack.Dlahqr.dlahqr(wantt,wantz,n,ilo,ihi,h,_h_offset,ldh,wr,_wr_offset,wi,_wi_offset,ilo,ihi,z,_z_offset,ldz,info);
/***
*
*              ==== A rare DLAHQR failure!  DLAQR0 sometimes succeeds
*              .    when DLAHQR fails. ====
*
***/

/***
*
*        ==== Clear out the trash, if necessary. ====
*
***/
}              //  Close else.
/***
*
*        ==== Ensure reported workspace size is backward-compatible with
*        .    previous LAPACK versions. ====
*
***/
if ((((wantt || (info.val != 0))) && (n > 2))) {
    com.polytechnik.lapack.Dlaset.dlaset('L',(n-2),(n-2),0.0,0.0,h,(3-(1))+(1-(1)) * (ldh)+ _h_offset,ldh);
}
    work[(1-(1))+ _work_offset] = Math.max((double)(Math.max(1, n)), work[(1-(1))+ _work_offset]);
/***
*
*     ==== End of DHSEQR ====
*
***/
}              //  Close else.
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dhseqr",999999);
return;
   }
} // End class.
