/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlae2.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlae2.java,v 1.2 2012/12/15 15:24:41 mal Exp $
 *
 */

package com.polytechnik.lapack;


public class Dlae2 {

// *> \brief \b DLAE2 computes the eigenvalues of a 2-by-2 symmetric matrix
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAE2 + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAE2( A, B, C, RT1, RT2 )
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   A, B, C, RT1, RT2
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAE2  computes the eigenvalues of a 2-by-2 symmetric matrix
// *>    [  A   B  ]
// *>    [  B   C  ].
// *> On return, RT1 is the eigenvalue of larger absolute value, and RT2
// *> is the eigenvalue of smaller absolute value.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION
// *>          The (1,1) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION
// *>          The (1,2) and (2,1) elements of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] C
// *> \verbatim
// *>          C is DOUBLE PRECISION
// *>          The (2,2) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[out] RT1
// *> \verbatim
// *>          RT1 is DOUBLE PRECISION
// *>          The eigenvalue of larger absolute value.
// *> \endverbatim
// *>
// *> \param[out] RT2
// *> \verbatim
// *>          RT2 is DOUBLE PRECISION
// *>          The eigenvalue of smaller absolute value.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  RT1 is accurate to a few ulps barring over/underflow.
// *>
// *>  RT2 may be inaccurate if there is massive cancellation in the
// *>  determinant A*C-B*B; higher precision or correctly rounded or
// *>  correctly truncated arithmetic would be needed to compute RT2
// *>  accurately in all cases.
// *>
// *>  Overflow is possible only if RT1 is within a factor of 5 of overflow
// *>  Underflow is harmless if the input data is 0 or exceeds
// *>     underflow_threshold / macheps.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Compute the eigenvalues
*
***/

public static void dlae2 (double a,
double b,
double c,
doubleW rt1,
doubleW rt2)  {

double ab= 0.0d;
double acmn= 0.0d;
double acmx= 0.0d;
double adf= 0.0d;
double df= 0.0d;
double rt= 0.0d;
double sm= 0.0d;
double tb= 0.0d;
sm = (a+c);
df = (a-c);
adf = Math.abs(df);
tb = (b+b);
ab = Math.abs(tb);
if ((Math.abs(a) > Math.abs(c)))  {
    acmx = a;
acmn = c;
}
else  {
  acmx = c;
acmn = a;
}              //  Close else.
if ((adf > ab))  {
    rt = (adf*Math.sqrt((1.0+( Math.pow(((ab/adf)), 2)))));
}
else if ((adf < ab))  {
    rt = (ab*Math.sqrt((1.0+( Math.pow(((adf/ab)), 2)))));
/***
*
*        Includes case AB=ADF=0
*
***/
}              // Close else if()
else  {
  rt = (ab*Math.sqrt(2.0));
}              //  Close else.
if ((sm < 0.0))  {
    /***
*
*        Order of execution important.
*        To get fully accurate smaller eigenvalue,
*        next line needs to be executed in higher precision.
*
***/
rt1.val = (0.5*((sm-rt)));
rt2.val = ((((acmx/rt1.val))*acmn)-(((b/rt1.val))*b));
}
else if ((sm > 0.0))  {
    /***
*
*        Order of execution important.
*        To get fully accurate smaller eigenvalue,
*        next line needs to be executed in higher precision.
*
***/
rt1.val = (0.5*((sm+rt)));
rt2.val = ((((acmx/rt1.val))*acmn)-(((b/rt1.val))*b));
/***
*
*        Includes case RT1 = RT2 = 0
*
***/
}              // Close else if()
else  {
  rt1.val = (0.5*rt);
rt2.val = (-((0.5*rt)));
}              //  Close else.
/***
*
*     End of DLAE2
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlae2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlae2",999999);
return;
   }
} // End class.
