/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaebz.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlaebz.java,v 1.8 2012/12/16 12:23:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlaebz {

// *> \brief \b DLAEBZ computes the number of eigenvalues of a real symmetr
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAEBZ + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAEBZ( IJOB, NITMAX, N, MMAX, MINP, NBMIN, ABSTOL,
// *                          RELTOL, PIVMIN, D, E, E2, NVAL, AB, C, MOUT,

// *                          NAB, WORK, IWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IJOB, INFO, MINP, MMAX, MOUT, N, NBMIN, NITMA
// *       DOUBLE PRECISION   ABSTOL, PIVMIN, RELTOL
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IWORK( * ), NAB( MMAX, * ), NVAL( * )
// *       DOUBLE PRECISION   AB( MMAX, * ), C( * ), D( * ), E( * ), E2( * 
// *      $                   WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAEBZ contains the iteration loops which compute and use the
// *> function N(w), which is the count of eigenvalues of a symmetric
// *> tridiagonal matrix T less than or equal to its argument  w.  It
// *> performs a choice of two types of loops:
// *>
// *> IJOB=1, followed by
// *> IJOB=2: It takes as input a list of intervals and returns a list of
// *>         sufficiently small intervals whose union contains the same
// *>         eigenvalues as the union of the original intervals.
// *>         The input intervals are (AB(j,1),AB(j,2)], j=1,...,MINP.
// *>         The output interval (AB(j,1),AB(j,2)] will contain
// *>         eigenvalues NAB(j,1)+1,...,NAB(j,2), where 1 <= j <= MOUT.
// *>
// *> IJOB=3: It performs a binary search in each input interval
// *>         (AB(j,1),AB(j,2)] for a point  w(j)  such that
// *>         N(w(j))=NVAL(j), and uses  C(j)  as the starting point of
// *>         the search.  If such a w(j) is found, then on output
// *>         AB(j,1)=AB(j,2)=w.  If no such w(j) is found, then on output

// *>         (AB(j,1),AB(j,2)] will be a small interval containing the
// *>         point where N(w) jumps through NVAL(j), unless that point
// *>         lies outside the initial interval.
// *>
// *> Note that the intervals are in all cases half-open intervals,
// *> i.e., of the form  (a,b] , which includes  b  but not  a .
// *>
// *> To avoid underflow, the matrix should be scaled so that its largest
// *> element is no greater than  overflow**(1/2) * underflow**(1/4)
// *> in absolute value.  To assure the most accurate computation
// *> of small eigenvalues, the matrix should be scaled to be
// *> not much smaller than that, either.
// *>
// *> See W. Kahan "Accurate Eigenvalues of a Symmetric Tridiagonal
// *> Matrix", Report CS41, Computer Science Dept., Stanford
// *> University, July 21, 1966
// *>
// *> Note: the arguments are, in general, *not* checked for unreasonable
// *> values.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] IJOB
// *> \verbatim
// *>          IJOB is INTEGER
// *>          Specifies what is to be done:
// *>          = 1:  Compute NAB for the initial intervals.
// *>          = 2:  Perform bisection iteration to find eigenvalues of T.

// *>          = 3:  Perform bisection iteration to invert N(w), i.e.,
// *>                to find a point which has a specified number of
// *>                eigenvalues of T to its left.
// *>          Other values will cause DLAEBZ to return with INFO=-1.
// *> \endverbatim
// *>
// *> \param[in] NITMAX
// *> \verbatim
// *>          NITMAX is INTEGER
// *>          The maximum number of "levels" of bisection to be
// *>          performed, i.e., an interval of width W will not be made
// *>          smaller than 2^(-NITMAX) * W.  If not all intervals
// *>          have converged after NITMAX iterations, then INFO is set
// *>          to the number of non-converged intervals.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The dimension n of the tridiagonal matrix T.  It must be at

// *>          least 1.
// *> \endverbatim
// *>
// *> \param[in] MMAX
// *> \verbatim
// *>          MMAX is INTEGER
// *>          The maximum number of intervals.  If more than MMAX interval
// *>          are generated, then DLAEBZ will quit with INFO=MMAX+1.
// *> \endverbatim
// *>
// *> \param[in] MINP
// *> \verbatim
// *>          MINP is INTEGER
// *>          The initial number of intervals.  It may not be greater than
// *>          MMAX.
// *> \endverbatim
// *>
// *> \param[in] NBMIN
// *> \verbatim
// *>          NBMIN is INTEGER
// *>          The smallest number of intervals that should be processed
// *>          using a vector loop.  If zero, then only the scalar loop
// *>          will be used.
// *> \endverbatim
// *>
// *> \param[in] ABSTOL
// *> \verbatim
// *>          ABSTOL is DOUBLE PRECISION
// *>          The minimum (absolute) width of an interval.  When an
// *>          interval is narrower than ABSTOL, or than RELTOL times the
// *>          larger (in magnitude) endpoint, then it is considered to be

// *>          sufficiently small, i.e., converged.  This must be at least

// *>          zero.
// *> \endverbatim
// *>
// *> \param[in] RELTOL
// *> \verbatim
// *>          RELTOL is DOUBLE PRECISION
// *>          The minimum relative width of an interval.  When an interval
// *>          is narrower than ABSTOL, or than RELTOL times the larger (in
// *>          magnitude) endpoint, then it is considered to be
// *>          sufficiently small, i.e., converged.  Note: this should
// *>          always be at least radix*machine epsilon.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum absolute value of a "pivot" in the Sturm
// *>          sequence loop.
// *>          This must be at least  max |e(j)**2|*safe_min  and at
// *>          least safe_min, where safe_min is at least
// *>          the smallest number that can divide one without overflow.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>          The offdiagonal elements of the tridiagonal matrix T in
// *>          positions 1 through N-1.  E(N) is arbitrary.
// *> \endverbatim
// *>
// *> \param[in] E2
// *> \verbatim
// *>          E2 is DOUBLE PRECISION array, dimension (N)
// *>          The squares of the offdiagonal elements of the tridiagonal
// *>          matrix T.  E2(N) is ignored.
// *> \endverbatim
// *>
// *> \param[in,out] NVAL
// *> \verbatim
// *>          NVAL is INTEGER array, dimension (MINP)
// *>          If IJOB=1 or 2, not referenced.
// *>          If IJOB=3, the desired values of N(w).  The elements of NVAL
// *>          will be reordered to correspond with the intervals in AB.
// *>          Thus, NVAL(j) on output will not, in general be the same as

// *>          NVAL(j) on input, but it will correspond with the interval
// *>          (AB(j,1),AB(j,2)] on output.
// *> \endverbatim
// *>
// *> \param[in,out] AB
// *> \verbatim
// *>          AB is DOUBLE PRECISION array, dimension (MMAX,2)
// *>          The endpoints of the intervals.  AB(j,1) is  a(j), the left

// *>          endpoint of the j-th interval, and AB(j,2) is b(j), the
// *>          right endpoint of the j-th interval.  The input intervals
// *>          will, in general, be modified, split, and reordered by the
// *>          calculation.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (MMAX)
// *>          If IJOB=1, ignored.
// *>          If IJOB=2, workspace.
// *>          If IJOB=3, then on input C(j) should be initialized to the
// *>          first search point in the binary search.
// *> \endverbatim
// *>
// *> \param[out] MOUT
// *> \verbatim
// *>          MOUT is INTEGER
// *>          If IJOB=1, the number of eigenvalues in the intervals.
// *>          If IJOB=2 or 3, the number of intervals output.
// *>          If IJOB=3, MOUT will equal MINP.
// *> \endverbatim
// *>
// *> \param[in,out] NAB
// *> \verbatim
// *>          NAB is INTEGER array, dimension (MMAX,2)
// *>          If IJOB=1, then on output NAB(i,j) will be set to N(AB(i,j))
// *>          If IJOB=2, then on input, NAB(i,j) should be set.  It must
// *>             satisfy the condition:
// *>             N(AB(i,1)) <= NAB(i,1) <= NAB(i,2) <= N(AB(i,2)),
// *>             which means that in interval i only eigenvalues
// *>             NAB(i,1)+1,...,NAB(i,2) will be considered.  Usually,
// *>             NAB(i,j)=N(AB(i,j)), from a previous call to DLAEBZ with

// *>             IJOB=1.
// *>             On output, NAB(i,j) will contain
// *>             max(na(k),min(nb(k),N(AB(i,j)))), where k is the index of
// *>             the input interval that the output interval
// *>             (AB(j,1),AB(j,2)] came from, and na(k) and nb(k) are the

// *>             the input values of NAB(k,1) and NAB(k,2).
// *>          If IJOB=3, then on output, NAB(i,j) contains N(AB(i,j)),
// *>             unless N(w) > NVAL(i) for all search points  w , in which
// *>             case NAB(i,1) will not be modified, i.e., the output
// *>             value will be the same as the input value (modulo
// *>             reorderings -- see NVAL and AB), or unless N(w) < NVAL(i)
// *>             for all search points  w , in which case NAB(i,2) will
// *>             not be modified.  Normally, NAB should be set to some
// *>             distinctive value(s) before DLAEBZ is called.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MMAX)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (MMAX)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:       All intervals converged.
// *>          = 1--MMAX: The last INFO intervals did not converge.
// *>          = MMAX+1:  More than MMAX intervals were generated.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>      This routine is intended to be called only by other LAPACK
// *>  routines, thus the interface is less user-friendly.  It is intended

// *>  for two purposes:
// *>
// *>  (a) finding eigenvalues.  In this case, DLAEBZ should have one or
// *>      more initial intervals set up in AB, and DLAEBZ should be called
// *>      with IJOB=1.  This sets up NAB, and also counts the eigenvalues.
// *>      Intervals with no eigenvalues would usually be thrown out at
// *>      this point.  Also, if not all the eigenvalues in an interval i
// *>      are desired, NAB(i,1) can be increased or NAB(i,2) decreased.
// *>      For example, set NAB(i,1)=NAB(i,2)-1 to get the largest
// *>      eigenvalue.  DLAEBZ is then called with IJOB=2 and MMAX
// *>      no smaller than the value of MOUT returned by the call with
// *>      IJOB=1.  After this (IJOB=2) call, eigenvalues NAB(i,1)+1
// *>      through NAB(i,2) are approximately AB(i,1) (or AB(i,2)) to the
// *>      tolerance specified by ABSTOL and RELTOL.
// *>
// *>  (b) finding an interval (a',b'] containing eigenvalues w(f),...,w(l)
// *>      In this case, start with a Gershgorin interval  (a,b).  Set up
// *>      AB to contain 2 search intervals, both initially (a,b).  One
// *>      NVAL element should contain  f-1  and the other should contain  
// *>      , while C should contain a and b, resp.  NAB(i,1) should be -1
// *>      and NAB(i,2) should be N+1, to flag an error if the desired
// *>      interval does not lie in (a,b).  DLAEBZ is then called with
// *>      IJOB=3.  On exit, if w(f-1) < w(f), then one of the intervals --
// *>      j -- will have AB(j,1)=AB(j,2) and NAB(j,1)=NAB(j,2)=f-1, while

// *>      if, to the specified tolerance, w(f-k)=...=w(f+r), k > 0 and r
// *>      >= 0, then the interval will have  N(AB(j,1))=NAB(j,1)=f-k and
// *>      N(AB(j,2))=NAB(j,2)=f+r.  The cases w(l) < w(l+1) and
// *>      w(l-r)=...=w(l+k) are handled similarly.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Check for Errors
*
***/

public static void dlaebz (int ijob,
int nitmax,
int n,
int mmax,
int minp,
int nbmin,
double abstol,
double reltol,
double pivmin,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] e2, int _e2_offset,
int [] nval, int _nval_offset,
double [] ab, int _ab_offset,
double [] c, int _c_offset,
intW mout,
int [] nab, int _nab_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

int itmp1= 0;
int itmp2= 0;
int j= 0;
int ji= 0;
int jit= 0;
int jp= 0;
int kf= 0;
int kfnew= 0;
int kl= 0;
int klnew= 0;
double tmp1= 0.0d;
double tmp2= 0.0d;
info.val = 0;
if (((ijob < 1) || (ijob > 3)))  {
    info.val = -1;
return;//Dummy.go_to("com/polytechnik/lapack/Dlaebz",999999);
/***
*
*     Initialize NAB
*
***/
}
/***
*
*        Compute the number of eigenvalues in the initial intervals.
*
***/
if ((ijob == 1))  {
    mout.val = 0;
{
for (ji = 1; ji <= minp; ji++) {
{
for (jp = 1; jp <= 2; jp++) {
tmp1 = (d[(1-(1))+ _d_offset]-ab[(ji-(1))+(jp-(1)) * (mmax)+ _ab_offset]);
if ((Math.abs(tmp1) < pivmin)) {
    tmp1 = (-(pivmin));
}
    nab[(ji-(1))+(jp-(1)) * (mmax)+ _nab_offset] = 0;
// *
if ((tmp1 <= 0.0)) {
    nab[(ji-(1))+(jp-(1)) * (mmax)+ _nab_offset] = 1;
}
    {
for (j = 2; j <= n; j++) {
tmp1 = ((d[(j-(1))+ _d_offset]-(e2[((j-1)-(1))+ _e2_offset]/tmp1))-ab[(ji-(1))+(jp-(1)) * (mmax)+ _ab_offset]);
if ((Math.abs(tmp1) < pivmin)) {
    tmp1 = (-(pivmin));
}
    if ((tmp1 <= 0.0)) {
    nab[(ji-(1))+(jp-(1)) * (mmax)+ _nab_offset] = (nab[(ji-(1))+(jp-(1)) * (mmax)+ _nab_offset]+1);
}
    //Dummy.label("com/polytechnik/lapack/Dlaebz",10);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlaebz",20);
}              //  Close for() loop. 
}
mout.val = ((mout.val+nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset])-nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset]);
//Dummy.label("com/polytechnik/lapack/Dlaebz",30);
}              //  Close for() loop. 
}
return;//Dummy.go_to("com/polytechnik/lapack/Dlaebz",999999);
/***
*
*     Initialize for loop
*
*     KF and KL have the following meaning:
*        Intervals 1,...,KF-1 have converged.
*        Intervals KF,...,KL  still need to be refined.
*
***/
}
kf = 1;
/***
*
*     If IJOB=2, initialize C.
*     If IJOB=3, use the user-supplied starting point.
*
***/
kl = minp;
if ((ijob == 2))  {
    {
for (ji = 1; ji <= minp; ji++) {
c[(ji-(1))+ _c_offset] = (0.5*((ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset]+ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset])));
//Dummy.label("com/polytechnik/lapack/Dlaebz",40);
}              //  Close for() loop. 
}
/***
*
*     Iteration loop
*
***/
}
/***
*
*        Loop over intervals
*
***/
{
label130:for (jit = 1; jit <= nitmax; jit++) {

/***
*
*           Begin of Parallel Version of the loop
*
***/
if (((((kl-kf)+1) >= nbmin) && (nbmin > 0)))  {
/***
*
*              Compute N(c), the number of eigenvalues less than c
*
***/
{
for (ji = kf; ji <= kl; ji++) {
work[(ji-(1))+ _work_offset] = (d[(1-(1))+ _d_offset]-c[(ji-(1))+ _c_offset]);
iwork[(ji-(1))+ _iwork_offset] = 0;
if ((work[(ji-(1))+ _work_offset] <= pivmin))  {
    iwork[(ji-(1))+ _iwork_offset] = 1;
work[(ji-(1))+ _work_offset] = Math.min(work[(ji-(1))+ _work_offset], (-(pivmin)));
// *
}
{
for (j = 2; j <= n; j++) {
work[(ji-(1))+ _work_offset] = ((d[(j-(1))+ _d_offset]-(e2[((j-1)-(1))+ _e2_offset]/work[(ji-(1))+ _work_offset]))-c[(ji-(1))+ _c_offset]);
if ((work[(ji-(1))+ _work_offset] <= pivmin))  {
    iwork[(ji-(1))+ _iwork_offset] = (iwork[(ji-(1))+ _iwork_offset]+1);
work[(ji-(1))+ _work_offset] = Math.min(work[(ji-(1))+ _work_offset], (-(pivmin)));
}
//Dummy.label("com/polytechnik/lapack/Dlaebz",50);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlaebz",60);
}              //  Close for() loop. 
}
/***
*
*              IJOB=2: Choose all intervals containing eigenvalues.
*
***/
if ((ijob <= 2))  {
    klnew = kl;
/***
*
*                 Insure that N(w) is monotone
*
***/
{
for (ji = kf; ji <= kl; ji++) {
/***
*
*                 Update the Queue -- add intervals if both halves
*                 contain eigenvalues.
*
***/
iwork[(ji-(1))+ _iwork_offset] = Math.min(nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset], Math.max(nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset], iwork[(ji-(1))+ _iwork_offset]));
/***
*
*                    No eigenvalue in the upper interval:
*                    just use the lower interval.
*
***/
if ((iwork[(ji-(1))+ _iwork_offset] == nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset]))  {
    // *
ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = c[(ji-(1))+ _c_offset];
/***
*
*                    No eigenvalue in the lower interval:
*                    just use the upper interval.
*
***/
}
else if ((iwork[(ji-(1))+ _iwork_offset] == nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset]))  {
    ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset] = c[(ji-(1))+ _c_offset];
}              // Close else if()
else  {
  klnew = (klnew+1);
/***
*
*                       Eigenvalue in both intervals -- add upper to
*                       queue.
*
***/
if ((klnew <= mmax))  {
    ab[(klnew-(1))+(2-(1)) * (mmax)+ _ab_offset] = ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset];
nab[(klnew-(1))+(2-(1)) * (mmax)+ _nab_offset] = nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset];
ab[(klnew-(1))+(1-(1)) * (mmax)+ _ab_offset] = c[(ji-(1))+ _c_offset];
nab[(klnew-(1))+(1-(1)) * (mmax)+ _nab_offset] = iwork[(ji-(1))+ _iwork_offset];
ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = c[(ji-(1))+ _c_offset];
nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset] = iwork[(ji-(1))+ _iwork_offset];
}
else  {
  info.val = (mmax+1);
}              //  Close else.
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlaebz",70);
}              //  Close for() loop. 
}
if ((info.val != 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlaebz",999999);
}
    kl = klnew;
/***
*
*              IJOB=3: Binary search.  Keep only the interval containing
*                      w   s.t. N(w) = NVAL
*
***/
}
else  {
  {
for (ji = kf; ji <= kl; ji++) {
if ((iwork[(ji-(1))+ _iwork_offset] <= nval[(ji-(1))+ _nval_offset]))  {
    ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset] = c[(ji-(1))+ _c_offset];
nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset] = iwork[(ji-(1))+ _iwork_offset];
}
if ((iwork[(ji-(1))+ _iwork_offset] >= nval[(ji-(1))+ _nval_offset]))  {
    ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = c[(ji-(1))+ _c_offset];
nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset] = iwork[(ji-(1))+ _iwork_offset];
}
//Dummy.label("com/polytechnik/lapack/Dlaebz",80);
}              //  Close for() loop. 
}
// *
}              //  Close else.
/***
*
*           End of Parallel Version of the loop
*
*           Begin of Serial Version of the loop
*
***/
}
else  {
  klnew = kl;
/***
*
*              Compute N(w), the number of eigenvalues less than w
*
***/
{
for (ji = kf; ji <= kl; ji++) {
tmp1 = c[(ji-(1))+ _c_offset];
tmp2 = (d[(1-(1))+ _d_offset]-tmp1);
itmp1 = 0;
if ((tmp2 <= pivmin))  {
    itmp1 = 1;
tmp2 = Math.min(tmp2, (-(pivmin)));
// *
}
{
for (j = 2; j <= n; j++) {
tmp2 = ((d[(j-(1))+ _d_offset]-(e2[((j-1)-(1))+ _e2_offset]/tmp2))-tmp1);
if ((tmp2 <= pivmin))  {
    itmp1 = (itmp1+1);
tmp2 = Math.min(tmp2, (-(pivmin)));
}
// *
//Dummy.label("com/polytechnik/lapack/Dlaebz",90);
}              //  Close for() loop. 
}
/***
*
*                 IJOB=2: Choose all intervals containing eigenvalues.
*
*                 Insure that N(w) is monotone
*
***/
if ((ijob <= 2))  {
    /***
*
*                 Update the Queue -- add intervals if both halves
*                 contain eigenvalues.
*
***/
itmp1 = Math.min(nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset], Math.max(nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset], itmp1));
/***
*
*                    No eigenvalue in the upper interval:
*                    just use the lower interval.
*
***/
if ((itmp1 == nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset]))  {
    // *
ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = tmp1;
/***
*
*                    No eigenvalue in the lower interval:
*                    just use the upper interval.
*
***/
}
else if ((itmp1 == nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset]))  {
    ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset] = tmp1;
/***
*
*                    Eigenvalue in both intervals -- add upper to queue.
*
***/
}              // Close else if()
else if ((klnew < mmax))  {
    klnew = (klnew+1);
ab[(klnew-(1))+(2-(1)) * (mmax)+ _ab_offset] = ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset];
nab[(klnew-(1))+(2-(1)) * (mmax)+ _nab_offset] = nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset];
ab[(klnew-(1))+(1-(1)) * (mmax)+ _ab_offset] = tmp1;
nab[(klnew-(1))+(1-(1)) * (mmax)+ _nab_offset] = itmp1;
ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = tmp1;
nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset] = itmp1;
}              // Close else if()
else  {
  info.val = (mmax+1);
  return;//Dummy.go_to("com/polytechnik/lapack/Dlaebz",999999);
}              //  Close else.
/***
*
*                 IJOB=3: Binary search.  Keep only the interval
*                         containing  w  s.t. N(w) = NVAL
*
***/
}
else  {
  if ((itmp1 <= nval[(ji-(1))+ _nval_offset]))  {
    ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset] = tmp1;
nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset] = itmp1;
}
if ((itmp1 >= nval[(ji-(1))+ _nval_offset]))  {
    ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = tmp1;
nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset] = itmp1;
}
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlaebz",100);
}              //  Close for() loop. 
}
// *
kl = klnew;
/***
*
*        Check for convergence
*
***/
}              //  Close else.
kfnew = kf;
{
for (ji = kf; ji <= kl; ji++) {
tmp1 = Math.abs((ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset]-ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset]));
tmp2 = Math.max(Math.abs(ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset]), Math.abs(ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset]));
/***
*
*              Converged -- Swap with position KFNEW,
*                           then increment KFNEW
*
***/
if (((tmp1 < Util.max(abstol, pivmin, (reltol*tmp2))) || (nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset] >= nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset])))  {
    if ((ji > kfnew))  {
    tmp1 = ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset];
tmp2 = ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset];
itmp1 = nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset];
itmp2 = nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset];
ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset] = ab[(kfnew-(1))+(1-(1)) * (mmax)+ _ab_offset];
ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset] = ab[(kfnew-(1))+(2-(1)) * (mmax)+ _ab_offset];
nab[(ji-(1))+(1-(1)) * (mmax)+ _nab_offset] = nab[(kfnew-(1))+(1-(1)) * (mmax)+ _nab_offset];
nab[(ji-(1))+(2-(1)) * (mmax)+ _nab_offset] = nab[(kfnew-(1))+(2-(1)) * (mmax)+ _nab_offset];
ab[(kfnew-(1))+(1-(1)) * (mmax)+ _ab_offset] = tmp1;
ab[(kfnew-(1))+(2-(1)) * (mmax)+ _ab_offset] = tmp2;
nab[(kfnew-(1))+(1-(1)) * (mmax)+ _nab_offset] = itmp1;
nab[(kfnew-(1))+(2-(1)) * (mmax)+ _nab_offset] = itmp2;
if ((ijob == 3))  {
    itmp1 = nval[(ji-(1))+ _nval_offset];
nval[(ji-(1))+ _nval_offset] = nval[(kfnew-(1))+ _nval_offset];
nval[(kfnew-(1))+ _nval_offset] = itmp1;
}
}
kfnew = (kfnew+1);
}
//Dummy.label("com/polytechnik/lapack/Dlaebz",110);
}              //  Close for() loop. 
}
/***
*
*        Choose Midpoints
*
***/
kf = kfnew;
{
for (ji = kf; ji <= kl; ji++) {
c[(ji-(1))+ _c_offset] = (0.5*((ab[(ji-(1))+(1-(1)) * (mmax)+ _ab_offset]+ab[(ji-(1))+(2-(1)) * (mmax)+ _ab_offset])));
/***
*
*        If no more intervals to refine, quit.
*
***/
//Dummy.label("com/polytechnik/lapack/Dlaebz",120);
}              //  Close for() loop. 
}
if ((kf > kl)) {
    //System.err.println("jit="+jit);
    break label130;//Dummy.go_to("com/polytechnik/lapack/Dlaebz",140);
}
    /***
*
*     Converged
*
***/
//Dummy.label("com/polytechnik/lapack/Dlaebz",130);
    } //label130               //  Close for() loop. 
}
//label140: Dummy.label("com/polytechnik/lapack/Dlaebz",140);
info.val = Math.max(((kl+1)-kf), 0);
// *
mout.val = kl;
/***
*
*     End of DLAEBZ
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlaebz",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlaebz",999999);
return;
   }
} // End class.
