/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlaev2.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlaev2.java,v 1.2 2012/12/15 15:25:54 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlaev2 {

// *> \brief \b DLAEV2 computes the eigenvalues and eigenvectors of a 2-by-
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAEV2 + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAEV2( A, B, C, RT1, RT2, CS1, SN1 )
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION   A, B, C, CS1, RT1, RT2, SN1
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAEV2 computes the eigendecomposition of a 2-by-2 symmetric matrix
// *>    [  A   B  ]
// *>    [  B   C  ].
// *> On return, RT1 is the eigenvalue of larger absolute value, RT2 is the
// *> eigenvalue of smaller absolute value, and (CS1,SN1) is the unit right
// *> eigenvector for RT1, giving the decomposition
// *>
// *>    [ CS1  SN1 ] [  A   B  ] [ CS1 -SN1 ]  =  [ RT1  0  ]
// *>    [-SN1  CS1 ] [  B   C  ] [ SN1  CS1 ]     [  0  RT2 ].
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION
// *>          The (1,1) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] B
// *> \verbatim
// *>          B is DOUBLE PRECISION
// *>          The (1,2) element and the conjugate of the (2,1) element of

// *>          the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[in] C
// *> \verbatim
// *>          C is DOUBLE PRECISION
// *>          The (2,2) element of the 2-by-2 matrix.
// *> \endverbatim
// *>
// *> \param[out] RT1
// *> \verbatim
// *>          RT1 is DOUBLE PRECISION
// *>          The eigenvalue of larger absolute value.
// *> \endverbatim
// *>
// *> \param[out] RT2
// *> \verbatim
// *>          RT2 is DOUBLE PRECISION
// *>          The eigenvalue of smaller absolute value.
// *> \endverbatim
// *>
// *> \param[out] CS1
// *> \verbatim
// *>          CS1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] SN1
// *> \verbatim
// *>          SN1 is DOUBLE PRECISION
// *>          The vector (CS1, SN1) is a unit right eigenvector for RT1.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  RT1 is accurate to a few ulps barring over/underflow.
// *>
// *>  RT2 may be inaccurate if there is massive cancellation in the
// *>  determinant A*C-B*B; higher precision or correctly rounded or
// *>  correctly truncated arithmetic would be needed to compute RT2
// *>  accurately in all cases.
// *>
// *>  CS1 and SN1 are accurate to a few ulps barring over/underflow.
// *>
// *>  Overflow is possible only if RT1 is within a factor of 5 of overflow
// *>  Underflow is harmless if the input data is 0 or exceeds
// *>     underflow_threshold / macheps.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Compute the eigenvalues
*
***/

public static void dlaev2 (double a,
double b,
double c,
doubleW rt1,
doubleW rt2,
doubleW cs1,
doubleW sn1)  {

int sgn1= 0;
int sgn2= 0;
double ab= 0.0d;
double acmn= 0.0d;
double acmx= 0.0d;
double acs= 0.0d;
double adf= 0.0d;
double cs= 0.0d;
double ct= 0.0d;
double df= 0.0d;
double rt= 0.0d;
double sm= 0.0d;
double tb= 0.0d;
double tn= 0.0d;
sm = (a+c);
df = (a-c);
adf = Math.abs(df);
tb = (b+b);
ab = Math.abs(tb);
if ((Math.abs(a) > Math.abs(c)))  {
    acmx = a;
acmn = c;
}
else  {
  acmx = c;
acmn = a;
}              //  Close else.
if ((adf > ab))  {
    rt = (adf*Math.sqrt((1.0+( Math.pow(((ab/adf)), 2)))));
}
else if ((adf < ab))  {
    rt = (ab*Math.sqrt((1.0+( Math.pow(((adf/ab)), 2)))));
/***
*
*        Includes case AB=ADF=0
*
***/
}              // Close else if()
else  {
  rt = (ab*Math.sqrt(2.0));
}              //  Close else.
if ((sm < 0.0))  {
    rt1.val = (0.5*((sm-rt)));
/***
*
*        Order of execution important.
*        To get fully accurate smaller eigenvalue,
*        next line needs to be executed in higher precision.
*
***/
sgn1 = -1;
rt2.val = ((((acmx/rt1.val))*acmn)-(((b/rt1.val))*b));
}
else if ((sm > 0.0))  {
    rt1.val = (0.5*((sm+rt)));
/***
*
*        Order of execution important.
*        To get fully accurate smaller eigenvalue,
*        next line needs to be executed in higher precision.
*
***/
sgn1 = 1;
rt2.val = ((((acmx/rt1.val))*acmn)-(((b/rt1.val))*b));
/***
*
*        Includes case RT1 = RT2 = 0
*
***/
}              // Close else if()
else  {
  rt1.val = (0.5*rt);
rt2.val = (-((0.5*rt)));
sgn1 = 1;
/***
*
*     Compute the eigenvector
*
***/
}              //  Close else.
if ((df >= 0.0))  {
    cs = (df+rt);
sgn2 = 1;
}
else  {
  cs = (df-rt);
sgn2 = -1;
}              //  Close else.
acs = Math.abs(cs);
if ((acs > ab))  {
    ct = (-((tb/cs)));
sn1.val = (1.0/Math.sqrt((1.0+(ct*ct))));
cs1.val = (ct*sn1.val);
}
else  {
  if ((ab == 0.0))  {
    cs1.val = 1.0;
sn1.val = 0.0;
}
else  {
  tn = (-((cs/tb)));
cs1.val = (1.0/Math.sqrt((1.0+(tn*tn))));
sn1.val = (tn*cs1.val);
}              //  Close else.
}              //  Close else.
if ((sgn1 == sgn2))  {
    tn = cs1.val;
cs1.val = (-(sn1.val));
sn1.val = tn;
}
/***
*
*     End of DLAEV2
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlaev2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlaev2",999999);
return;
   }
} // End class.
