/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlagtf.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlagtf.java,v 1.3 2012/12/16 14:06:28 mal Exp $
 *
 */

package com.polytechnik.lapack;


public class Dlagtf {

// *> \brief \b DLAGTF computes an LU factorization of a matrix T-λI, wher
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAGTF + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAGTF( N, A, LAMBDA, B, C, TOL, D, IN, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INFO, N
// *       DOUBLE PRECISION   LAMBDA, TOL
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IN( * )
// *       DOUBLE PRECISION   A( * ), B( * ), C( * ), D( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAGTF factorizes the matrix (T - lambda*I), where T is an n by n
// *> tridiagonal matrix and lambda is a scalar, as
// *>
// *>    T - lambda*I = PLU,
// *>
// *> where P is a permutation matrix, L is a unit lower tridiagonal matrix
// *> with at most one non-zero sub-diagonal elements per column and U is
// *> an upper triangular matrix with at most two non-zero super-diagonal
// *> elements per column.
// *>
// *> The factorization is obtained by Gaussian elimination with partial
// *> pivoting and implicit row scaling.
// *>
// *> The parameter LAMBDA is included in the routine so that DLAGTF may
// *> be used, in conjunction with DLAGTS, to obtain eigenvectors of T by
// *> inverse iteration.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix T.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (N)
// *>          On entry, A must contain the diagonal elements of T.
// *>
// *>          On exit, A is overwritten by the n diagonal elements of the

// *>          upper triangular matrix U of the factorization of T.
// *> \endverbatim
// *>
// *> \param[in] LAMBDA
// *> \verbatim
// *>          LAMBDA is DOUBLE PRECISION
// *>          On entry, the scalar lambda.
// *> \endverbatim
// *>
// *> \param[in,out] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, B must contain the (n-1) super-diagonal elements o
// *>          T.
// *>
// *>          On exit, B is overwritten by the (n-1) super-diagonal
// *>          elements of the matrix U of the factorization of T.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (N-1)
// *>          On entry, C must contain the (n-1) sub-diagonal elements of

// *>          T.
// *>
// *>          On exit, C is overwritten by the (n-1) sub-diagonal elements
// *>          of the matrix L of the factorization of T.
// *> \endverbatim
// *>
// *> \param[in] TOL
// *> \verbatim
// *>          TOL is DOUBLE PRECISION
// *>          On entry, a relative tolerance used to indicate whether or
// *>          not the matrix (T - lambda*I) is nearly singular. TOL should
// *>          normally be chose as approximately the largest relative erro
// *>          in the elements of T. For example, if the elements of T are

// *>          correct to about 4 significant figures, then TOL should be
// *>          set to about 5*10**(-4). If TOL is supplied as less than eps
// *>          where eps is the relative machine precision, then the value

// *>          eps is used in place of TOL.
// *> \endverbatim
// *>
// *> \param[out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N-2)
// *>          On exit, D is overwritten by the (n-2) second super-diagonal
// *>          elements of the matrix U of the factorization of T.
// *> \endverbatim
// *>
// *> \param[out] IN
// *> \verbatim
// *>          IN is INTEGER array, dimension (N)
// *>          On exit, IN contains details of the permutation matrix P. If
// *>          an interchange occurred at the kth step of the elimination,

// *>          then IN(k) = 1, otherwise IN(k) = 0. The element IN(n)
// *>          returns the smallest positive integer j such that
// *>
// *>             abs( u(j,j) ).le. norm( (T - lambda*I)(j) )*TOL,
// *>
// *>          where norm( A(j) ) denotes the sum of the absolute values of
// *>          the jth row of the matrix A. If no such j exists then IN(n)

// *>          is returned as zero. If IN(n) is returned as positive, then 
// *>          diagonal element of U is small, indicating that
// *>          (T - lambda*I) is singular or nearly singular,
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0   : successful exit
// *>          .lt. 0: if INFO = -k, the kth argument had an illegal value

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlagtf (int n,
double [] a, int _a_offset,
double lambda,
double [] b, int _b_offset,
double [] c, int _c_offset,
double tol,
double [] d, int _d_offset,
int [] in, int _in_offset,
intW info)  {

int k= 0;
double eps= 0.0d;
double mult= 0.0d;
double piv1= 0.0d;
double piv2= 0.0d;
double scale1= 0.0d;
double scale2= 0.0d;
double temp= 0.0d;
double tl= 0.0d;
info.val = 0;
if ((n < 0))  {
    info.val = -1;
com.polytechnik.lapack.Xerbla.xerbla("DLAGTF",(-(info.val)));
return;//Dummy.go_to("com/polytechnik/lapack/Dlagtf",999999);
// *
}
// *
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagtf",999999);
}
    a[(1-(1))+ _a_offset] = (a[(1-(1))+ _a_offset]-lambda);
in[(n-(1))+ _in_offset] = 0;
if ((n == 1))  {
    if ((a[(1-(1))+ _a_offset] == 0.0)) {
    in[(1-(1))+ _in_offset] = 1;
}
    return;//Dummy.go_to("com/polytechnik/lapack/Dlagtf",999999);
// *
}
// *
eps = com.polytechnik.lapack.Dlamch.dlamch('E');
tl = Math.max(tol, eps);
scale1 = (Math.abs(a[(1-(1))+ _a_offset])+Math.abs(b[(1-(1))+ _b_offset]));
{
for (k = 1; k <= (n-1); k++) {
a[((k+1)-(1))+ _a_offset] = (a[((k+1)-(1))+ _a_offset]-lambda);
scale2 = (Math.abs(c[(k-(1))+ _c_offset])+Math.abs(a[((k+1)-(1))+ _a_offset]));
if ((k < ((n-1)))) {
    scale2 = (scale2+Math.abs(b[((k+1)-(1))+ _b_offset]));
}
    if ((a[(k-(1))+ _a_offset] == 0.0))  {
    piv1 = 0.0;
}
else  {
  piv1 = (Math.abs(a[(k-(1))+ _a_offset])/scale1);
}              //  Close else.
if ((c[(k-(1))+ _c_offset] == 0.0))  {
    in[(k-(1))+ _in_offset] = 0;
piv2 = 0.0;
scale1 = scale2;
if ((k < ((n-1)))) {
    d[(k-(1))+ _d_offset] = 0.0;
}
    }
else  {
  piv2 = (Math.abs(c[(k-(1))+ _c_offset])/scale2);
if ((piv2 <= piv1))  {
    in[(k-(1))+ _in_offset] = 0;
scale1 = scale2;
c[(k-(1))+ _c_offset] = (c[(k-(1))+ _c_offset]/a[(k-(1))+ _a_offset]);
a[((k+1)-(1))+ _a_offset] = (a[((k+1)-(1))+ _a_offset]-(c[(k-(1))+ _c_offset]*b[(k-(1))+ _b_offset]));
if ((k < ((n-1)))) {
    d[(k-(1))+ _d_offset] = 0.0;
}
    }
else  {
  in[(k-(1))+ _in_offset] = 1;
mult = (a[(k-(1))+ _a_offset]/c[(k-(1))+ _c_offset]);
a[(k-(1))+ _a_offset] = c[(k-(1))+ _c_offset];
temp = a[((k+1)-(1))+ _a_offset];
a[((k+1)-(1))+ _a_offset] = (b[(k-(1))+ _b_offset]-(mult*temp));
if ((k < ((n-1))))  {
    d[(k-(1))+ _d_offset] = b[((k+1)-(1))+ _b_offset];
b[((k+1)-(1))+ _b_offset] = (-((mult*d[(k-(1))+ _d_offset])));
}
b[(k-(1))+ _b_offset] = temp;
c[(k-(1))+ _c_offset] = mult;
}              //  Close else.
}              //  Close else.
if ((((Math.max(piv1, piv2) <= tl)) && ((in[(n-(1))+ _in_offset] == 0)))) {
    in[(n-(1))+ _in_offset] = k;
}
//Dummy.label("com/polytechnik/lapack/Dlagtf",10);
}              //  Close for() loop. 
}
// *
if ((((Math.abs(a[(n-(1))+ _a_offset]) <= (scale1*tl))) && ((in[(n-(1))+ _in_offset] == 0)))) {
    in[(n-(1))+ _in_offset] = n;
}
    /***
*
*     End of DLAGTF
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlagtf",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlagtf",999999);
return;
   }
} // End class.
