package com.polytechnik.lapack;


/*
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlamch.java,v 1.4 2012/12/23 19:17:39 mal Exp $
 */
public class Dlamch {


    private static final double eps=0.5*Math.ulp(1.0),sfmin=Double.MIN_NORMAL,base=2;

 
    /**
*>          Specifies the value to be returned by DLAMCH:
*>          = 'E' or 'e',   DLAMCH := eps
*>          = 'S' or 's ,   DLAMCH := sfmin
*>          = 'B' or 'b',   DLAMCH := base
*>          = 'P' or 'p',   DLAMCH := eps*base
*>          = 'N' or 'n',   DLAMCH := t
*>          = 'R' or 'r',   DLAMCH := rnd
*>          = 'M' or 'm',   DLAMCH := emin
*>          = 'U' or 'u',   DLAMCH := rmin
*>          = 'L' or 'l',   DLAMCH := emax
*>          = 'O' or 'o',   DLAMCH := rmax
*>          where
*>          eps   = relative machine precision
*>          sfmin = safe minimum, such that 1/sfmin does not overflow
*>          base  = base of the machine
*>          prec  = eps*base
*>          t     = number of (base) digits in the mantissa
*>          rnd   = 1.0 when rounding occurs in addition, 0.0 otherwise
*>          emin  = minimum exponent before (gradual) underflow
*>          rmin  = underflow threshold - base**(emin-1)
*>          emax  = largest exponent before overflow
*>          rmax  = overflow threshold  - (base**emax)*(1-eps)
*/

    public static double dlamch(final char s)
    {
	if(Lsame.lsame('E',s)) return eps;
	if(Lsame.lsame('S',s)) return sfmin;
	if(Lsame.lsame('B',s)) return base;
	if(Lsame.lsame('P',s)) return eps*base;
	if(Lsame.lsame('U',s)) return sfmin;
	if(true)
	    throw new RuntimeException("s="+s);
	return Double.NaN;
    }

    public static void main(String [] args)
    {
	for(char s:new char[]{'E','S','B','P'})
	{
	    System.err.println(s+"="+dlamch(s));
	}
    }
   
}
