/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlansy.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlansy.java,v 1.5 2012/12/20 04:48:58 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlansy {

// *> \brief \b DLANSY returns the value of the 1-norm, or the Frobenius no
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLANSY + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       DOUBLE PRECISION FUNCTION DLANSY( NORM, UPLO, N, A, LDA, WORK )

// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          NORM, UPLO
// *       INTEGER            LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLANSY  returns the value of the one norm,  or the Frobenius norm, or
// *> the  infinity norm,  or the  element of  largest absolute value  of a
// *> real symmetric matrix A.
// *> \endverbatim
// *>
// *> \return DLANSY
// *> \verbatim
// *>
// *>    DLANSY = ( max(abs(A(i,j))), NORM = 'M' or 'm'
// *>             (
// *>             ( norm1(A),         NORM = '1', 'O' or 'o'
// *>             (
// *>             ( normI(A),         NORM = 'I' or 'i'
// *>             (
// *>             ( normF(A),         NORM = 'F', 'f', 'E' or 'e'
// *>
// *> where  norm1  denotes the  one norm of a matrix (maximum column sum),
// *> normI  denotes the  infinity norm  of a matrix  (maximum row sum) and
// *> normF  denotes the  Frobenius norm of a matrix (square root of sum of
// *> squares).  Note that  max(abs(A(i,j)))  is not a consistent matrix no
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] NORM
// *> \verbatim
// *>          NORM is CHARACTER*1
// *>          Specifies the value to be returned in DLANSY as described
// *>          above.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies whether the upper or lower triangular part of the

// *>          symmetric matrix A is to be referenced.
// *>          = 'U':  Upper triangular part of A is referenced
// *>          = 'L':  Lower triangular part of A is referenced
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.  When N = 0, DLANSY is
// *>          set to zero.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          The symmetric matrix A.  If UPLO = 'U', the leading n by n
// *>          upper triangular part of A contains the upper triangular par
// *>          of the matrix A, and the strictly lower triangular part of A
// *>          is not referenced.  If UPLO = 'L', the leading n by n lower

// *>          triangular part of A contains the lower triangular part of
// *>          the matrix A, and the strictly upper triangular part of A is
// *>          not referenced.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(N,1).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK)),
// *>          where LWORK >= N when NORM = 'I' or '1' or 'O'; otherwise,
// *>          WORK is not referenced.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleSYauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static double dlansy (final char norm,
final char uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] work, int _work_offset)  {

double dlansy= 0.0d;
int i= 0;
int j= 0;
double absa= 0.0d;
doubleW scale= new doubleW(0.0d);
doubleW sum= new doubleW(0.0d);
double value= 0.0d;
//norm = norm.substring((1)-1,1);
//uplo = uplo.substring((1)-1,1);
if ((n == 0))  {
    value = 0.0;
/***
*
*        Find max(abs(A(i,j))).
*
***/
}
else if (com.polytechnik.lapack.Lsame.lsame(norm,'M'))  {
    value = 0.0;
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= j; i++) {
sum.val = Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]);
if ((!(value >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    value = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlansy",10);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlansy",20);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= n; i++) {
sum.val = Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]);
if ((!(value >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    value = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlansy",30);
}              //  Close for() loop. 
}
//Dummy.label("com/polytechnik/lapack/Dlansy",40);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Find normI(A) ( = norm1(A), since A is symmetric).
*
***/
}              // Close else if()
else if ((((com.polytechnik.lapack.Lsame.lsame(norm,'I')) || (com.polytechnik.lapack.Lsame.lsame(norm,'O'))) || ((com.polytechnik.lapack.Lsame.lsame(norm, '1')))))  {
    value = 0.0;
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    {
for (j = 1; j <= n; j++) {
sum.val = 0.0;
{
for (i = 1; i <= (j-1); i++) {
absa = Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]);
sum.val = (sum.val+absa);
work[(i-(1))+ _work_offset] = (work[(i-(1))+ _work_offset]+absa);
//Dummy.label("com/polytechnik/lapack/Dlansy",50);
}              //  Close for() loop. 
}
work[(j-(1))+ _work_offset] = (sum.val+Math.abs(a[(j-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dlansy",60);
}              //  Close for() loop. 
}
{
for (i = 1; i <= n; i++) {
sum.val = work[(i-(1))+ _work_offset];
if ((!(value >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    value = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlansy",70);
}              //  Close for() loop. 
}
}
else  {
  {
for (i = 1; i <= n; i++) {
work[(i-(1))+ _work_offset] = 0.0;
//Dummy.label("com/polytechnik/lapack/Dlansy",80);
}              //  Close for() loop. 
}
{
for (j = 1; j <= n; j++) {
sum.val = (work[(j-(1))+ _work_offset]+Math.abs(a[(j-(1))+(j-(1)) * (lda)+ _a_offset]));
{
for (i = (j+1); i <= n; i++) {
absa = Math.abs(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]);
sum.val = (sum.val+absa);
work[(i-(1))+ _work_offset] = (work[(i-(1))+ _work_offset]+absa);
//Dummy.label("com/polytechnik/lapack/Dlansy",90);
}              //  Close for() loop. 
}
if ((!(value >= sum.val) || com.polytechnik.lapack.Disnan.disnan(sum.val))) {
    value = sum.val;
}
//Dummy.label("com/polytechnik/lapack/Dlansy",100);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Find normF(A).
*
***/
}              // Close else if()
else if (((com.polytechnik.lapack.Lsame.lsame(norm,'F')) || (com.polytechnik.lapack.Lsame.lsame(norm,'E'))))  {
    scale.val = 0.0;
sum.val = 1.0;
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    {
for (j = 2; j <= n; j++) {
com.polytechnik.lapack.Dlassq.dlassq((j-1),a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1,scale,sum);
//Dummy.label("com/polytechnik/lapack/Dlansy",110);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= (n-1); j++) {
com.polytechnik.lapack.Dlassq.dlassq((n-j),a,((j+1)-(1))+(j-(1)) * (lda)+ _a_offset,1,scale,sum);
//Dummy.label("com/polytechnik/lapack/Dlansy",120);
}              //  Close for() loop. 
}
}              //  Close else.
sum.val = (2*sum.val);
com.polytechnik.lapack.Dlassq.dlassq(n,a,_a_offset,(lda+1),scale,sum);
value = (scale.val*Math.sqrt(sum.val));
// *
}              // Close else if()
dlansy = value;
/***
*
*     End of DLANSY
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlansy",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlansy",999999);
return dlansy;
   }
} // End class.
