/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlar1v.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlar1v.java,v 1.2 2012/12/16 05:50:25 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlar1v {

// *> \brief \b DLAR1V computes the (scaled) r-th column of the inverse of 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLAR1V + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLAR1V( N, B1, BN, LAMBDA, D, L, LD, LLD,
// *                  PIVMIN, GAPTOL, Z, WANTNC, NEGCNT, ZTZ, MINGMA,
// *                  R, ISUPPZ, NRMINV, RESID, RQCORR, WORK )
// * 
// *       .. Scalar Arguments ..
// *       LOGICAL            WANTNC
// *       INTEGER   B1, BN, N, NEGCNT, R
// *       DOUBLE PRECISION   GAPTOL, LAMBDA, MINGMA, NRMINV, PIVMIN, RESID
// *      $                   RQCORR, ZTZ
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISUPPZ( * )
// *       DOUBLE PRECISION   D( * ), L( * ), LD( * ), LLD( * ),
// *      $                  WORK( * )
// *       DOUBLE PRECISION Z( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLAR1V computes the (scaled) r-th column of the inverse of
// *> the sumbmatrix in rows B1 through BN of the tridiagonal matrix
// *> L D L**T - sigma I. When sigma is close to an eigenvalue, the
// *> computed vector is an accurate eigenvector. Usually, r corresponds
// *> to the index where the eigenvector is largest in magnitude.
// *> The following steps accomplish this computation :
// *> (a) Stationary qd transform,  L D L**T - sigma I = L(+) D(+) L(+)**T,
// *> (b) Progressive qd transform, L D L**T - sigma I = U(-) D(-) U(-)**T,
// *> (c) Computation of the diagonal elements of the inverse of
// *>     L D L**T - sigma I by combining the above transforms, and choosin
// *>     r as the index where the diagonal of the inverse is (one of the)

// *>     largest in magnitude.
// *> (d) Computation of the (scaled) r-th column of the inverse using the

// *>     twisted factorization obtained by combining the top part of the
// *>     the stationary and the bottom part of the progressive transform.

// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           The order of the matrix L D L**T.
// *> \endverbatim
// *>
// *> \param[in] B1
// *> \verbatim
// *>          B1 is INTEGER
// *>           First index of the submatrix of L D L**T.
// *> \endverbatim
// *>
// *> \param[in] BN
// *> \verbatim
// *>          BN is INTEGER
// *>           Last index of the submatrix of L D L**T.
// *> \endverbatim
// *>
// *> \param[in] LAMBDA
// *> \verbatim
// *>          LAMBDA is DOUBLE PRECISION
// *>           The shift. In order to compute an accurate eigenvector,
// *>           LAMBDA should be a good approximation to an eigenvalue
// *>           of L D L**T.
// *> \endverbatim
// *>
// *> \param[in] L
// *> \verbatim
// *>          L is DOUBLE PRECISION array, dimension (N-1)
// *>           The (n-1) subdiagonal elements of the unit bidiagonal matri
// *>           L, in elements 1 to N-1.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>           The n diagonal elements of the diagonal matrix D.
// *> \endverbatim
// *>
// *> \param[in] LD
// *> \verbatim
// *>          LD is DOUBLE PRECISION array, dimension (N-1)
// *>           The n-1 elements L(i)*D(i).
// *> \endverbatim
// *>
// *> \param[in] LLD
// *> \verbatim
// *>          LLD is DOUBLE PRECISION array, dimension (N-1)
// *>           The n-1 elements L(i)*L(i)*D(i).
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>           The minimum pivot in the Sturm sequence.
// *> \endverbatim
// *>
// *> \param[in] GAPTOL
// *> \verbatim
// *>          GAPTOL is DOUBLE PRECISION
// *>           Tolerance that indicates when eigenvector entries are negli
// *>           w.r.t. their contribution to the residual.
// *> \endverbatim
// *>
// *> \param[in,out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (N)
// *>           On input, all entries of Z must be set to 0.
// *>           On output, Z contains the (scaled) r-th column of the
// *>           inverse. The scaling is such that Z(R) equals 1.
// *> \endverbatim
// *>
// *> \param[in] WANTNC
// *> \verbatim
// *>          WANTNC is LOGICAL
// *>           Specifies whether NEGCNT has to be computed.
// *> \endverbatim
// *>
// *> \param[out] NEGCNT
// *> \verbatim
// *>          NEGCNT is INTEGER
// *>           If WANTNC is .TRUE. then NEGCNT = the number of pivots < pi
// *>           in the  matrix factorization L D L**T, and NEGCNT = -1 othe
// *> \endverbatim
// *>
// *> \param[out] ZTZ
// *> \verbatim
// *>          ZTZ is DOUBLE PRECISION
// *>           The square of the 2-norm of Z.
// *> \endverbatim
// *>
// *> \param[out] MINGMA
// *> \verbatim
// *>          MINGMA is DOUBLE PRECISION
// *>           The reciprocal of the largest (in magnitude) diagonal
// *>           element of the inverse of L D L**T - sigma I.
// *> \endverbatim
// *>
// *> \param[in,out] R
// *> \verbatim
// *>          R is INTEGER
// *>           The twist index for the twisted factorization used to
// *>           compute Z.
// *>           On input, 0 <= R <= N. If R is input as 0, R is set to
// *>           the index where (L D L**T - sigma I)^{-1} is largest
// *>           in magnitude. If 1 <= R <= N, R is unchanged.
// *>           On output, R contains the twist index used to compute Z.
// *>           Ideally, R designates the position of the maximum entry in 
// *>           eigenvector.
// *> \endverbatim
// *>
// *> \param[out] ISUPPZ
// *> \verbatim
// *>          ISUPPZ is INTEGER array, dimension (2)
// *>           The support of the vector in Z, i.e., the vector Z is
// *>           nonzero only in elements ISUPPZ(1) through ISUPPZ( 2 ).
// *> \endverbatim
// *>
// *> \param[out] NRMINV
// *> \verbatim
// *>          NRMINV is DOUBLE PRECISION
// *>           NRMINV = 1/SQRT( ZTZ )
// *> \endverbatim
// *>
// *> \param[out] RESID
// *> \verbatim
// *>          RESID is DOUBLE PRECISION
// *>           The residual of the FP vector.
// *>           RESID = ABS( MINGMA )/SQRT( ZTZ )
// *> \endverbatim
// *>
// *> \param[out] RQCORR
// *> \verbatim
// *>          RQCORR is DOUBLE PRECISION
// *>           The Rayleigh Quotient correction to LAMBDA.
// *>           RQCORR = MINGMA*TMP
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (4*N)
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***

*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
/***


***/

public static void dlar1v (int n,
int b1,
int bn,
double lambda,
double [] d, int _d_offset,
double [] l, int _l_offset,
double [] ld, int _ld_offset,
double [] lld, int _lld_offset,
double pivmin,
double gaptol,
double [] z, int _z_offset,
boolean wantnc,
intW negcnt,
doubleW ztz,
doubleW mingma,
intW r,
int [] isuppz, int _isuppz_offset,
doubleW nrminv,
doubleW resid,
doubleW rqcorr,
double [] work, int _work_offset)  {

boolean sawnan1= false;
boolean sawnan2= false;
int i= 0;
int indlpl= 0;
int indp= 0;
int inds= 0;
int indumn= 0;
int neg1= 0;
int neg2= 0;
int r1= 0;
int r2= 0;
double dminus= 0.0d;
double dplus= 0.0d;
double eps= 0.0d;
double s= 0.0d;
double tmp= 0.0d;
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
if ((r.val == 0))  {
    r1 = b1;
r2 = bn;
}
else  {
  r1 = r.val;
r2 = r.val;
/***

*     Storage for LPLUS
***/
}              //  Close else.
// *     Storage for UMINUS
indlpl = 0;
indumn = n;
inds = ((2*n)+1);
// 
indp = ((3*n)+1);
if ((b1 == 1))  {
    work[(inds-(1))+ _work_offset] = 0.0;
}
else  {
  work[(((inds+b1)-1)-(1))+ _work_offset] = lld[((b1-1)-(1))+ _lld_offset];
/***

*
*     Compute the stationary transform (using the differential form)
*     until the index R2.
*
***/
}              //  Close else.
sawnan1 = false;
neg1 = 0;
s = (work[(((inds+b1)-1)-(1))+ _work_offset]-lambda);
{
for (i = b1; i <= (r1-1); i++) {
dplus = (d[(i-(1))+ _d_offset]+s);
work[((indlpl+i)-(1))+ _work_offset] = (ld[(i-(1))+ _ld_offset]/dplus);
if ((dplus < 0.0)) {
    neg1 = (neg1+1);
}
    work[((inds+i)-(1))+ _work_offset] = ((s*work[((indlpl+i)-(1))+ _work_offset])*l[(i-(1))+ _l_offset]);
s = (work[((inds+i)-(1))+ _work_offset]-lambda);
//Dummy.label("com/polytechnik/lapack/Dlar1v",50);
}              //  Close for() loop. 
}
sawnan1 = com.polytechnik.lapack.Disnan.disnan(s);
if (sawnan1) {
//    Dummy.go_to("com/polytechnik/lapack/Dlar1v",60);
}else{
    {
for (i = r1; i <= (r2-1); i++) {
dplus = (d[(i-(1))+ _d_offset]+s);
work[((indlpl+i)-(1))+ _work_offset] = (ld[(i-(1))+ _ld_offset]/dplus);
work[((inds+i)-(1))+ _work_offset] = ((s*work[((indlpl+i)-(1))+ _work_offset])*l[(i-(1))+ _l_offset]);
s = (work[((inds+i)-(1))+ _work_offset]-lambda);
//Dummy.label("com/polytechnik/lapack/Dlar1v",51);
}              //  Close for() loop. 
}
// *
sawnan1 = com.polytechnik.lapack.Disnan.disnan(s);
//label60:   Dummy.label("com/polytechnik/lapack/Dlar1v",60);
}
// *        Runs a slower version of the above loop if a NaN is detected
if (sawnan1)  {
    neg1 = 0;
s = (work[(((inds+b1)-1)-(1))+ _work_offset]-lambda);
{
for (i = b1; i <= (r1-1); i++) {
dplus = (d[(i-(1))+ _d_offset]+s);
if ((Math.abs(dplus) < pivmin)) {
    dplus = (-(pivmin));
}
    work[((indlpl+i)-(1))+ _work_offset] = (ld[(i-(1))+ _ld_offset]/dplus);
if ((dplus < 0.0)) {
    neg1 = (neg1+1);
}
    work[((inds+i)-(1))+ _work_offset] = ((s*work[((indlpl+i)-(1))+ _work_offset])*l[(i-(1))+ _l_offset]);
if ((work[((indlpl+i)-(1))+ _work_offset] == 0.0)) {
    work[((inds+i)-(1))+ _work_offset] = lld[(i-(1))+ _lld_offset];
}
    s = (work[((inds+i)-(1))+ _work_offset]-lambda);
//Dummy.label("com/polytechnik/lapack/Dlar1v",70);
}              //  Close for() loop. 
}
{
for (i = r1; i <= (r2-1); i++) {
dplus = (d[(i-(1))+ _d_offset]+s);
if ((Math.abs(dplus) < pivmin)) {
    dplus = (-(pivmin));
}
    work[((indlpl+i)-(1))+ _work_offset] = (ld[(i-(1))+ _ld_offset]/dplus);
work[((inds+i)-(1))+ _work_offset] = ((s*work[((indlpl+i)-(1))+ _work_offset])*l[(i-(1))+ _l_offset]);
if ((work[((indlpl+i)-(1))+ _work_offset] == 0.0)) {
    work[((inds+i)-(1))+ _work_offset] = lld[(i-(1))+ _lld_offset];
}
    s = (work[((inds+i)-(1))+ _work_offset]-lambda);
//Dummy.label("com/polytechnik/lapack/Dlar1v",71);
}              //  Close for() loop. 
}
/***
*
*     Compute the progressive transform (using the differential form)
*     until the index R1
*
***/
}
sawnan2 = false;
neg2 = 0;
work[(((indp+bn)-1)-(1))+ _work_offset] = (d[(bn-(1))+ _d_offset]-lambda);
{
int _i_inc = -1;
for (i = (bn-1); i >= r1; i += _i_inc) {
dminus = (lld[(i-(1))+ _lld_offset]+work[((indp+i)-(1))+ _work_offset]);
tmp = (d[(i-(1))+ _d_offset]/dminus);
if ((dminus < 0.0)) {
    neg2 = (neg2+1);
}
    work[((indumn+i)-(1))+ _work_offset] = (l[(i-(1))+ _l_offset]*tmp);
work[(((indp+i)-1)-(1))+ _work_offset] = ((work[((indp+i)-(1))+ _work_offset]*tmp)-lambda);
//Dummy.label("com/polytechnik/lapack/Dlar1v",80);
}              //  Close for() loop. 
}
tmp = work[(((indp+r1)-1)-(1))+ _work_offset];
// 
sawnan2 = com.polytechnik.lapack.Disnan.disnan(tmp);
// *        Runs a slower version of the above loop if a NaN is detected
if (sawnan2)  {
    neg2 = 0;
{
int _i_inc = -1;
for (i = (bn-1); i >= r1; i += _i_inc) {
dminus = (lld[(i-(1))+ _lld_offset]+work[((indp+i)-(1))+ _work_offset]);
if ((Math.abs(dminus) < pivmin)) {
    dminus = (-(pivmin));
}
    tmp = (d[(i-(1))+ _d_offset]/dminus);
if ((dminus < 0.0)) {
    neg2 = (neg2+1);
}
    work[((indumn+i)-(1))+ _work_offset] = (l[(i-(1))+ _l_offset]*tmp);
work[(((indp+i)-1)-(1))+ _work_offset] = ((work[((indp+i)-(1))+ _work_offset]*tmp)-lambda);
if ((tmp == 0.0)) {
    work[(((indp+i)-1)-(1))+ _work_offset] = (d[(i-(1))+ _d_offset]-lambda);
}
//    Dummy.label("com/polytechnik/lapack/Dlar1v",100);
}              //  Close for() loop. 
}
/***
*
*     Find the index (from R1 to R2) of the largest (in magnitude)
*     diagonal element of the inverse
*
***/
}
mingma.val = (work[(((inds+r1)-1)-(1))+ _work_offset]+work[(((indp+r1)-1)-(1))+ _work_offset]);
if ((mingma.val < 0.0)) {
    neg1 = (neg1+1);
}
    if (wantnc)  {
    negcnt.val = (neg1+neg2);
}
else  {
  negcnt.val = -1;
}              //  Close else.
if ((Math.abs(mingma.val) == 0.0)) {
    mingma.val = (eps*work[(((inds+r1)-1)-(1))+ _work_offset]);
}
    r.val = r1;
{
for (i = r1; i <= (r2-1); i++) {
tmp = (work[((inds+i)-(1))+ _work_offset]+work[((indp+i)-(1))+ _work_offset]);
if ((tmp == 0.0)) {
    tmp = (eps*work[((inds+i)-(1))+ _work_offset]);
}
    if ((Math.abs(tmp) <= Math.abs(mingma.val)))  {
    mingma.val = tmp;
r.val = (i+1);
}
/***
*
*     Compute the FP vector: solve N^T v = e_r
*
***/
//Dummy.label("com/polytechnik/lapack/Dlar1v",110);
}              //  Close for() loop. 
}
isuppz[(1-(1))+ _isuppz_offset] = b1;
isuppz[(2-(1))+ _isuppz_offset] = bn;
z[(r.val-(1))+ _z_offset] = 1.0;
/***
*
*     Compute the FP vector upwards from R
*
***/
ztz.val = 1.0;
if (((!sawnan1) && (!sawnan2)))  {
    {
int _i_inc = -1;
label210:for (i = (r.val-1); i >= b1; i += _i_inc) {
z[(i-(1))+ _z_offset] = (-(((work[((indlpl+i)-(1))+ _work_offset]*z[((i+1)-(1))+ _z_offset]))));
if (((((Math.abs(z[(i-(1))+ _z_offset])+Math.abs(z[((i+1)-(1))+ _z_offset])))*Math.abs(ld[(i-(1))+ _ld_offset])) < gaptol))  {
    z[(i-(1))+ _z_offset] = 0.0;
isuppz[(1-(1))+ _isuppz_offset] = (i+1);
break label210;//Dummy.go_to("com/polytechnik/lapack/Dlar1v",220);
}
ztz.val = (ztz.val+(z[(i-(1))+ _z_offset]*z[(i-(1))+ _z_offset]));
//Dummy.label("com/polytechnik/lapack/Dlar1v",210);
}              //  Close for() loop. 
}
//label220:  Dummy.label("com/polytechnik/lapack/Dlar1v",220);
// *        Run slower loop if NaN occurred.
}
else  {
  {
int _i_inc = -1;
label230:for (i = (r.val-1); i >= b1; i += _i_inc) {
if ((z[((i+1)-(1))+ _z_offset] == 0.0))  {
    z[(i-(1))+ _z_offset] = (-((((ld[((i+1)-(1))+ _ld_offset]/ld[(i-(1))+ _ld_offset]))*z[((i+2)-(1))+ _z_offset])));
}
else  {
  z[(i-(1))+ _z_offset] = (-(((work[((indlpl+i)-(1))+ _work_offset]*z[((i+1)-(1))+ _z_offset]))));
}              //  Close else.
if (((((Math.abs(z[(i-(1))+ _z_offset])+Math.abs(z[((i+1)-(1))+ _z_offset])))*Math.abs(ld[(i-(1))+ _ld_offset])) < gaptol))  {
    z[(i-(1))+ _z_offset] = 0.0;
isuppz[(1-(1))+ _isuppz_offset] = (i+1);
break label230;//Dummy.go_to("com/polytechnik/lapack/Dlar1v",240);
}
ztz.val = (ztz.val+(z[(i-(1))+ _z_offset]*z[(i-(1))+ _z_offset]));
//Dummy.label("com/polytechnik/lapack/Dlar1v",230);
}              //  Close for() loop. 
}
//label240: Dummy.label("com/polytechnik/lapack/Dlar1v",240);
/***

*     Compute the FP vector downwards from R in blocks of size BLKSIZ
***/
}              //  Close else.
if (((!sawnan1) && (!sawnan2)))  {
    {
label250:for (i = r.val; i <= (bn-1); i++) {
z[((i+1)-(1))+ _z_offset] = (-(((work[((indumn+i)-(1))+ _work_offset]*z[(i-(1))+ _z_offset]))));
if (((((Math.abs(z[(i-(1))+ _z_offset])+Math.abs(z[((i+1)-(1))+ _z_offset])))*Math.abs(ld[(i-(1))+ _ld_offset])) < gaptol))  {
    z[((i+1)-(1))+ _z_offset] = 0.0;
isuppz[(2-(1))+ _isuppz_offset] = i;
break label250;//Dummy.go_to("com/polytechnik/lapack/Dlar1v",260);
}
ztz.val = (ztz.val+(z[((i+1)-(1))+ _z_offset]*z[((i+1)-(1))+ _z_offset]));
//Dummy.label("com/polytechnik/lapack/Dlar1v",250);
}              //  Close for() loop. 
}
//label260: Dummy.label("com/polytechnik/lapack/Dlar1v",260);
// *        Run slower loop if NaN occurred.
}
else  {
  {
label270:for (i = r.val; i <= (bn-1); i++) {
if ((z[(i-(1))+ _z_offset] == 0.0))  {
    z[((i+1)-(1))+ _z_offset] = (-((((ld[((i-1)-(1))+ _ld_offset]/ld[(i-(1))+ _ld_offset]))*z[((i-1)-(1))+ _z_offset])));
}
else  {
  z[((i+1)-(1))+ _z_offset] = (-(((work[((indumn+i)-(1))+ _work_offset]*z[(i-(1))+ _z_offset]))));
}              //  Close else.
if (((((Math.abs(z[(i-(1))+ _z_offset])+Math.abs(z[((i+1)-(1))+ _z_offset])))*Math.abs(ld[(i-(1))+ _ld_offset])) < gaptol))  {
    z[((i+1)-(1))+ _z_offset] = 0.0;
isuppz[(2-(1))+ _isuppz_offset] = i;
break label270;//Dummy.go_to("com/polytechnik/lapack/Dlar1v",280);
}
ztz.val = (ztz.val+(z[((i+1)-(1))+ _z_offset]*z[((i+1)-(1))+ _z_offset]));
//Dummy.label("com/polytechnik/lapack/Dlar1v",270);
}              //  Close for() loop. 
}
//label280: Dummy.label("com/polytechnik/lapack/Dlar1v",280);
/***
*
*     Compute quantities for convergence test
*
***/
}              //  Close else.
tmp = (1.0/ztz.val);
nrminv.val = Math.sqrt(tmp);
resid.val = (Math.abs(mingma.val)*nrminv.val);
/***
*
*
***/
rqcorr.val = (mingma.val*tmp);
/***
*
*     End of DLAR1V
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlar1v",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlar1v",999999);
return;
   }
} // End class.
