/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarfg.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarfg.java,v 1.4 2012/12/19 22:00:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarfg {

// *> \brief \b DLARFG generates an elementary reflector (Householder matri
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARFG + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARFG( N, ALPHA, X, INCX, TAU )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            INCX, N
// *       DOUBLE PRECISION   ALPHA, TAU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   X( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARFG generates a real elementary reflector H of order n, such
// *> that
// *>
// *>       H * ( alpha ) = ( beta ),   H**T * H = I.
// *>           (   x   )   (   0  )
// *>
// *> where alpha and beta are scalars, and x is an (n-1)-element real
// *> vector. H is represented in the form
// *>
// *>       H = I - tau * ( 1 ) * ( 1 v**T ) ,
// *>                     ( v )
// *>
// *> where tau is a real scalar and v is a real (n-1)-element
// *> vector.
// *>
// *> If the elements of x are all zero, then tau = 0 and H is taken to be

// *> the unit matrix.
// *>
// *> Otherwise  1 <= tau <= 2.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the elementary reflector.
// *> \endverbatim
// *>
// *> \param[in,out] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION
// *>          On entry, the value alpha.
// *>          On exit, it is overwritten with the value beta.
// *> \endverbatim
// *>
// *> \param[in,out] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array, dimension
// *>                         (1+(N-2)*abs(INCX))
// *>          On entry, the vector x.
// *>          On exit, it is overwritten with the vector v.
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>          The increment between elements of X. INCX > 0.
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION
// *>          The value tau.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarfg (int n,
doubleW alpha,
double [] x, int _x_offset,
int incx,
doubleW tau)  {

int j= 0;
int knt= 0;
double beta= 0.0d;
double rsafmn= 0.0d;
double safmin= 0.0d;
double xnorm= 0.0d;
if ((n <= 1))  {
    tau.val = 0.0;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarfg",999999);
// *
}
// *
xnorm = com.polytechnik.lapack.Dnrm2.dnrm2((n-1),x,_x_offset,incx);
/***
*
*        H  =  I
*
***/
if ((xnorm == 0.0))  {
    tau.val = 0.0;
/***
*
*        general case
*
***/
}
else  {
  beta = (-(Util.dsign(com.polytechnik.lapack.Dlapy2.dlapy2(alpha.val,xnorm),alpha.val)));
safmin = (com.polytechnik.lapack.Dlamch.dlamch('S')/com.polytechnik.lapack.Dlamch.dlamch('E'));
knt = 0;
/***
*
*           XNORM, BETA may be inaccurate; scale X and recompute them
*
***/
if ((Math.abs(beta) < safmin))  {
    rsafmn = (1.0/safmin);
label10:for(int jknt=0;jknt<Util.ITMAX;jknt++){ //Dummy.label("com/polytechnik/lapack/Dlarfg",10);
knt = (knt+1);
com.polytechnik.lapack.Dscal.dscal((n-1),rsafmn,x,_x_offset,incx);
beta = (beta*rsafmn);
alpha.val = (alpha.val*rsafmn);
/***
*
*           New BETA is at most 1, at least SAFMIN
*
***/
if (!(Math.abs(beta) < safmin)) {
    break label10;//Dummy.go_to("com/polytechnik/lapack/Dlarfg",10);
}
}// end label10

    xnorm = com.polytechnik.lapack.Dnrm2.dnrm2((n-1),x,_x_offset,incx);
beta = (-(Util.dsign(com.polytechnik.lapack.Dlapy2.dlapy2(alpha.val,xnorm),alpha.val)));
}
tau.val = (((beta-alpha.val))/beta);
/***
*
*        If ALPHA is subnormal, it may lose relative accuracy
*
***/
com.polytechnik.lapack.Dscal.dscal((n-1),(1.0/((alpha.val-beta))),x,_x_offset,incx);
{
for (j = 1; j <= knt; j++) {
beta = (beta*safmin);
//Dummy.label("com/polytechnik/lapack/Dlarfg",20);
}              //  Close for() loop. 
}
alpha.val = beta;
// *
}              //  Close else.
/***
*
*     End of DLARFG
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarfg",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarfg",999999);
return;
   }

} // End class.
