/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrb.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrb.java,v 1.4 2012/12/19 22:00:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrb {

// *> \brief \b DLARRB provides limited bisection to locate eigenvalues for
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRB + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRB( N, D, LLD, IFIRST, ILAST, RTOL1,
// *                          RTOL2, OFFSET, W, WGAP, WERR, WORK, IWORK,
// *                          PIVMIN, SPDIAM, TWIST, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IFIRST, ILAST, INFO, N, OFFSET, TWIST
// *       DOUBLE PRECISION   PIVMIN, RTOL1, RTOL2, SPDIAM
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IWORK( * )
// *       DOUBLE PRECISION   D( * ), LLD( * ), W( * ),
// *      $                   WERR( * ), WGAP( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Given the relatively robust representation(RRR) L D L^T, DLARRB
// *> does "limited" bisection to refine the eigenvalues of L D L^T,
// *> W( IFIRST-OFFSET ) through W( ILAST-OFFSET ), to more accuracy. Initi
// *> guesses for these eigenvalues are input in W, the corresponding estim
// *> of the error in these guesses and their gaps are input in WERR
// *> and WGAP, respectively. During bisection, intervals
// *> [left, right] are maintained by storing their mid-points and
// *> semi-widths in the arrays W and WERR respectively.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The N diagonal elements of the diagonal matrix D.
// *> \endverbatim
// *>
// *> \param[in] LLD
// *> \verbatim
// *>          LLD is DOUBLE PRECISION array, dimension (N-1)
// *>          The (N-1) elements L(i)*L(i)*D(i).
// *> \endverbatim
// *>
// *> \param[in] IFIRST
// *> \verbatim
// *>          IFIRST is INTEGER
// *>          The index of the first eigenvalue to be computed.
// *> \endverbatim
// *>
// *> \param[in] ILAST
// *> \verbatim
// *>          ILAST is INTEGER
// *>          The index of the last eigenvalue to be computed.
// *> \endverbatim
// *>
// *> \param[in] RTOL1
// *> \verbatim
// *>          RTOL1 is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] RTOL2
// *> \verbatim
// *>          RTOL2 is DOUBLE PRECISION
// *>          Tolerance for the convergence of the bisection intervals.
// *>          An interval [LEFT,RIGHT] has converged if
// *>          RIGHT-LEFT.LT.MAX( RTOL1*GAP, RTOL2*MAX(|LEFT|,|RIGHT|) )
// *>          where GAP is the (estimated) distance to the nearest
// *>          eigenvalue.
// *> \endverbatim
// *>
// *> \param[in] OFFSET
// *> \verbatim
// *>          OFFSET is INTEGER
// *>          Offset for the arrays W, WGAP and WERR, i.e., the IFIRST-OFF
// *>          through ILAST-OFFSET elements of these arrays are to be used
// *> \endverbatim
// *>
// *> \param[in,out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          On input, W( IFIRST-OFFSET ) through W( ILAST-OFFSET ) are
// *>          estimates of the eigenvalues of L D L^T indexed IFIRST throu
// *>          ILAST.
// *>          On output, these estimates are refined.
// *> \endverbatim
// *>
// *> \param[in,out] WGAP
// *> \verbatim
// *>          WGAP is DOUBLE PRECISION array, dimension (N-1)
// *>          On input, the (estimated) gaps between consecutive
// *>          eigenvalues of L D L^T, i.e., WGAP(I-OFFSET) is the gap betw
// *>          eigenvalues I and I+1. Note that if IFIRST.EQ.ILAST
// *>          then WGAP(IFIRST-OFFSET) must be set to ZERO.
// *>          On output, these gaps are refined.
// *> \endverbatim
// *>
// *> \param[in,out] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION array, dimension (N)
// *>          On input, WERR( IFIRST-OFFSET ) through WERR( ILAST-OFFSET )
// *>          the errors in the estimates of the corresponding elements in
// *>          On output, these errors are refined.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (2*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (2*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot in the Sturm sequence.
// *> \endverbatim
// *>
// *> \param[in] SPDIAM
// *> \verbatim
// *>          SPDIAM is DOUBLE PRECISION
// *>          The spectral diameter of the matrix.
// *> \endverbatim
// *>
// *> \param[in] TWIST
// *> \verbatim
// *>          TWIST is INTEGER
// *>          The twist index for the twisted factorization that is used
// *>          for the negcount.
// *>          TWIST = N: Compute negcount from L D L^T - LAMBDA I = L+ D+ 
// *>          TWIST = 1: Compute negcount from L D L^T - LAMBDA I = U- D- 
// *>          TWIST = R: Compute negcount from L D L^T - LAMBDA I = N(r) D
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          Error flag.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
// *

public static void dlarrb (int n,
double [] d, int _d_offset,
double [] lld, int _lld_offset,
int ifirst,
int ilast,
double rtol1,
double rtol2,
int offset,
double [] w, int _w_offset,
double [] wgap, int _wgap_offset,
double [] werr, int _werr_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
double pivmin,
double spdiam,
int twist,
intW info)  {

final int NLOOPMAX=2*n+Util.ITMAX;

int maxitr= 0;
int i= 0;
int i1= 0;
int ii= 0;
int ip= 0;
int iter= 0;
int k= 0;
int negcnt= 0;
int next= 0;
int nint= 0;
int olnint= 0;
int prev= 0;
int r= 0;
double back= 0.0d;
double cvrgd= 0.0d;
double gap= 0.0d;
double left= 0.0d;
double lgap= 0.0d;
double mid= 0.0d;
double mnwdth= 0.0d;
double rgap= 0.0d;
double right= 0.0d;
double tmp= 0.0d;
double width= 0.0d;
info.val = 0;
maxitr = ((int)((((Math.log((spdiam+pivmin))-Math.log(pivmin)))/Math.log(2.0)))+2);
// *
mnwdth = (2.0*pivmin);
r = twist;
/***
*
*     Initialize unconverged intervals in [ WORK(2*I-1), WORK(2*I) ].
*     The Sturm Count, Count( WORK(2*I-1) ) is arranged to be I-1, while
*     Count( WORK(2*I) ) is stored in IWORK( 2*I ). The integer IWORK( 2*I-1 )
*     for an unconverged interval is set to the index of the next unconverged
*     interval, and is -1 or 0 for a converged interval. Thus a linked
*     list of unconverged intervals is set up.
*
***/
if ((((r < 1)) || ((r > n)))) {
    r = n;
}
    // *     The number of unconverged intervals
i1 = ifirst;
// *     The last unconverged interval found
nint = 0;
// 
prev = 0;
rgap = wgap[((i1-offset)-(1))+ _wgap_offset];
{
for (i = i1; i <= ilast; i++) {
k = (2*i);
ii = (i-offset);
left = (w[(ii-(1))+ _w_offset]-werr[(ii-(1))+ _werr_offset]);
right = (w[(ii-(1))+ _w_offset]+werr[(ii-(1))+ _werr_offset]);
lgap = rgap;
rgap = wgap[(ii-(1))+ _wgap_offset];
/***

*        Make sure that [LEFT,RIGHT] contains the desired eigenvalue
*        Compute negcount from dstqds facto L+D+L+^T = L D L^T - LEFT
*
*        Do while( NEGCNT(LEFT).GT.I-1 )
*
***/
gap = Math.min(lgap, rgap);
back = werr[(ii-(1))+ _werr_offset];
label20:for(int jj20=0;jj20<NLOOPMAX;jj20++){
    //Dummy.label("com/polytechnik/lapack/Dlarrb",20);
negcnt = com.polytechnik.lapack.Dlaneg.dlaneg(n,d,_d_offset,lld,_lld_offset,left,pivmin,r);
if ((negcnt > (i-1)))  {
    left = (left-back);
back = (2.0*back);
continue label20;//Dummy.go_to("com/polytechnik/lapack/Dlarrb",20);
/***
*
*        Do while( NEGCNT(RIGHT).LT.I )
*        Compute negcount from dstqds facto L+D+L+^T = L D L^T - RIGHT
*
***/
}
break label20;}

back = werr[(ii-(1))+ _werr_offset];
// 
label50:for(int jj50=0;jj50<NLOOPMAX;jj50++){
    //Dummy.label("com/polytechnik/lapack/Dlarrb",50);
negcnt = com.polytechnik.lapack.Dlaneg.dlaneg(n,d,_d_offset,lld,_lld_offset,right,pivmin,r);
if ((negcnt < i))  {
    right = (right+back);
back = (2.0*back);
continue label50;//Dummy.go_to("com/polytechnik/lapack/Dlarrb",50);
}
break label50;}
width = (0.5*Math.abs((left-right)));
tmp = Math.max(Math.abs(left), Math.abs(right));
cvrgd = Math.max((rtol1*gap), (rtol2*tmp));
/***
*           This interval has already converged and does not need refinement.
*           (Note that the gaps might change through refining the
*            eigenvalues, however, they can only get bigger.)
*           Remove it from the list.
***/
if (((width <= cvrgd) || (width <= mnwdth)))  {
    // *           Make sure that I1 always points to the first unconverged interval
iwork[((k-1)-(1))+ _iwork_offset] = -1;
if ((((i == i1)) && ((i < ilast)))) {
    i1 = (i+1);
}
    if ((((prev >= i1)) && ((i <= ilast)))) {
    iwork[(((2*prev)-1)-(1))+ _iwork_offset] = (i+1);
}
    // *           unconverged interval found
}
else  {
  prev = i;
nint = (nint+1);
iwork[((k-1)-(1))+ _iwork_offset] = (i+1);
iwork[(k-(1))+ _iwork_offset] = negcnt;
}              //  Close else.
work[((k-1)-(1))+ _work_offset] = left;
work[(k-(1))+ _work_offset] = right;
/***

*
*     Do while( NINT.GT.0 ), i.e. there are still unconverged intervals
*     and while (ITER.LT.MAXITR)
*
***/
//Dummy.label("com/polytechnik/lapack/Dlarrb",75);
}              //  Close for() loop. 
}
iter = 0;
    label80:for(int jj80=0;jj80<NLOOPMAX;jj80++){
    //Dummy.label("com/polytechnik/lapack/Dlarrb",80);
prev = (i1-1);
i = i1;
// 
olnint = nint;
{
label100:for (ip = 1; ip <= olnint; ip++) {
k = (2*i);
ii = (i-offset);
rgap = wgap[(ii-(1))+ _wgap_offset];
lgap = rgap;
if ((ii > 1)) {
    lgap = wgap[((ii-1)-(1))+ _wgap_offset];
}
    gap = Math.min(lgap, rgap);
next = iwork[((k-1)-(1))+ _iwork_offset];
left = work[((k-1)-(1))+ _work_offset];
right = work[(k-(1))+ _work_offset];
/***

*        semiwidth of interval
***/
mid = (0.5*((left+right)));
width = (right-mid);
tmp = Math.max(Math.abs(left), Math.abs(right));
cvrgd = Math.max((rtol1*gap), (rtol2*tmp));
// *           reduce number of unconverged intervals
if (((((width <= cvrgd)) || ((width <= mnwdth))) || ((iter == maxitr))))  {
    // *           Mark interval as converged.
nint = (nint-1);
iwork[((k-1)-(1))+ _iwork_offset] = 0;
if ((i1 == i))  {
    i1 = next;
// *              Prev holds the last unconverged interval previously examined
}
else  {
  if ((prev >= i1)) {
    iwork[(((2*prev)-1)-(1))+ _iwork_offset] = next;
}
    }              //  Close else.
i = next;
continue label100;//Dummy.go_to("com/polytechnik/lapack/Dlarrb",100);
}
/***
*
*        Perform one bisection step
*
***/
prev = i;
negcnt = com.polytechnik.lapack.Dlaneg.dlaneg(n,d,_d_offset,lld,_lld_offset,mid,pivmin,r);
if ((negcnt <= (i-1)))  {
    work[((k-1)-(1))+ _work_offset] = mid;
}
else  {
  work[(k-(1))+ _work_offset] = mid;
}              //  Close else.
i = next;
continue label100;//Dummy.label("com/polytechnik/lapack/Dlarrb",100);
}              //  Close for() loop. 
}
/***
*     do another loop if there are still unconverged intervals
*     However, in the last iteration, all intervals are accepted
*     since this is the best we can do.
***/
iter = (iter+1);
/***
*
*
*     At this point, all the intervals have converged
***/
if ((((nint > 0)) && ((iter <= maxitr)))) {
    continue label80;//Dummy.go_to("com/polytechnik/lapack/Dlarrb",80);
}
break label80;}

    {
for (i = ifirst; i <= ilast; i++) {
k = (2*i);
// *        All intervals marked by '0' have been refined.
ii = (i-offset);
if ((iwork[((k-1)-(1))+ _iwork_offset] == 0))  {
    w[(ii-(1))+ _w_offset] = (0.5*((work[((k-1)-(1))+ _work_offset]+work[(k-(1))+ _work_offset])));
werr[(ii-(1))+ _werr_offset] = (work[(k-(1))+ _work_offset]-w[(ii-(1))+ _w_offset]);
}
// *
//Dummy.label("com/polytechnik/lapack/Dlarrb",110);
}              //  Close for() loop. 
}
{
for (i = (ifirst+1); i <= ilast; i++) {
k = (2*i);
ii = (i-offset);
wgap[((ii-1)-(1))+ _wgap_offset] = Math.max(0.0, (((w[(ii-(1))+ _w_offset]-werr[(ii-(1))+ _werr_offset])-w[((ii-1)-(1))+ _w_offset])-werr[((ii-1)-(1))+ _werr_offset]));
// 
//Dummy.label("com/polytechnik/lapack/Dlarrb",111);
}              //  Close for() loop. 
}
/***
*
*     End of DLARRB
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrb",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrb",999999);
return;
   }
} // End class.
