/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrc.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrc.java,v 1.2 2012/12/15 15:28:05 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrc {

// *> \brief \b DLARRC computes the number of eigenvalues of the symmetric 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRC + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRC( JOBT, N, VL, VU, D, E, PIVMIN,
// *                                   EIGCNT, LCNT, RCNT, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBT
// *       INTEGER            EIGCNT, INFO, LCNT, N, RCNT
// *       DOUBLE PRECISION   PIVMIN, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), E( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Find the number of eigenvalues of the symmetric tridiagonal matrix T

// *> that are in the interval (VL,VU] if JOBT = 'T', and of L D L^T
// *> if JOBT = 'L'.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBT
// *> \verbatim
// *>          JOBT is CHARACTER*1
// *>          = 'T':  Compute Sturm count for matrix T.
// *>          = 'L':  Compute Sturm count for matrix L D L^T.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix. N > 0.
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          The lower and upper bounds for the eigenvalues.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          JOBT = 'T': The N diagonal elements of the tridiagonal matri
// *>          JOBT = 'L': The N diagonal elements of the diagonal matrix D
// *> \endverbatim
// *>
// *> \param[in] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N)
// *>          JOBT = 'T': The N-1 offdiagonal elements of the matrix T.
// *>          JOBT = 'L': The N-1 offdiagonal elements of the matrix L.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot in the Sturm sequence for T.
// *> \endverbatim
// *>
// *> \param[out] EIGCNT
// *> \verbatim
// *>          EIGCNT is INTEGER
// *>          The number of eigenvalues of the symmetric tridiagonal matri
// *>          that are in the interval (VL,VU]
// *> \endverbatim
// *>
// *> \param[out] LCNT
// *> \verbatim
// *>          LCNT is INTEGER
// *> \endverbatim
// *>
// *> \param[out] RCNT
// *> \verbatim
// *>          RCNT is INTEGER
// *>          The left and right negcounts of the interval.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***

*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/

public static void dlarrc (final char jobt,
int n,
double vl,
double vu,
double [] d, int _d_offset,
double [] e, int _e_offset,
double pivmin,
intW eigcnt,
intW lcnt,
intW rcnt,
intW info)  {

int i= 0;
boolean matt= false;
double lpivot= 0.0d;
double rpivot= 0.0d;
double sl= 0.0d;
double su= 0.0d;
double tmp= 0.0d;
double tmp2= 0.0d;
//jobt = jobt.substring((1)-1,1);
info.val = 0;
lcnt.val = 0;
rcnt.val = 0;
eigcnt.val = 0;
/***


***/
matt = com.polytechnik.lapack.Lsame.lsame(jobt,'T');
// *        Sturm sequence count on T
if (matt)  {
    lpivot = (d[(1-(1))+ _d_offset]-vl);
rpivot = (d[(1-(1))+ _d_offset]-vu);
if ((lpivot <= 0.0))  {
    lcnt.val = (lcnt.val+1);
}
if ((rpivot <= 0.0))  {
    rcnt.val = (rcnt.val+1);
}
{
for (i = 1; i <= (n-1); i++) {
tmp = ( Math.pow(e[(i-(1))+ _e_offset], 2));
lpivot = (((d[((i+1)-(1))+ _d_offset]-vl))-(tmp/lpivot));
rpivot = (((d[((i+1)-(1))+ _d_offset]-vu))-(tmp/rpivot));
if ((lpivot <= 0.0))  {
    lcnt.val = (lcnt.val+1);
}
if ((rpivot <= 0.0))  {
    rcnt.val = (rcnt.val+1);
}
//Dummy.label("com/polytechnik/lapack/Dlarrc",10);
}              //  Close for() loop. 
}
// *        Sturm sequence count on L D L^T
}
else  {
  sl = (-(vl));
su = (-(vu));
{
for (i = 1; i <= (n-1); i++) {
lpivot = (d[(i-(1))+ _d_offset]+sl);
rpivot = (d[(i-(1))+ _d_offset]+su);
if ((lpivot <= 0.0))  {
    lcnt.val = (lcnt.val+1);
}
if ((rpivot <= 0.0))  {
    rcnt.val = (rcnt.val+1);
}
// *
tmp = ((e[(i-(1))+ _e_offset]*d[(i-(1))+ _d_offset])*e[(i-(1))+ _e_offset]);
tmp2 = (tmp/lpivot);
if ((tmp2 == 0.0))  {
    sl = (tmp-vl);
}
else  {
  sl = ((sl*tmp2)-vl);
// *
}              //  Close else.
tmp2 = (tmp/rpivot);
if ((tmp2 == 0.0))  {
    su = (tmp-vu);
}
else  {
  su = ((su*tmp2)-vu);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlarrc",20);
}              //  Close for() loop. 
}
lpivot = (d[(n-(1))+ _d_offset]+sl);
rpivot = (d[(n-(1))+ _d_offset]+su);
if ((lpivot <= 0.0))  {
    lcnt.val = (lcnt.val+1);
}
if ((rpivot <= 0.0))  {
    rcnt.val = (rcnt.val+1);
}
}              //  Close else.
// 
eigcnt.val = (rcnt.val-lcnt.val);
/***
*
*     end of DLARRC
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrc",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrc",999999);
return;
   }
} // End class.
