/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrd.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrd.java,v 1.4 2012/12/16 08:30:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrd {

// *> \brief \b DLARRD computes the eigenvalues of a symmetric tridiagonal 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRD + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRD( RANGE, ORDER, N, VL, VU, IL, IU, GERS,
// *                           RELTOL, D, E, E2, PIVMIN, NSPLIT, ISPLIT,
// *                           M, W, WERR, WL, WU, IBLOCK, INDEXW,
// *                           WORK, IWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          ORDER, RANGE
// *       INTEGER            IL, INFO, IU, M, N, NSPLIT
// *       DOUBLE PRECISION    PIVMIN, RELTOL, VL, VU, WL, WU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IBLOCK( * ), INDEXW( * ),
// *      $                   ISPLIT( * ), IWORK( * )
// *       DOUBLE PRECISION   D( * ), E( * ), E2( * ),
// *      $                   GERS( * ), W( * ), WERR( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARRD computes the eigenvalues of a symmetric tridiagonal
// *> matrix T to suitable accuracy. This is an auxiliary code to be
// *> called from DSTEMR.
// *> The user may ask for all eigenvalues, all eigenvalues
// *> in the half-open interval (VL, VU], or the IL-th through IU-th
// *> eigenvalues.
// *>
// *> To avoid overflow, the matrix must be scaled so that its
// *> largest element is no greater than overflow**(1/2) * underflow**(1/4)
// *> accuracy, it should not be much smaller than that.
// *>
// *> See W. Kahan "Accurate Eigenvalues of a Symmetric Tridiagonal
// *> Matrix", Report CS41, Computer Science Dept., Stanford
// *> University, July 21, 1966.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': ("All")   all eigenvalues will be found.
// *>          = 'V': ("Value") all eigenvalues in the half-open interval
// *>                           (VL, VU] will be found.
// *>          = 'I': ("Index") the IL-th through IU-th eigenvalues (of the
// *>                           entire matrix) will be found.
// *> \endverbatim
// *>
// *> \param[in] ORDER
// *> \verbatim
// *>          ORDER is CHARACTER*1
// *>          = 'B': ("By Block") the eigenvalues will be grouped by
// *>                              split-off block (see IBLOCK, ISPLIT) and
// *>                              ordered from smallest to largest within

// *>                              the block.
// *>          = 'E': ("Entire matrix")
// *>                              the eigenvalues for the entire matrix
// *>                              will be ordered from smallest to
// *>                              largest.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the tridiagonal matrix T.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          If RANGE='V', the lower and upper bounds of the interval to

// *>          be searched for eigenvalues.  Eigenvalues less than or equal
// *>          to VL, or greater than VU, will not be returned.  VL < VU.
// *>          Not referenced if RANGE = 'A' or 'I'.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N, if N > 0; IL = 1 and IU = 0 if N = 0.
// *>          Not referenced if RANGE = 'A' or 'V'.
// *> \endverbatim
// *>
// *> \param[in] GERS
// *> \verbatim
// *>          GERS is DOUBLE PRECISION array, dimension (2*N)
// *>          The N Gerschgorin intervals (the i-th Gerschgorin interval
// *>          is (GERS(2*i-1), GERS(2*i)).
// *> \endverbatim
// *>
// *> \param[in] RELTOL
// *> \verbatim
// *>          RELTOL is DOUBLE PRECISION
// *>          The minimum relative width of an interval.  When an interval
// *>          is narrower than RELTOL times the larger (in
// *>          magnitude) endpoint, then it is considered to be
// *>          sufficiently small, i.e., converged.  Note: this should
// *>          always be at least radix*machine epsilon.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The n diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          The (n-1) off-diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in] E2
// *> \verbatim
// *>          E2 is DOUBLE PRECISION array, dimension (N-1)
// *>          The (n-1) squared off-diagonal elements of the tridiagonal m
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot allowed in the Sturm sequence for T.
// *> \endverbatim
// *>
// *> \param[in] NSPLIT
// *> \verbatim
// *>          NSPLIT is INTEGER
// *>          The number of diagonal blocks in the matrix T.
// *>          1 <= NSPLIT <= N.
// *> \endverbatim
// *>
// *> \param[in] ISPLIT
// *> \verbatim
// *>          ISPLIT is INTEGER array, dimension (N)
// *>          The splitting points, at which T breaks up into submatrices.
// *>          The first submatrix consists of rows/columns 1 to ISPLIT(1),
// *>          the second of rows/columns ISPLIT(1)+1 through ISPLIT(2),
// *>          etc., and the NSPLIT-th consists of rows/columns
// *>          ISPLIT(NSPLIT-1)+1 through ISPLIT(NSPLIT)=N.
// *>          (Only the first NSPLIT elements will actually be used, but
// *>          since the user cannot know a priori what value NSPLIT will
// *>          have, N words must be reserved for ISPLIT.)
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The actual number of eigenvalues found. 0 <= M <= N.
// *>          (See also the description of INFO=2,3.)
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          On exit, the first M elements of W will contain the
// *>          eigenvalue approximations. DLARRD computes an interval
// *>          I_j = (a_j, b_j] that includes eigenvalue j. The eigenvalue

// *>          approximation is given as the interval midpoint
// *>          W(j)= ( a_j + b_j)/2. The corresponding error is bounded by

// *>          WERR(j) = abs( a_j - b_j)/2
// *> \endverbatim
// *>
// *> \param[out] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION array, dimension (N)
// *>          The error bound on the corresponding eigenvalue approximatio
// *>          in W.
// *> \endverbatim
// *>
// *> \param[out] WL
// *> \verbatim
// *>          WL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] WU
// *> \verbatim
// *>          WU is DOUBLE PRECISION
// *>          The interval (WL, WU] contains all the wanted eigenvalues.
// *>          If RANGE='V', then WL=VL and WU=VU.
// *>          If RANGE='A', then WL and WU are the global Gerschgorin boun
// *>                        on the spectrum.
// *>          If RANGE='I', then WL and WU are computed by DLAEBZ from the
// *>                        index range specified.
// *> \endverbatim
// *>
// *> \param[out] IBLOCK
// *> \verbatim
// *>          IBLOCK is INTEGER array, dimension (N)
// *>          At each row/column j where E(j) is zero or small, the
// *>          matrix T is considered to split into a block diagonal
// *>          matrix.  On exit, if INFO = 0, IBLOCK(i) specifies to which

// *>          block (from 1 to the number of blocks) the eigenvalue W(i)
// *>          belongs.  (DLARRD may use the remaining N-M elements as
// *>          workspace.)
// *> \endverbatim
// *>
// *> \param[out] INDEXW
// *> \verbatim
// *>          INDEXW is INTEGER array, dimension (N)
// *>          The indices of the eigenvalues within each block (submatrix)
// *>          for example, INDEXW(i)= j and IBLOCK(i)=k imply that the
// *>          i-th eigenvalue W(i) is the j-th eigenvalue in block k.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (4*N)
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (3*N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  some or all of the eigenvalues failed to converge or
// *>                were not computed:
// *>                =1 or 3: Bisection failed to converge for some
// *>                        eigenvalues; these eigenvalues are flagged by 
// *>                        negative block number.  The effect is that the
// *>                        eigenvalues may not be as accurate as the
// *>                        absolute and relative tolerances.  This is
// *>                        generally caused by unexpectedly inaccurate
// *>                        arithmetic.
// *>                =2 or 3: RANGE='I' only: Not all of the eigenvalues
// *>                        IL:IU were found.
// *>                        Effect: M < IU+1-IL
// *>                        Cause:  non-monotonic arithmetic, causing the

// *>                                Sturm sequence to be non-monotonic.
// *>                        Cure:   recalculate, using RANGE='A', and pick
// *>                                out eigenvalues IL:IU.  In some cases,
// *>                                increasing the PARAMETER "FUDGE" may
// *>                                make things work.
// *>                = 4:    RANGE='I', and the Gershgorin interval
// *>                        initially used was too small.  No eigenvalues

// *>                        were computed.
// *>                        Probable cause: your machine has sloppy
// *>                                        floating-point arithmetic.
// *>                        Cure: Increase the PARAMETER "FUDGE",
// *>                              recompile, and try again.
// *> \endverbatim
// *
// *> \par Internal Parameters:
// *  =========================
// *>
// *> \verbatim
// *>  FUDGE   DOUBLE PRECISION, default = 2
// *>          A "fudge factor" to widen the Gershgorin intervals.  Ideally
// *>          a value of 1 should work, but on machines with sloppy
// *>          arithmetic, this needs to be larger.  The default for
// *>          publicly released versions should be large enough to handle

// *>          the worst machine around.  Note that this has no effect
// *>          on accuracy of the solution.
// *> \endverbatim
// *>
// *> \par Contributors:
// *  ==================
// *>
// *>     W. Kahan, University of California, Berkeley, USA \n
// *>     Beresford Parlett, University of California, Berkeley, USA \n
// *>     Jim Demmel, University of California, Berkeley, USA \n
// *>     Inderjit Dhillon, University of Texas, Austin, USA \n
// *>     Osni Marques, LBNL/NERSC, USA \n
// *>     Christof Voemel, University of California, Berkeley, USA \n
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***

*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
/***
*
*     Decode RANGE
*
***/

public static void dlarrd (final char range,
final char order,
int n,
double vl,
double vu,
int il,
int iu,
double [] gers, int _gers_offset,
double reltol,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] e2, int _e2_offset,
double pivmin,
int nsplit,
int [] isplit, int _isplit_offset,
intW m,
double [] w, int _w_offset,
double [] werr, int _werr_offset,
doubleW wl,
doubleW wu,
int [] iblock, int _iblock_offset,
int [] indexw, int _indexw_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

boolean ncnvrg= false;
boolean toofew= false;
int i= 0;
int ib= 0;
int ibegin= 0;
int idiscl= 0;
int idiscu= 0;
int ie= 0;
int iend= 0;
intW iinfo= new intW(0);
intW im= new intW(0);
int in= 0;
int ioff= 0;
intW iout= new intW(0);
int irange= 0;
int itmax= 0;
int itmp1= 0;
int itmp2= 0;
int iw= 0;
int iwoff= 0;
int j= 0;
int jblk= 0;
int jdisc= 0;
int je= 0;
int jee= 0;
int nb= 0;
int nwl= 0;
int nwu= 0;
double atoli= 0.0d;
double eps= 0.0d;
double gl= 0.0d;
double gu= 0.0d;
double rtoli= 0.0d;
double tmp1= 0.0d;
double tmp2= 0.0d;
double tnorm= 0.0d;
double uflow= 0.0d;
double wkill= 0.0d;
double wlu= 0.0d;
double wul= 0.0d;
int [] idumma= new int[(1)];
//range = range.substring((1)-1,1);
//order = order.substring((1)-1,1);
info.val = 0;
if (com.polytechnik.lapack.Lsame.lsame(range,'A'))  {
    irange = 1;
}
else if (com.polytechnik.lapack.Lsame.lsame(range,'V'))  {
    irange = 2;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(range,'I'))  {
    irange = 3;
}              // Close else if()
else  {
  irange = 0;
/***
*
*     Check for Errors
*
***/
}              //  Close else.
if ((irange <= 0))  {
    info.val = -1;
}
else if ((!((com.polytechnik.lapack.Lsame.lsame(order,'B') || com.polytechnik.lapack.Lsame.lsame(order,'E')))))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if ((irange == 2))  {
    if ((vl >= vu)) {
    info.val = -5;
}
    }              // Close else if()
else if (((irange == 3) && (((il < 1) || (il > Math.max(1, n))))))  {
    info.val = -6;
}              // Close else if()
else if (((irange == 3) && (((iu < Math.min(n, il)) || (iu > n)))))  {
    info.val = -7;
// *
}              // Close else if()
if ((info.val != 0))  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
/***

*     Initialize error flags
***/
}
info.val = 0;
ncnvrg = false;
/***

*     Quick return if possible
***/
toofew = false;
m.val = 0;
/***

*     Simplification:
***/
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
}
    /***

*     Get machine constants
***/
if ((((irange == 3) && (il == 1)) && (iu == n))) {
    irange = 1;
}
    eps = com.polytechnik.lapack.Dlamch.dlamch('P');
/***


*     Special Case when N=1
*     Treat case of 1x1 matrix for quick return
***/
uflow = com.polytechnik.lapack.Dlamch.dlamch('U');
if ((n == 1))  {
    if (((((irange == 1)) || (((((irange == 2)) && ((d[(1-(1))+ _d_offset] > vl))) && ((d[(1-(1))+ _d_offset] <= vu))))) || (((((irange == 3)) && ((il == 1))) && ((iu == 1))))))  {
    m.val = 1;
// *           The computation error of the eigenvalue is zero
w[(1-(1))+ _w_offset] = d[(1-(1))+ _d_offset];
werr[(1-(1))+ _werr_offset] = 0.0;
iblock[(1-(1))+ _iblock_offset] = 1;
indexw[(1-(1))+ _indexw_offset] = 1;
}
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
/***

*     NB is the minimum vector length for vector bisection, or 0
*     if only scalar is to be done.
***/
}
nb = 1;//com.polytechnik.lapack.Ilaenv.ilaenv(1,"DSTEBZ"," ",n,-1,-1,-1);
/***

*     Find global spectral radius
***/
if ((nb <= 1)) {
    nb = 0;
}
    gl = d[(1-(1))+ _d_offset];
gu = d[(1-(1))+ _d_offset];
{
for (i = 1; i <= n; i++) {
gl = Math.min(gl, gers[(((2*i)-1)-(1))+ _gers_offset]);
gu = Math.max(gu, gers[((2*i)-(1))+ _gers_offset]);
// *     Compute global Gerschgorin bounds and spectral diameter
//Dummy.label("com/polytechnik/lapack/Dlarrd",5);
}              //  Close for() loop. 
}
tnorm = Math.max(Math.abs(gl), Math.abs(gu));
gl = ((gl-(((2.0*tnorm)*eps)*n))-((2.0*2.0)*pivmin));
/***
*     [JAN/28/2009] remove the line below since SPDIAM variable not use
*     SPDIAM = GU - GL
*     Input arguments for DLAEBZ:
*     The relative tolerance.  An interval (a,b] lies within
*     "relative tolerance" if  b-a < RELTOL*max(|a|,|b|),
***/
gu = ((gu+(((2.0*tnorm)*eps)*n))+((2.0*2.0)*pivmin));
/***
*     Set the absolute tolerance for interval convergence to zero to force
*     interval convergence based on relative size of the interval.
*     This is dangerous because intervals might not converge when RELTOL is
*     small. But at least a very small number should be selected so that for
*     strongly graded matrices, the code can get relatively accurate
*     eigenvalues.
***/
rtoli = reltol;
// 
atoli = (((2.0*2.0)*uflow)+((2.0*2.0)*pivmin));
/***

*        RANGE='I': Compute an interval containing eigenvalues
*        IL through IU. The initial interval [GL,GU] from the global
*        Gerschgorin bounds GL and GU is refined by DLAEBZ.
***/
if ((irange == 3))  {
    itmax = ((int)((((Math.log((tnorm+pivmin))-Math.log(pivmin)))/Math.log(2.0)))+2);
work[((n+1)-(1))+ _work_offset] = gl;
work[((n+2)-(1))+ _work_offset] = gl;
work[((n+3)-(1))+ _work_offset] = gu;
work[((n+4)-(1))+ _work_offset] = gu;
work[((n+5)-(1))+ _work_offset] = gl;
work[((n+6)-(1))+ _work_offset] = gu;
iwork[(1-(1))+ _iwork_offset] = -1;
iwork[(2-(1))+ _iwork_offset] = -1;
iwork[(3-(1))+ _iwork_offset] = (n+1);
iwork[(4-(1))+ _iwork_offset] = (n+1);
iwork[(5-(1))+ _iwork_offset] = (il-1);
// *
iwork[(6-(1))+ _iwork_offset] = iu;
com.polytechnik.lapack.Dlaebz.dlaebz(3,itmax,n,2,2,nb,atoli,rtoli,pivmin,d,_d_offset,e,_e_offset,e2,_e2_offset,iwork,(5-(1))+ _iwork_offset,work,((n+1)-(1))+ _work_offset,work,((n+5)-(1))+ _work_offset,iout,iwork,_iwork_offset,w,_w_offset,iblock,_iblock_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = iinfo.val;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
// *        On exit, output intervals may not be ordered by ascending negcount
}
if ((iwork[(6-(1))+ _iwork_offset] == iu))  {
    wl.val = work[((n+1)-(1))+ _work_offset];
wlu = work[((n+3)-(1))+ _work_offset];
nwl = iwork[(1-(1))+ _iwork_offset];
wu.val = work[((n+4)-(1))+ _work_offset];
wul = work[((n+2)-(1))+ _work_offset];
nwu = iwork[(4-(1))+ _iwork_offset];
}
else  {
  wl.val = work[((n+2)-(1))+ _work_offset];
wlu = work[((n+4)-(1))+ _work_offset];
nwl = iwork[(2-(1))+ _iwork_offset];
wu.val = work[((n+3)-(1))+ _work_offset];
wul = work[((n+1)-(1))+ _work_offset];
nwu = iwork[(3-(1))+ _iwork_offset];
/***
*        On exit, the interval [WL, WLU] contains a value with negcount NWL,
*        and [WUL, WU] contains a value with negcount NWU.
***/
}              //  Close else.
if (((((nwl < 0) || (nwl >= n)) || (nwu < 1)) || (nwu > n)))  {
    info.val = 4;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
// 
}
}
else if ((irange == 2))  {
    wl.val = vl;
// 
wu.val = vu;
}              // Close else if()
else if ((irange == 1))  {
    wl.val = gl;
wu.val = gu;
/***



*     Find Eigenvalues -- Loop Over blocks and recompute NWL and NWU.
*     NWL accumulates the number of eigenvalues .le. WL,
*     NWU accumulates the number of eigenvalues .le. WU
***/
}              // Close else if()
m.val = 0;
iend = 0;
info.val = 0;
nwl = 0;
// *
nwu = 0;
{
label70:for (jblk = 1; jblk <= nsplit; jblk++) {
ioff = iend;
ibegin = (ioff+1);
iend = isplit[(jblk-(1))+ _isplit_offset];
// *
in = (iend-ioff);
// *           1x1 block
if ((in == 1))  {
    if ((wl.val >= (d[(ibegin-(1))+ _d_offset]-pivmin))) {
    nwl = (nwl+1);
}
    if ((wu.val >= (d[(ibegin-(1))+ _d_offset]-pivmin))) {
    nwu = (nwu+1);
}
    if (((irange == 1) || (((wl.val < (d[(ibegin-(1))+ _d_offset]-pivmin)) && (wu.val >= (d[(ibegin-(1))+ _d_offset]-pivmin))))))  {
    m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = d[(ibegin-(1))+ _d_offset];
/***
*              The gap for a single block doesn't matter for the later
*              algorithm and is assigned an arbitrary large value
***/
werr[(m.val-(1))+ _werr_offset] = 0.0;
iblock[(m.val-(1))+ _iblock_offset] = jblk;
indexw[(m.val-(1))+ _indexw_offset] = 1;
/***

*        Disabled 2x2 case because of a failure on the following matrix
*        RANGE = 'I', IL = IU = 4
*          Original Tridiagonal, d = [
*           -0.150102010615740E+00
*           -0.849897989384260E+00
*           -0.128208148052635E-15
*            0.128257718286320E-15
*          ];
*          e = [
*           -0.357171383266986E+00
*           -0.180411241501588E-15
*           -0.175152352710251E-15
*          ];
*
*         ELSE IF( IN.EQ.2 ) THEN
**           2x2 block
*            DISC = SQRT( (HALF*(D(IBEGIN)-D(IEND)))**2 + E(IBEGIN)**2 )
*            TMP1 = HALF*(D(IBEGIN)+D(IEND))
*            L1 = TMP1 - DISC
*            IF( WL.GE. L1-PIVMIN )
*     $         NWL = NWL + 1
*            IF( WU.GE. L1-PIVMIN )
*     $         NWU = NWU + 1
*            IF( IRANGE.EQ.ALLRNG .OR. ( WL.LT.L1-PIVMIN .AND. WU.GE.
*     $          L1-PIVMIN ) ) THEN
*               M = M + 1
*               W( M ) = L1
**              The uncertainty of eigenvalues of a 2x2 matrix is very small
*               WERR( M ) = EPS * ABS( W( M ) ) * TWO
*               IBLOCK( M ) = JBLK
*               INDEXW( M ) = 1
*            ENDIF
*            L2 = TMP1 + DISC
*            IF( WL.GE. L2-PIVMIN )
*     $         NWL = NWL + 1
*            IF( WU.GE. L2-PIVMIN )
*     $         NWU = NWU + 1
*            IF( IRANGE.EQ.ALLRNG .OR. ( WL.LT.L2-PIVMIN .AND. WU.GE.
*     $          L2-PIVMIN ) ) THEN
*               M = M + 1
*               W( M ) = L2
**              The uncertainty of eigenvalues of a 2x2 matrix is very small
*               WERR( M ) = EPS * ABS( W( M ) ) * TWO
*               IBLOCK( M ) = JBLK
*               INDEXW( M ) = 2
*            ENDIF
***/
}
/***
*           General Case - block of size IN >= 2
*           Compute local Gerschgorin interval and use it as the initial
*           interval for DLAEBZ
***/
}
else  {
  gu = d[(ibegin-(1))+ _d_offset];
gl = d[(ibegin-(1))+ _d_offset];
// 
tmp1 = 0.0;
{
for (j = ibegin; j <= iend; j++) {
gl = Math.min(gl, gers[(((2*j)-1)-(1))+ _gers_offset]);
gu = Math.max(gu, gers[((2*j)-(1))+ _gers_offset]);
/***
*           [JAN/28/2009]
*           change SPDIAM by TNORM in lines 2 and 3 thereafter
*           line 1: remove computation of SPDIAM (not useful anymore)
*           SPDIAM = GU - GL
*           GL = GL - FUDGE*SPDIAM*EPS*IN - FUDGE*PIVMIN
*           GU = GU + FUDGE*SPDIAM*EPS*IN + FUDGE*PIVMIN
***/
//Dummy.label("com/polytechnik/lapack/Dlarrd",40);
}              //  Close for() loop. 
}
gl = ((gl-(((2.0*tnorm)*eps)*in))-(2.0*pivmin));
// *
gu = ((gu+(((2.0*tnorm)*eps)*in))+(2.0*pivmin));
if ((irange > 1))  {
    // *                 the local block contains none of the wanted eigenvalues
if ((gu < wl.val))  {
    nwl = (nwl+in);
nwu = (nwu+in);
continue label70;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",70);
// *              refine search interval if possible, only range (WL,WU] matters
}
gl = Math.max(gl, wl.val);
gu = Math.min(gu, wu.val);
if ((gl >= gu)) {
    continue label70;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",70);
}
    /***

*           Find negcount of initial interval boundaries GL and GU
***/
}
work[((n+1)-(1))+ _work_offset] = gl;
work[(((n+in)+1)-(1))+ _work_offset] = gu;
com.polytechnik.lapack.Dlaebz.dlaebz(1,0,in,in,1,nb,atoli,rtoli,pivmin,d,(ibegin-(1))+ _d_offset,e,(ibegin-(1))+ _e_offset,e2,(ibegin-(1))+ _e2_offset,idumma,0,work,((n+1)-(1))+ _work_offset,work,(((n+(2*in))+1)-(1))+ _work_offset,im,iwork,_iwork_offset,w,((m.val+1)-(1))+ _w_offset,iblock,((m.val+1)-(1))+ _iblock_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = iinfo.val;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
// *
}
nwl = (nwl+iwork[(1-(1))+ _iwork_offset]);
nwu = (nwu+iwork[((in+1)-(1))+ _iwork_offset]);
/***

*           Compute Eigenvalues
***/
iwoff = (m.val-iwork[(1-(1))+ _iwork_offset]);
itmax = ((int)((((Math.log(((gu-gl)+pivmin))-Math.log(pivmin)))/Math.log(2.0)))+2);
com.polytechnik.lapack.Dlaebz.dlaebz(2,itmax,in,in,1,nb,atoli,rtoli,pivmin,d,(ibegin-(1))+ _d_offset,e,(ibegin-(1))+ _e_offset,e2,(ibegin-(1))+ _e2_offset,idumma,0,work,((n+1)-(1))+ _work_offset,work,(((n+(2*in))+1)-(1))+ _work_offset,iout,iwork,_iwork_offset,w,((m.val+1)-(1))+ _w_offset,iblock,((m.val+1)-(1))+ _iblock_offset,iinfo);
if ((iinfo.val != 0))  {
    info.val = iinfo.val;
    return;//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
/***
*
*           Copy eigenvalues into W and IBLOCK
*           Use -JBLK for block number for unconverged eigenvalues.
*           Loop over the number of output intervals from DLAEBZ
***/
}
// *              eigenvalue approximation is middle point of interval
{
for (j = 1; j <= iout.val; j++) {
// *              semi length of error interval
tmp1 = (0.5*((work[((j+n)-(1))+ _work_offset]+work[(((j+in)+n)-(1))+ _work_offset])));
tmp2 = (0.5*Math.abs((work[((j+n)-(1))+ _work_offset]-work[(((j+in)+n)-(1))+ _work_offset])));
// *                 Flag non-convergence.
if ((j > (iout.val-iinfo.val)))  {
    ncnvrg = true;
ib = (-(jblk));
}
else  {
  ib = jblk;
}              //  Close else.
{
for (je = ((iwork[(j-(1))+ _iwork_offset]+1)+iwoff); je <= (iwork[((j+in)-(1))+ _iwork_offset]+iwoff); je++) {
w[(je-(1))+ _w_offset] = tmp1;
werr[(je-(1))+ _werr_offset] = tmp2;
indexw[(je-(1))+ _indexw_offset] = (je-iwoff);
iblock[(je-(1))+ _iblock_offset] = ib;
//Dummy.label("com/polytechnik/lapack/Dlarrd",50);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlarrd",60);
}              //  Close for() loop. 
}
m.val = (m.val+im.val);
}              //  Close else.
/***

*     If RANGE='I', then (WL,WU) contains eigenvalues NWL+1,...,NWU
*     If NWL+1 < IL or NWU > IU, discard extra eigenvalues.
***/
//Dummy.label("com/polytechnik/lapack/Dlarrd",70);
}              //  Close for() loop. 
}
if ((irange == 3))  {
    idiscl = ((il-1)-nwl);
// *
idiscu = (nwu-iu);
if ((idiscl > 0))  {
    im.val = 0;
/***
*              Remove some of the smallest eigenvalues from the left so that
*              at the end IDISCL =0. Move all eigenvalues up to the left.
***/
{
for (je = 1; je <= m.val; je++) {
if (((w[(je-(1))+ _w_offset] <= wlu) && (idiscl > 0)))  {
    idiscl = (idiscl-1);
}
else  {
  im.val = (im.val+1);
w[(im.val-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
werr[(im.val-(1))+ _werr_offset] = werr[(je-(1))+ _werr_offset];
indexw[(im.val-(1))+ _indexw_offset] = indexw[(je-(1))+ _indexw_offset];
iblock[(im.val-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlarrd",80);
}              //  Close for() loop. 
}
m.val = im.val;
}
/***
*           Remove some of the largest eigenvalues from the right so that
*           at the end IDISCU =0. Move all eigenvalues up to the left.
***/
if ((idiscu > 0))  {
    im.val = (m.val+1);
{
int _je_inc = -1;
for (je = m.val; je >= 1; je += _je_inc) {
if (((w[(je-(1))+ _w_offset] >= wul) && (idiscu > 0)))  {
    idiscu = (idiscu-1);
}
else  {
  im.val = (im.val-1);
w[(im.val-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
werr[(im.val-(1))+ _werr_offset] = werr[(je-(1))+ _werr_offset];
indexw[(im.val-(1))+ _indexw_offset] = indexw[(je-(1))+ _indexw_offset];
iblock[(im.val-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dlarrd",81);
}              //  Close for() loop. 
}
jee = 0;
{
for (je = im.val; je <= m.val; je++) {
jee = (jee+1);
w[(jee-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
werr[(jee-(1))+ _werr_offset] = werr[(je-(1))+ _werr_offset];
indexw[(jee-(1))+ _indexw_offset] = indexw[(je-(1))+ _indexw_offset];
iblock[(jee-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
//Dummy.label("com/polytechnik/lapack/Dlarrd",82);
}              //  Close for() loop. 
}
m.val = ((m.val-im.val)+1);
// 
}
/***
*           Code to deal with effects of bad arithmetic. (If N(w) is
*           monotone non-decreasing, this should never happen.)
*           Some low eigenvalues to be discarded are not in (WL,WLU],
*           or high eigenvalues to be discarded are not in (WUL,WU]
*           so just kill off the smallest IDISCL/largest IDISCU
*           eigenvalues, by marking the corresponding IBLOCK = 0
***/
if (((idiscl > 0) || (idiscu > 0)))  {
    if ((idiscl > 0))  {
    wkill = wu.val;
{
for (jdisc = 1; jdisc <= idiscl; jdisc++) {
iw = 0;
{
for (je = 1; je <= m.val; je++) {
if (((iblock[(je-(1))+ _iblock_offset] != 0) && (((w[(je-(1))+ _w_offset] < wkill) || (iw == 0)))))  {
    iw = je;
wkill = w[(je-(1))+ _w_offset];
}
//Dummy.label("com/polytechnik/lapack/Dlarrd",90);
}              //  Close for() loop. 
}
iblock[(iw-(1))+ _iblock_offset] = 0;
//Dummy.label("com/polytechnik/lapack/Dlarrd",100);
}              //  Close for() loop. 
}
}
if ((idiscu > 0))  {
    wkill = wl.val;
{
for (jdisc = 1; jdisc <= idiscu; jdisc++) {
iw = 0;
{
for (je = 1; je <= m.val; je++) {
if (((iblock[(je-(1))+ _iblock_offset] != 0) && (((w[(je-(1))+ _w_offset] >= wkill) || (iw == 0)))))  {
    iw = je;
wkill = w[(je-(1))+ _w_offset];
}
//Dummy.label("com/polytechnik/lapack/Dlarrd",110);
}              //  Close for() loop. 
}
iblock[(iw-(1))+ _iblock_offset] = 0;
//Dummy.label("com/polytechnik/lapack/Dlarrd",120);
}              //  Close for() loop. 
}
// *           Now erase all eigenvalues with IBLOCK set to zero
}
im.val = 0;
{
for (je = 1; je <= m.val; je++) {
if ((iblock[(je-(1))+ _iblock_offset] != 0))  {
    im.val = (im.val+1);
w[(im.val-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
werr[(im.val-(1))+ _werr_offset] = werr[(je-(1))+ _werr_offset];
indexw[(im.val-(1))+ _indexw_offset] = indexw[(je-(1))+ _indexw_offset];
iblock[(im.val-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
}
//Dummy.label("com/polytechnik/lapack/Dlarrd",130);
}              //  Close for() loop. 
}
m.val = im.val;
}
if (((idiscl < 0) || (idiscu < 0)))  {
    toofew = true;
}
// *
}
if (((((irange == 1) && (m.val != n))) || (((irange == 3) && (m.val != ((iu-il)+1))))))  {
    toofew = true;
/***

*     If ORDER='B', do nothing the eigenvalues are already sorted by
*        block.
*     If ORDER='E', sort the eigenvalues from smallest to largest

***/
}
if ((com.polytechnik.lapack.Lsame.lsame(order,'E') && (nsplit > 1)))  {
    {
for (je = 1; je <= (m.val-1); je++) {
ie = 0;
tmp1 = w[(je-(1))+ _w_offset];
{
for (j = (je+1); j <= m.val; j++) {
if ((w[(j-(1))+ _w_offset] < tmp1))  {
    ie = j;
tmp1 = w[(j-(1))+ _w_offset];
}
//Dummy.label("com/polytechnik/lapack/Dlarrd",140);
}              //  Close for() loop. 
}
if ((ie != 0))  {
    tmp2 = werr[(ie-(1))+ _werr_offset];
itmp1 = iblock[(ie-(1))+ _iblock_offset];
itmp2 = indexw[(ie-(1))+ _indexw_offset];
w[(ie-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
werr[(ie-(1))+ _werr_offset] = werr[(je-(1))+ _werr_offset];
iblock[(ie-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
indexw[(ie-(1))+ _indexw_offset] = indexw[(je-(1))+ _indexw_offset];
w[(je-(1))+ _w_offset] = tmp1;
werr[(je-(1))+ _werr_offset] = tmp2;
iblock[(je-(1))+ _iblock_offset] = itmp1;
indexw[(je-(1))+ _indexw_offset] = itmp2;
}
//Dummy.label("com/polytechnik/lapack/Dlarrd",150);
}              //  Close for() loop. 
}
// *
}
info.val = 0;
if (ncnvrg) {
    info.val = (info.val+1);
}
    if (toofew) {
    info.val = (info.val+2);
}
    /***
*
*     End of DLARRD
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrd",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrd",999999);
return;
   }
} // End class.
