/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrj.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrj.java,v 1.4 2012/12/19 22:00:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrj {

// *> \brief \b DLARRJ performs refinement of the initial estimates of the 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRJ + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRJ( N, D, E2, IFIRST, ILAST,
// *                          RTOL, OFFSET, W, WERR, WORK, IWORK,
// *                          PIVMIN, SPDIAM, INFO )
// * 
// *       .. Scalar Arguments ..
// *       INTEGER            IFIRST, ILAST, INFO, N, OFFSET
// *       DOUBLE PRECISION   PIVMIN, RTOL, SPDIAM
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IWORK( * )
// *       DOUBLE PRECISION   D( * ), E2( * ), W( * ),
// *      $                   WERR( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> Given the initial eigenvalue approximations of T, DLARRJ
// *> does  bisection to refine the eigenvalues of T,
// *> W( IFIRST-OFFSET ) through W( ILAST-OFFSET ), to more accuracy. Initi
// *> guesses for these eigenvalues are input in W, the corresponding estim
// *> of the error in these guesses in WERR. During bisection, intervals
// *> [left, right] are maintained by storing their mid-points and
// *> semi-widths in the arrays W and WERR respectively.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The N diagonal elements of T.
// *> \endverbatim
// *>
// *> \param[in] E2
// *> \verbatim
// *>          E2 is DOUBLE PRECISION array, dimension (N-1)
// *>          The Squares of the (N-1) subdiagonal elements of T.
// *> \endverbatim
// *>
// *> \param[in] IFIRST
// *> \verbatim
// *>          IFIRST is INTEGER
// *>          The index of the first eigenvalue to be computed.
// *> \endverbatim
// *>
// *> \param[in] ILAST
// *> \verbatim
// *>          ILAST is INTEGER
// *>          The index of the last eigenvalue to be computed.
// *> \endverbatim
// *>
// *> \param[in] RTOL
// *> \verbatim
// *>          RTOL is DOUBLE PRECISION
// *>          Tolerance for the convergence of the bisection intervals.
// *>          An interval [LEFT,RIGHT] has converged if
// *>          RIGHT-LEFT.LT.RTOL*MAX(|LEFT|,|RIGHT|).
// *> \endverbatim
// *>
// *> \param[in] OFFSET
// *> \verbatim
// *>          OFFSET is INTEGER
// *>          Offset for the arrays W and WERR, i.e., the IFIRST-OFFSET
// *>          through ILAST-OFFSET elements of these arrays are to be used
// *> \endverbatim
// *>
// *> \param[in,out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          On input, W( IFIRST-OFFSET ) through W( ILAST-OFFSET ) are
// *>          estimates of the eigenvalues of L D L^T indexed IFIRST throu
// *>          ILAST.
// *>          On output, these estimates are refined.
// *> \endverbatim
// *>
// *> \param[in,out] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION array, dimension (N)
// *>          On input, WERR( IFIRST-OFFSET ) through WERR( ILAST-OFFSET )
// *>          the errors in the estimates of the corresponding elements in
// *>          On output, these errors are refined.
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (2*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (2*N)
// *>          Workspace.
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot in the Sturm sequence for T.
// *> \endverbatim
// *>
// *> \param[in] SPDIAM
// *> \verbatim
// *>          SPDIAM is DOUBLE PRECISION
// *>          The spectral diameter of T.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          Error flag.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *> \par Contributors:
// *  ==================
// *>
// *> Beresford Parlett, University of California, Berkeley, USA \n
// *> Jim Demmel, University of California, Berkeley, USA \n
// *> Inderjit Dhillon, University of Texas, Austin, USA \n
// *> Osni Marques, LBNL/NERSC, USA \n
// *> Christof Voemel, University of California, Berkeley, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
// *

public static void dlarrj (int n,
double [] d, int _d_offset,
double [] e2, int _e2_offset,
int ifirst,
int ilast,
double rtol,
int offset,
double [] w, int _w_offset,
double [] werr, int _werr_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
double pivmin,
double spdiam,
intW info)  {

    final int NLOOPMAX=2*n+Util.ITMAX;

int maxitr= 0;
int cnt= 0;
int i= 0;
int i1= 0;
int i2= 0;
int ii= 0;
int iter= 0;
int j= 0;
int k= 0;
int next= 0;
int nint= 0;
int olnint= 0;
int p= 0;
int prev= 0;
int savi1= 0;
double dplus= 0.0d;
double fac= 0.0d;
double left= 0.0d;
double mid= 0.0d;
double right= 0.0d;
double s= 0.0d;
double tmp= 0.0d;
double width= 0.0d;
info.val = 0;
/***
*
*     Initialize unconverged intervals in [ WORK(2*I-1), WORK(2*I) ].
*     The Sturm Count, Count( WORK(2*I-1) ) is arranged to be I-1, while
*     Count( WORK(2*I) ) is stored in IWORK( 2*I ). The integer IWORK( 2*I-1 )
*     for an unconverged interval is set to the index of the next unconverged
*     interval, and is -1 or 0 for a converged interval. Thus a linked
*     list of unconverged intervals is set up.
*

***/
maxitr = ((int)((((Math.log((spdiam+pivmin))-Math.log(pivmin)))/Math.log(2.0)))+2);
i1 = ifirst;
// *     The number of unconverged intervals
i2 = ilast;
// *     The last unconverged interval found
nint = 0;
prev = 0;
{
for (i = i1; i <= i2; i++) {
k = (2*i);
ii = (i-offset);
left = (w[(ii-(1))+ _w_offset]-werr[(ii-(1))+ _werr_offset]);
mid = w[(ii-(1))+ _w_offset];
right = (w[(ii-(1))+ _w_offset]+werr[(ii-(1))+ _werr_offset]);
width = (right-mid);
/***

*        The following test prevents the test of converged intervals
***/
tmp = Math.max(Math.abs(left), Math.abs(right));
/***
*           This interval has already converged and does not need refinement.
*           (Note that the gaps might change through refining the
*            eigenvalues, however, they can only get bigger.)
*           Remove it from the list.
***/
if ((width < (rtol*tmp)))  {
    // *           Make sure that I1 always points to the first unconverged interval
iwork[((k-1)-(1))+ _iwork_offset] = -1;
if ((((i == i1)) && ((i < i2)))) {
    i1 = (i+1);
}
    if ((((prev >= i1)) && ((i <= i2)))) {
    iwork[(((2*prev)-1)-(1))+ _iwork_offset] = (i+1);
}
    // *           unconverged interval found
}
else  {
  /***
*           Make sure that [LEFT,RIGHT] contains the desired eigenvalue
*
*           Do while( CNT(LEFT).GT.I-1 )
*
***/
prev = i;
fac = 1.0;
label20: for(int jj20=0;jj20<NLOOPMAX;jj20++){// Dummy.label("com/polytechnik/lapack/Dlarrj",20);
cnt = 0;
s = left;
dplus = (d[(1-(1))+ _d_offset]-s);
if ((dplus < 0.0)) {
    cnt = (cnt+1);
}
    {
for (j = 2; j <= n; j++) {
dplus = ((d[(j-(1))+ _d_offset]-s)-(e2[((j-1)-(1))+ _e2_offset]/dplus));
if ((dplus < 0.0)) {
    cnt = (cnt+1);
}
//Dummy.label("com/polytechnik/lapack/Dlarrj",30);
}              //  Close for() loop. 
}
if ((cnt > (i-1)))  {
    left = (left-(werr[(ii-(1))+ _werr_offset]*fac));
fac = (2.0*fac);
continue label20;//Dummy.go_to("com/polytechnik/lapack/Dlarrj",20);
/***
*
*           Do while( CNT(RIGHT).LT.I )
*
***/
}
break label20;}
fac = 1.0;
label50:for(int jj50=0;jj50<NLOOPMAX;jj50++){
    //Dummy.label("com/polytechnik/lapack/Dlarrj",50);
cnt = 0;
s = right;
dplus = (d[(1-(1))+ _d_offset]-s);
if ((dplus < 0.0)) {
    cnt = (cnt+1);
}
    {
for (j = 2; j <= n; j++) {
dplus = ((d[(j-(1))+ _d_offset]-s)-(e2[((j-1)-(1))+ _e2_offset]/dplus));
if ((dplus < 0.0)) {
    cnt = (cnt+1);
}
//Dummy.label("com/polytechnik/lapack/Dlarrj",60);
}              //  Close for() loop. 
}
if ((cnt < i))  {
    right = (right+(werr[(ii-(1))+ _werr_offset]*fac));
fac = (2.0*fac);
continue label50;//Dummy.go_to("com/polytechnik/lapack/Dlarrj",50);
}
break label50;}

nint = (nint+1);
iwork[((k-1)-(1))+ _iwork_offset] = (i+1);
iwork[(k-(1))+ _iwork_offset] = cnt;
}              //  Close else.
work[((k-1)-(1))+ _work_offset] = left;
work[(k-(1))+ _work_offset] = right;
/***


***/
//Dummy.label("com/polytechnik/lapack/Dlarrj",75);
}              //  Close for() loop. 
}
/***
*
*     Do while( NINT.GT.0 ), i.e. there are still unconverged intervals
*     and while (ITER.LT.MAXITR)
*
***/
savi1 = i1;
iter = 0;
label80:for(int jj80=0;jj80<NLOOPMAX;jj80++) {
    //Dummy.label("com/polytechnik/lapack/Dlarrj",80);
prev = (i1-1);
i = i1;
// 
olnint = nint;
{
label100:for (p = 1; p <= olnint; p++) {
k = (2*i);
ii = (i-offset);
next = iwork[((k-1)-(1))+ _iwork_offset];
left = work[((k-1)-(1))+ _work_offset];
right = work[(k-(1))+ _work_offset];
/***

*        semiwidth of interval
***/
mid = (0.5*((left+right)));
width = (right-mid);
// 
tmp = Math.max(Math.abs(left), Math.abs(right));
// *           reduce number of unconverged intervals
if ((((width < (rtol*tmp))) || ((iter == maxitr))))  {
    // *           Mark interval as converged.
nint = (nint-1);
iwork[((k-1)-(1))+ _iwork_offset] = 0;
if ((i1 == i))  {
    i1 = next;
// *              Prev holds the last unconverged interval previously examined
}
else  {
  if ((prev >= i1)) {
    iwork[(((2*prev)-1)-(1))+ _iwork_offset] = next;
}
    }              //  Close else.
i = next;
continue label100;//Dummy.go_to("com/polytechnik/lapack/Dlarrj",100);
}
/***
*
*        Perform one bisection step
*
***/
prev = i;
cnt = 0;
s = mid;
dplus = (d[(1-(1))+ _d_offset]-s);
if ((dplus < 0.0)) {
    cnt = (cnt+1);
}
    {
for (j = 2; j <= n; j++) {
dplus = ((d[(j-(1))+ _d_offset]-s)-(e2[((j-1)-(1))+ _e2_offset]/dplus));
if ((dplus < 0.0)) {
    cnt = (cnt+1);
}
//Dummy.label("com/polytechnik/lapack/Dlarrj",90);
}              //  Close for() loop. 
}
if ((cnt <= (i-1)))  {
    work[((k-1)-(1))+ _work_offset] = mid;
}
else  {
  work[(k-(1))+ _work_offset] = mid;
}              //  Close else.
// 
i = next;
//Dummy.label("com/polytechnik/lapack/Dlarrj",100);
}              //  Close for() loop. 
}
/***
*     do another loop if there are still unconverged intervals
*     However, in the last iteration, all intervals are accepted
*     since this is the best we can do.
***/
iter = (iter+1);
/***
*
*
*     At this point, all the intervals have converged
***/
if ((((nint > 0)) && ((iter <= maxitr)))) {
    continue label80;//Dummy.go_to("com/polytechnik/lapack/Dlarrj",80);
}
break label80;}

    {
for (i = savi1; i <= ilast; i++) {
k = (2*i);
// *        All intervals marked by '0' have been refined.
ii = (i-offset);
if ((iwork[((k-1)-(1))+ _iwork_offset] == 0))  {
    w[(ii-(1))+ _w_offset] = (0.5*((work[((k-1)-(1))+ _work_offset]+work[(k-(1))+ _work_offset])));
werr[(ii-(1))+ _werr_offset] = (work[(k-(1))+ _work_offset]-w[(ii-(1))+ _w_offset]);
}
/***
*

***/
//Dummy.label("com/polytechnik/lapack/Dlarrj",110);
}              //  Close for() loop. 
}
/***
*
*     End of DLARRJ
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrj",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrj",999999);
return;
   }
} // End class.
