/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlarrk.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlarrk.java,v 1.2 2012/12/15 19:04:15 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlarrk {

// *> \brief \b DLARRK computes one eigenvalue of a symmetric tridiagonal m
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLARRK + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLARRK( N, IW, GL, GU,
// *                           D, E2, PIVMIN, RELTOL, W, WERR, INFO)
// * 
// *       .. Scalar Arguments ..
// *       INTEGER   INFO, IW, N
// *       DOUBLE PRECISION    PIVMIN, RELTOL, GL, GU, W, WERR
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   D( * ), E2( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLARRK computes one eigenvalue of a symmetric tridiagonal
// *> matrix T to suitable accuracy. This is an auxiliary code to be
// *> called from DSTEMR.
// *>
// *> To avoid overflow, the matrix must be scaled so that its
// *> largest element is no greater than overflow**(1/2) * underflow**(1/4)
// *> accuracy, it should not be much smaller than that.
// *>
// *> See W. Kahan "Accurate Eigenvalues of a Symmetric Tridiagonal
// *> Matrix", Report CS41, Computer Science Dept., Stanford
// *> University, July 21, 1966.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the tridiagonal matrix T.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] IW
// *> \verbatim
// *>          IW is INTEGER
// *>          The index of the eigenvalues to be returned.
// *> \endverbatim
// *>
// *> \param[in] GL
// *> \verbatim
// *>          GL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] GU
// *> \verbatim
// *>          GU is DOUBLE PRECISION
// *>          An upper and a lower bound on the eigenvalue.
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The n diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in] E2
// *> \verbatim
// *>          E2 is DOUBLE PRECISION array, dimension (N-1)
// *>          The (n-1) squared off-diagonal elements of the tridiagonal m
// *> \endverbatim
// *>
// *> \param[in] PIVMIN
// *> \verbatim
// *>          PIVMIN is DOUBLE PRECISION
// *>          The minimum pivot allowed in the Sturm sequence for T.
// *> \endverbatim
// *>
// *> \param[in] RELTOL
// *> \verbatim
// *>          RELTOL is DOUBLE PRECISION
// *>          The minimum relative width of an interval.  When an interval
// *>          is narrower than RELTOL times the larger (in
// *>          magnitude) endpoint, then it is considered to be
// *>          sufficiently small, i.e., converged.  Note: this should
// *>          always be at least radix*machine epsilon.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[out] WERR
// *> \verbatim
// *>          WERR is DOUBLE PRECISION
// *>          The error bound on the corresponding eigenvalue approximatio
// *>          in W.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:       Eigenvalue converged
// *>          = -1:      Eigenvalue did NOT converge
// *> \endverbatim
// *
// *> \par Internal Parameters:
// *  =========================
// *>
// *> \verbatim
// *>  FUDGE   DOUBLE PRECISION, default = 2
// *>          A "fudge factor" to widen the Gershgorin intervals.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Get machine constants
***/
// 

public static void dlarrk (int n,
int iw,
double gl,
double gu,
double [] d, int _d_offset,
double [] e2, int _e2_offset,
double pivmin,
double reltol,
doubleW w,
doubleW werr,
intW info)  {

int i= 0;
int it= 0;
int itmax= 0;
int negcnt= 0;
double atoli= 0.0d;
double eps= 0.0d;
double left= 0.0d;
double mid= 0.0d;
double right= 0.0d;
double rtoli= 0.0d;
double tmp1= 0.0d;
double tmp2= 0.0d;
double tnorm= 0.0d;
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
tnorm = Math.max(Math.abs(gl), Math.abs(gu));
rtoli = reltol;
// 
atoli = ((2.0*2.0)*pivmin);
// 
itmax = ((int)((((Math.log((tnorm+pivmin))-Math.log(pivmin)))/Math.log(2.0)))+2);
// 
info.val = -1;
left = ((gl-(((2.0*tnorm)*eps)*n))-((2.0*2.0)*pivmin));
right = ((gu+(((2.0*tnorm)*eps)*n))+((2.0*2.0)*pivmin));
// 
it = 0;
/***
*
*     Check if interval converged or maximum number of iterations reached
*
***/
label10:for(;;){
    //Dummy.label("com/polytechnik/lapack/Dlarrk",10);
tmp1 = Math.abs((right-left));
tmp2 = Math.max(Math.abs(right), Math.abs(left));
if ((tmp1 < Util.max(atoli, pivmin, (rtoli*tmp2))))  {
    info.val = 0;
    break label10;//Dummy.go_to("com/polytechnik/lapack/Dlarrk",30);
}
/***

*
*     Count number of negative pivots for mid-point
*
***/
if ((it > itmax)) {
    break label10;//Dummy.go_to("com/polytechnik/lapack/Dlarrk",30);
}
    it = (it+1);
mid = (0.5*((left+right)));
negcnt = 0;
tmp1 = (d[(1-(1))+ _d_offset]-mid);
if ((Math.abs(tmp1) < pivmin)) {
    tmp1 = (-(pivmin));
}
    // *
if ((tmp1 <= 0.0)) {
    negcnt = (negcnt+1);
}
    {
for (i = 2; i <= n; i++) {
tmp1 = ((d[(i-(1))+ _d_offset]-(e2[((i-1)-(1))+ _e2_offset]/tmp1))-mid);
if ((Math.abs(tmp1) < pivmin)) {
    tmp1 = (-(pivmin));
}
    if ((tmp1 <= 0.0)) {
    negcnt = (negcnt+1);
}
    // 
//Dummy.label("com/polytechnik/lapack/Dlarrk",20);
}              //  Close for() loop. 
}
if ((negcnt >= iw))  {
    right = mid;
}
else  {
  left = mid;
}              //  Close else.
// 
}//Dummy.go_to("com/polytechnik/lapack/Dlarrk",10);
/***
*
*     Converged or maximum number of iterations reached
*
***/
//label30: Dummy.label("com/polytechnik/lapack/Dlarrk",30);
w.val = (0.5*((left+right)));
// 
werr.val = (0.5*Math.abs((right-left)));
/***
*
*     End of DLARRK
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlarrk",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlarrk",999999);
return;
   }
} // End class.
