/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlascl.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlascl.java,v 1.3 2012/12/19 22:16:46 mal Exp $
 */

package com.polytechnik.lapack;


public class Dlascl {

// *> \brief \b DLASCL
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLASCL + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLASCL( TYPE, KL, KU, CFROM, CTO, M, N, A, LDA, INFO 
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          TYPE
// *       INTEGER            INFO, KL, KU, LDA, M, N
// *       DOUBLE PRECISION   CFROM, CTO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLASCL multiplies the M by N real matrix A by the real scalar
// *> CTO/CFROM.  This is done without over/underflow as long as the final

// *> result CTO*A(I,J)/CFROM does not over/underflow. TYPE specifies that

// *> A may be full, upper triangular, lower triangular, upper Hessenberg,

// *> or banded.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] TYPE
// *> \verbatim
// *>          TYPE is CHARACTER*1
// *>          TYPE indices the storage type of the input matrix.
// *>          = 'G':  A is a full matrix.
// *>          = 'L':  A is a lower triangular matrix.
// *>          = 'U':  A is an upper triangular matrix.
// *>          = 'H':  A is an upper Hessenberg matrix.
// *>          = 'B':  A is a symmetric band matrix with lower bandwidth KL
// *>                  and upper bandwidth KU and with the only the lower
// *>                  half stored.
// *>          = 'Q':  A is a symmetric band matrix with lower bandwidth KL
// *>                  and upper bandwidth KU and with the only the upper
// *>                  half stored.
// *>          = 'Z':  A is a band matrix with lower bandwidth KL and upper
// *>                  bandwidth KU. See DGBTRF for storage details.
// *> \endverbatim
// *>
// *> \param[in] KL
// *> \verbatim
// *>          KL is INTEGER
// *>          The lower bandwidth of A.  Referenced only if TYPE = 'B',
// *>          'Q' or 'Z'.
// *> \endverbatim
// *>
// *> \param[in] KU
// *> \verbatim
// *>          KU is INTEGER
// *>          The upper bandwidth of A.  Referenced only if TYPE = 'B',
// *>          'Q' or 'Z'.
// *> \endverbatim
// *>
// *> \param[in] CFROM
// *> \verbatim
// *>          CFROM is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] CTO
// *> \verbatim
// *>          CTO is DOUBLE PRECISION
// *>
// *>          The matrix A is multiplied by CTO/CFROM. A(I,J) is computed

// *>          without over/underflow if the final result CTO*A(I,J)/CFROM

// *>          can be represented without over/underflow.  CFROM must be
// *>          nonzero.
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix A.  M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          The matrix to be multiplied by CTO/CFROM.  See TYPE for the

// *>          storage type.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          0  - successful exit
// *>          <0 - if INFO = -i, the i-th argument had an illegal value.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup auxOTHERauxiliary
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/
// *

public static void dlascl (final char type,
int kl,
int ku,
double cfrom,
double cto,
int m,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

boolean done= false;
int i= 0;
int itype= 0;
int j= 0;
int k1= 0;
int k2= 0;
int k3= 0;
int k4= 0;
double bignum= 0.0d;
double cfrom1= 0.0d;
double cfromc= 0.0d;
double cto1= 0.0d;
double ctoc= 0.0d;
double mul= 0.0d;
double smlnum= 0.0d;
//type = type.substring((1)-1,1);
info.val = 0;
if (com.polytechnik.lapack.Lsame.lsame(type,'G'))  {
    itype = 0;
}
else if (com.polytechnik.lapack.Lsame.lsame(type,'L'))  {
    itype = 1;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(type,'U'))  {
    itype = 2;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(type,'H'))  {
    itype = 3;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(type,'B'))  {
    itype = 4;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(type,'Q'))  {
    itype = 5;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(type,'Z'))  {
    itype = 6;
}              // Close else if()
else  {
  itype = -1;
// *
}              //  Close else.
if ((itype == -1))  {
    info.val = -1;
}
else if (((cfrom == 0.0) || Double.isNaN(cfrom)))  {
    info.val = -4;
}              // Close else if()
else if (Double.isNaN(cto))  {
    info.val = -5;
}              // Close else if()
else if ((m < 0))  {
    info.val = -6;
}              // Close else if()
else if ((((n < 0) || (((itype == 4) && (n != m)))) || (((itype == 5) && (n != m)))))  {
    info.val = -7;
}              // Close else if()
else if (((itype <= 3) && (lda < Math.max(1, m))))  {
    info.val = -9;
}              // Close else if()
else if ((itype >= 4))  {
    if (((kl < 0) || (kl > Math.max((m-1), 0))))  {
    info.val = -2;
}
else if ((((ku < 0) || (ku > Math.max((n-1), 0))) || (((((itype == 4) || (itype == 5))) && (kl != ku)))))  {
    info.val = -3;
}              // Close else if()
else if ((((((itype == 4) && (lda < (kl+1)))) || (((itype == 5) && (lda < (ku+1))))) || (((itype == 6) && (lda < (((2*kl)+ku)+1))))))  {
    info.val = -9;
}              // Close else if()
// *
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DLASCL",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dlascl",999999);
/***
*
*     Quick return if possible
*
***/
}
/***
*
*     Get machine parameters
*
***/
if (((n == 0) || (m == 0))) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlascl",999999);
}
    smlnum = com.polytechnik.lapack.Dlamch.dlamch('S');
// *
bignum = (1.0/smlnum);
cfromc = cfrom;
// *
ctoc = cto;
label10:for(int jj10=0;jj10<Util.ITMAX;jj10++){//label10: Dummy.label("com/polytechnik/lapack/Dlascl",10);
cfrom1 = (cfromc*smlnum);
/***
!        CFROMC is an inf.  Multiply by a correctly signed zero for
!        finite CTOC, or a NaN if CTOC is infinite.
***/
if ((cfrom1 == cfromc))  {
    mul = (ctoc/cfromc);
done = true;
cto1 = ctoc;
}
else  {
  cto1 = (ctoc/bignum);
/***
!           CTOC is either 0 or an inf.  In both cases, CTOC itself
!           serves as the correct multiplication factor.
***/
if ((cto1 == ctoc))  {
    mul = ctoc;
done = true;
cfromc = 1.0;
}
else if (((Math.abs(cfrom1) > Math.abs(ctoc)) && (ctoc != 0.0)))  {
    mul = smlnum;
done = false;
cfromc = cfrom1;
}              // Close else if()
else if ((Math.abs(cto1) > Math.abs(cfromc)))  {
    mul = bignum;
done = false;
ctoc = cto1;
}              // Close else if()
else  {
  mul = (ctoc/cfromc);
done = true;
}              //  Close else.
// *
}              //  Close else.
/***
*
*        Full matrix
*
***/
if ((itype == 0))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",20);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",30);
}              //  Close for() loop. 
}
/***
*
*        Lower triangular matrix
*
***/
}
else if ((itype == 1))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = j; i <= m; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",40);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",50);
}              //  Close for() loop. 
}
/***
*
*        Upper triangular matrix
*
***/
}              // Close else if()
else if ((itype == 2))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= Math.min(j, m); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",60);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",70);
}              //  Close for() loop. 
}
/***
*
*        Upper Hessenberg matrix
*
***/
}              // Close else if()
else if ((itype == 3))  {
    {
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= Math.min((j+1), m); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",80);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",90);
}              //  Close for() loop. 
}
/***
*
*        Lower half of a symmetric band matrix
*
***/
}              // Close else if()
else if ((itype == 4))  {
    k3 = (kl+1);
k4 = (n+1);
{
for (j = 1; j <= n; j++) {
{
for (i = 1; i <= Math.min(k3, (k4-j)); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",100);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",110);
}              //  Close for() loop. 
}
/***
*
*        Upper half of a symmetric band matrix
*
***/
}              // Close else if()
else if ((itype == 5))  {
    k1 = (ku+2);
k3 = (ku+1);
{
for (j = 1; j <= n; j++) {
{
for (i = Math.max((k1-j), 1); i <= k3; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",120);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",130);
}              //  Close for() loop. 
}
/***
*
*        Band matrix
*
***/
}              // Close else if()
else if ((itype == 6))  {
    k1 = ((kl+ku)+2);
k2 = (kl+1);
k3 = (((2*kl)+ku)+1);
k4 = (((kl+ku)+1)+m);
{
for (j = 1; j <= n; j++) {
{
for (i = Math.max((k1-j), k2); i <= Math.min(k3, (k4-j)); i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = (a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*mul);
//Dummy.label("com/polytechnik/lapack/Dlascl",140);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dlascl",150);
}              //  Close for() loop. 
}
// *
}              // Close else if()
// *
if (!(!done)) {
    break label10;//Dummy.go_to("com/polytechnik/lapack/Dlascl",10);
}
}
    /***
*
*     End of DLASCL
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlascl",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlascl",999999);
return;
   }
} // End class.
