/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dlatrd.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dlatrd.java,v 1.5 2012/12/20 16:59:33 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dlatrd {

// *> \brief \b DLATRD reduces the first nb rows and columns of a symmetric
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DLATRD + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DLATRD( UPLO, N, NB, A, LDA, E, TAU, W, LDW )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            LDA, LDW, N, NB
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), E( * ), TAU( * ), W( LDW, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DLATRD reduces NB rows and columns of a real symmetric matrix A to
// *> symmetric tridiagonal form by an orthogonal similarity
// *> transformation Q**T * A * Q, and returns the matrices V and W which a
// *> needed to apply the transformation to the unreduced part of A.
// *>
// *> If UPLO = 'U', DLATRD reduces the last NB rows and columns of a
// *> matrix, of which the upper triangle is supplied;
// *> if UPLO = 'L', DLATRD reduces the first NB rows and columns of a
// *> matrix, of which the lower triangle is supplied.
// *>
// *> This is an auxiliary routine called by DSYTRD.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies whether the upper or lower triangular part of the

// *>          symmetric matrix A is stored:
// *>          = 'U': Upper triangular
// *>          = 'L': Lower triangular
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.
// *> \endverbatim
// *>
// *> \param[in] NB
// *> \verbatim
// *>          NB is INTEGER
// *>          The number of rows and columns to be reduced.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          n-by-n upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading n-by-n lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>          On exit:
// *>          if UPLO = 'U', the last NB columns have been reduced to
// *>            tridiagonal form, with the diagonal elements overwriting
// *>            the diagonal elements of A; the elements above the diagona
// *>            with the array TAU, represent the orthogonal matrix Q as a
// *>            product of elementary reflectors;
// *>          if UPLO = 'L', the first NB columns have been reduced to
// *>            tridiagonal form, with the diagonal elements overwriting
// *>            the diagonal elements of A; the elements below the diagona
// *>            with the array TAU, represent the  orthogonal matrix Q as 
// *>            product of elementary reflectors.
// *>          See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= (1,N).
// *> \endverbatim
// *>
// *> \param[out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          If UPLO = 'U', E(n-nb:n-1) contains the superdiagonal
// *>          elements of the last NB columns of the reduced matrix;
// *>          if UPLO = 'L', E(1:nb) contains the subdiagonal elements of

// *>          the first NB columns of the reduced matrix.
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          The scalar factors of the elementary reflectors, stored in
// *>          TAU(n-nb:n-1) if UPLO = 'U', and in TAU(1:nb) if UPLO = 'L'.
// *>          See Further Details.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (LDW,NB)
// *>          The n-by-nb matrix W required to update the unreduced part
// *>          of A.
// *> \endverbatim
// *>
// *> \param[in] LDW
// *> \verbatim
// *>          LDW is INTEGER
// *>          The leading dimension of the array W. LDW >= max(1,N).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERauxiliary
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  If UPLO = 'U', the matrix Q is represented as a product of elementar
// *>  reflectors
// *>
// *>     Q = H(n) H(n-1) . . . H(n-nb+1).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(i:n) = 0 and v(i-1) = 1; v(1:i-1) is stored on exit in A(1:i-1,i),
// *>  and tau in TAU(i-1).
// *>
// *>  If UPLO = 'L', the matrix Q is represented as a product of elementar
// *>  reflectors
// *>
// *>     Q = H(1) H(2) . . . H(nb).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i) = 0 and v(i+1) = 1; v(i+1:n) is stored on exit in A(i+1:n,i),
// *>  and tau in TAU(i).
// *>
// *>  The elements of the vectors v together form the n-by-nb matrix V
// *>  which is needed, with W, to apply the transformation to the unreduce
// *>  part of the matrix, using a symmetric rank-2k update of the form:
// *>  A := A - V*W**T - W*V**T.
// *>
// *>  The contents of A on exit are illustrated by the following examples

// *>  with n = 5 and nb = 2:
// *>
// *>  if UPLO = 'U':                       if UPLO = 'L':
// *>
// *>    (  a   a   a   v4  v5 )              (  d                  )
// *>    (      a   a   v4  v5 )              (  1   d              )
// *>    (          a   1   v5 )              (  v1  1   a          )
// *>    (              d   1  )              (  v1  v2  a   a      )
// *>    (                  d  )              (  v1  v2  a   a   a  )
// *>
// *>  where d denotes a diagonal element of the reduced matrix, a denotes

// *>  an element of the original matrix that is unchanged, and vi denotes

// *>  an element of the vector defining H(i).
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
***/
// *

public static void dlatrd (final char uplo,
int n,
int nb,
double [] a, int _a_offset,
int lda,
double [] e, int _e_offset,
double [] tau, int _tau_offset,
double [] w, int _w_offset,
int ldw)  {

int i= 0;
int iw= 0;
double alpha= 0.0d;
//uplo = uplo.substring((1)-1,1);
if ((n <= 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dlatrd",999999);
}
    /***
*
*        Reduce last NB columns of upper triangle
*
***/
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    {
int _i_inc = -1;
for (i = n; i >= ((n-nb)+1); i += _i_inc) {
iw = ((i-n)+nb);
/***
*
*              Update A(1:i,i)
*
***/
if ((i < n))  {
    com.polytechnik.lapack.Dgemv.dgemv('N',i,(n-i),-1.0,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,w,(i-(1))+((iw+1)-(1)) * (ldw)+ _w_offset,ldw,1.0,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('N',i,(n-i),-1.0,w,(1-(1))+((iw+1)-(1)) * (ldw)+ _w_offset,ldw,a,(i-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,1.0,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1);
}
/***
*
*              Generate elementary reflector H(i) to annihilate
*              A(1:i-2,i)
*
***/
if ((i > 1))  {
    dlarfg_adapter((i-1),a,((i-1)-(1))+(i-(1)) * (lda)+ _a_offset,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,((i-1)-(1))+ _tau_offset);
e[((i-1)-(1))+ _e_offset] = a[((i-1)-(1))+(i-(1)) * (lda)+ _a_offset];
/***
*
*              Compute W(1:i-1,i)
*
***/
a[((i-1)-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack.Dsymv.dsymv('U',(i-1),1.0,a,_a_offset,lda,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,w,(1-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
if ((i < n))  {
    com.polytechnik.lapack.Dgemv.dgemv('T',(i-1),(n-i),1.0,w,(1-(1))+((iw+1)-(1)) * (ldw)+ _w_offset,ldw,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,w,((i+1)-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('N',(i-1),(n-i),-1.0,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,w,((i+1)-(1))+(iw-(1)) * (ldw)+ _w_offset,1,1.0,w,(1-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('T',(i-1),(n-i),1.0,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,w,((i+1)-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('N',(i-1),(n-i),-1.0,w,(1-(1))+((iw+1)-(1)) * (ldw)+ _w_offset,ldw,w,((i+1)-(1))+(iw-(1)) * (ldw)+ _w_offset,1,1.0,w,(1-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
}
com.polytechnik.lapack.Dscal.dscal((i-1),tau[((i-1)-(1))+ _tau_offset],w,(1-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
alpha = (-(((0.5*tau[((i-1)-(1))+ _tau_offset])*com.polytechnik.lapack.Ddot.ddot((i-1),w,(1-(1))+(iw-(1)) * (ldw)+ _w_offset,1,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1))));
com.polytechnik.lapack.Daxpy.daxpy((i-1),alpha,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,w,(1-(1))+(iw-(1)) * (ldw)+ _w_offset,1);
// *
}
//Dummy.label("com/polytechnik/lapack/Dlatrd",10);
}              //  Close for() loop. 
}
/***
*
*        Reduce first NB columns of lower triangle
*
***/
}
else  {
  /***
*
*           Update A(i:n,i)
*
***/
{
for (i = 1; i <= nb; i++) {
com.polytechnik.lapack.Dgemv.dgemv('N',((n-i)+1),(i-1),-1.0,a,(i-(1))+(1-(1)) * (lda)+ _a_offset,lda,w,(i-(1))+(1-(1)) * (ldw)+ _w_offset,ldw,1.0,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('N',((n-i)+1),(i-1),-1.0,w,(i-(1))+(1-(1)) * (ldw)+ _w_offset,ldw,a,(i-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,a,(i-(1))+(i-(1)) * (lda)+ _a_offset,1);
/***
*
*              Generate elementary reflector H(i) to annihilate
*              A(i+2:n,i)
*
***/
if ((i < n))  {
    dlarfg_adapter((n-i),a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,a,(Math.min((i+2), n)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,(i-(1))+ _tau_offset);
e[(i-(1))+ _e_offset] = a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset];
/***
*
*              Compute W(i+1:n,i)
*
***/
a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack.Dsymv.dsymv('L',(n-i),1.0,a,((i+1)-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,w,((i+1)-(1))+(i-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('T',(n-i),(i-1),1.0,w,((i+1)-(1))+(1-(1)) * (ldw)+ _w_offset,ldw,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,w,(1-(1))+(i-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('N',(n-i),(i-1),-1.0,a,((i+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,w,(1-(1))+(i-(1)) * (ldw)+ _w_offset,1,1.0,w,((i+1)-(1))+(i-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('T',(n-i),(i-1),1.0,a,((i+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,w,(1-(1))+(i-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dgemv.dgemv('N',(n-i),(i-1),-1.0,w,((i+1)-(1))+(1-(1)) * (ldw)+ _w_offset,ldw,w,(1-(1))+(i-(1)) * (ldw)+ _w_offset,1,1.0,w,((i+1)-(1))+(i-(1)) * (ldw)+ _w_offset,1);
com.polytechnik.lapack.Dscal.dscal((n-i),tau[(i-(1))+ _tau_offset],w,((i+1)-(1))+(i-(1)) * (ldw)+ _w_offset,1);
alpha = (-(((0.5*tau[(i-(1))+ _tau_offset])*com.polytechnik.lapack.Ddot.ddot((n-i),w,((i+1)-(1))+(i-(1)) * (ldw)+ _w_offset,1,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1))));
com.polytechnik.lapack.Daxpy.daxpy((n-i),alpha,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,w,((i+1)-(1))+(i-(1)) * (ldw)+ _w_offset,1);
// *
}
//Dummy.label("com/polytechnik/lapack/Dlatrd",20);
}              //  Close for() loop. 
}
// *
}              //  Close else.
/***
*
*     End of DLATRD
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dlatrd",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dlatrd",999999);
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,double [] arg4 , int arg4_offset )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);
doubleW _f2j_tmp4 = new doubleW(arg4[arg4_offset]);

com.polytechnik.lapack.Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,_f2j_tmp4);

arg1[arg1_offset] = _f2j_tmp1.val;
arg4[arg4_offset] = _f2j_tmp4.val;
}

} // End class.
