/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dnrm2.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dnrm2.java,v 1.4 2012/12/20 14:04:01 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dnrm2 {

// *> \brief \b DNRM2
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       DOUBLE PRECISION FUNCTION DNRM2(N,X,INCX)
// * 
// *       .. Scalar Arguments ..
// *       INTEGER INCX,N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION X(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DNRM2 returns the euclidean norm of a vector via the function
// *> name, so that
// *>
// *>    DNRM2 := sqrt( x'*x )
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level1
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  -- This version written on 25-October-1982.
// *>     Modified on 14-October-1993 to inline the call to DLASSQ.
// *>     Sven Hammarling, Nag Ltd.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
// *     ..

public static double dnrm2 (int n,
double [] x, int _x_offset,
int incx)  {

double dnrm2= 0.0d;
double absxi= 0.0d;
double norm= 0.0d;
double scale= 0.0d;
double ssq= 0.0d;
int ix= 0;
if (((n < 1) || (incx < 1)))  {
    norm = 0.0;
}
else if ((n == 1))  {
    norm = Math.abs(x[(1-(1))+ _x_offset]);
}              // Close else if()
else  {
  scale = 0.0;
/***
*        The following loop is equivalent to this call to the LAPACK
*        auxiliary routine:
*        CALL DLASSQ( N, X, INCX, SCALE, SSQ )
*
***/
ssq = 1.0;
{
int _ix_inc = incx;
for (ix = 1; (_ix_inc < 0) ? ix >= (1+(((n-1))*incx)) : ix <= (1+(((n-1))*incx)); ix += _ix_inc) {
if ((x[(ix-(1))+ _x_offset] != 0.0))  {
    absxi = Math.abs(x[(ix-(1))+ _x_offset]);
if ((scale < absxi))  {
    ssq = (1.0+(ssq*( Math.pow(((scale/absxi)), 2))));
scale = absxi;
}
else  {
  ssq = (ssq+( Math.pow(((absxi/scale)), 2)));
}              //  Close else.
}
//Dummy.label("com/polytechnik/lapack/Dnrm2",10);
}              //  Close for() loop. 
}
norm = (scale*Math.sqrt(ssq));
// *
}              //  Close else.
dnrm2 = norm;
/***
*
*     End of DNRM2.
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dnrm2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dnrm2",999999);
return dnrm2;
   }
} // End class.
