/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dorm2l.f
 *  f2java version: 0.8.1
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dorm2l.java,v 1.4 2012/12/20 16:45:27 mal Exp $ 
 *
 */

package com.polytechnik.lapack;



public class Dorm2l {

// *> \brief \b DORM2L multiplies a general matrix by the orthogonal matrix
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DORM2L + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DORM2L( SIDE, TRANS, M, N, K, A, LDA, TAU, C, LDC,
// *                          WORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          SIDE, TRANS
// *       INTEGER            INFO, K, LDA, LDC, M, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), C( LDC, * ), TAU( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DORM2L overwrites the general real m by n matrix C with
// *>
// *>       Q * C  if SIDE = 'L' and TRANS = 'N', or
// *>
// *>       Q**T * C  if SIDE = 'L' and TRANS = 'T', or
// *>
// *>       C * Q  if SIDE = 'R' and TRANS = 'N', or
// *>
// *>       C * Q**T if SIDE = 'R' and TRANS = 'T',
// *>
// *> where Q is a real orthogonal matrix defined as the product of k
// *> elementary reflectors
// *>
// *>       Q = H(k) . . . H(2) H(1)
// *>
// *> as returned by DGEQLF. Q is of order m if SIDE = 'L' and of order n
// *> if SIDE = 'R'.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] SIDE
// *> \verbatim
// *>          SIDE is CHARACTER*1
// *>          = 'L': apply Q or Q**T from the Left
// *>          = 'R': apply Q or Q**T from the Right
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>          = 'N': apply Q  (No transpose)
// *>          = 'T': apply Q**T (Transpose)
// *> \endverbatim
// *>
// *> \param[in] M
// *> \verbatim
// *>          M is INTEGER
// *>          The number of rows of the matrix C. M >= 0.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The number of columns of the matrix C. N >= 0.
// *> \endverbatim
// *>
// *> \param[in] K
// *> \verbatim
// *>          K is INTEGER
// *>          The number of elementary reflectors whose product defines
// *>          the matrix Q.
// *>          If SIDE = 'L', M >= K >= 0;
// *>          if SIDE = 'R', N >= K >= 0.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,K)
// *>          The i-th column must contain the vector which defines the
// *>          elementary reflector H(i), for i = 1,2,...,k, as returned by
// *>          DGEQLF in the last k columns of its array argument A.
// *>          A is modified by the routine but restored on exit.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.
// *>          If SIDE = 'L', LDA >= max(1,M);
// *>          if SIDE = 'R', LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (K)
// *>          TAU(i) must contain the scalar factor of the elementary
// *>          reflector H(i), as returned by DGEQLF.
// *> \endverbatim
// *>
// *> \param[in,out] C
// *> \verbatim
// *>          C is DOUBLE PRECISION array, dimension (LDC,N)
// *>          On entry, the m by n matrix C.
// *>          On exit, C is overwritten by Q*C or Q**T*C or C*Q**T or C*Q.
// *> \endverbatim
// *>
// *> \param[in] LDC
// *> \verbatim
// *>          LDC is INTEGER
// *>          The leading dimension of the array C. LDC >= max(1,M).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension
// *>                                   (N) if SIDE = 'L',
// *>                                   (M) if SIDE = 'R'
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -i, the i-th argument had an illegal value
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input arguments
*
***/

public static void dorm2l (final char side,
final char trans,
int m,
int n,
int k,
double [] a, int _a_offset,
int lda,
double [] tau, int _tau_offset,
double [] c, int _c_offset,
int ldc,
double [] work, int _work_offset,
intW info)  {

boolean left= false;
boolean notran= false;
int i= 0;
int i1= 0;
int i2= 0;
int i3= 0;
int mi= 0;
int ni= 0;
int nq= 0;
double aii= 0.0d;
//side = side.substring((1)-1,1);
//trans = trans.substring((1)-1,1);
info.val = 0;
left = com.polytechnik.lapack.Lsame.lsame(side,'L');
/***
*
*     NQ is the order of Q
*
***/
notran = com.polytechnik.lapack.Lsame.lsame(trans,'N');
if (left)  {
    nq = m;
}
else  {
  nq = n;
}              //  Close else.
if (((!left) && (!com.polytechnik.lapack.Lsame.lsame(side,'R'))))  {
    info.val = -1;
}
else if (((!notran) && (!com.polytechnik.lapack.Lsame.lsame(trans,'T'))))  {
    info.val = -2;
}              // Close else if()
else if ((m < 0))  {
    info.val = -3;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if (((k < 0) || (k > nq)))  {
    info.val = -5;
}              // Close else if()
else if ((lda < Math.max(1, nq)))  {
    info.val = -7;
}              // Close else if()
else if ((ldc < Math.max(1, m)))  {
    info.val = -10;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DORM2L",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dorm2l",999999);
/***
*
*     Quick return if possible
*
***/
}
// *
if ((((m == 0) || (n == 0)) || (k == 0))) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dorm2l",999999);
}
    if ((((left && notran)) || (((!left) && (!notran)))))  {
    i1 = 1;
i2 = k;
i3 = 1;
}
else  {
  i1 = k;
i2 = 1;
i3 = -1;
// *
}              //  Close else.
if (left)  {
    ni = n;
}
else  {
  mi = m;
// *
}              //  Close else.
{
int _i_inc = i3;
for (i = i1; (_i_inc < 0) ? i >= i2 : i <= i2; i += _i_inc) {
/***
*
*           H(i) is applied to C(1:m-k+i,1:n)
*
***/
if (left)  {
    mi = ((m-k)+i);
/***
*
*           H(i) is applied to C(1:m,1:n-k+i)
*
***/
}
else  {
  ni = ((n-k)+i);
/***
*
*        Apply H(i)
*
***/
}              //  Close else.
aii = a[(((nq-k)+i)-(1))+(i-(1)) * (lda)+ _a_offset];
a[(((nq-k)+i)-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
com.polytechnik.lapack.Dlarf.dlarf(side,mi,ni,a,(1-(1))+(i-(1)) * (lda)+ _a_offset,1,tau[(i-(1))+ _tau_offset],c,_c_offset,ldc,work,_work_offset);
a[(((nq-k)+i)-(1))+(i-(1)) * (lda)+ _a_offset] = aii;
//Dummy.label("com/polytechnik/lapack/Dorm2l",10);
}              //  Close for() loop. 
}
/***
*
*     End of DORM2L
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dorm2l",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dorm2l",999999);
return;
   }
} // End class.
