/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dpotf2.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dpotf2.java,v 1.5 2012/12/20 04:36:57 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dpotf2 {

// *> \brief \b DPOTF2 computes the Cholesky factorization of a symmetric/H
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DPOTF2 + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DPOTF2( UPLO, N, A, LDA, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DPOTF2 computes the Cholesky factorization of a real symmetric
// *> positive definite matrix A.
// *>
// *> The factorization has the form
// *>    A = U**T * U ,  if UPLO = 'U', or
// *>    A = L  * L**T,  if UPLO = 'L',
// *> where U is an upper triangular matrix and L is lower triangular.
// *>
// *> This is the unblocked version of the algorithm, calling Level 2 BLAS.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies whether the upper or lower triangular part of the

// *>          symmetric matrix A is stored.
// *>          = 'U':  Upper triangular
// *>          = 'L':  Lower triangular
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          n by n upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading n by n lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>
// *>          On exit, if INFO = 0, the factor U or L from the Cholesky
// *>          factorization A = U**T *U  or A = L*L**T.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0: successful exit
// *>          < 0: if INFO = -k, the k-th argument had an illegal value
// *>          > 0: if INFO = k, the leading minor of order k is not
// *>               positive definite, and the factorization could not be
// *>               completed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doublePOcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dpotf2 (final char uplo,
int n,
double [] a, int _a_offset,
int lda,
intW info)  {

boolean upper= false;
int j= 0;
double ajj= 0.0d;
//uplo = uplo.substring((1)-1,1);
info.val = 0;
upper = com.polytechnik.lapack.Lsame.lsame(uplo,'U');
if (((!upper) && (!com.polytechnik.lapack.Lsame.lsame(uplo,'L'))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -4;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DPOTF2",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dpotf2",999999);
/***
*
*     Quick return if possible
*
***/
}

// *
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dpotf2",999999);
}

label30:{
    /***
*
*        Compute the Cholesky factorization A = U**T *U.
*
***/
if (upper)  {
    /***
*
*           Compute U(J,J) and test for non-positive-definiteness.
*
***/
{
for (j = 1; j <= n; j++) {
ajj = (a[(j-(1))+(j-(1)) * (lda)+ _a_offset]-com.polytechnik.lapack.Ddot.ddot((j-1),a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1));
if ((!(ajj > 0.0) || com.polytechnik.lapack.Disnan.disnan(ajj)))  {
    a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = ajj;
    break label30;//Dummy.go_to("com/polytechnik/lapack/Dpotf2",30);
}
ajj = Math.sqrt(ajj);
/***
*
*           Compute elements J+1:N of row J.
*
***/
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = ajj;
if ((j < n))  {
    com.polytechnik.lapack.Dgemv.dgemv('T',(j-1),(n-j),-1.0,a,(1-(1))+((j+1)-(1)) * (lda)+ _a_offset,lda,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1,1.0,a,(j-(1))+((j+1)-(1)) * (lda)+ _a_offset,lda);
com.polytechnik.lapack.Dscal.dscal((n-j),(1.0/ajj),a,(j-(1))+((j+1)-(1)) * (lda)+ _a_offset,lda);
}
//Dummy.label("com/polytechnik/lapack/Dpotf2",10);
}              //  Close for() loop. 
}
/***
*
*        Compute the Cholesky factorization A = L*L**T.
*
***/
}
else  {
  /***
*
*           Compute L(J,J) and test for non-positive-definiteness.
*
***/
{
for (j = 1; j <= n; j++) {
ajj = (a[(j-(1))+(j-(1)) * (lda)+ _a_offset]-com.polytechnik.lapack.Ddot.ddot((j-1),a,(j-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,(j-(1))+(1-(1)) * (lda)+ _a_offset,lda));
if ((!(ajj > 0.0) || com.polytechnik.lapack.Disnan.disnan(ajj)))  {
    a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = ajj;
    break label30;//Dummy.go_to("com/polytechnik/lapack/Dpotf2",30);
}
ajj = Math.sqrt(ajj);
/***
*
*           Compute elements J+1:N of column J.
*
***/
a[(j-(1))+(j-(1)) * (lda)+ _a_offset] = ajj;
if ((j < n))  {
    com.polytechnik.lapack.Dgemv.dgemv('N',(n-j),(j-1),-1.0,a,((j+1)-(1))+(1-(1)) * (lda)+ _a_offset,lda,a,(j-(1))+(1-(1)) * (lda)+ _a_offset,lda,1.0,a,((j+1)-(1))+(j-(1)) * (lda)+ _a_offset,1);
com.polytechnik.lapack.Dscal.dscal((n-j),(1.0/ajj),a,((j+1)-(1))+(j-(1)) * (lda)+ _a_offset,1);
}
//Dummy.label("com/polytechnik/lapack/Dpotf2",20);
}              //  Close for() loop. 
}
}              //  Close else.
// *
return;//Dummy.go_to("com/polytechnik/lapack/Dpotf2",40);

} //label30:
//Dummy.label("com/polytechnik/lapack/Dpotf2",30);
// *
info.val = j;
//label40:
//Dummy.label("com/polytechnik/lapack/Dpotf2",40);
/***
*
*     End of DPOTF2
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dpotf2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dpotf2",999999);
return;
   }
} // End class.
