/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dstebz.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dstebz.java,v 1.16 2012/12/19 22:00:16 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dstebz {

// *> \brief \b DSTEBZ
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSTEBZ + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSTEBZ( RANGE, ORDER, N, VL, VU, IL, IU, ABSTOL, D, E
// *                          M, NSPLIT, W, IBLOCK, ISPLIT, WORK, IWORK,
// *                          INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          ORDER, RANGE
// *       INTEGER            IL, INFO, IU, M, N, NSPLIT
// *       DOUBLE PRECISION   ABSTOL, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IBLOCK( * ), ISPLIT( * ), IWORK( * )
// *       DOUBLE PRECISION   D( * ), E( * ), W( * ), WORK( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSTEBZ computes the eigenvalues of a symmetric tridiagonal
// *> matrix T.  The user may ask for all eigenvalues, all eigenvalues
// *> in the half-open interval (VL, VU], or the IL-th through IU-th
// *> eigenvalues.
// *>
// *> To avoid overflow, the matrix must be scaled so that its
// *> largest element is no greater than overflow**(1/2) * underflow**(1/4)
// *> accuracy, it should not be much smaller than that.
// *>
// *> See W. Kahan "Accurate Eigenvalues of a Symmetric Tridiagonal
// *> Matrix", Report CS41, Computer Science Dept., Stanford
// *> University, July 21, 1966.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': ("All")   all eigenvalues will be found.
// *>          = 'V': ("Value") all eigenvalues in the half-open interval
// *>                           (VL, VU] will be found.
// *>          = 'I': ("Index") the IL-th through IU-th eigenvalues (of the
// *>                           entire matrix) will be found.
// *> \endverbatim
// *>
// *> \param[in] ORDER
// *> \verbatim
// *>          ORDER is CHARACTER*1
// *>          = 'B': ("By Block") the eigenvalues will be grouped by
// *>                              split-off block (see IBLOCK, ISPLIT) and
// *>                              ordered from smallest to largest within

// *>                              the block.
// *>          = 'E': ("Entire matrix")
// *>                              the eigenvalues for the entire matrix
// *>                              will be ordered from smallest to
// *>                              largest.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the tridiagonal matrix T.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>
// *>          If RANGE='V', the lower and upper bounds of the interval to

// *>          be searched for eigenvalues.  Eigenvalues less than or equal
// *>          to VL, or greater than VU, will not be returned.  VL < VU.
// *>          Not referenced if RANGE = 'A' or 'I'.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N, if N > 0; IL = 1 and IU = 0 if N = 0.
// *>          Not referenced if RANGE = 'A' or 'V'.
// *> \endverbatim
// *>
// *> \param[in] ABSTOL
// *> \verbatim
// *>          ABSTOL is DOUBLE PRECISION
// *>          The absolute tolerance for the eigenvalues.  An eigenvalue
// *>          (or cluster) is considered to be located if it has been
// *>          determined to lie in an interval whose width is ABSTOL or
// *>          less.  If ABSTOL is less than or equal to zero, then ULP*|T|
// *>          will be used, where |T| means the 1-norm of T.
// *>
// *>          Eigenvalues will be computed most accurately when ABSTOL is

// *>          set to twice the underflow threshold 2*DLAMCH('S'), not zero
// *> \endverbatim
// *>
// *> \param[in] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The n diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[in] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          The (n-1) off-diagonal elements of the tridiagonal matrix T.
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The actual number of eigenvalues found. 0 <= M <= N.
// *>          (See also the description of INFO=2,3.)
// *> \endverbatim
// *>
// *> \param[out] NSPLIT
// *> \verbatim
// *>          NSPLIT is INTEGER
// *>          The number of diagonal blocks in the matrix T.
// *>          1 <= NSPLIT <= N.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          On exit, the first M elements of W will contain the
// *>          eigenvalues.  (DSTEBZ may use the remaining N-M elements as

// *>          workspace.)
// *> \endverbatim
// *>
// *> \param[out] IBLOCK
// *> \verbatim
// *>          IBLOCK is INTEGER array, dimension (N)
// *>          At each row/column j where E(j) is zero or small, the
// *>          matrix T is considered to split into a block diagonal
// *>          matrix.  On exit, if INFO = 0, IBLOCK(i) specifies to which

// *>          block (from 1 to the number of blocks) the eigenvalue W(i)
// *>          belongs.  (DSTEBZ may use the remaining N-M elements as
// *>          workspace.)
// *> \endverbatim
// *>
// *> \param[out] ISPLIT
// *> \verbatim
// *>          ISPLIT is INTEGER array, dimension (N)
// *>          The splitting points, at which T breaks up into submatrices.
// *>          The first submatrix consists of rows/columns 1 to ISPLIT(1),
// *>          the second of rows/columns ISPLIT(1)+1 through ISPLIT(2),
// *>          etc., and the NSPLIT-th consists of rows/columns
// *>          ISPLIT(NSPLIT-1)+1 through ISPLIT(NSPLIT)=N.
// *>          (Only the first NSPLIT elements will actually be used, but
// *>          since the user cannot know a priori what value NSPLIT will
// *>          have, N words must be reserved for ISPLIT.)
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (4*N)
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (3*N)
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  some or all of the eigenvalues failed to converge or
// *>                were not computed:
// *>                =1 or 3: Bisection failed to converge for some
// *>                        eigenvalues; these eigenvalues are flagged by 
// *>                        negative block number.  The effect is that the
// *>                        eigenvalues may not be as accurate as the
// *>                        absolute and relative tolerances.  This is
// *>                        generally caused by unexpectedly inaccurate
// *>                        arithmetic.
// *>                =2 or 3: RANGE='I' only: Not all of the eigenvalues
// *>                        IL:IU were found.
// *>                        Effect: M < IU+1-IL
// *>                        Cause:  non-monotonic arithmetic, causing the

// *>                                Sturm sequence to be non-monotonic.
// *>                        Cure:   recalculate, using RANGE='A', and pick
// *>                                out eigenvalues IL:IU.  In some cases,
// *>                                increasing the PARAMETER "FUDGE" may
// *>                                make things work.
// *>                = 4:    RANGE='I', and the Gershgorin interval
// *>                        initially used was too small.  No eigenvalues

// *>                        were computed.
// *>                        Probable cause: your machine has sloppy
// *>                                        floating-point arithmetic.
// *>                        Cure: Increase the PARAMETER "FUDGE",
// *>                              recompile, and try again.
// *> \endverbatim
// *
// *> \par Internal Parameters:
// *  =========================
// *>
// *> \verbatim
// *>  RELFAC  DOUBLE PRECISION, default = 2.0e0
// *>          The relative tolerance.  An interval (a,b] lies within
// *>          "relative tolerance" if  b-a < RELFAC*ulp*max(|a|,|b|),
// *>          where "ulp" is the machine precision (distance from 1 to
// *>          the next larger floating point number.)
// *>
// *>  FUDGE   DOUBLE PRECISION, default = 2
// *>          A "fudge factor" to widen the Gershgorin intervals.  Ideally
// *>          a value of 1 should work, but on machines with sloppy
// *>          arithmetic, this needs to be larger.  The default for
// *>          publicly released versions should be large enough to handle

// *>          the worst machine around.  Note that this has no effect
// *>          on accuracy of the solution.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup auxOTHERcomputational
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. Local Arrays ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
***/
/***
*
*     Decode RANGE
*
***/

public static void dstebz (final char range,
final char order,
int n,
double vl,
double vu,
int il,
int iu,
double abstol,
double [] d, int _d_offset,
double [] e, int _e_offset,
intW m,
intW nsplit,
double [] w, int _w_offset,
int [] iblock, int _iblock_offset,
int [] isplit, int _isplit_offset,
double [] work, int _work_offset,
int [] iwork, int _iwork_offset,
intW info)  {

boolean ncnvrg= false;
boolean toofew= false;
int ib= 0;
int ibegin= 0;
int idiscl= 0;
int idiscu= 0;
int ie= 0;
int iend= 0;
intW iinfo= new intW(0);
intW im= new intW(0);
int in= 0;
int ioff= 0;
int iorder= 0;
intW iout= new intW(0);
int irange= 0;
int itmax= 0;
int itmp1= 0;
int iw= 0;
int iwoff= 0;
int j= 0;
int jb= 0;
int jdisc= 0;
int je= 0;
int nb= 0;
int nwl= 0;
int nwu= 0;
double atoli= 0.0d;
double bnorm= 0.0d;
double gl= 0.0d;
double gu= 0.0d;
double pivmin= 0.0d;
double rtoli= 0.0d;
double safemn= 0.0d;
double tmp1= 0.0d;
double tmp2= 0.0d;
double tnorm= 0.0d;
double ulp= 0.0d;
double wkill= 0.0d;
double wl= 0.0d;
double wlu= 0.0d;
double wu= 0.0d;
double wul= 0.0d;
int [] idumma= new int[(1)];
final double FUDGE = 2.1, RELFAC = 2.0;

//range = range.substring((1)-1,1);
//order = order.substring((1)-1,1);
info.val = 0;
if (com.polytechnik.lapack.Lsame.lsame(range,'A'))  {
    irange = 1;
}
else if (com.polytechnik.lapack.Lsame.lsame(range,'V'))  {
    irange = 2;
}              // Close else if()
else if (com.polytechnik.lapack.Lsame.lsame(range,'I'))  {
    irange = 3;
}              // Close else if()
else  {
  irange = 0;
/***
*
*     Decode ORDER
*
***/
}              //  Close else.
if (com.polytechnik.lapack.Lsame.lsame(order,'B'))  {
    iorder = 2;
}
else if (com.polytechnik.lapack.Lsame.lsame(order,'E'))  {
    iorder = 1;
}              // Close else if()
else  {
  iorder = 0;
/***
*
*     Check for Errors
*
***/
}              //  Close else.
if ((irange <= 0))  {
    info.val = -1;
}
else if ((iorder <= 0))  {
    info.val = -2;
}              // Close else if()
else if ((n < 0))  {
    info.val = -3;
}              // Close else if()
else if ((irange == 2))  {
    if ((vl >= vu)) {
    info.val = -5;
}
    }              // Close else if()
else if (((irange == 3) && (((il < 1) || (il > Math.max(1, n))))))  {
    info.val = -6;
}              // Close else if()
else if (((irange == 3) && (((iu < Math.min(n, il)) || (iu > n)))))  {
    info.val = -7;
// *
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSTEBZ",(-(info.val)));
    return;// Dummy.go_to("com/polytechnik/lapack/Dstebz",999999);
/***
*
*     Initialize error flags
*
***/
}
info.val = 0;
ncnvrg = false;
/***
*
*     Quick return if possible
*
***/
toofew = false;
m.val = 0;
/***
*
*     Simplifications:
*
***/
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dstebz",999999);
}
    /***
*
*     Get machine constants
*     NB is the minimum vector length for vector bisection, or 0
*     if only scalar is to be done.
*
***/
if ((((irange == 3) && (il == 1)) && (iu == n))) {
    irange = 1;
}
safemn = com.polytechnik.lapack.Dlamch.dlamch('S');
ulp = com.polytechnik.lapack.Dlamch.dlamch('P');
rtoli = (ulp*RELFAC);
nb = 1; //com.polytechnik.lapack.Ilaenv.ilaenv(1,"DSTEBZ"," ",n,-1,-1,-1);
/***
*
*     Special Case when N=1
*
***/
if ((nb <= 1)) {
    nb = 0;
}
    if ((n == 1))  {
    nsplit.val = 1;
isplit[(1-(1))+ _isplit_offset] = 1;
if (((irange == 2) && (((vl >= d[(1-(1))+ _d_offset]) || (vu < d[(1-(1))+ _d_offset])))))  {
    m.val = 0;
}
else  {
  w[(1-(1))+ _w_offset] = d[(1-(1))+ _d_offset];
iblock[(1-(1))+ _iblock_offset] = 1;
m.val = 1;
}              //  Close else.
return;//Dummy.go_to("com/polytechnik/lapack/Dstebz",999999);
/***
*
*     Compute Splitting Points
*
***/
}
nsplit.val = 1;
work[(n-(1))+ _work_offset] = 0.0;
// *
pivmin = 1.0;
{
for (j = 2; j <= n; j++) {
tmp1 = ( Math.pow(e[((j-1)-(1))+ _e_offset], 2));
if ((((Math.abs((d[(j-(1))+ _d_offset]*d[((j-1)-(1))+ _d_offset]))*( Math.pow(ulp, 2)))+safemn) > tmp1))  {
    isplit[(nsplit.val-(1))+ _isplit_offset] = (j-1);
nsplit.val = (nsplit.val+1);
work[((j-1)-(1))+ _work_offset] = 0.0;
}
else  {
  work[((j-1)-(1))+ _work_offset] = tmp1;
pivmin = Math.max(pivmin, tmp1);
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dstebz",10);
}              //  Close for() loop. 
}
isplit[(nsplit.val-(1))+ _isplit_offset] = n;
/***
*
*     Compute Interval and ATOLI
*
***/
pivmin = (pivmin*safemn);
/***
*
*        RANGE='I': Compute the interval containing eigenvalues
*                   IL through IU.
*
*        Compute Gershgorin interval for entire (split) matrix
*        and use it as the initial interval
*
***/
if ((irange == 3))  {
    gu = d[(1-(1))+ _d_offset];
gl = d[(1-(1))+ _d_offset];
// *
tmp1 = 0.0;
{
for (j = 1; j <= (n-1); j++) {
tmp2 = Math.sqrt(work[(j-(1))+ _work_offset]);
gu = Math.max(gu, ((d[(j-(1))+ _d_offset]+tmp1)+tmp2));
gl = Math.min(gl, ((d[(j-(1))+ _d_offset]-tmp1)-tmp2));
tmp1 = tmp2;
// *
//Dummy.label("com/polytechnik/lapack/Dstebz",20);
}              //  Close for() loop. 
}
gu = Math.max(gu, (d[(n-(1))+ _d_offset]+tmp1));
gl = Math.min(gl, (d[(n-(1))+ _d_offset]-tmp1));
tnorm = Math.max(Math.abs(gl), Math.abs(gu));
gl = ((gl-(((FUDGE*tnorm)*ulp)*n))-((FUDGE*2.0)*pivmin));
/***
*
*        Compute Iteration parameters
*
***/
gu = ((gu+(((FUDGE*tnorm)*ulp)*n))+(FUDGE*pivmin));
itmax = ((int)((((Math.log((tnorm+pivmin))-Math.log(pivmin)))/LOG2))+2);
if(itmax>=Util.ITMAX) itmax=Util.ITMAX;
//System.err.println("itmax 1="+itmax);
if ((abstol <= 0.0))  {
    atoli = (ulp*tnorm);
}
else  {
  atoli = abstol;
// *
}              //  Close else.
work[((n+1)-(1))+ _work_offset] = gl;
work[((n+2)-(1))+ _work_offset] = gl;
work[((n+3)-(1))+ _work_offset] = gu;
work[((n+4)-(1))+ _work_offset] = gu;
work[((n+5)-(1))+ _work_offset] = gl;
work[((n+6)-(1))+ _work_offset] = gu;
iwork[(1-(1))+ _iwork_offset] = -1;
iwork[(2-(1))+ _iwork_offset] = -1;
iwork[(3-(1))+ _iwork_offset] = (n+1);
iwork[(4-(1))+ _iwork_offset] = (n+1);
iwork[(5-(1))+ _iwork_offset] = (il-1);
// *
iwork[(6-(1))+ _iwork_offset] = iu;
// *
com.polytechnik.lapack.Dlaebz.dlaebz(3,itmax,n,2,2,nb,atoli,rtoli,pivmin,d,_d_offset,e,_e_offset,work,_work_offset,iwork,(5-(1))+ _iwork_offset,work,((n+1)-(1))+ _work_offset,work,((n+5)-(1))+ _work_offset,iout,iwork,_iwork_offset,w,_w_offset,iblock,_iblock_offset,iinfo);
if ((iwork[(6-(1))+ _iwork_offset] == iu))  {
    wl = work[((n+1)-(1))+ _work_offset];
wlu = work[((n+3)-(1))+ _work_offset];
nwl = iwork[(1-(1))+ _iwork_offset];
wu = work[((n+4)-(1))+ _work_offset];
wul = work[((n+2)-(1))+ _work_offset];
nwu = iwork[(4-(1))+ _iwork_offset];
}
else  {
  wl = work[((n+2)-(1))+ _work_offset];
wlu = work[((n+4)-(1))+ _work_offset];
nwl = iwork[(2-(1))+ _iwork_offset];
wu = work[((n+3)-(1))+ _work_offset];
wul = work[((n+1)-(1))+ _work_offset];
nwu = iwork[(3-(1))+ _iwork_offset];
// *
}              //  Close else.
if (((((nwl < 0) || (nwl >= n)) || (nwu < 1)) || (nwu > n)))  {
    info.val = 4;
    return;//Dummy.go_to("com/polytechnik/lapack/Dstebz",999999);
}
/***
*
*        RANGE='A' or 'V' -- Set ATOLI
*
***/
}
else  {
  // *
tnorm = Math.max((Math.abs(d[(1-(1))+ _d_offset])+Math.abs(e[(1-(1))+ _e_offset])), (Math.abs(d[(n-(1))+ _d_offset])+Math.abs(e[((n-1)-(1))+ _e_offset])));
{
for (j = 2; j <= (n-1); j++) {
tnorm = Math.max(tnorm, ((Math.abs(d[(j-(1))+ _d_offset])+Math.abs(e[((j-1)-(1))+ _e_offset]))+Math.abs(e[(j-(1))+ _e_offset])));
// *
//Dummy.label("com/polytechnik/lapack/Dstebz",30);
}              //  Close for() loop. 
}
if ((abstol <= 0.0))  {
    atoli = (ulp*tnorm);
}
else  {
  atoli = abstol;
// *
}              //  Close else.
if ((irange == 2))  {
    wl = vl;
wu = vu;
}
else  {
  wl = 0.0;
wu = 0.0;
}              //  Close else.
/***
*
*     Find Eigenvalues -- Loop Over Blocks and recompute NWL and NWU.
*     NWL accumulates the number of eigenvalues .le. WL,
*     NWU accumulates the number of eigenvalues .le. WU
*
***/
}              //  Close else.
m.val = 0;
iend = 0;
info.val = 0;
nwl = 0;
// *
nwu = 0;
{
label70:for (jb = 1; jb <= nsplit.val; jb++) {
ioff = iend;
ibegin = (ioff+1);
iend = isplit[(jb-(1))+ _isplit_offset];
// *
in = (iend-ioff);
/***
*
*           Special Case -- IN=1
*
***/
if ((in == 1))  {
    if (((irange == 1) || (wl >= (d[(ibegin-(1))+ _d_offset]-pivmin)))) {
    nwl = (nwl+1);
}
    if (((irange == 1) || (wu >= (d[(ibegin-(1))+ _d_offset]-pivmin)))) {
    nwu = (nwu+1);
}
    if (((irange == 1) || (((wl < (d[(ibegin-(1))+ _d_offset]-pivmin)) && (wu >= (d[(ibegin-(1))+ _d_offset]-pivmin))))))  {
    m.val = (m.val+1);
w[(m.val-(1))+ _w_offset] = d[(ibegin-(1))+ _d_offset];
iblock[(m.val-(1))+ _iblock_offset] = jb;
}
/***
*
*           General Case -- IN > 1
*
*           Compute Gershgorin Interval
*           and use it as the initial interval
*
***/
}
else  {
  gu = d[(ibegin-(1))+ _d_offset];
gl = d[(ibegin-(1))+ _d_offset];
// *
tmp1 = 0.0;
{
for (j = ibegin; j <= (iend-1); j++) {
tmp2 = Math.abs(e[(j-(1))+ _e_offset]);
gu = Math.max(gu, ((d[(j-(1))+ _d_offset]+tmp1)+tmp2));
gl = Math.min(gl, ((d[(j-(1))+ _d_offset]-tmp1)-tmp2));
tmp1 = tmp2;
// *
//Dummy.label("com/polytechnik/lapack/Dstebz",40);
}              //  Close for() loop. 
}
gu = Math.max(gu, (d[(iend-(1))+ _d_offset]+tmp1));
gl = Math.min(gl, (d[(iend-(1))+ _d_offset]-tmp1));
bnorm = Math.max(Math.abs(gl), Math.abs(gu));
gl = ((gl-(((FUDGE*bnorm)*ulp)*in))-(FUDGE*pivmin));
/***
*
*           Compute ATOLI for the current submatrix
*
***/
gu = ((gu+(((FUDGE*bnorm)*ulp)*in))+(FUDGE*pivmin));
if ((abstol <= 0.0))  {
    atoli = (ulp*Math.max(Math.abs(gl), Math.abs(gu)));
}
else  {
  atoli = abstol;
// *
}              //  Close else.
if ((irange > 1))  {
    if ((gu < wl))  {
    nwl = (nwl+in);
nwu = (nwu+in);
continue label70;//Dummy.go_to("com/polytechnik/lapack/Dstebz",70);
}
gl = Math.max(gl, wl);
gu = Math.min(gu, wu);
if ((gl >= gu)) {
continue label70;//Dummy.go_to("com/polytechnik/lapack/Dstebz",70);
}
    /***
*
*           Set Up Initial Interval
*
***/
}
work[((n+1)-(1))+ _work_offset] = gl;
work[(((n+in)+1)-(1))+ _work_offset] = gu;
// *
com.polytechnik.lapack.Dlaebz.dlaebz(1,0,in,in,1,nb,atoli,rtoli,pivmin,d,(ibegin-(1))+ _d_offset,e,(ibegin-(1))+ _e_offset,work,(ibegin-(1))+ _work_offset,idumma,0,work,((n+1)-(1))+ _work_offset,work,(((n+(2*in))+1)-(1))+ _work_offset,im,iwork,_iwork_offset,w,((m.val+1)-(1))+ _w_offset,iblock,((m.val+1)-(1))+ _iblock_offset,iinfo);
nwl = (nwl+iwork[(1-(1))+ _iwork_offset]);
nwu = (nwu+iwork[((in+1)-(1))+ _iwork_offset]);
/***
*
*           Compute Eigenvalues
*
***/
iwoff = (m.val-iwork[(1-(1))+ _iwork_offset]);
itmax = ((int)((((Math.log(((gu-gl)+pivmin))-Math.log(pivmin)))/LOG2))+2);
if(itmax>=Util.ITMAX) itmax=Util.ITMAX;
//System.err.println("itmax 2="+itmax+" gu="+gu+" gl="+gl+" pivmin="+pivmin);
/***
*
*           Copy Eigenvalues Into W and IBLOCK
*           Use -JB for block number for unconverged eigenvalues.
*
***/
com.polytechnik.lapack.Dlaebz.dlaebz(2,itmax,in,in,1,nb,atoli,rtoli,pivmin,d,(ibegin-(1))+ _d_offset,e,(ibegin-(1))+ _e_offset,work,(ibegin-(1))+ _work_offset,idumma,0,work,((n+1)-(1))+ _work_offset,work,(((n+(2*in))+1)-(1))+ _work_offset,iout,iwork,_iwork_offset,w,((m.val+1)-(1))+ _w_offset,iblock,((m.val+1)-(1))+ _iblock_offset,iinfo);
{
for (j = 1; j <= iout.val; j++) {
/***
*
*              Flag non-convergence.
*
***/
tmp1 = (0.5*((work[((j+n)-(1))+ _work_offset]+work[(((j+in)+n)-(1))+ _work_offset])));
if ((j > (iout.val-iinfo.val)))  {
    ncnvrg = true;
ib = (-(jb));
}
else  {
  ib = jb;
}              //  Close else.
{
for (je = ((iwork[(j-(1))+ _iwork_offset]+1)+iwoff); je <= (iwork[((j+in)-(1))+ _iwork_offset]+iwoff); je++) {
w[(je-(1))+ _w_offset] = tmp1;
iblock[(je-(1))+ _iblock_offset] = ib;
//Dummy.label("com/polytechnik/lapack/Dstebz",50);
}              //  Close for() loop. 
}
// *
//Dummy.label("com/polytechnik/lapack/Dstebz",60);
}              //  Close for() loop. 
}
m.val = (m.val+im.val);
}              //  Close else.
/***
*
*     If RANGE='I', then (WL,WU) contains eigenvalues NWL+1,...,NWU
*     If NWL+1 < IL or NWU > IU, discard extra eigenvalues.
*
***/
//Dummy.label("com/polytechnik/lapack/Dstebz",70);
} //label70              //  Close for() loop. 
}
if ((irange == 3))  {
    im.val = 0;
idiscl = ((il-1)-nwl);
// *
idiscu = (nwu-iu);
if (((idiscl > 0) || (idiscu > 0)))  {
    {
for (je = 1; je <= m.val; je++) {
if (((w[(je-(1))+ _w_offset] <= wlu) && (idiscl > 0)))  {
    idiscl = (idiscl-1);
}
else if (((w[(je-(1))+ _w_offset] >= wul) && (idiscu > 0)))  {
    idiscu = (idiscu-1);
}              // Close else if()
else  {
  im.val = (im.val+1);
w[(im.val-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
iblock[(im.val-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
}              //  Close else.
//Dummy.label("com/polytechnik/lapack/Dstebz",80);
}              //  Close for() loop. 
}
m.val = im.val;
}
/***
*
*           Code to deal with effects of bad arithmetic:
*           Some low eigenvalues to be discarded are not in (WL,WLU],
*           or high eigenvalues to be discarded are not in (WUL,WU]
*           so just kill off the smallest IDISCL/largest IDISCU
*           eigenvalues, by simply finding the smallest/largest
*           eigenvalue(s).
*
*           (If N(w) is monotone non-decreasing, this should never
*               happen.)
*
***/
if (((idiscl > 0) || (idiscu > 0)))  {
    if ((idiscl > 0))  {
    wkill = wu;
{
for (jdisc = 1; jdisc <= idiscl; jdisc++) {
iw = 0;
{
for (je = 1; je <= m.val; je++) {
if (((iblock[(je-(1))+ _iblock_offset] != 0) && (((w[(je-(1))+ _w_offset] < wkill) || (iw == 0)))))  {
    iw = je;
wkill = w[(je-(1))+ _w_offset];
}
//Dummy.label("com/polytechnik/lapack/Dstebz",90);
}              //  Close for() loop. 
}
iblock[(iw-(1))+ _iblock_offset] = 0;
//Dummy.label("com/polytechnik/lapack/Dstebz",100);
}              //  Close for() loop. 
}
}
// *
if ((idiscu > 0))  {
    wkill = wl;
{
for (jdisc = 1; jdisc <= idiscu; jdisc++) {
iw = 0;
{
for (je = 1; je <= m.val; je++) {
if (((iblock[(je-(1))+ _iblock_offset] != 0) && (((w[(je-(1))+ _w_offset] > wkill) || (iw == 0)))))  {
    iw = je;
wkill = w[(je-(1))+ _w_offset];
}
//Dummy.label("com/polytechnik/lapack/Dstebz",110);
}              //  Close for() loop. 
}
iblock[(iw-(1))+ _iblock_offset] = 0;
//Dummy.label("com/polytechnik/lapack/Dstebz",120);
}              //  Close for() loop. 
}
}
im.val = 0;
{
for (je = 1; je <= m.val; je++) {
if ((iblock[(je-(1))+ _iblock_offset] != 0))  {
    im.val = (im.val+1);
w[(im.val-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
iblock[(im.val-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
}
//Dummy.label("com/polytechnik/lapack/Dstebz",130);
}              //  Close for() loop. 
}
m.val = im.val;
}
if (((idiscl < 0) || (idiscu < 0)))  {
    toofew = true;
}
/***
*
*     If ORDER='B', do nothing -- the eigenvalues are already sorted
*        by block.
*     If ORDER='E', sort the eigenvalues from smallest to largest
*
***/
}
if (((iorder == 1) && (nsplit.val > 1)))  {
    {
for (je = 1; je <= (m.val-1); je++) {
ie = 0;
tmp1 = w[(je-(1))+ _w_offset];
{
for (j = (je+1); j <= m.val; j++) {
if ((w[(j-(1))+ _w_offset] < tmp1))  {
    ie = j;
tmp1 = w[(j-(1))+ _w_offset];
}
// *
//Dummy.label("com/polytechnik/lapack/Dstebz",140);
}              //  Close for() loop. 
}
if ((ie != 0))  {
    itmp1 = iblock[(ie-(1))+ _iblock_offset];
w[(ie-(1))+ _w_offset] = w[(je-(1))+ _w_offset];
iblock[(ie-(1))+ _iblock_offset] = iblock[(je-(1))+ _iblock_offset];
w[(je-(1))+ _w_offset] = tmp1;
iblock[(je-(1))+ _iblock_offset] = itmp1;
}
//Dummy.label("com/polytechnik/lapack/Dstebz",150);
}              //  Close for() loop. 
}
// *
}
info.val = 0;
if (ncnvrg) {
    info.val = (info.val+1);
}
    if (toofew) {
    info.val = (info.val+2);
}
    /***
*
*     End of DSTEBZ
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dstebz",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dstebz",999999);
return;
   }

private static final double LOG2=Math.log(2.0);
} // End class.
