/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsyevr.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsyevr.java,v 1.10 2012/12/26 14:27:45 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dsyevr {

// *> \brief <b> DSYEVR computes the eigenvalues and, optionally, the left 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSYEVR + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYEVR( JOBZ, RANGE, UPLO, N, A, LDA, VL, VU, IL, IU,
// *                          ABSTOL, M, W, Z, LDZ, ISUPPZ, WORK, LWORK,
// *                          IWORK, LIWORK, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBZ, RANGE, UPLO
// *       INTEGER            IL, INFO, IU, LDA, LDZ, LIWORK, LWORK, M, N
// *       DOUBLE PRECISION   ABSTOL, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            ISUPPZ( * ), IWORK( * )
// *       DOUBLE PRECISION   A( LDA, * ), W( * ), WORK( * ), Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYEVR computes selected eigenvalues and, optionally, eigenvectors
// *> of a real symmetric matrix A.  Eigenvalues and eigenvectors can be
// *> selected by specifying either a range of values or a range of
// *> indices for the desired eigenvalues.
// *>
// *> DSYEVR first reduces the matrix A to tridiagonal form T with a call
// *> to DSYTRD.  Then, whenever possible, DSYEVR calls DSTEMR to compute
// *> the eigenspectrum using Relatively Robust Representations.  DSTEMR
// *> computes eigenvalues by the dqds algorithm, while orthogonal
// *> eigenvectors are computed from various "good" L D L^T representations
// *> (also known as Relatively Robust Representations). Gram-Schmidt
// *> orthogonalization is avoided as far as possible. More specifically,
// *> the various steps of the algorithm are as follows.
// *>
// *> For each unreduced block (submatrix) of T,
// *>    (a) Compute T - sigma I  = L D L^T, so that L and D
// *>        define all the wanted eigenvalues to high relative accuracy.
// *>        This means that small relative changes in the entries of D and
// *>        cause only small relative changes in the eigenvalues and
// *>        eigenvectors. The standard (unfactored) representation of the

// *>        tridiagonal matrix T does not have this property in general.
// *>    (b) Compute the eigenvalues to suitable accuracy.
// *>        If the eigenvectors are desired, the algorithm attains full
// *>        accuracy of the computed eigenvalues only right before
// *>        the corresponding vectors have to be computed, see steps c) an
// *>    (c) For each cluster of close eigenvalues, select a new
// *>        shift close to the cluster, find a new factorization, and refi
// *>        the shifted eigenvalues to suitable accuracy.
// *>    (d) For each eigenvalue with a large enough relative separation co
// *>        the corresponding eigenvector by forming a rank revealing twis
// *>        factorization. Go back to (c) for any clusters that remain.
// *>
// *> The desired accuracy of the output can be specified by the input
// *> parameter ABSTOL.
// *>
// *> For more details, see DSTEMR's documentation and:
// *> - Inderjit S. Dhillon and Beresford N. Parlett: "Multiple representat
// *>   to compute orthogonal eigenvectors of symmetric tridiagonal matrice
// *>   Linear Algebra and its Applications, 387(1), pp. 1-28, August 2004.
// *> - Inderjit Dhillon and Beresford Parlett: "Orthogonal Eigenvectors an
// *>   Relative Gaps," SIAM Journal on Matrix Analysis and Applications, V
// *>   2004.  Also LAPACK Working Note 154.
// *> - Inderjit Dhillon: "A new O(n^2) algorithm for the symmetric
// *>   tridiagonal eigenvalue/eigenvector problem",
// *>   Computer Science Division Technical Report No. UCB/CSD-97-971,
// *>   UC Berkeley, May 1997.
// *>
// *>
// *> Note 1 : DSYEVR calls DSTEMR when the full spectrum is requested
// *> on machines which conform to the ieee-754 floating point standard.
// *> DSYEVR calls DSTEBZ and SSTEIN on non-ieee machines and
// *> when partial spectrum requests are made.
// *>
// *> Normal execution of DSTEMR may create NaNs and infinities and
// *> hence may abort due to a floating point exception in environments
// *> which do not handle NaNs and infinities in the ieee standard default

// *> manner.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBZ
// *> \verbatim
// *>          JOBZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only;
// *>          = 'V':  Compute eigenvalues and eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': all eigenvalues will be found.
// *>          = 'V': all eigenvalues in the half-open interval (VL,VU]
// *>                 will be found.
// *>          = 'I': the IL-th through IU-th eigenvalues will be found.
// *>          For RANGE = 'V' or 'I' and IU - IL < N - 1, DSTEBZ and
// *>          DSTEIN are called
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored;
// *>          = 'L':  Lower triangle of A is stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA, N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of A contains the
// *>          upper triangular part of the matrix A.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of A contains
// *>          the lower triangular part of the matrix A.
// *>          On exit, the lower triangle (if UPLO='L') or the upper
// *>          triangle (if UPLO='U') of A, including the diagonal, is
// *>          destroyed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          If RANGE='V', the lower and upper bounds of the interval to

// *>          be searched for eigenvalues. VL < VU.
// *>          Not referenced if RANGE = 'A' or 'I'.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N, if N > 0; IL = 1 and IU = 0 if N = 0.
// *>          Not referenced if RANGE = 'A' or 'V'.
// *> \endverbatim
// *>
// *> \param[in] ABSTOL
// *> \verbatim
// *>          ABSTOL is DOUBLE PRECISION
// *>          The absolute error tolerance for the eigenvalues.
// *>          An approximate eigenvalue is accepted as converged
// *>          when it is determined to lie in an interval [a,b]
// *>          of width less than or equal to
// *>
// *>                  ABSTOL + EPS *   max( |a|,|b| ) ,
// *>
// *>          where EPS is the machine precision.  If ABSTOL is less than

// *>          or equal to zero, then  EPS*|T|  will be used in its place,

// *>          where |T| is the 1-norm of the tridiagonal matrix obtained
// *>          by reducing A to tridiagonal form.
// *>
// *>          See "Computing Small Singular Values of Bidiagonal Matrices

// *>          with Guaranteed High Relative Accuracy," by Demmel and
// *>          Kahan, LAPACK Working Note #3.
// *>
// *>          If high relative accuracy is important, set ABSTOL to
// *>          DLAMCH( 'Safe minimum' ).  Doing so will guarantee that
// *>          eigenvalues are computed to high relative accuracy when
// *>          possible in future releases.  The current code does not
// *>          make any guarantees about high relative accuracy, but
// *>          future releases will. See J. Barlow and J. Demmel,
// *>          "Computing Accurate Eigensystems of Scaled Diagonally
// *>          Dominant Matrices", LAPACK Working Note #7, for a discussion
// *>          of which matrices define their eigenvalues to high relative

// *>          accuracy.
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The total number of eigenvalues found.  0 <= M <= N.
// *>          If RANGE = 'A', M = N, and if RANGE = 'I', M = IU-IL+1.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          The first M elements contain the selected eigenvalues in
// *>          ascending order.
// *> \endverbatim
// *>
// *> \param[out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, max(1,M))
// *>          If JOBZ = 'V', then if INFO = 0, the first M columns of Z
// *>          contain the orthonormal eigenvectors of the matrix A
// *>          corresponding to the selected eigenvalues, with the i-th
// *>          column of Z holding the eigenvector associated with W(i).
// *>          If JOBZ = 'N', then Z is not referenced.
// *>          Note: the user must ensure that at least max(1,M) columns ar
// *>          supplied in the array Z; if RANGE = 'V', the exact value of 
// *>          is not known in advance and an upper bound must be used.
// *>          Supplying N columns is always safe.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1, and if
// *>          JOBZ = 'V', LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] ISUPPZ
// *> \verbatim
// *>          ISUPPZ is INTEGER array, dimension ( 2*max(1,M) )
// *>          The support of the eigenvectors in Z, i.e., the indices
// *>          indicating the nonzero elements in Z. The i-th eigenvector
// *>          is nonzero only in elements ISUPPZ( 2*i-1 ) through
// *>          ISUPPZ( 2*i ).
// *>          Implemented only for RANGE = 'A' or 'I' and IU - IL = N - 1

// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The dimension of the array WORK.  LWORK >= max(1,26*N).
// *>          For optimal efficiency, LWORK >= (NB+6)*N,
// *>          where NB is the max of the blocksize for DSYTRD and DORMTR
// *>          returned by ILAENV.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (MAX(1,LIWORK))
// *>          On exit, if INFO = 0, IWORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LIWORK
// *> \verbatim
// *>          LIWORK is INTEGER
// *>          The dimension of the array IWORK.  LIWORK >= max(1,10*N).
// *>
// *>          If LIWORK = -1, then a workspace query is assumed; the
// *>          routine only calculates the optimal size of the IWORK array,
// *>          returns this value as the first entry of the IWORK array, an
// *>          no error message related to LIWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  Internal error
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleSYeigen
// *
// *> \par Contributors:
// *  ==================
// *>
// *>     Inderjit Dhillon, IBM Almaden, USA \n
// *>     Osni Marques, LBNL/NERSC, USA \n
// *>     Ken Stanley, Computer Science Division, University of
// *>       California at Berkeley, USA \n
// *>     Jason Riedy, Computer Science Division, University of
// *>       California at Berkeley, USA \n
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/
// *

public static void dsyevr (final char jobz,
final char range,
final char uplo,
int n,
double [] a, int _a_offset,
int lda,
double vl,
double vu,
int il,
int iu,
double abstol,
intW m,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
int [] isuppz, int _isuppz_offset,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int liwork,
intW info)  {

boolean alleig= false;
boolean indeig= false;
boolean lower= false;
boolean lquery= false;
boolean valeig= false;
boolean wantz= false;
booleanW tryrac= new booleanW(false);
//String order= new String(" ");
final char order;
int i= 0;
int ieeeok= 0;
intW iinfo= new intW(0);
int imax= 0;
int indd= 0;
int inddd= 0;
int inde= 0;
int indee= 0;
int indibl= 0;
int indifl= 0;
int indisp= 0;
int indiwo= 0;
int indtau= 0;
int indwk= 0;
int indwkn= 0;
int iscale= 0;
int j= 0;
int jj= 0;
int liwmin= 0;
int llwork= 0;
int llwrkn= 0;
int lwkopt= 0;
int lwmin= 0;
int nb= 0;
intW nsplit= new intW(0);
double abstll= 0.0d;
double anrm= 0.0d;
double bignum= 0.0d;
double eps= 0.0d;
double rmax= 0.0d;
double rmin= 0.0d;
double safmin= 0.0d;
double sigma= 0.0d;
double smlnum= 0.0d;
double tmp1= 0.0d;
double vll= 0.0d;
double vuu= 0.0d;
//jobz = jobz.substring((1)-1,1);
//range = range.substring((1)-1,1);
//uplo = uplo.substring((1)-1,1);
ieeeok = 1;//com.polytechnik.lapack.Ilaenv.ilaenv(10,"DSYEVR","N",1,2,3,4);
lower = com.polytechnik.lapack.Lsame.lsame(uplo,'L');
wantz = com.polytechnik.lapack.Lsame.lsame(jobz,'V');
alleig = com.polytechnik.lapack.Lsame.lsame(range,'A');
valeig = com.polytechnik.lapack.Lsame.lsame(range,'V');
// *
indeig = com.polytechnik.lapack.Lsame.lsame(range,'I');
// *
lquery = ((((lwork == -1)) || ((liwork == -1))));
lwmin = Math.max(1, (26*n));
// *
liwmin = Math.max(1, (10*n));
info.val = 0;
if ((!((wantz || com.polytechnik.lapack.Lsame.lsame(jobz,'N')))))  {
    info.val = -1;
}
else if ((!(((alleig || valeig) || indeig))))  {
    info.val = -2;
}              // Close else if()
else if ((!((lower || com.polytechnik.lapack.Lsame.lsame(uplo,'U')))))  {
    info.val = -3;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -6;
}              // Close else if()
else  {
  if (valeig)  {
    if (((n > 0) && (vu <= vl))) {
    info.val = -8;
}
    }
else if (indeig)  {
    if (((il < 1) || (il > Math.max(1, n))))  {
    info.val = -9;
}
else if (((iu < Math.min(n, il)) || (iu > n)))  {
    info.val = -10;
}              // Close else if()
}              // Close else if()
}              //  Close else.
if ((info.val == 0))  {
    if (((ldz < 1) || ((wantz && (ldz < n)))))  {
    info.val = -15;
}
else if (((lwork < lwmin) && (!lquery)))  {
    info.val = -18;
}              // Close else if()
else if (((liwork < liwmin) && (!lquery)))  {
    info.val = -20;
}              // Close else if()
// *
}
/***
!         NB = ILAENV( 1, 'DSYTRD', UPLO, N, -1, -1, -1 )
!         NB = MAX( NB, ILAENV( 1, 'DORMTR', UPLO, N, -1, -1, -1 ) )
***/
if ((info.val == 0))  {
    nb = 1;
lwkopt = Math.max((((nb+1))*n), lwmin);
work[(1-(1))+ _work_offset] = (double)(lwkopt);
iwork[(1-(1))+ _iwork_offset] = liwmin;
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYEVR",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyevr",999999);
}
else if (lquery)  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyevr",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
m.val = 0;
if ((n == 0))  {
    work[(1-(1))+ _work_offset] = (double)(1);
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyevr",999999);
// *
}
if ((n == 1))  {
    work[(1-(1))+ _work_offset] = (double)(7);
if ((alleig || indeig))  {
    m.val = 1;
w[(1-(1))+ _w_offset] = a[(1-(1))+(1-(1)) * (lda)+ _a_offset];
}
else  {
  if (((vl < a[(1-(1))+(1-(1)) * (lda)+ _a_offset]) && (vu >= a[(1-(1))+(1-(1)) * (lda)+ _a_offset])))  {
    m.val = 1;
w[(1-(1))+ _w_offset] = a[(1-(1))+(1-(1)) * (lda)+ _a_offset];
}
}              //  Close else.
if (wantz)  {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = 1.0;
isuppz[(1-(1))+ _isuppz_offset] = 1;
isuppz[(2-(1))+ _isuppz_offset] = 1;
}
return;//Dummy.go_to("com/polytechnik/lapack/Dsyevr",999999);
/***
*
*     Get machine constants.
*
***/
}
safmin = com.polytechnik.lapack.Dlamch.dlamch('S');
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
smlnum = (safmin/eps);
bignum = (1.0/smlnum);
rmin = Math.sqrt(smlnum);
/***
*
*     Scale matrix to allowable range, if necessary.
*
***/
rmax = Math.min(Math.sqrt(bignum), (1.0/Math.sqrt(Math.sqrt(safmin))));
iscale = 0;
abstll = abstol;
if (valeig)  {
    vll = vl;
vuu = vu;
}
anrm = com.polytechnik.lapack.Dlansy.dlansy('M',uplo,n,a,_a_offset,lda,work,_work_offset);
if (((anrm > 0.0) && (anrm < rmin)))  {
    iscale = 1;
sigma = (rmin/anrm);
}
else if ((anrm > rmax))  {
    iscale = 1;
sigma = (rmax/anrm);
}              // Close else if()
if ((iscale == 1))  {
    if (lower)  {
    {
for (j = 1; j <= n; j++) {
com.polytechnik.lapack.Dscal.dscal(((n-j)+1),sigma,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,1);
//Dummy.label("com/polytechnik/lapack/Dsyevr",10);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
com.polytechnik.lapack.Dscal.dscal(j,sigma,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1);
//Dummy.label("com/polytechnik/lapack/Dsyevr",20);
}              //  Close for() loop. 
}
}              //  Close else.
if ((abstol > 0)) {
    abstll = (abstol*sigma);
}
    if (valeig)  {
    vll = (vl*sigma);
vuu = (vu*sigma);
}
/***

*     Initialize indices into workspaces.  Note: The IWORK indices are
*     used only if DSTERF or DSTEMR fail.

*     WORK(INDTAU:INDTAU+N-1) stores the scalar factors of the
*     elementary reflectors used in DSYTRD.
***/
}
// *     WORK(INDD:INDD+N-1) stores the tridiagonal's diagonal entries.
indtau = 1;
/***
*     WORK(INDE:INDE+N-1) stores the off-diagonal entries of the
*     tridiagonal matrix from DSYTRD.
***/
indd = (indtau+n);
/***
*     WORK(INDDD:INDDD+N-1) is a copy of the diagonal entries over
*     -written by DSTEMR (the DSTERF path copies the diagonal to W).
***/
inde = (indd+n);
/***
*     WORK(INDEE:INDEE+N-1) is a copy of the off-diagonal entries over
*     -written while computing the eigenvalues in DSTERF and DSTEMR.
***/
inddd = (inde+n);
/***
*     INDWK is the starting offset of the left-over workspace, and
*     LLWORK is the remaining workspace size.
***/
indee = (inddd+n);
indwk = (indee+n);
/***

*     IWORK(INDIBL:INDIBL+M-1) corresponds to IBLOCK in DSTEBZ and
*     stores the block indices of each of the M<=N eigenvalues.
***/
llwork = ((lwork-indwk)+1);
/***
*     IWORK(INDISP:INDISP+NSPLIT-1) corresponds to ISPLIT in DSTEBZ and
*     stores the starting and finishing indices of each block.
***/
indibl = 1;
/***
*     IWORK(INDIFL:INDIFL+N-1) stores the indices of eigenvectors
*     that corresponding to eigenvectors that fail to converge in
*     DSTEIN.  This information is discarded; if any fail, the driver
*     returns INFO > 0.
***/
indisp = (indibl+n);
// *     INDIWO is the offset of the remaining integer workspace.
indifl = (indisp+n);
/***

*
*     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
*
***/
indiwo = (indifl+n);
/***
*
*     If all eigenvalues are desired
*     then call DSTERF or DSTEMR and DORMTR.
*
***/
com.polytechnik.lapack.Dsytrd.dsytrd(uplo,n,a,_a_offset,lda,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,work,(indtau-(1))+ _work_offset,work,(indwk-(1))+ _work_offset,llwork,info);
if ((info.val != 0))  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyevr",999999);
}

label30:{
// 
if ((((alleig || (((indeig && (il == 1)) && (iu == n))))) && (ieeeok == 1)))  {
    com.polytechnik.lapack.Dcopy.dcopy((n-1),work,(inde-(1))+ _work_offset,1,work,(indee-(1))+ _work_offset,1);
// *
com.polytechnik.lapack.Dcopy.dcopy(n,work,(indd-(1))+ _work_offset,1,work,(inddd-(1))+ _work_offset,1);
if ((abstol <= ((2.0*n)*eps)))  {
    tryrac.val = true;
}
else  {
  tryrac.val = false;
}              //  Close else.
// 
com.polytechnik.lapack.Dstemr.dstemr(jobz,'A',n,work,(inddd-(1))+ _work_offset,work,(indee-(1))+ _work_offset,vl,vu,il,iu,m,w,_w_offset,z,_z_offset,ldz,n,isuppz,_isuppz_offset,tryrac,work,(indwk-(1))+ _work_offset,lwork,iwork,_iwork_offset,liwork,info);
/***
*
*
*
*        Apply orthogonal matrix used in reduction to tridiagonal
*        form to eigenvectors returned by DSTEIN.
*
***/
if (wantz)  {
    if ((wantz && (info.val == 0)))  {
    indwkn = inde;
llwrkn = ((lwork-indwkn)+1);
com.polytechnik.lapack.Dormtr.dormtr('L',uplo,'N',n,m.val,a,_a_offset,lda,work,(indtau-(1))+ _work_offset,z,_z_offset,ldz,work,(indwkn-(1))+ _work_offset,llwrkn,iinfo);
if(info.val==0){info.val=iinfo.val;}
}
/***
*
*
***/
}
/***
*           Everything worked.  Skip DSTEBZ/DSTEIN.  IWORK(:) are
*           undefined.
***/
if ((info.val == 0 && m.val == n))  {
    //m.val = n;
    break label30;//Dummy.go_to("com/polytechnik/lapack/Dsyevr",30);
}
info.val = 0;
/***
*
*     Otherwise, call DSTEBZ and, if eigenvectors are desired, DSTEIN.
*     Also call DSTEBZ and DSTEIN if DSTEMR fails.
*
***/
}
if (wantz)  {
    order = 'B';
}
else  {
  order = 'E';
// 
}              //  Close else.
// *
com.polytechnik.lapack.Dstebz.dstebz(range,order,n,vll,vuu,il,iu,abstll,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,m,nsplit,w,_w_offset,iwork,(indibl-(1))+ _iwork_offset,iwork,(indisp-(1))+ _iwork_offset,work,(indwk-(1))+ _work_offset,iwork,(indiwo-(1))+ _iwork_offset,info);
if (wantz)  {
    /***
*
*        Apply orthogonal matrix used in reduction to tridiagonal
*        form to eigenvectors returned by DSTEIN.
*
***/
com.polytechnik.lapack.Dstein.dstein(n,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,m.val,w,_w_offset,iwork,(indibl-(1))+ _iwork_offset,iwork,(indisp-(1))+ _iwork_offset,z,_z_offset,ldz,work,(indwk-(1))+ _work_offset,iwork,(indiwo-(1))+ _iwork_offset,iwork,(indifl-(1))+ _iwork_offset,iinfo);
if(info.val==0){info.val=iinfo.val;}
indwkn = inde;
llwrkn = ((lwork-indwkn)+1);
com.polytechnik.lapack.Dormtr.dormtr('L',uplo,'N',n,m.val,a,_a_offset,lda,work,(indtau-(1))+ _work_offset,z,_z_offset,ldz,work,(indwkn-(1))+ _work_offset,llwrkn,iinfo);
if(info.val==0){info.val=iinfo.val;}
/***
*
*     If matrix was scaled, then rescale eigenvalues appropriately.
*
*  Jump here if DSTEMR/DSTEIN succeeded.
***/
}
} //label30:Dummy.label("com/polytechnik/lapack/Dsyevr",30);

if ((iscale == 1))  {
    if ((info.val == 0))  {
    imax = m.val;
}
else  {
    imax = m.val;//(info.val-1);
}              //  Close else.
com.polytechnik.lapack.Dscal.dscal(imax,(1.0/sigma),w,_w_offset,1);
/***
*
*     If eigenvalues are not in order, then sort them, along with
*     eigenvectors.  Note: We do not sort the IFAIL portion of IWORK.
*     It may not be initialized (if DSTEMR/DSTEIN succeeded), and we do
*     not return this detailed information to the user.
*
***/
}
if (wantz)  {
    {
for (j = 1; j <= (m.val-1); j++) {
i = 0;
tmp1 = w[(j-(1))+ _w_offset];
{
for (jj = (j+1); jj <= m.val; jj++) {
if ((w[(jj-(1))+ _w_offset] < tmp1))  {
    i = jj;
tmp1 = w[(jj-(1))+ _w_offset];
}
// *
//Dummy.label("com/polytechnik/lapack/Dsyevr",40);
}              //  Close for() loop. 
}
if ((i != 0))  {
    w[(i-(1))+ _w_offset] = w[(j-(1))+ _w_offset];
w[(j-(1))+ _w_offset] = tmp1;
com.polytechnik.lapack.Dswap.dswap(n,z,(1-(1))+(i-(1)) * (ldz)+ _z_offset,1,z,(1-(1))+(j-(1)) * (ldz)+ _z_offset,1);
}
//Dummy.label("com/polytechnik/lapack/Dsyevr",50);
}              //  Close for() loop. 
}
/***
*
*     Set WORK(1) to optimal workspace size.
*
***/
}
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
iwork[(1-(1))+ _iwork_offset] = liwmin;
/***
*
*     End of DSYEVR
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsyevr",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsyevr",999999);
return;
   }
} // End class.
