/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsyevx.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsyevx.java,v 1.9 2015/06/13 08:18:12 mal Exp $
 */

package com.polytechnik.lapack;


public class Dsyevx {

// *> \brief <b> DSYEVX computes the eigenvalues and, optionally, the left 
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSYEVX + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYEVX( JOBZ, RANGE, UPLO, N, A, LDA, VL, VU, IL, IU,
// *                          ABSTOL, M, W, Z, LDZ, WORK, LWORK, IWORK,
// *                          IFAIL, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBZ, RANGE, UPLO
// *       INTEGER            IL, INFO, IU, LDA, LDZ, LWORK, M, N
// *       DOUBLE PRECISION   ABSTOL, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IFAIL( * ), IWORK( * )
// *       DOUBLE PRECISION   A( LDA, * ), W( * ), WORK( * ), Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYEVX computes selected eigenvalues and, optionally, eigenvectors
// *> of a real symmetric matrix A.  Eigenvalues and eigenvectors can be
// *> selected by specifying either a range of values or a range of indices
// *> for the desired eigenvalues.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] JOBZ
// *> \verbatim
// *>          JOBZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only;
// *>          = 'V':  Compute eigenvalues and eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': all eigenvalues will be found.
// *>          = 'V': all eigenvalues in the half-open interval (VL,VU]
// *>                 will be found.
// *>          = 'I': the IL-th through IU-th eigenvalues will be found.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A is stored;
// *>          = 'L':  Lower triangle of A is stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA, N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of A contains the
// *>          upper triangular part of the matrix A.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of A contains
// *>          the lower triangular part of the matrix A.
// *>          On exit, the lower triangle (if UPLO='L') or the upper
// *>          triangle (if UPLO='U') of A, including the diagonal, is
// *>          destroyed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          If RANGE='V', the lower and upper bounds of the interval to

// *>          be searched for eigenvalues. VL < VU.
// *>          Not referenced if RANGE = 'A' or 'I'.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N, if N > 0; IL = 1 and IU = 0 if N = 0.
// *>          Not referenced if RANGE = 'A' or 'V'.
// *> \endverbatim
// *>
// *> \param[in] ABSTOL
// *> \verbatim
// *>          ABSTOL is DOUBLE PRECISION
// *>          The absolute error tolerance for the eigenvalues.
// *>          An approximate eigenvalue is accepted as converged
// *>          when it is determined to lie in an interval [a,b]
// *>          of width less than or equal to
// *>
// *>                  ABSTOL + EPS *   max( |a|,|b| ) ,
// *>
// *>          where EPS is the machine precision.  If ABSTOL is less than

// *>          or equal to zero, then  EPS*|T|  will be used in its place,

// *>          where |T| is the 1-norm of the tridiagonal matrix obtained
// *>          by reducing A to tridiagonal form.
// *>
// *>          Eigenvalues will be computed most accurately when ABSTOL is

// *>          set to twice the underflow threshold 2*DLAMCH('S'), not zero
// *>          If this routine returns with INFO>0, indicating that some
// *>          eigenvectors did not converge, try setting ABSTOL to
// *>          2*DLAMCH('S').
// *>
// *>          See "Computing Small Singular Values of Bidiagonal Matrices

// *>          with Guaranteed High Relative Accuracy," by Demmel and
// *>          Kahan, LAPACK Working Note #3.
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The total number of eigenvalues found.  0 <= M <= N.
// *>          If RANGE = 'A', M = N, and if RANGE = 'I', M = IU-IL+1.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          On normal exit, the first M elements contain the selected
// *>          eigenvalues in ascending order.
// *> \endverbatim
// *>
// *> \param[out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, max(1,M))
// *>          If JOBZ = 'V', then if INFO = 0, the first M columns of Z
// *>          contain the orthonormal eigenvectors of the matrix A
// *>          corresponding to the selected eigenvalues, with the i-th
// *>          column of Z holding the eigenvector associated with W(i).
// *>          If an eigenvector fails to converge, then that column of Z
// *>          contains the latest approximation to the eigenvector, and th
// *>          index of the eigenvector is returned in IFAIL.
// *>          If JOBZ = 'N', then Z is not referenced.
// *>          Note: the user must ensure that at least max(1,M) columns ar
// *>          supplied in the array Z; if RANGE = 'V', the exact value of 
// *>          is not known in advance and an upper bound must be used.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1, and if
// *>          JOBZ = 'V', LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The length of the array WORK.  LWORK >= 1, when N <= 1;
// *>          otherwise 8*N.
// *>          For optimal efficiency, LWORK >= (NB+3)*N,
// *>          where NB is the max of the blocksize for DSYTRD and DORMTR
// *>          returned by ILAENV.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (5*N)
// *> \endverbatim
// *>
// *> \param[out] IFAIL
// *> \verbatim
// *>          IFAIL is INTEGER array, dimension (N)
// *>          If JOBZ = 'V', then if INFO = 0, the first M elements of
// *>          IFAIL are zero.  If INFO > 0, then IFAIL contains the
// *>          indices of the eigenvectors that failed to converge.
// *>          If JOBZ = 'N', then IFAIL is not referenced.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  if INFO = i, then i eigenvectors failed to converge.
// *>                Their indices are stored in array IFAIL.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleSYeigen
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dsyevx (final char jobz,
final char uplo,
int n,
double [] a, int _a_offset,
int lda,
double abstol,
intW m,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int [] ifail, int _ifail_offset,
intW info)  {

//boolean alleig= false;
//boolean indeig= false;
//boolean lower= false;
boolean lquery= false;
boolean test= false;
//boolean valeig= false;
//boolean wantz= false;
//String order= new String(" ");
int i= 0;
final intW iinfo= new intW(0);
int imax= 0;
int indd= 0;
int inde= 0;
int indee= 0;
int indibl= 0;
int indisp= 0;
int indiwo= 0;
int indtau= 0;
int indwkn= 0;
int indwrk= 0;
int iscale= 0;
int itmp1= 0;
int j= 0;
int jj= 0;
int llwork= 0;
int llwrkn= 0;
int lwkmin= 0;
int lwkopt= 0;
int nb= 0;
intW nsplit= new intW(0);
double abstll= 0.0d;
double anrm= 0.0d;
double bignum= 0.0d;
double eps= 0.0d;
double rmax= 0.0d;
double rmin= 0.0d;
double safmin= 0.0d;
double sigma= 0.0d;
double smlnum= 0.0d;
double tmp1= 0.0d;
double vll= 0.0d;
double vuu= 0.0d;
//jobz = jobz.substring((1)-1,1);
//range = range.substring((1)-1,1);
//uplo = uplo.substring((1)-1,1);
final char range='A';
final double vl=0,vu=0;
final int il=-1,iu=-1;
final boolean lower = com.polytechnik.lapack.Lsame.lsame(uplo,'L');
final boolean wantz = com.polytechnik.lapack.Lsame.lsame(jobz,'V');
final boolean alleig = com.polytechnik.lapack.Lsame.lsame(range,'A');
final boolean valeig = com.polytechnik.lapack.Lsame.lsame(range,'V');
final boolean indeig = com.polytechnik.lapack.Lsame.lsame(range,'I');
// *
lquery = ((lwork == -1));
info.val = 0;
if ((!((wantz || com.polytechnik.lapack.Lsame.lsame(jobz,'N')))))  {
    info.val = -1;
}
else if ((!(((alleig || valeig) || indeig))))  {
    info.val = -2;
}              // Close else if()
else if ((!((lower || com.polytechnik.lapack.Lsame.lsame(uplo,'U')))))  {
    info.val = -3;
}              // Close else if()
else if ((n < 0))  {
    info.val = -4;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -6;
}              // Close else if()
else  {
  if (valeig)  {
    if (((n > 0) && (vu <= vl))) {
    info.val = -8;
}
    }
else if (indeig)  {
    if (((il < 1) || (il > Math.max(1, n))))  {
    info.val = -9;
}
else if (((iu < Math.min(n, il)) || (iu > n)))  {
    info.val = -10;
}              // Close else if()
}              // Close else if()
}              //  Close else.
if ((info.val == 0))  {
    if (((ldz < 1) || ((wantz && (ldz < n)))))  {
    info.val = -15;
}
// *
}
if ((info.val == 0))  {
    if ((n <= 1))  {
    lwkmin = 1;
work[(1-(1))+ _work_offset] = (double)(lwkmin);
}
else  {
  lwkmin = (8*n);
  nb = 1;//com.polytechnik.lapack.Ilaenv.ilaenv(1,"DSYTRD",uplo,n,-1,-1,-1);
//nb = Math.max(nb, com.polytechnik.lapack.Ilaenv.ilaenv(1,"DORMTR",uplo,n,-1,-1,-1));
lwkopt = Math.max(lwkmin, (((nb+3))*n));
work[(1-(1))+ _work_offset] = (double)(lwkopt);
// *
}              //  Close else.
if (((lwork < lwkmin) && (!lquery))) {
    info.val = -17;
}
    // *
}
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYEVX",(-(info.val)));
//Dummy.go_to("com/polytechnik/lapack/Dsyevx",999999);
    return;
}
else if (lquery)  {
    //Dummy.go_to("com/polytechnik/lapack/Dsyevx",999999);
    return;
}              // Close else if()

/***
*
*     Quick return if possible
*
***/
m.val = 0;
if ((n == 0))  {
    //Dummy.go_to("com/polytechnik/lapack/Dsyevx",999999);
    return;
// *
}
if ((n == 1))  {
    if ((alleig || indeig))  {
    m.val = 1;
w[(1-(1))+ _w_offset] = a[(1-(1))+(1-(1)) * (lda)+ _a_offset];
}
else  {
  if (((vl < a[(1-(1))+(1-(1)) * (lda)+ _a_offset]) && (vu >= a[(1-(1))+(1-(1)) * (lda)+ _a_offset])))  {
    m.val = 1;
w[(1-(1))+ _w_offset] = a[(1-(1))+(1-(1)) * (lda)+ _a_offset];
}
}              //  Close else.
if (wantz) {
    z[(1-(1))+(1-(1)) * (ldz)+ _z_offset] = 1.0;
}
//Dummy.go_to("com/polytechnik/lapack/Dsyevx",999999);
return;
}
/***
*
*     Get machine constants.
*
***/
safmin = com.polytechnik.lapack.Dlamch.dlamch('S');
eps = com.polytechnik.lapack.Dlamch.dlamch('P');
smlnum = (safmin/eps);
bignum = (1.0/smlnum);
rmin = Math.sqrt(smlnum);
/***
*
*     Scale matrix to allowable range, if necessary.
*
***/
rmax = Math.min(Math.sqrt(bignum), (1.0/Math.sqrt(Math.sqrt(safmin))));
iscale = 0;
abstll = abstol;
if (valeig)  {
    vll = vl;
vuu = vu;
}
anrm = com.polytechnik.lapack.Dlansy.dlansy('M',uplo,n,a,_a_offset,lda,work,_work_offset);
if (((anrm > 0.0) && (anrm < rmin)))  {
    iscale = 1;
sigma = (rmin/anrm);
}
else if ((anrm > rmax))  {
    iscale = 1;
sigma = (rmax/anrm);
}              // Close else if()
if ((iscale == 1))  {
    if (lower)  {
    {
for (j = 1; j <= n; j++) {
com.polytechnik.lapack.Dscal.dscal(((n-j)+1),sigma,a,(j-(1))+(j-(1)) * (lda)+ _a_offset,1);
//Dummy.label("com/polytechnik/lapack/Dsyevx",10);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
com.polytechnik.lapack.Dscal.dscal(j,sigma,a,(1-(1))+(j-(1)) * (lda)+ _a_offset,1);
//Dummy.label("com/polytechnik/lapack/Dsyevx",20);
}              //  Close for() loop. 
}
}              //  Close else.
if ((abstol > 0)) {
    abstll = (abstol*sigma);
}
    if (valeig)  {
    vll = (vl*sigma);
vuu = (vu*sigma);
}
/***
*
*     Call DSYTRD to reduce symmetric matrix to tridiagonal form.
*
***/
}
indtau = 1;
inde = (indtau+n);
indd = (inde+n);
indwrk = (indd+n);
llwork = ((lwork-indwrk)+1);
/***
*
*     If all eigenvalues are desired and ABSTOL is less than or equal to
*     zero, then call DSTERF or DORGTR and SSTEQR.  If this fails for
*     some eigenvalue, then try DSTEBZ.
*
***/
com.polytechnik.lapack.Dsytrd.dsytrd(uplo,n,a,_a_offset,lda,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,work,(indtau-(1))+ _work_offset,work,(indwrk-(1))+ _work_offset,llwork,info);
if ((info.val != 0))  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyevx",999999);
}
test = false;
if (indeig)  {
    if (((il == 1) && (iu == n)))  {
    test = true;
}
}



/***
*
*     Otherwise, call DSTEBZ and, if eigenvectors are desired, SSTEIN.
*
***/

final char order;
if (wantz)  {
    order = 'B';
}
else  {
  order = 'E';
}              //  Close else.
indibl = 1;
indisp = (indibl+n);
indiwo = (indisp+n);
// *
com.polytechnik.lapack.Dstebz.dstebz(range,order,n,vll,vuu,il,iu,abstll,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,m,nsplit,w,_w_offset,iwork,(indibl-(1))+ _iwork_offset,iwork,(indisp-(1))+ _iwork_offset,work,(indwrk-(1))+ _work_offset,iwork,(indiwo-(1))+ _iwork_offset,info);
//System.err.println("!!!!!!!!!!!!!!!!info.val="+info.val+" m.val="+m.val);
if (wantz)  {
    /***
*
*        Apply orthogonal matrix used in reduction to tridiagonal
*        form to eigenvectors returned by DSTEIN.
*
***/
    if(false)
    {
	/*
	final org.netlib.util.intW ainfo=new org.netlib.util.intW(0);
	org.netlib.lapack.Dstein.dstein(n,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,m.val,w,_w_offset,iwork,(indibl-(1))+ _iwork_offset,iwork,(indisp-(1))+ _iwork_offset,z,_z_offset,ldz,work,(indwrk-(1))+ _work_offset,iwork,(indiwo-(1))+ _iwork_offset,ifail,_ifail_offset,ainfo);
	if(info.val==0){info.val=ainfo.val;}
	*/
    }
    else
    {
com.polytechnik.lapack.Dstein.dstein(n,work,(indd-(1))+ _work_offset,work,(inde-(1))+ _work_offset,m.val,w,_w_offset,iwork,(indibl-(1))+ _iwork_offset,iwork,(indisp-(1))+ _iwork_offset,z,_z_offset,ldz,work,(indwrk-(1))+ _work_offset,iwork,(indiwo-(1))+ _iwork_offset,ifail,_ifail_offset,iinfo);
if(info.val==0){info.val=iinfo.val;}
    }
//System.err.println("!!!!!!!!!!!!!!!!dstein:info.val="+info.val+" m.val="+m.val);
indwkn = inde;
llwrkn = ((lwork-indwkn)+1);
com.polytechnik.lapack.Dormtr.dormtr('L',uplo,'N',n,m.val,a,_a_offset,lda,work,(indtau-(1))+ _work_offset,z,_z_offset,ldz,work,(indwkn-(1))+ _work_offset,llwrkn,iinfo);
if(info.val==0){info.val=iinfo.val;}
//System.err.println("!!!!!!!!!!!!!!!!dormtr:info.val="+info.val+" m.val="+m.val);
}
/***
*
*     If matrix was scaled, then rescale eigenvalues appropriately.
*
***/

//label40: Dummy.label("com/polytechnik/lapack/Dsyevx",40);
if ((iscale == 1))  {
    if ((info.val == 0))  {
    imax = m.val;
}
else  {
    imax = m.val;//(info.val-1);
}              //  Close else.
com.polytechnik.lapack.Dscal.dscal(imax,(1.0/sigma),w,_w_offset,1);
/***
*
*     If eigenvalues are not in order, then sort them, along with
*     eigenvectors.
*
***/
}
if (wantz)  {
    {
for (j = 1; j <= (m.val-1); j++) {
i = 0;
tmp1 = w[(j-(1))+ _w_offset];
{
for (jj = (j+1); jj <= m.val; jj++) {
if ((w[(jj-(1))+ _w_offset] < tmp1))  {
    i = jj;
tmp1 = w[(jj-(1))+ _w_offset];
}
// *
//Dummy.label("com/polytechnik/lapack/Dsyevx",50);
}              //  Close for() loop. 
}
if ((i != 0))  {
    itmp1 = iwork[(((indibl+i)-1)-(1))+ _iwork_offset];
w[(i-(1))+ _w_offset] = w[(j-(1))+ _w_offset];
iwork[(((indibl+i)-1)-(1))+ _iwork_offset] = iwork[(((indibl+j)-1)-(1))+ _iwork_offset];
w[(j-(1))+ _w_offset] = tmp1;
iwork[(((indibl+j)-1)-(1))+ _iwork_offset] = itmp1;
com.polytechnik.lapack.Dswap.dswap(n,z,(1-(1))+(i-(1)) * (ldz)+ _z_offset,1,z,(1-(1))+(j-(1)) * (ldz)+ _z_offset,1);
if ((info.val != 0))  {
    itmp1 = ifail[(i-(1))+ _ifail_offset];
ifail[(i-(1))+ _ifail_offset] = ifail[(j-(1))+ _ifail_offset];
ifail[(j-(1))+ _ifail_offset] = itmp1;
}
}
//Dummy.label("com/polytechnik/lapack/Dsyevx",60);
}              //  Close for() loop. 
}
/***
*
*     Set WORK(1) to optimal workspace size.
*
***/
}
// *
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DSYEVX
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsyevx",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsyevx",999999);
return;
   }
} // End class.
