/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsygvx.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsygvx.java,v 1.7 2012/12/26 14:27:45 mal Exp $
 */

package com.polytechnik.lapack;



public class Dsygvx {

// *> \brief \b DSYGST
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSYGVX + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYGVX( ITYPE, JOBZ, RANGE, UPLO, N, A, LDA, B, LDB,

// *                          VL, VU, IL, IU, ABSTOL, M, W, Z, LDZ, WORK,
// *                          LWORK, IWORK, IFAIL, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          JOBZ, RANGE, UPLO
// *       INTEGER            IL, INFO, ITYPE, IU, LDA, LDB, LDZ, LWORK, M,
// *       DOUBLE PRECISION   ABSTOL, VL, VU
// *       ..
// *       .. Array Arguments ..
// *       INTEGER            IFAIL( * ), IWORK( * )
// *       DOUBLE PRECISION   A( LDA, * ), B( LDB, * ), W( * ), WORK( * ),

// *      $                   Z( LDZ, * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYGVX computes selected eigenvalues, and optionally, eigenvectors
// *> of a real generalized symmetric-definite eigenproblem, of the form
// *> A*x=(lambda)*B*x,  A*Bx=(lambda)*x,  or B*A*x=(lambda)*x.  Here A
// *> and B are assumed to be symmetric and B is also positive definite.
// *> Eigenvalues and eigenvectors can be selected by specifying either a
// *> range of values or a range of indices for the desired eigenvalues.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] ITYPE
// *> \verbatim
// *>          ITYPE is INTEGER
// *>          Specifies the problem type to be solved:
// *>          = 1:  A*x = (lambda)*B*x
// *>          = 2:  A*B*x = (lambda)*x
// *>          = 3:  B*A*x = (lambda)*x
// *> \endverbatim
// *>
// *> \param[in] JOBZ
// *> \verbatim
// *>          JOBZ is CHARACTER*1
// *>          = 'N':  Compute eigenvalues only;
// *>          = 'V':  Compute eigenvalues and eigenvectors.
// *> \endverbatim
// *>
// *> \param[in] RANGE
// *> \verbatim
// *>          RANGE is CHARACTER*1
// *>          = 'A': all eigenvalues will be found.
// *>          = 'V': all eigenvalues in the half-open interval (VL,VU]
// *>                 will be found.
// *>          = 'I': the IL-th through IU-th eigenvalues will be found.
// *> \endverbatim
// *>
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          = 'U':  Upper triangle of A and B are stored;
// *>          = 'L':  Lower triangle of A and B are stored.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix pencil (A,B).  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA, N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of A contains the
// *>          upper triangular part of the matrix A.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of A contains
// *>          the lower triangular part of the matrix A.
// *>
// *>          On exit, the lower triangle (if UPLO='L') or the upper
// *>          triangle (if UPLO='U') of A, including the diagonal, is
// *>          destroyed.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in,out] B
// *> \verbatim
// *>          B is DOUBLE PRECISION array, dimension (LDB, N)
// *>          On entry, the symmetric matrix B.  If UPLO = 'U', the
// *>          leading N-by-N upper triangular part of B contains the
// *>          upper triangular part of the matrix B.  If UPLO = 'L',
// *>          the leading N-by-N lower triangular part of B contains
// *>          the lower triangular part of the matrix B.
// *>
// *>          On exit, if INFO <= N, the part of B containing the matrix i
// *>          overwritten by the triangular factor U or L from the Cholesk
// *>          factorization B = U**T*U or B = L*L**T.
// *> \endverbatim
// *>
// *> \param[in] LDB
// *> \verbatim
// *>          LDB is INTEGER
// *>          The leading dimension of the array B.  LDB >= max(1,N).
// *> \endverbatim
// *>
// *> \param[in] VL
// *> \verbatim
// *>          VL is DOUBLE PRECISION
// *> \endverbatim
// *>
// *> \param[in] VU
// *> \verbatim
// *>          VU is DOUBLE PRECISION
// *>          If RANGE='V', the lower and upper bounds of the interval to

// *>          be searched for eigenvalues. VL < VU.
// *>          Not referenced if RANGE = 'A' or 'I'.
// *> \endverbatim
// *>
// *> \param[in] IL
// *> \verbatim
// *>          IL is INTEGER
// *> \endverbatim
// *>
// *> \param[in] IU
// *> \verbatim
// *>          IU is INTEGER
// *>          If RANGE='I', the indices (in ascending order) of the
// *>          smallest and largest eigenvalues to be returned.
// *>          1 <= IL <= IU <= N, if N > 0; IL = 1 and IU = 0 if N = 0.
// *>          Not referenced if RANGE = 'A' or 'V'.
// *> \endverbatim
// *>
// *> \param[in] ABSTOL
// *> \verbatim
// *>          ABSTOL is DOUBLE PRECISION
// *>          The absolute error tolerance for the eigenvalues.
// *>          An approximate eigenvalue is accepted as converged
// *>          when it is determined to lie in an interval [a,b]
// *>          of width less than or equal to
// *>
// *>                  ABSTOL + EPS *   max( |a|,|b| ) ,
// *>
// *>          where EPS is the machine precision.  If ABSTOL is less than

// *>          or equal to zero, then  EPS*|T|  will be used in its place,

// *>          where |T| is the 1-norm of the tridiagonal matrix obtained
// *>          by reducing C to tridiagonal form, where C is the symmetric

// *>          matrix of the standard symmetric problem to which the
// *>          generalized problem is transformed.
// *>
// *>          Eigenvalues will be computed most accurately when ABSTOL is

// *>          set to twice the underflow threshold 2*DLAMCH('S'), not zero
// *>          If this routine returns with INFO>0, indicating that some
// *>          eigenvectors did not converge, try setting ABSTOL to
// *>          2*DLAMCH('S').
// *> \endverbatim
// *>
// *> \param[out] M
// *> \verbatim
// *>          M is INTEGER
// *>          The total number of eigenvalues found.  0 <= M <= N.
// *>          If RANGE = 'A', M = N, and if RANGE = 'I', M = IU-IL+1.
// *> \endverbatim
// *>
// *> \param[out] W
// *> \verbatim
// *>          W is DOUBLE PRECISION array, dimension (N)
// *>          On normal exit, the first M elements contain the selected
// *>          eigenvalues in ascending order.
// *> \endverbatim
// *>
// *> \param[out] Z
// *> \verbatim
// *>          Z is DOUBLE PRECISION array, dimension (LDZ, max(1,M))
// *>          If JOBZ = 'N', then Z is not referenced.
// *>          If JOBZ = 'V', then if INFO = 0, the first M columns of Z
// *>          contain the orthonormal eigenvectors of the matrix A
// *>          corresponding to the selected eigenvalues, with the i-th
// *>          column of Z holding the eigenvector associated with W(i).
// *>          The eigenvectors are normalized as follows:
// *>          if ITYPE = 1 or 2, Z**T*B*Z = I;
// *>          if ITYPE = 3, Z**T*inv(B)*Z = I.
// *>
// *>          If an eigenvector fails to converge, then that column of Z
// *>          contains the latest approximation to the eigenvector, and th
// *>          index of the eigenvector is returned in IFAIL.
// *>          Note: the user must ensure that at least max(1,M) columns ar
// *>          supplied in the array Z; if RANGE = 'V', the exact value of 
// *>          is not known in advance and an upper bound must be used.
// *> \endverbatim
// *>
// *> \param[in] LDZ
// *> \verbatim
// *>          LDZ is INTEGER
// *>          The leading dimension of the array Z.  LDZ >= 1, and if
// *>          JOBZ = 'V', LDZ >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] WORK
// *> \verbatim
// *>          WORK is DOUBLE PRECISION array, dimension (MAX(1,LWORK))
// *>          On exit, if INFO = 0, WORK(1) returns the optimal LWORK.
// *> \endverbatim
// *>
// *> \param[in] LWORK
// *> \verbatim
// *>          LWORK is INTEGER
// *>          The length of the array WORK.  LWORK >= max(1,8*N).
// *>          For optimal efficiency, LWORK >= (NB+3)*N,
// *>          where NB is the blocksize for DSYTRD returned by ILAENV.
// *>
// *>          If LWORK = -1, then a workspace query is assumed; the routin
// *>          only calculates the optimal size of the WORK array, returns

// *>          this value as the first entry of the WORK array, and no erro
// *>          message related to LWORK is issued by XERBLA.
// *> \endverbatim
// *>
// *> \param[out] IWORK
// *> \verbatim
// *>          IWORK is INTEGER array, dimension (5*N)
// *> \endverbatim
// *>
// *> \param[out] IFAIL
// *> \verbatim
// *>          IFAIL is INTEGER array, dimension (N)
// *>          If JOBZ = 'V', then if INFO = 0, the first M elements of
// *>          IFAIL are zero.  If INFO > 0, then IFAIL contains the
// *>          indices of the eigenvectors that failed to converge.
// *>          If JOBZ = 'N', then IFAIL is not referenced.
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value
// *>          > 0:  DPOTRF or DSYEVX returned an error code:
// *>             <= N:  if INFO = i, DSYEVX failed to converge;
// *>                    i eigenvectors failed to converge.  Their indices

// *>                    are stored in array IFAIL.
// *>             > N:   if INFO = N + i, for 1 <= i <= N, then the leading
// *>                    minor of order i of B is not positive definite.
// *>                    The factorization of B could not be completed and

// *>                    no eigenvalues or eigenvectors were computed.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup doubleSYeigen
// *
// *> \par Contributors:
// *  ==================
// *>
// *>     Mark Fahey, Department of Mathematics, Univ. of Kentucky, USA
// *
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
* =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters.
*
***/

public static void dsygvx (int itype,
final char jobz,
final char uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] b, int _b_offset,
int ldb,
double abstol,
intW m,
double [] w, int _w_offset,
double [] z, int _z_offset,
int ldz,
double [] work, int _work_offset,
int lwork,
int [] iwork, int _iwork_offset,
int [] ifail, int _ifail_offset,
intW info)  {

boolean alleig= false;
boolean indeig= false;
boolean lquery= false;
boolean upper= false;
boolean valeig= false;
boolean wantz= false;
//String trans= new String(" ");
int lwkmin= 0;
int lwkopt= 0;
int nb= 0;
//jobz = jobz.substring((1)-1,1);
//range = range.substring((1)-1,1);
//uplo = uplo.substring((1)-1,1);
final char range='A';
final double vl=0,vu=0;
final int il=-1,iu=-1;
upper = com.polytechnik.lapack.Lsame.lsame(uplo,'U');
wantz = com.polytechnik.lapack.Lsame.lsame(jobz,'V');
alleig = com.polytechnik.lapack.Lsame.lsame(range,'A');
valeig = com.polytechnik.lapack.Lsame.lsame(range,'V');
indeig = com.polytechnik.lapack.Lsame.lsame(range,'I');
// *
lquery = ((lwork == -1));
info.val = 0;
if (((itype < 1) || (itype > 3)))  {
    info.val = -1;
}
else if ((!((wantz || com.polytechnik.lapack.Lsame.lsame(jobz,'N')))))  {
    info.val = -2;
}              // Close else if()
else if ((!(((alleig || valeig) || indeig))))  {
    info.val = -3;
}              // Close else if()
else if ((!((upper || com.polytechnik.lapack.Lsame.lsame(uplo,'L')))))  {
    info.val = -4;
}              // Close else if()
else if ((n < 0))  {
    info.val = -5;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -7;
}              // Close else if()
else if ((ldb < Math.max(1, n)))  {
    info.val = -9;
}              // Close else if()
else  {
  if (valeig)  {
    if (((n > 0) && (vu <= vl))) {
    info.val = -11;
}
    }
else if (indeig)  {
    if (((il < 1) || (il > Math.max(1, n))))  {
    info.val = -12;
}
else if (((iu < Math.min(n, il)) || (iu > n)))  {
    info.val = -13;
}              // Close else if()
}              // Close else if()
}              //  Close else.
if ((info.val == 0))  {
    if (((ldz < 1) || ((wantz && (ldz < n)))))  {
    info.val = -18;
}
// *
}
if ((info.val == 0))  {
    lwkmin = Math.max(1, (8*n));
    nb = 1;//com.polytechnik.lapack.Ilaenv.ilaenv(1,"DSYTRD",uplo,n,-1,-1,-1);
lwkopt = Math.max(lwkmin, (((nb+3))*n));
// *
work[(1-(1))+ _work_offset] = (double)(lwkopt);
if (((lwork < lwkmin) && (!lquery)))  {
    info.val = -20;
}
// *
}
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYGVX",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dsygvx",999999);
}
else if (lquery)  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsygvx",999999);
/***
*
*     Quick return if possible
*
***/
}              // Close else if()
m.val = 0;
if ((n == 0))  {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsygvx",999999);
/***
*
*     Form a Cholesky factorization of B.
*
***/
}
com.polytechnik.lapack.Dpotrf.dpotrf(uplo,n,b,_b_offset,ldb,info);
if ((info.val != 0))  {
    info.val = (n+info.val);
return;//Dummy.go_to("com/polytechnik/lapack/Dsygvx",999999);
/***
*
*     Transform problem to standard eigenvalue problem and solve.
*
***/
}
com.polytechnik.lapack.Dsygst.dsygst(itype,uplo,n,a,_a_offset,lda,b,_b_offset,ldb,info);
// *
com.polytechnik.lapack.Dsyevx.dsyevx(jobz,uplo,n,a,_a_offset,lda,abstol,m,w,_w_offset,z,_z_offset,ldz,work,_work_offset,lwork,iwork,_iwork_offset,ifail,_ifail_offset,info);
/***
*
*        Backtransform eigenvectors to the original problem.
*
***/
if (wantz)  {
    if ((info.val > 0)) {
	//m.val = (info.val-1);
}
    /***
*
*           For A*x=(lambda)*B*x and A*B*x=(lambda)*x;
*           backtransform eigenvectors: x = inv(L)**T*y or inv(U)*y
*
***/
final char trans;
if (((itype == 1) || (itype == 2)))  {
    if (upper)  {
    trans = 'N';
}
else  {
  trans = 'T';
// *
}              //  Close else.
// *
com.polytechnik.lapack.Dtrsm.dtrsm('L',uplo,trans,'N',n,m.val,1.0,b,_b_offset,ldb,z,_z_offset,ldz);
/***
*
*           For B*A*x=(lambda)*x;
*           backtransform eigenvectors: x = L*y or U**T*y
*
***/
}
else if ((itype == 3))  {
    if (upper)  {
    trans = 'T';
}
else  {
  trans = 'N';
// *
}              //  Close else.
com.polytechnik.lapack.Dtrmm.dtrmm('L',uplo,trans,'N',n,m.val,1.0,b,_b_offset,ldb,z,_z_offset,ldz);
}              // Close else if()
/***
*
*     Set WORK(1) to optimal workspace size.
*
***/
}
// *
work[(1-(1))+ _work_offset] = (double)(lwkopt);
/***
*
*     End of DSYGVX
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsygvx",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsygvx",999999);
return;
   }
} // End class.
