/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsyr2.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsyr2.java,v 1.4 2012/12/20 17:43:30 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dsyr2 {

// *> \brief \b DSYR2
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYR2(UPLO,N,ALPHA,X,INCX,Y,INCY,A,LDA)
// * 
// *       .. Scalar Arguments ..
// *       DOUBLE PRECISION ALPHA
// *       INTEGER INCX,INCY,LDA,N
// *       CHARACTER UPLO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),X(*),Y(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYR2  performs the symmetric rank 2 operation
// *>
// *>    A := alpha*x*y**T + alpha*y*x**T + A,
// *>
// *> where alpha is a scalar, x and y are n element vectors and A is an n

// *> by n symmetric matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>           On entry, UPLO specifies whether the upper or lower
// *>           triangular part of the array A is to be referenced as
// *>           follows:
// *>
// *>              UPLO = 'U' or 'u'   Only the upper triangular part of A

// *>                                  is to be referenced.
// *>
// *>              UPLO = 'L' or 'l'   Only the lower triangular part of A

// *>                                  is to be referenced.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry, N specifies the order of the matrix A.
// *>           N must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] ALPHA
// *> \verbatim
// *>          ALPHA is DOUBLE PRECISION.
// *>           On entry, ALPHA specifies the scalar alpha.
// *> \endverbatim
// *>
// *> \param[in] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array of dimension at least
// *>           ( 1 + ( n - 1 )*abs( INCX ) ).
// *>           Before entry, the incremented array X must contain the n
// *>           element vector x.
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>           On entry, INCX specifies the increment for the elements of

// *>           X. INCX must not be zero.
// *> \endverbatim
// *>
// *> \param[in] Y
// *> \verbatim
// *>          Y is DOUBLE PRECISION array of dimension at least
// *>           ( 1 + ( n - 1 )*abs( INCY ) ).
// *>           Before entry, the incremented array Y must contain the n
// *>           element vector y.
// *> \endverbatim
// *>
// *> \param[in] INCY
// *> \verbatim
// *>          INCY is INTEGER
// *>           On entry, INCY specifies the increment for the elements of

// *>           Y. INCY must not be zero.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *>           Before entry with  UPLO = 'U' or 'u', the leading n by n
// *>           upper triangular part of the array A must contain the upper
// *>           triangular part of the symmetric matrix and the strictly
// *>           lower triangular part of A is not referenced. On exit, the

// *>           upper triangular part of the array A is overwritten by the

// *>           upper triangular part of the updated matrix.
// *>           Before entry with UPLO = 'L' or 'l', the leading n by n
// *>           lower triangular part of the array A must contain the lower
// *>           triangular part of the symmetric matrix and the strictly
// *>           upper triangular part of A is not referenced. On exit, the

// *>           lower triangular part of the array A is overwritten by the

// *>           lower triangular part of the updated matrix.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program. LDA must be at least
// *>           max( 1, n ).
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level2
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 2 Blas routine.
// *>
// *>  -- Written on 22-October-1986.
// *>     Jack Dongarra, Argonne National Lab.
// *>     Jeremy Du Croz, Nag Central Office.
// *>     Sven Hammarling, Nag Central Office.
// *>     Richard Hanson, Sandia National Labs.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dsyr2 (final char uplo,
int n,
double alpha,
double [] x, int _x_offset,
int incx,
double [] y, int _y_offset,
int incy,
double [] a, int _a_offset,
int lda)  {

double temp1= 0.0d;
double temp2= 0.0d;
int i= 0;
int info= 0;
int ix= 0;
int iy= 0;
int j= 0;
int jx= 0;
int jy= 0;
int kx= 0;
int ky= 0;
//uplo = uplo.substring((1)-1,1);
info = 0;
if (((!com.polytechnik.lapack.Lsame.lsame(uplo,'U')) && (!com.polytechnik.lapack.Lsame.lsame(uplo,'L'))))  {
    info = 1;
}
else if ((n < 0))  {
    info = 2;
}              // Close else if()
else if ((incx == 0))  {
    info = 5;
}              // Close else if()
else if ((incy == 0))  {
    info = 7;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info = 9;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYR2 ",info);
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyr2",999999);
/***
*
*     Quick return if possible.
*
***/
}
/***
*
*     Set up the start points in X and Y if the increments are not both
*     unity.
*
***/
if ((((n == 0)) || ((alpha == 0.0)))) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsyr2",999999);
}
    if ((((incx != 1)) || ((incy != 1))))  {
    if ((incx > 0))  {
    kx = 1;
}
else  {
  kx = (1-(((n-1))*incx));
}              //  Close else.
if ((incy > 0))  {
    ky = 1;
}
else  {
  ky = (1-(((n-1))*incy));
}              //  Close else.
jx = kx;
jy = ky;
/***
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through the triangular part
*     of A.
*
***/
}
/***
*
*        Form  A  when A is stored in the upper triangle.
*
***/
if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    if ((((incx == 1)) && ((incy == 1))))  {
    {
for (j = 1; j <= n; j++) {
if ((((x[(j-(1))+ _x_offset] != 0.0)) || ((y[(j-(1))+ _y_offset] != 0.0))))  {
    temp1 = (alpha*y[(j-(1))+ _y_offset]);
temp2 = (alpha*x[(j-(1))+ _x_offset]);
{
for (i = 1; i <= j; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((a[(i-(1))+(j-(1)) * (lda)+ _a_offset]+(x[(i-(1))+ _x_offset]*temp1))+(y[(i-(1))+ _y_offset]*temp2));
//Dummy.label("com/polytechnik/lapack/Dsyr2",10);
}              //  Close for() loop. 
}
}
//Dummy.label("com/polytechnik/lapack/Dsyr2",20);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
if ((((x[(jx-(1))+ _x_offset] != 0.0)) || ((y[(jy-(1))+ _y_offset] != 0.0))))  {
    temp1 = (alpha*y[(jy-(1))+ _y_offset]);
temp2 = (alpha*x[(jx-(1))+ _x_offset]);
ix = kx;
iy = ky;
{
for (i = 1; i <= j; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((a[(i-(1))+(j-(1)) * (lda)+ _a_offset]+(x[(ix-(1))+ _x_offset]*temp1))+(y[(iy-(1))+ _y_offset]*temp2));
ix = (ix+incx);
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack/Dsyr2",30);
}              //  Close for() loop. 
}
}
jx = (jx+incx);
jy = (jy+incy);
//Dummy.label("com/polytechnik/lapack/Dsyr2",40);
}              //  Close for() loop. 
}
}              //  Close else.
/***
*
*        Form  A  when A is stored in the lower triangle.
*
***/
}
else  {
  if ((((incx == 1)) && ((incy == 1))))  {
    {
for (j = 1; j <= n; j++) {
if ((((x[(j-(1))+ _x_offset] != 0.0)) || ((y[(j-(1))+ _y_offset] != 0.0))))  {
    temp1 = (alpha*y[(j-(1))+ _y_offset]);
temp2 = (alpha*x[(j-(1))+ _x_offset]);
{
for (i = j; i <= n; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((a[(i-(1))+(j-(1)) * (lda)+ _a_offset]+(x[(i-(1))+ _x_offset]*temp1))+(y[(i-(1))+ _y_offset]*temp2));
//Dummy.label("com/polytechnik/lapack/Dsyr2",50);
}              //  Close for() loop. 
}
}
//Dummy.label("com/polytechnik/lapack/Dsyr2",60);
}              //  Close for() loop. 
}
}
else  {
  {
for (j = 1; j <= n; j++) {
if ((((x[(jx-(1))+ _x_offset] != 0.0)) || ((y[(jy-(1))+ _y_offset] != 0.0))))  {
    temp1 = (alpha*y[(jy-(1))+ _y_offset]);
temp2 = (alpha*x[(jx-(1))+ _x_offset]);
ix = jx;
iy = jy;
{
for (i = j; i <= n; i++) {
a[(i-(1))+(j-(1)) * (lda)+ _a_offset] = ((a[(i-(1))+(j-(1)) * (lda)+ _a_offset]+(x[(ix-(1))+ _x_offset]*temp1))+(y[(iy-(1))+ _y_offset]*temp2));
ix = (ix+incx);
iy = (iy+incy);
//Dummy.label("com/polytechnik/lapack/Dsyr2",70);
}              //  Close for() loop. 
}
}
jx = (jx+incx);
jy = (jy+incy);
//Dummy.label("com/polytechnik/lapack/Dsyr2",80);
}              //  Close for() loop. 
}
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DSYR2 .
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsyr2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsyr2",999999);
return;
   }
} // End class.
