/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dsytd2.f
 *  f2java version: 0.8.1
 *
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dsytd2.java,v 1.3 2012/12/19 21:45:44 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dsytd2 {

// *> \brief \b DSYTD2 reduces a symmetric matrix to real symmetric tridiag
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *> \htmlonly
// *> Download DSYTD2 + dependencies 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&fil
// *> [TGZ]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&fil
// *> [ZIP]</a> 
// *> <a href="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&fil
// *> [TXT]</a>
// *> \endhtmlonly 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DSYTD2( UPLO, N, A, LDA, D, E, TAU, INFO )
// * 
// *       .. Scalar Arguments ..
// *       CHARACTER          UPLO
// *       INTEGER            INFO, LDA, N
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION   A( LDA, * ), D( * ), E( * ), TAU( * )
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DSYTD2 reduces a real symmetric matrix A to symmetric tridiagonal
// *> form T by an orthogonal similarity transformation: Q**T * A * Q = T.

// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>          Specifies whether the upper or lower triangular part of the

// *>          symmetric matrix A is stored:
// *>          = 'U':  Upper triangular
// *>          = 'L':  Lower triangular
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>          The order of the matrix A.  N >= 0.
// *> \endverbatim
// *>
// *> \param[in,out] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array, dimension (LDA,N)
// *>          On entry, the symmetric matrix A.  If UPLO = 'U', the leadin
// *>          n-by-n upper triangular part of A contains the upper
// *>          triangular part of the matrix A, and the strictly lower
// *>          triangular part of A is not referenced.  If UPLO = 'L', the

// *>          leading n-by-n lower triangular part of A contains the lower
// *>          triangular part of the matrix A, and the strictly upper
// *>          triangular part of A is not referenced.
// *>          On exit, if UPLO = 'U', the diagonal and first superdiagonal
// *>          of A are overwritten by the corresponding elements of the
// *>          tridiagonal matrix T, and the elements above the first
// *>          superdiagonal, with the array TAU, represent the orthogonal

// *>          matrix Q as a product of elementary reflectors; if UPLO
// *>          = 'L', the diagonal and first subdiagonal of A are over-
// *>          written by the corresponding elements of the tridiagonal
// *>          matrix T, and the elements below the first subdiagonal, with
// *>          the array TAU, represent the orthogonal matrix Q as a produc
// *>          of elementary reflectors. See Further Details.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>          The leading dimension of the array A.  LDA >= max(1,N).
// *> \endverbatim
// *>
// *> \param[out] D
// *> \verbatim
// *>          D is DOUBLE PRECISION array, dimension (N)
// *>          The diagonal elements of the tridiagonal matrix T:
// *>          D(i) = A(i,i).
// *> \endverbatim
// *>
// *> \param[out] E
// *> \verbatim
// *>          E is DOUBLE PRECISION array, dimension (N-1)
// *>          The off-diagonal elements of the tridiagonal matrix T:
// *>          E(i) = A(i,i+1) if UPLO = 'U', E(i) = A(i+1,i) if UPLO = 'L'
// *> \endverbatim
// *>
// *> \param[out] TAU
// *> \verbatim
// *>          TAU is DOUBLE PRECISION array, dimension (N-1)
// *>          The scalar factors of the elementary reflectors (see Further
// *>          Details).
// *> \endverbatim
// *>
// *> \param[out] INFO
// *> \verbatim
// *>          INFO is INTEGER
// *>          = 0:  successful exit
// *>          < 0:  if INFO = -i, the i-th argument had an illegal value.

// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date September 2012
// *
// *> \ingroup doubleSYcomputational
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  If UPLO = 'U', the matrix Q is represented as a product of elementar
// *>  reflectors
// *>
// *>     Q = H(n-1) . . . H(2) H(1).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(i+1:n) = 0 and v(i) = 1; v(1:i-1) is stored on exit in
// *>  A(1:i-1,i+1), and tau in TAU(i).
// *>
// *>  If UPLO = 'L', the matrix Q is represented as a product of elementar
// *>  reflectors
// *>
// *>     Q = H(1) H(2) . . . H(n-1).
// *>
// *>  Each H(i) has the form
// *>
// *>     H(i) = I - tau * v * v**T
// *>
// *>  where tau is a real scalar, and v is a real vector with
// *>  v(1:i) = 0 and v(i+1) = 1; v(i+2:n) is stored on exit in A(i+2:n,i),
// *>  and tau in TAU(i).
// *>
// *>  The contents of A on exit are illustrated by the following examples

// *>  with n = 5:
// *>
// *>  if UPLO = 'U':                       if UPLO = 'L':
// *>
// *>    (  d   e   v2  v3  v4 )              (  d                  )
// *>    (      d   e   v3  v4 )              (  e   d              )
// *>    (          d   e   v4 )              (  v1  e   d          )
// *>    (              d   e  )              (  v1  v2  e   d      )
// *>    (                  d  )              (  v1  v2  v3  e   d  )
// *>
// *>  where d and e denote diagonal and off-diagonal elements of T, and vi
// *>  denotes an element of the vector defining H(i).
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*     .. Executable Statements ..
*
*     Test the input parameters
*
***/

public static void dsytd2 (final char uplo,
int n,
double [] a, int _a_offset,
int lda,
double [] d, int _d_offset,
double [] e, int _e_offset,
double [] tau, int _tau_offset,
intW info)  {

boolean upper= false;
int i= 0;
double alpha= 0.0d;
doubleW taui= new doubleW(0.0d);
//uplo = uplo.substring((1)-1,1);
info.val = 0;
upper = com.polytechnik.lapack.Lsame.lsame(uplo,'U');
if (((!upper) && (!com.polytechnik.lapack.Lsame.lsame(uplo,'L'))))  {
    info.val = -1;
}
else if ((n < 0))  {
    info.val = -2;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info.val = -4;
}              // Close else if()
if ((info.val != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DSYTD2",(-(info.val)));
    return;//Dummy.go_to("com/polytechnik/lapack/Dsytd2",999999);
/***
*
*     Quick return if possible
*
***/
}
// *
if ((n <= 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dsytd2",999999);
}
    /***
*
*        Reduce the upper triangle of A
*
***/
if (upper)  {
    /***
*
*           Generate elementary reflector H(i) = I - tau * v * v**T
*           to annihilate A(1:i-1,i+1)
*
***/
{
int _i_inc = -1;
for (i = (n-1); i >= 1; i += _i_inc) {
dlarfg_adapter(i,a,(i-(1))+((i+1)-(1)) * (lda)+ _a_offset,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,1,taui);
// *
e[(i-(1))+ _e_offset] = a[(i-(1))+((i+1)-(1)) * (lda)+ _a_offset];
/***
*
*              Apply H(i) from both sides to A(1:i,1:i)
*
***/
if ((taui.val != 0.0))  {
    /***
*
*              Compute  x := tau * A * v  storing x in TAU(1:i)
*
***/
a[(i-(1))+((i+1)-(1)) * (lda)+ _a_offset] = 1.0;
/***
*
*              Compute  w := x - 1/2 * tau * (x**T * v) * v
*
***/
com.polytechnik.lapack.Dsymv.dsymv(uplo,i,taui.val,a,_a_offset,lda,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,1,0.0,tau,_tau_offset,1);
alpha = (-(((0.5*taui.val)*com.polytechnik.lapack.Ddot.ddot(i,tau,_tau_offset,1,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,1))));
/***
*
*              Apply the transformation as a rank-2 update:
*                 A := A - v * w**T - w * v**T
*
***/
com.polytechnik.lapack.Daxpy.daxpy(i,alpha,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,1,tau,_tau_offset,1);
// *
com.polytechnik.lapack.Dsyr2.dsyr2(uplo,i,-1.0,a,(1-(1))+((i+1)-(1)) * (lda)+ _a_offset,1,tau,_tau_offset,1,a,_a_offset,lda);
a[(i-(1))+((i+1)-(1)) * (lda)+ _a_offset] = e[(i-(1))+ _e_offset];
}
d[((i+1)-(1))+ _d_offset] = a[((i+1)-(1))+((i+1)-(1)) * (lda)+ _a_offset];
tau[(i-(1))+ _tau_offset] = taui.val;
//Dummy.label("com/polytechnik/lapack/Dsytd2",10);
}              //  Close for() loop. 
}
d[(1-(1))+ _d_offset] = a[(1-(1))+(1-(1)) * (lda)+ _a_offset];
/***
*
*        Reduce the lower triangle of A
*
***/
}
else  {
  /***
*
*           Generate elementary reflector H(i) = I - tau * v * v**T
*           to annihilate A(i+2:n,i)
*
***/
{
for (i = 1; i <= (n-1); i++) {
dlarfg_adapter((n-i),a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,a,(Math.min((i+2), n)-(1))+(i-(1)) * (lda)+ _a_offset,1,taui);
// *
e[(i-(1))+ _e_offset] = a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset];
/***
*
*              Apply H(i) from both sides to A(i+1:n,i+1:n)
*
***/
if ((taui.val != 0.0))  {
    /***
*
*              Compute  x := tau * A * v  storing y in TAU(i:n-1)
*
***/
a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] = 1.0;
/***
*
*              Compute  w := x - 1/2 * tau * (x**T * v) * v
*
***/
com.polytechnik.lapack.Dsymv.dsymv(uplo,(n-i),taui.val,a,((i+1)-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,0.0,tau,(i-(1))+ _tau_offset,1);
alpha = (-(((0.5*taui.val)*com.polytechnik.lapack.Ddot.ddot((n-i),tau,(i-(1))+ _tau_offset,1,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1))));
/***
*
*              Apply the transformation as a rank-2 update:
*                 A := A - v * w**T - w * v**T
*
***/
com.polytechnik.lapack.Daxpy.daxpy((n-i),alpha,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,(i-(1))+ _tau_offset,1);
// *
com.polytechnik.lapack.Dsyr2.dsyr2(uplo,(n-i),-1.0,a,((i+1)-(1))+(i-(1)) * (lda)+ _a_offset,1,tau,(i-(1))+ _tau_offset,1,a,((i+1)-(1))+((i+1)-(1)) * (lda)+ _a_offset,lda);
a[((i+1)-(1))+(i-(1)) * (lda)+ _a_offset] = e[(i-(1))+ _e_offset];
}
d[(i-(1))+ _d_offset] = a[(i-(1))+(i-(1)) * (lda)+ _a_offset];
tau[(i-(1))+ _tau_offset] = taui.val;
//Dummy.label("com/polytechnik/lapack/Dsytd2",20);
}              //  Close for() loop. 
}
d[(n-(1))+ _d_offset] = a[(n-(1))+(n-(1)) * (lda)+ _a_offset];
// *
}              //  Close else.
/***
*
*     End of DSYTD2
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dsytd2",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dsytd2",999999);
return;
   }
// adapter for dlarfg(ILorg/netlib/util/doubleW;[DIILorg/netlib/util/doubleW;)V
private static void dlarfg_adapter(int arg0 ,double [] arg1 , int arg1_offset ,double [] arg2 , int arg2_offset ,int arg3 ,doubleW arg4 )
{
doubleW _f2j_tmp1 = new doubleW(arg1[arg1_offset]);

com.polytechnik.lapack.Dlarfg.dlarfg(arg0,_f2j_tmp1,arg2, arg2_offset,arg3,arg4);

arg1[arg1_offset] = _f2j_tmp1.val;
}

} // End class.
