/*
 *  Produced by f2java.  f2java is part of the Fortran-
 *  -to-Java project at the University of Tennessee Netlib
 *  numerical software repository.
 *
 *  Original authorship for the BLAS and LAPACK numerical
 *  routines may be found in the Fortran source, available at
 *  http://www.netlib.org.
 *
 *  Fortran input file: dtrmv.f
 *  f2java version: 0.8.1
 *
 *  Manually adjusted by
 *  @author Vladislav Malyshkin mal@gromco.com
 *  @version $Id: Dtrmv.java,v 1.4 2012/12/20 17:58:50 mal Exp $
 *
 */

package com.polytechnik.lapack;



public class Dtrmv {

// *> \brief \b DTRMV
// *
// *  =========== DOCUMENTATION ===========
// *
// * Online html documentation available at 
// *            http://www.netlib.org/lapack/explore-html/ 
// *
// *  Definition:
// *  ===========
// *
// *       SUBROUTINE DTRMV(UPLO,TRANS,DIAG,N,A,LDA,X,INCX)
// * 
// *       .. Scalar Arguments ..
// *       INTEGER INCX,LDA,N
// *       CHARACTER DIAG,TRANS,UPLO
// *       ..
// *       .. Array Arguments ..
// *       DOUBLE PRECISION A(LDA,*),X(*)
// *       ..
// *  
// *
// *> \par Purpose:
// *  =============
// *>
// *> \verbatim
// *>
// *> DTRMV  performs one of the matrix-vector operations
// *>
// *>    x := A*x,   or   x := A**T*x,
// *>
// *> where x is an n element vector and  A is an n by n unit, or non-unit,
// *> upper or lower triangular matrix.
// *> \endverbatim
// *
// *  Arguments:
// *  ==========
// *
// *> \param[in] UPLO
// *> \verbatim
// *>          UPLO is CHARACTER*1
// *>           On entry, UPLO specifies whether the matrix is an upper or

// *>           lower triangular matrix as follows:
// *>
// *>              UPLO = 'U' or 'u'   A is an upper triangular matrix.
// *>
// *>              UPLO = 'L' or 'l'   A is a lower triangular matrix.
// *> \endverbatim
// *>
// *> \param[in] TRANS
// *> \verbatim
// *>          TRANS is CHARACTER*1
// *>           On entry, TRANS specifies the operation to be performed as

// *>           follows:
// *>
// *>              TRANS = 'N' or 'n'   x := A*x.
// *>
// *>              TRANS = 'T' or 't'   x := A**T*x.
// *>
// *>              TRANS = 'C' or 'c'   x := A**T*x.
// *> \endverbatim
// *>
// *> \param[in] DIAG
// *> \verbatim
// *>          DIAG is CHARACTER*1
// *>           On entry, DIAG specifies whether or not A is unit
// *>           triangular as follows:
// *>
// *>              DIAG = 'U' or 'u'   A is assumed to be unit triangular.

// *>
// *>              DIAG = 'N' or 'n'   A is not assumed to be unit
// *>                                  triangular.
// *> \endverbatim
// *>
// *> \param[in] N
// *> \verbatim
// *>          N is INTEGER
// *>           On entry, N specifies the order of the matrix A.
// *>           N must be at least zero.
// *> \endverbatim
// *>
// *> \param[in] A
// *> \verbatim
// *>          A is DOUBLE PRECISION array of DIMENSION ( LDA, n ).
// *>           Before entry with  UPLO = 'U' or 'u', the leading n by n
// *>           upper triangular part of the array A must contain the upper
// *>           triangular matrix and the strictly lower triangular part of
// *>           A is not referenced.
// *>           Before entry with UPLO = 'L' or 'l', the leading n by n
// *>           lower triangular part of the array A must contain the lower
// *>           triangular matrix and the strictly upper triangular part of
// *>           A is not referenced.
// *>           Note that when  DIAG = 'U' or 'u', the diagonal elements of
// *>           A are not referenced either, but are assumed to be unity.
// *> \endverbatim
// *>
// *> \param[in] LDA
// *> \verbatim
// *>          LDA is INTEGER
// *>           On entry, LDA specifies the first dimension of A as declare
// *>           in the calling (sub) program. LDA must be at least
// *>           max( 1, n ).
// *> \endverbatim
// *>
// *> \param[in,out] X
// *> \verbatim
// *>          X is DOUBLE PRECISION array of dimension at least
// *>           ( 1 + ( n - 1 )*abs( INCX ) ).
// *>           Before entry, the incremented array X must contain the n
// *>           element vector x. On exit, X is overwritten with the
// *>           tranformed vector x.
// *> \endverbatim
// *>
// *> \param[in] INCX
// *> \verbatim
// *>          INCX is INTEGER
// *>           On entry, INCX specifies the increment for the elements of

// *>           X. INCX must not be zero.
// *> \endverbatim
// *
// *  Authors:
// *  ========
// *
// *> \author Univ. of Tennessee 
// *> \author Univ. of California Berkeley 
// *> \author Univ. of Colorado Denver 
// *> \author NAG Ltd. 
// *
// *> \date November 2011
// *
// *> \ingroup double_blas_level2
// *
// *> \par Further Details:
// *  =====================
// *>
// *> \verbatim
// *>
// *>  Level 2 Blas routine.
// *>  The vector and matrix arguments are not referenced when N = 0, or M 
// *>
// *>  -- Written on 22-October-1986.
// *>     Jack Dongarra, Argonne National Lab.
// *>     Jeremy Du Croz, Nag Central Office.
// *>     Sven Hammarling, Nag Central Office.
// *>     Richard Hanson, Sandia National Labs.
// *> \endverbatim
// *>
// *  =====================================================================
// 
/***
*     ..
*     .. Array Arguments ..
***/
/***
*     ..
*
*  =====================================================================
*
*     .. Parameters ..
***/
/***
*     ..
*     .. Local Scalars ..
***/
/***
*     ..
*     .. External Functions ..
***/
/***
*     ..
*     .. External Subroutines ..
***/
/***
*     ..
*     .. Intrinsic Functions ..
***/
/***
*     ..
*
*     Test the input parameters.
*
***/

public static void dtrmv (final char uplo,
final char trans,
final char diag,
int n,
double [] a, int _a_offset,
int lda,
double [] x, int _x_offset,
int incx)  {

double temp= 0.0d;
int i= 0;
int info= 0;
int ix= 0;
int j= 0;
int jx= 0;
int kx= 0;
boolean nounit= false;
//uplo = uplo.substring((1)-1,1);
//trans = trans.substring((1)-1,1);
//diag = diag.substring((1)-1,1);
info = 0;
if (((!com.polytechnik.lapack.Lsame.lsame(uplo,'U')) && (!com.polytechnik.lapack.Lsame.lsame(uplo,'L'))))  {
    info = 1;
}
else if ((((!com.polytechnik.lapack.Lsame.lsame(trans,'N')) && (!com.polytechnik.lapack.Lsame.lsame(trans,'T'))) && (!com.polytechnik.lapack.Lsame.lsame(trans,'C'))))  {
    info = 2;
}              // Close else if()
else if (((!com.polytechnik.lapack.Lsame.lsame(diag,'U')) && (!com.polytechnik.lapack.Lsame.lsame(diag,'N'))))  {
    info = 3;
}              // Close else if()
else if ((n < 0))  {
    info = 4;
}              // Close else if()
else if ((lda < Math.max(1, n)))  {
    info = 6;
}              // Close else if()
else if ((incx == 0))  {
    info = 8;
}              // Close else if()
if ((info != 0))  {
    com.polytechnik.lapack.Xerbla.xerbla("DTRMV ",info);
    return;//Dummy.go_to("com/polytechnik/lapack/Dtrmv",999999);
/***
*
*     Quick return if possible.
*
***/
}
// *
if ((n == 0)) {
    return;//Dummy.go_to("com/polytechnik/lapack/Dtrmv",999999);
}
    /***
*
*     Set up the start point in X if the increment is not unity. This
*     will be  ( N - 1 )*INCX  too small for descending loops.
*
***/
nounit = com.polytechnik.lapack.Lsame.lsame(diag,'N');
if ((incx <= 0))  {
    kx = (1-(((n-1))*incx));
}
else if ((incx != 1))  {
    kx = 1;
/***
*
*     Start the operations. In this version the elements of A are
*     accessed sequentially with one pass through A.
*
***/
}              // Close else if()
/***
*
*        Form  x := A*x.
*
***/
if (com.polytechnik.lapack.Lsame.lsame(trans,'N'))  {
    if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    if ((incx == 1))  {
    {
for (j = 1; j <= n; j++) {
if ((x[(j-(1))+ _x_offset] != 0.0))  {
    temp = x[(j-(1))+ _x_offset];
{
for (i = 1; i <= (j-1); i++) {
x[(i-(1))+ _x_offset] = (x[(i-(1))+ _x_offset]+(temp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dtrmv",10);
}              //  Close for() loop. 
}
if (nounit) {
    x[(j-(1))+ _x_offset] = (x[(j-(1))+ _x_offset]*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    }
//Dummy.label("com/polytechnik/lapack/Dtrmv",20);
}              //  Close for() loop. 
}
}
else  {
  jx = kx;
{
for (j = 1; j <= n; j++) {
if ((x[(jx-(1))+ _x_offset] != 0.0))  {
    temp = x[(jx-(1))+ _x_offset];
ix = kx;
{
for (i = 1; i <= (j-1); i++) {
x[(ix-(1))+ _x_offset] = (x[(ix-(1))+ _x_offset]+(temp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
ix = (ix+incx);
//Dummy.label("com/polytechnik/lapack/Dtrmv",30);
}              //  Close for() loop. 
}
if (nounit) {
    x[(jx-(1))+ _x_offset] = (x[(jx-(1))+ _x_offset]*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    }
jx = (jx+incx);
//Dummy.label("com/polytechnik/lapack/Dtrmv",40);
}              //  Close for() loop. 
}
}              //  Close else.
}
else  {
  if ((incx == 1))  {
    {
int _j_inc = -1;
for (j = n; j >= 1; j += _j_inc) {
if ((x[(j-(1))+ _x_offset] != 0.0))  {
    temp = x[(j-(1))+ _x_offset];
{
int _i_inc = -1;
for (i = n; i >= (j+1); i += _i_inc) {
x[(i-(1))+ _x_offset] = (x[(i-(1))+ _x_offset]+(temp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
//Dummy.label("com/polytechnik/lapack/Dtrmv",50);
}              //  Close for() loop. 
}
if (nounit) {
    x[(j-(1))+ _x_offset] = (x[(j-(1))+ _x_offset]*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    }
//Dummy.label("com/polytechnik/lapack/Dtrmv",60);
}              //  Close for() loop. 
}
}
else  {
  kx = (kx+(((n-1))*incx));
jx = kx;
{
int _j_inc = -1;
for (j = n; j >= 1; j += _j_inc) {
if ((x[(jx-(1))+ _x_offset] != 0.0))  {
    temp = x[(jx-(1))+ _x_offset];
ix = kx;
{
int _i_inc = -1;
for (i = n; i >= (j+1); i += _i_inc) {
x[(ix-(1))+ _x_offset] = (x[(ix-(1))+ _x_offset]+(temp*a[(i-(1))+(j-(1)) * (lda)+ _a_offset]));
ix = (ix-incx);
//Dummy.label("com/polytechnik/lapack/Dtrmv",70);
}              //  Close for() loop. 
}
if (nounit) {
    x[(jx-(1))+ _x_offset] = (x[(jx-(1))+ _x_offset]*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    }
jx = (jx-incx);
//Dummy.label("com/polytechnik/lapack/Dtrmv",80);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
/***
*
*        Form  x := A**T*x.
*
***/
}
else  {
  if (com.polytechnik.lapack.Lsame.lsame(uplo,'U'))  {
    if ((incx == 1))  {
    {
int _j_inc = -1;
for (j = n; j >= 1; j += _j_inc) {
temp = x[(j-(1))+ _x_offset];
if (nounit) {
    temp = (temp*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    {
int _i_inc = -1;
for (i = (j-1); i >= 1; i += _i_inc) {
temp = (temp+(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*x[(i-(1))+ _x_offset]));
//Dummy.label("com/polytechnik/lapack/Dtrmv",90);
}              //  Close for() loop. 
}
x[(j-(1))+ _x_offset] = temp;
//Dummy.label("com/polytechnik/lapack/Dtrmv",100);
}              //  Close for() loop. 
}
}
else  {
  jx = (kx+(((n-1))*incx));
{
int _j_inc = -1;
for (j = n; j >= 1; j += _j_inc) {
temp = x[(jx-(1))+ _x_offset];
ix = jx;
if (nounit) {
    temp = (temp*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    {
int _i_inc = -1;
for (i = (j-1); i >= 1; i += _i_inc) {
ix = (ix-incx);
temp = (temp+(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*x[(ix-(1))+ _x_offset]));
//Dummy.label("com/polytechnik/lapack/Dtrmv",110);
}              //  Close for() loop. 
}
x[(jx-(1))+ _x_offset] = temp;
jx = (jx-incx);
//Dummy.label("com/polytechnik/lapack/Dtrmv",120);
}              //  Close for() loop. 
}
}              //  Close else.
}
else  {
  if ((incx == 1))  {
    {
for (j = 1; j <= n; j++) {
temp = x[(j-(1))+ _x_offset];
if (nounit) {
    temp = (temp*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    {
for (i = (j+1); i <= n; i++) {
temp = (temp+(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*x[(i-(1))+ _x_offset]));
//Dummy.label("com/polytechnik/lapack/Dtrmv",130);
}              //  Close for() loop. 
}
x[(j-(1))+ _x_offset] = temp;
//Dummy.label("com/polytechnik/lapack/Dtrmv",140);
}              //  Close for() loop. 
}
}
else  {
  jx = kx;
{
for (j = 1; j <= n; j++) {
temp = x[(jx-(1))+ _x_offset];
ix = jx;
if (nounit) {
    temp = (temp*a[(j-(1))+(j-(1)) * (lda)+ _a_offset]);
}
    {
for (i = (j+1); i <= n; i++) {
ix = (ix+incx);
temp = (temp+(a[(i-(1))+(j-(1)) * (lda)+ _a_offset]*x[(ix-(1))+ _x_offset]));
//Dummy.label("com/polytechnik/lapack/Dtrmv",150);
}              //  Close for() loop. 
}
x[(jx-(1))+ _x_offset] = temp;
jx = (jx+incx);
//Dummy.label("com/polytechnik/lapack/Dtrmv",160);
}              //  Close for() loop. 
}
}              //  Close else.
}              //  Close else.
// *
}              //  Close else.
/***
*
*     End of DTRMV .
*
***/
//Dummy.go_to("com/polytechnik/lapack/Dtrmv",999999);
// dummy stmt
//Dummy.label("com/polytechnik/lapack/Dtrmv",999999);
return;
   }
} // End class.
