package com.polytechnik.lapack;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */
import com.polytechnik.utils.TMatr;
import com.polytechnik.utils.UnitTests;

/** Eigenvalues
 *  @version $Id: Eigenvalues.java,v 1.30 2013/01/27 17:59:22 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

public class Eigenvalues {


public static double [] getEV_dstebz(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev)
{
    if(n<=0) return new double []{};

    final double [] work,res;
    final int [] iwork=new int [n*5];
    final char sort;
    if(flag_ev)
    {
	res=new double [n+n*n];
	work=new double [n*5];
	sort='B';
    }
    else
    {
	res=new double [n];
	work=new double [n*4];
	sort='E';
    }

    final intW m=new intW(0),info=new intW(0),nsplit=new intW(0);
    Dstebz.dstebz(
	'A',sort,n,
	Double.NaN,Double.NaN, // VL, VU
	-1,-1, // IL,IU
	-1, // ABSTOL, use default
	diag,diag_off,offdiag,offdiag_off, // diag & offdiag. No need to clone arrays, the data is preserved.
	m, // number of eigenvalues calculated
	nsplit,
	res,0, // result, first m.val elements
	iwork,0+0,
	iwork,0+n,
	work,0,
	iwork,0+n+n,
	info);
    //System.err.println("getEV:n="+n+" work[0]="+work[0]+" iwork[0]="+iwork[0]+" m.val="+m.val+" res="+java.util.Arrays.toString(res));
    //System.err.println("nsplit="+nsplit.val);

    if(m.val!=n||info.val!=0) {java.util.Arrays.fill(res,Double.NaN);return res;}

    if(!flag_ev){return res;}

    Dstein.dstein(n,
		  diag,diag_off,offdiag,offdiag_off, // diag & offdiag. No need to clone arrays, the data is preserved.
		  m.val,
		  res,0,
		  iwork,0+0,
		  iwork,0+n,
		  res,n,n,
		  work,0,
		  iwork,0+n+n,
		  iwork,0+n+n+n,
		  info);
    if(info.val!=0) {java.util.Arrays.fill(res,Double.NaN);return res;}

    LapackUtils.sortArrayWithLoad(n,res,0+0,n,res,n,0+n,-1,null,-1,-1);
    
    return res;
}


public static double [] getEV_dstemr(final int n,final double [] diag,final int diag_off,final double [] offdiag,final int offdiag_off,final boolean flag_ev)
{
    if(n<=0) return new double []{};
    final double [] work,res;
    final int [] iwork,isplit;
    final char jobz;
    // work/iwork sizes is 18/10, or 12/8 if no eigenvalues required.
    if(flag_ev)
    {
	work=new double [n*18];
	res=new double [n+n*n];
	iwork=new int [n*10];
	isplit=new int [n*2];
	jobz='V';
    }
    else
    {
	work=new double [n*12];
	res=new double [n];
	iwork=new int [n*8];
	isplit=null;
	jobz='N';
    }
    final double [] d=new double [2*n];
    System.arraycopy(diag,diag_off,d,0,n);
    System.arraycopy(offdiag,offdiag_off,d,n,n-1);

    final booleanW tryrac=new booleanW(true);
    final intW m=new intW(0),info=new intW(0);
    Dstemr.dstemr(
	jobz,'A',n,
	d,0,d,n, // diag & offdiag.
	Double.NaN,Double.NaN, // VL, VU
	-1,-1, // IL,IU
	m,
	res,0,
        res,n,n,n,
	isplit,0,
	tryrac,
	work,0,work.length,
	iwork,0,iwork.length,
	info);

    if(m.val!=n||info.val!=0) java.util.Arrays.fill(res,Double.NaN);
    return res;
}


/** First n element -eigenvalues, if eigenvectors are required, then n eigenvectors follow.
 */
public static double [] getEV_dsyevx(final int n,final double [] matr,final int lda_matr,final boolean flag_ev)
{
    if(n<=0) return new double []{};
    final double [] mt=new double [lda_matr*n];
    System.arraycopy(matr,0,mt,0,mt.length);
    final double [] work=new double [n*8];
    final int [] iwork=new int [n*5],ifail;
    final double [] ev;
    final char job;
    if(flag_ev)
    {
	ifail=new int [n];
	ev=new double [n*(n+1)];
	job='V';
    }
    else
    {
	ifail=null;
	ev=new double [n];
	job='N';
    }
    final intW m=new intW(0),info=new intW(0);
    Dsyevx.dsyevx(
	job,'U',
	n,mt,0,lda_matr,
	0,
	m,
	ev,0,
	ev,n,n,
	work,0,work.length,
	iwork,0,
	ifail,0,
	info);

    if(m.val!=n||info.val!=0) java.util.Arrays.fill(ev,Double.NaN);
    return ev;
}


/** First n element -eigenvalues, if eigenvectors are required, then n eigenvectors follow.
 */
public static double [] getEV_dsyevr(final int n,final double [] matr,final int lda_matr,final boolean flag_ev)
{
    if(n<=0) return new double []{};
    final double [] mt=new double [lda_matr*n];
    System.arraycopy(matr,0,mt,0,mt.length);
    final double [] work=new double [n*26];
    final int [] iwork=new int [n*10],isupp;
    final double [] ev;
    final char job;
    if(flag_ev)
    {
	isupp=new int [2*n];
	ev=new double [n*(n+1)];
	job='V';
    }
    else
    {
	isupp=null;
	ev=new double [n];
	job='N';
    }
    final intW m=new intW(0),info=new intW(0);
    Dsyevr.dsyevr(
	job,'A','U',
	n,mt,0,lda_matr,
	0,0,-1,-1,
	0, // ABSTOL
	m,
	ev,0,
	ev,n,n,
	isupp,0,
	work,0,work.length,
	iwork,0,iwork.length,
	info);

    if(m.val!=n||info.val!=0) java.util.Arrays.fill(ev,Double.NaN);
    return ev;
}

/** Generalized eigenvalues.
 *  First n element -eigenvalues, if eigenvectors are required, then n eigenvectors follow.
 */
public static double [] getGEV_dsygvx(final int n,final double [] A,final int lda_A,final double [] B,final int lda_B,final boolean flag_ev)
{
    if(n<=0) return new double []{};
    final int o=lda_A*n,om=o+lda_B*n,nwork=n*8;
    final double [] mt=new double [om+nwork];
    System.arraycopy(A,0,mt,0,o);
    System.arraycopy(B,0,mt,o,lda_B*n);
    final int [] iwork=new int [n*5],ifail;
    final double [] ev;
    final char job;
    if(flag_ev)
    {
	ifail=new int [n];
	ev=new double [n*(n+1)];
	job='V';
    }
    else
    {
	ifail=null;
	ev=new double [n];
	job='N';
    }
    final intW m=new intW(0),info=new intW(0);
    Dsygvx.dsygvx(1,
	job,'U',
	n,mt,0,lda_A,mt,o,lda_B,
	0,
	m,
	ev,0,
	ev,n,n,
	mt,om,nwork,
	iwork,0,
	ifail,0,
	info);

    if(m.val!=n||info.val!=0) java.util.Arrays.fill(ev,Double.NaN);
    return ev;
}


/** Generalized eigenvalues.
 *  @return First n element -eigenvalues, if eigenvectors are required, then n eigenvectors follow.
 */
public static double [] getGEV_dsygvx_dsyevr(final int n,final double [] A,final int lda_A,final double [] B,final int lda_B,final boolean flag_ev)
{
    if(n<=0) return new double []{};
    final int o=lda_A*n,om=o+lda_B*n,nwork=n*26;
    final double [] mt=new double [om+nwork];
    System.arraycopy(A,0,mt,0,o);
    System.arraycopy(B,0,mt,o,lda_B*n);
    final int [] iwork=new int [n*10];
    final double [] ev;
    final char job;
    if(flag_ev)
    {
	ev=new double [n*(n+1)];
	job='V';
    }
    else
    {
	ev=new double [n];
	job='N';
    }
    final intW m=new intW(0),info=new intW(0);
    Dsygvx_dsyevr.dsygvr(1,
	job,'U',
	n,mt,0,lda_A,mt,o,lda_B,
	0, // ABSTOL
	m,
	ev,0,
	ev,n,n,
	mt,om,nwork,
	iwork,0,
	info);

    if(m.val!=n||info.val!=0) java.util.Arrays.fill(ev,Double.NaN);
    return ev;
}



/** Calls Dgees.dgees to calculate eigenvalues of non-symmetric matrix.
 * @param n Dimension.
 * @param matr Matrix, on exit matr is clobbered.
 * @param lda_matr Leading dimension of matr.
 * @param wr[n] Real part of eigenvalues.
 * @param wi[n] Imaginary part of eigenvalues.
 * @return error code, equal to 0 if success.
 */
public static int getNonSymEV_dgees(final int n,final double [] matr,final int lda_matr,final double [] wr,final double [] wi)
{
    if(n==0) return 0;

    final double [] work=new double [3*n];
    final intW info=new intW(0);
    Dgees.dgees('N',
		n,
		matr,0,lda_matr,
		wr,0,
		wi,0,
		null,0,
		n,
		work,0,work.length,
		info);
    return info.val;
}

private static void test_EV_getEV_dsyevx(final int n_tests,final int n,java.util.Random r,final double eps)
{
    final double [] matr=new double [n*n];
 for(int t=0;t<n_tests;t++)
 {
     final double scale;
     final int is=r.nextInt(10);
     if(is==0) scale=1e200;
     else if(is==1) scale=1e-200;
     else scale=1;

    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++) matr[k*n+l]=matr[l*n+l]=scale*r.nextDouble();	    
    }
    final double [] ev=getEV_dsyevx(n,matr,n,true);
    final double [] lv=new double [n],mv=new double [n];
    final int o=n;
    for(int k=n;k<n;k++)
    {
	final double lambda=ev[k];
	double nrm=0;
	for(int l=0;l<n;l++)
	{
	    lv[l]=lambda*ev[o+k*n+l];
	    nrm+=ev[o+k*n+l]*ev[o+k*n+l];
	    double s=0;
	    for(int j=0;j<n;j++)
	    {
		s+=matr[l*n+j]*ev[o+k*n+j];
	    }
	    mv[l]=s;
	}
	UnitTests.checkDoubleArrayRel(lv,"lv",mv,"mv",eps);
	UnitTests.checkDoubleRel(nrm,"nrm",1,"1",eps);
    }
 }
 System.err.println("Eigenvalues:test_EV_getEV_dsyevx(): n="+n+" n_tests="+n_tests+" eps="+eps+" OK");
}


private static void test_EV_getGEV_dsygvx(final int n_tests,final int n,java.util.Random r,final double eps)
{
    final double [] A=new double [n*n],B=new double [n*n];
 for(int t=0;t<n_tests;t++)
 {
     final double scale;
     final int is=r.nextInt(10);
     if(is==0) scale=1e200;
     else if(is==1) scale=1e-200;
     else scale=1;

    for(int k=0;k<n;k++)
    {
	for(int l=0;l<=k;l++){
	    A[k*n+l]=A[l*n+l]=scale*(1-2*r.nextDouble());
	    B[k*n+l]=B[l*n+l]=scale*(r.nextDouble());
	}
    }
    final double [] ev=getGEV_dsygvx(n,A,n,B,n,true);
    final double [] lv=new double [n],mv=new double [n];
    final int o=n;
    for(int k=n;k<n;k++)
    {
	final double lambda=ev[k];
	for(int l=0;l<n;l++)
	{
	    double sa=0,sb=0;
	    for(int j=0;j<n;j++)
	    {
		sa+=A[l*n+j]*ev[o+k*n+j];
		sb+=B[l*n+j]*ev[o+k*n+j];
	    }
	    mv[l]=sa;
	    lv[l]=lambda*sb;
	}
	UnitTests.checkDoubleArrayRel(lv,"lv",mv,"mv",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrB(n,ev,o+k*n,ev,o+k*n,B,n),"<psi|B|psi>",1,"1",eps);
    }
 }
 System.err.println("Eigenvalues:test_EV_getGEV_dsygvx(): n="+n+" n_tests="+n_tests+" eps="+eps+" OK");
}


private static void test_getEV_dstebz(final int n_tests,final int nmax,java.util.Random r,final double eps)
{
    for(int t=0;t<n_tests;t++)
    {
	final int n=1+r.nextInt(nmax+1);
	final double [] d=new double [n],e=new double [n-1];
	for(int k=0;k<n;k++)
	{
	    d[k]=r.nextDouble();
	    if(k<n-1) e[k]=r.nextDouble();
	}
	final double [] res=getEV_dstebz(n,d,0,e,0,false);
	UnitTests.checkDoubleRel(TMatr.sumElements(res),"sum(res)",TMatr.sumElements(d),"sum(d)",eps);
	for(int k=0;k<n;k++)
	{
	    boolean flag=false;	    
	    final double [] dt=d.clone();
	    for(int j=0;j<n;j++)
	    {
		dt[j]-=res[k];
	    }
	    final double [] res1=getEV_dstebz(n,dt,0,e,0,false);
	    for(int j=0;j<n;j++)
	    {
		if(Math.abs(res1[j])<=eps)
		{
		    flag=true;
		}
	    }	    
	    //System.err.println("n="+n+"; res1="+java.util.Arrays.toString(res1));
	    if(!flag) throw new RuntimeException("n="+n+"; res1="+java.util.Arrays.toString(res1));
	}
	//System.err.println("n="+n+"; res1="+java.util.Arrays.toString(res1));
    }
    System.err.println("Eigenvalues:test_getEV_dstebz(): nmax="+nmax+" n_tests="+n_tests+" tests OK eps="+eps);
}



public static void main(String [] args)
{
    final double eps=1e-13;
    final java.util.Random r=new java.util.Random(1167);
    final int n_tests=1000;
    for(int n=1;n<50;n++)
    {
	test_EV_getEV_dsyevx(n_tests,n,r,eps);
	test_EV_getGEV_dsygvx(n_tests,n,r,eps);
	test_getEV_dstebz(n_tests,n,r,eps);
    }
}



}
