package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2008
 * This file is under GPL version 3.
 *
 */
import java.util.*;


/** Basic Encode/Decode utilities.
 *  @version $Id: EncodeDecode.java,v 1.18 2009/03/15 10:10:00 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class EncodeDecode {

    private static final String ENCODING="UTF-8";

    public static final String urlencode(String s)
	throws java.io.UnsupportedEncodingException
    {
	return java.net.URLEncoder.encode(s,ENCODING);
    }

    public static final String urldecode(String s)
	throws java.io.UnsupportedEncodingException
    {
	return java.net.URLDecoder.decode(s,ENCODING);
    }

	/**
	 * Converts a string into a htmlencoded string.
	 *
	 * @param val the <CODE>String</CODE> to be converted.
         * @param buf A buffer to use for storing converted string.
	 */
	public static final String encode(String val,StringBuilder buf) {
		if(val != null) {
                        buf.setLength(0);
			final int val_len=val.length();
			buf.ensureCapacity(2*val_len+1);
			char c;

			for(int i = 0; i<val_len; i++) {
				c = val.charAt(i);
				switch(c) {
				case '<':
					buf.append("&lt;");
					break;
				case '>':
					buf.append("&gt;");
					break;
				case '\"':
					buf.append("&quot;");
					break;
				case '\'':
					buf.append("&#39;");
					break;
				case '&':
					buf.append("&amp;");
					break;
				default:
				    if(c<' ')
					{
					    buf.append("&#");
					    buf.append((int)c);
					    buf.append(";");
					}
				    else
					{
					    buf.append(c);
					}
				}
			}
			val=null;
			return buf.toString();
		} else {
			return null;
		}
	}


    /**
     * Converts a htmlencoded string into a string.
     *
     * @param val the <CODE>String</CODE> to be converted.
     * @param buf A buffer to use for storing converted string.
     */
    public static final String decode(String val) {
      if(val!=null)
      {
	final int val_len=val.length();
	final StringBuilder buf=new StringBuilder(1+val_len);

	char c;
	for(int i = 0; i<val_len; i++) {
            c = val.charAt(i);
            if (c == '&') {
                final int semi = val.indexOf(';', i + 1);
                if (semi == -1) {
                    buf.append(c);
                    continue;
                }
                final String entityName = val.substring(i + 1, semi);
                int entityValue=-1;
		if (entityName.charAt(0) == '#') {
		    try {
			entityValue = Integer.parseInt(entityName.substring(1));
		    }catch(Exception e){}
		} else {
		    if(entityName.equalsIgnoreCase("lt"))
		    {
			entityValue=(int)'<';
		    }
		    else if(entityName.equalsIgnoreCase("gt"))
		    {
			entityValue=(int)'>';
		    }
		    else if(entityName.equalsIgnoreCase("quot"))
		    {
			entityValue=(int)'\"';
		    }
		    else if(entityName.equalsIgnoreCase("amp"))
		    {
			entityValue=(int)'&';
		    }
		}
                if (entityValue == -1) {
		    buf.append(c);
                    continue;
                } else {
                    buf.append((char)entityValue);
		    i=semi;
                }
            } else {
                buf.append(c);
            }
        }
        return buf.toString();
      }
      else
      {
	  return null;
      }
    }


  /** This method decodes a string of URL encoded string
   *  and returns a dictionary name - array of values.
   *  @param encodedString Encoded String.
   *  @return A Disctionary of pair name to array of values.
   */
    public static final Map<String,String []> decodeURLString(String encodedString)
	throws java.io.IOException
    {
      final HashMap<String,ArrayList<String>> t=new HashMap<String,ArrayList<String>>();
      if(encodedString==null)
      {
	  return new HashMap<String,String []>();
      }
      final int val_len=encodedString.length(),val_len_1=val_len-1;
      int prevPosWord=0,possepar=-1;
      for(int n=0;n<val_len;n++)
	{
	  final char c=encodedString.charAt(n);
	  if(c=='&' || n>=val_len_1)
	    {
	      final String name,value;
	      final int last_pos;
	      if(c=='&')
		  {
		      last_pos=n;
		  }
	      else
		  {
		      last_pos=n+1;
		      if(c=='=') possepar=n;
		  }
	      if(possepar<0)
		{
		  name=urldecode(encodedString.substring(prevPosWord,last_pos));
		  value="";
		}
	      else
		{
		  name=urldecode(encodedString.substring(prevPosWord,possepar));
		  value= possepar>=val_len_1 ? "" : urldecode(encodedString.substring(possepar+1,last_pos));
		}

	      ArrayList<String> v=t.get(name);
	      if(null==v)
	      { // new parameter
		  v=new ArrayList<String>();
		  t.put(name,v);
	      }
	      v.add(value);
	      prevPosWord=n+1;
	      possepar=-1;
	    }
	  else if(c=='=')
	    {
	      possepar=n;
	    }
	}

      encodedString=null; // to help garbage collector

      final Map<String,String []> t1=new HashMap<String,String []>(t.size()+1,1.0f);
      for(final Iterator<String> e=t.keySet().iterator();e.hasNext();)
      {
	  final String nm=e.next();
	  final ArrayList<String> v=t.get(nm);
	  final String [] varr=new String [v.size()];
	  v.toArray(varr);
	  t1.put(nm,varr);
      }
      return t1;
    }


    public static final Map<String,String> decodeURLStringSingleValue(String encodedString)
	throws java.io.IOException
    {
      final Map<String,String []> d=decodeURLString(encodedString);
      encodedString=null;
      final HashMap<String,String> t=new HashMap<String,String>(1+d.size(),1.0f);
      for(final Iterator<String> e=d.keySet().iterator();e.hasNext();)
      {
	  final String k=e.next();
	  final String [] v=d.get(k);
	  if(v!=null && v.length>=1)
	  {
	      t.put(k,v[0]);
	  }
      }
      return t;
    }


    public static String [] decodeStringArray(String encodedString)
	throws java.io.UnsupportedEncodingException
    {
	final int val_len=encodedString.length(),val_len_1=val_len-1;
	final ArrayList<String> v=new ArrayList<String>();
	int prevPosWord=0;
	for(int n=0;n<val_len;n++)
	{
	    final char c=encodedString.charAt(n);
	    if(c=='&' || n>=val_len_1)
	    {
		final int last_pos=c=='&'?n:n+1;
		v.add(urldecode(encodedString.substring(prevPosWord,last_pos)));
		prevPosWord=n+1;
	    }
	}
	final String [] r=new String [v.size()];
	v.toArray(r);
	return r;
    }

    public static int [] decodeIntegerArray(String ar)
	throws java.io.UnsupportedEncodingException
    {
	final String [] a=decodeStringArray(ar);
	final int [] r=new int [a.length];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=Integer.valueOf(a[i]).intValue();
	}
	return r;
    }

   public static double [] decodeDoubleArray(String ar)
	throws java.io.UnsupportedEncodingException
    {
	final String [] a=decodeStringArray(ar);
	final double [] r=new double [a.length];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=Double.valueOf(a[i]).doubleValue();
	}
	return r;
    }

    public static double [][] decodeDoubleArray2(String ar)
	throws java.io.UnsupportedEncodingException
    {
	final String [] a=decodeStringArray(ar);
	final double [][] r=new double [a.length][];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=decodeDoubleArray(a[i]);
	}
	return r;
    }


    public static String [][] decodeStringArray2(String encodedString)
	throws java.io.UnsupportedEncodingException
    {
	final String [] arr1=decodeStringArray(encodedString);
	final String [][] r=new String [arr1.length][];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=decodeStringArray(arr1[i]);
	}
	return r;
    }


    public static String [] getStringFromArray(int [] ar)
    {
	final String [] r=new String [ar.length];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=String.valueOf(ar[i]);
	}
	return r;
    }

    public static String [] getStringFromArray(double [] ar)
    {
	final String [] r=new String [ar.length];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=String.valueOf(ar[i]);
	}
	return r;
    }

    public static String [][] getStringFromArray(double [][] ar)
    {
	final String [][] r=new String [ar.length][];
	for(int i=0;i<r.length;i++)
	{
	    r[i]=getStringFromArray(ar[i]);
	}
	return r;
    }


    public static void appendField(StringBuilder buf,String fieldname,String fieldvalue)
	throws java.io.UnsupportedEncodingException
    {
	buf.append(EncodeDecode.urlencode(fieldname));
	buf.append("=");
	buf.append(EncodeDecode.urlencode(fieldvalue));
	buf.append("&");
    }

    public static void appendField(StringBuilder buf,String fieldname,String [] fieldvalues)
	throws java.io.UnsupportedEncodingException
    {
	buf.append(EncodeDecode.urlencode(fieldname));
	buf.append("=");
	final String separ=EncodeDecode.urlencode("&");
	for(int i=0;i<fieldvalues.length;i++)
	{
	    buf.append(EncodeDecode.urlencode(EncodeDecode.urlencode(fieldvalues[i])));
	    buf.append(separ);
	}
	buf.append("&");
    }

    public static void appendField(StringBuilder buf,String fieldname,String [][] fieldvalues)
	throws java.io.UnsupportedEncodingException
    {
	buf.append(EncodeDecode.urlencode(fieldname));
	buf.append("=");
	final String separ=EncodeDecode.urlencode("&");
	final String separ2=EncodeDecode.urlencode(separ);
	for(int i=0;i<fieldvalues.length;i++)
	{
	    final String [] s=fieldvalues[i];
	    for(int j=0;j<s.length;j++)
	    {
		buf.append(EncodeDecode.urlencode(EncodeDecode.urlencode(EncodeDecode.urlencode(s[j]))));
		buf.append(separ2);
	    }
	    buf.append(separ);
	}
	buf.append("&");
    }






    public static String skipchars(String s)
    {
	return s.replace(",","").replace(" ","").replace("\t","");
    }

    public static double getdouble(String s,boolean flagfail)
    {
	double d=Double.NaN;
	try {
	    if(s!=null)
	    {
		d=new Double(skipchars(s)).doubleValue();
		if(Double.isInfinite(d))
		{
		    d=Double.NaN;
		}
	    }
	}catch(NumberFormatException ee){}
	if(flagfail && Double.isNaN(d))
	{
	    throw new NumberFormatException("invalid double "+s);
	}
	return d;
    }

    public static long getlong(String s,long bydefault,boolean flagfail)
    {
	long d=bydefault;
	try {
	    d=new Long(skipchars(s)).longValue();
	}catch(NumberFormatException ee){
	    if(flagfail)
	    {
		throw new NumberFormatException("invalid long "+s);
	    }
	}
	return d;
    }


    public static int getint(String s,int bydefault,boolean flagfail)
    {
	int d=bydefault;
	try {
	    d=new Integer(skipchars(s)).intValue();
	}catch(NumberFormatException ee){
	    if(flagfail)
	    {
		throw new NumberFormatException("invalid int "+s);
	    }
	}
	return d;
    }


    public static long gettime(String s,java.text.DateFormat dateformat,long bydefault,boolean flagfail)
    {
	long t=bydefault;
	try {
	    t=com.polytechnik.utils.TimeUtils.getTime00(dateformat.parse(s).getTime());
	}catch(java.text.ParseException pe){
	    if(flagfail)
	    {
		throw new NumberFormatException("invalid date"+s+" format="+dateformat.toString());
	    }
	}
	return t;
    }



    public static String getstringbeginning(String s)
    {
	final int pspace=s.indexOf(" ");
	if(pspace>=0)
	{
	    return s.substring(0,pspace);
	}
	return s;
    }


    public static String getstringparametervalue(String paramname,String s)
    {
	if(s.startsWith(paramname))
	{
	    final String s1=s.substring(paramname.length()).trim();
	    final int peq=s1.indexOf("=");
	    if(peq==0)
	    {
		return s1.substring(peq+1).trim();
	    }
	}
	return null;
    }



    /**
     * This method parses a string with attributes.
     * It is based on Java HTML Generator v0.9 from
     * <A HREF="http://artho.com/webtools/java/">http://artho.com/webtools/java/</A>
     * file java/html/Attributes.java .
     * @param attributes  String with HTML attributes like:
     * border="0" cellpadding="0" cellspacing="0".
     * @return An array of (name, value) pairs for parsed attibutes.
     */

    public static final java.util.Map<String,String> parseAttributes(String attributes)
    {
	final java.util.Map<String,String> v=new java.util.HashMap<String,String>(10);
	final int START = 0;
	final int ERR = -1;
	final int ATTR = 2;
	final int VAL0 = 3;
	final int VAL1 = 4;
	final int VALUE = 5;
	final int QVAL = 6;

	final int len=attributes.length();
	// finite state machine for parsing
	int state = START;
	int i = 0,pos=-1;
	String name=null;

	while ((i <= len) && (state != ERR)) {
	    final char chr=i<len? attributes.charAt(i):' ';
	    switch(state) {
	    case START:
		switch(chr) {
		case ' ': break;
		case '"': case '=':
		    state = ERR; break;
		default: pos=i; state = ATTR;
		}
		break;
	    case ATTR:
		switch(chr) {
		case ' ': case '\t': case '\r': case '\n':
		    name=attributes.substring(pos,i);
		    pos=-1;
		    state = VAL0;
		    break;
		case '=':
		    name=attributes.substring(pos,i);
		    pos=-1;
		    state = VAL1;
		    break;
		case '"':
		    state = ERR; break;
		}
		break;
	    case VAL0:
	        switch(chr) {
	            case '=': state = VAL1; break;
	            case ' ': case '\t': case '\r': case '\n': break;
	            default:
			v.put(name,"");
			pos=i;
			name=null;
			state = ATTR;
	        }
		break;
	    case VAL1:
	        switch(chr) {
	        case '"': state = QVAL;
		    pos=i+1;
		    break;
	        case '=': state = ERR; break;
	        case ' ': case '\t': case '\r': case '\n': break;
	        default:
		    pos=i;
		    state = VALUE;
		    break;
		}
		break;
	    case VALUE:
		switch(chr) {
		case ' ': case '\t': case '\r': case '\n':
		    v.put(name,attributes.substring(pos,i));
		    pos=-1;
		    name=null;
		    state = START;
		    break;
		case '=': case '"': state = ERR; break;
		}
		break;
	    case QVAL:
		switch(chr) {
		case '"':
		    v.put(name,attributes.substring(pos,i));
		    pos=-1;
		    name=null;
		    state = START;
		    break;
		}
		break;
	    }
	    i++;
	}
	if ((i <= len) || (state == ERR))
	    {
		throw new IllegalArgumentException("Invalid argument "+
						   attributes+".");
	    }
	if(state!=START)
	    {
		if(name!=null && state == VAL0)
		    {
			v.put(name,"");
		    }
		else
		    {
			throw new IllegalArgumentException("Invalid argument "+
							   attributes+".");
		    }
	    }
	return v;
    }

}
