package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */

/** Golden section minimizer.
 *  @version $Id: GoldenMin.java,v 1.4 2016/03/26 09:15:14 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

public class GoldenMin  {

static final double gr = (Math.sqrt(5) - 1) / 2;
public static double findMin(final FunctionCalculateable fun,
			     double a,double b,final int N_iters)
{
    double c = b - gr * (b - a),fc=fun.getF(c),
	   d = a + gr * (b - a),fd=fun.getF(d);
    for(int it=0;it<N_iters;it++)
    {
	//System.err.println("it="+it+" fc="+fc+" fd="+fd);
        if(fc < fd){
            b = d;
            d = c;
	    fd=fc;
            c = b - gr * (b - a);
	    fc=fun.getF(c);
	}
        else
	{
            a = c;
	    c = d;
	    fc=fd;
            d = a + gr * (b - a);
	    fd=fun.getF(d);
	}
    }
    return (b + a) / 2;
}


static double findMin2(final FunctionCalculateable f,
		       double a,double b,final int N_iters)
{
    final double gratio = 2/(Math.sqrt(5) + 1);

    //### Use the golden ratio to set the initial test points
    double x1 = b - gratio*(b - a),x2 = a + gratio*(b - a),
	//### Evaluate the function at the test points
	f1 = f.getF(x1),f2 = f.getF(x2);


   for(int it=0;it<N_iters;it++)
   {
      //cat('', '\n')
      //cat('Iteration #', iteration, '\n')
      //cat('f1 =', f1, '\n')
      //cat('f2 =', f2, '\n')

       //System.err.println("it="+it+" f1="+f1+" f2="+f2);

      if (f2 > f1) {
	  //# then the minimum is to the left of x2
	  //# let x2 be the new upper bound
	  //# let x1 be the new upper test point
	  //cat('f2 > f1', '\n')
	  //### Set the new upper bound
	  b = x2;
	  //cat('New Upper Bound =', b, '\n')
	  //cat('New Lower Bound =', a, '\n')
	  //### Set the new upper test point
	  //### Use the special result of the golden ratio
	  x2 = x1;
	  //cat('New Upper Test Point = ', x2, '\n')
	  f2 = f1;

	  //### Set the new lower test point
	  x1 = b - gratio*(b - a);
	  //cat('New Lower Test Point = ', x1, '\n')
	  f1 = f.getF(x1);
      }
      else
      {
	  //cat('f2 < f1', '\n')
	  // # the minimum is to the right of x1
	  //# let x1 be the new lower bound
	  //# let x2 be the new lower test point

         //### Set the new lower bound
         a = x1;
         //cat('New Upper Bound =', b, '\n')
         //cat('New Lower Bound =', a, '\n')

         //### Set the new lower test point
         x1 = x2;
         //cat('New Lower Test Point = ', x1, '\n')

         f1 = f2;

         //### Set the new upper test point
         x2 = a + gratio*(b - a);
         //cat('New Upper Test Point = ', x2, '\n')
         f2 = f.getF(x2);
      }
   }

   //### Use the mid-point of the final interval as the estimate of the optimzer
   //cat('', '\n')
   //cat('Final Lower Bound =', a, '\n')
   //cat('Final Upper Bound =', b, '\n')
   return (a + b)/2;
   //cat('Estimated Minimizer =', estimated.minimizer, '\n')
}

static void test_min(final int ntests,final java.util.Random r,final double eps)
{
    for(int t=0;t<ntests;t++)
    {
	final double a=2*r.nextDouble()-1,b=2*r.nextDouble()-1;
	final int pow=2*(1+r.nextInt(3));
	final FunctionCalculateable fun=new FunctionCalculateable(){
		public double getF(final double x)
		{
		    return Math.pow(x-a,pow)+2*b*x;
		}
	};
	final int niter=2+r.nextInt(20);
	final double left=-(3+r.nextDouble()),right=(3+r.nextDouble());
	final double x1 = findMin(fun,left,right,niter),x2=findMin2(fun,left,right,niter);
	UnitTests.checkDoubleRel(x1,"x1",x2,"x2",eps);
	final double xx1 = findMin(fun,right,left,niter),xx2=findMin2(fun,right,left,niter);
	UnitTests.checkDoubleRel(xx1,"xx1",xx2,"xx2",eps);
    }
    System.err.println("GoldenMin:test_min: test OK, ntests="+ntests+" eps="+eps);
}


public static void main(String [] args)
{
    test_min(10000,new java.util.Random(23),1e-6);
}


}
