package com.polytechnik.utils;

/** Linear system solver.
 * copied from http://netlib.org/benchmark/linpackjava/Linpack.java
 * @version $Id: Linsystems.java,v 1.54 2015/10/24 22:22:50 mal Exp $
 */

public class Linsystems {



    /** Simple test. */
    public static void main(String [] args)
    {
	final long t0=System.currentTimeMillis();

	final java.util.Random r=new java.util.Random(-79090);
	test_linsys("Linsystems",35,1000000,r);

	int ntests=1;
	checkSystem(new double [][] {
		{1  , 5.5 , 6.7 ,8.0},
		{2.4, 5.5 , 4.7 ,2.0},
		{1.2, 1.5 , 3.6 ,1.7},
		{1.9, 0.5 , 1.9 ,6.2}
	    },
	    new double []{
		1.2 , 7.2 , 3.9 , 7.0
	    },
	    true);

	// a test with random matrix

	checkInverted(200000,r,1e-9);


	for(int i=0;i<2000000;i++,ntests++)
	{
	    final int n=1+r.nextInt(1+100);
	    final double [][] matr=new double [n][n];
	    final double [] rightpart=new double [n];
	    for(int i1=0;i1<n;i1++)
	    {
		rightpart[i1]=r.nextDouble();
		final double [] m1=matr[i1];
		for(int i2=0;i2<n;i2++)
		{
		    m1[i2]=r.nextDouble();
		}
	    }
	    checkSystem(matr,rightpart,i<20);
	}

	final long t1=System.currentTimeMillis();
	System.err.println("Linsystems:main() "+ntests+" tests done in "+(t1-t0)+" milliseconds ver=$Id: Linsystems.java,v 1.54 2015/10/24 22:22:50 mal Exp $.");
    }


    private static void checkSystem(final double [][] matr,
				    final double [] rightpart,
				    final boolean diag)
    {
	final double eps=1e-8;
	final int n=rightpart.length;
	if(diag)
	{
	    System.err.println("====================== Testing linear system N="+n);
	}


	final double a[][] = new double[n][n];
	final int nlda=n+3;
	final double [] a1d=new double [nlda*n];
	for(int i=0,o=0;i<n;o+=nlda,i++)
	{
	    System.arraycopy(matr[i],0,a[i],0,n);
	    System.arraycopy(matr[i],0,a1d,o,n);
	}


	final int [] ipvt = new int[n],ipvt1d = new int[n];
	final double [] b = new double[n],b1d=new double[n];


        /* Factor a.  */
	final int info = dgefa(a,n,ipvt);
	final int info1d = dgefa_m1(a1d,n,nlda,ipvt1d);
	checkMatrs(n,nlda,info,info1d,a,a1d,ipvt,ipvt1d);
	if(info!=0)
	{
	    throw new RuntimeException("INFO="+info);
	}


	if(diag)
	{
	    System.err.println("---solve with job=0");
	}
	/* Solve ax=b. */
	System.arraycopy(rightpart,0,b,0,n);
	System.arraycopy(rightpart,0,b1d,0,n);
	dgesl(a,n,ipvt,b,0);
	dgesl_m1(a1d,n,nlda,ipvt1d,b1d,0);
	checkMatrs2(n,b,b1d);

	for(int i=0;i<n;i++)
	{
	    double s=0;
	    for(int j=0;j<n;j++)
	    {
		s+=matr[j][i]*b[j];
	    }
	    s-=rightpart[i];
	    if(diag)
	    {
		System.err.println("x["+i+"]="+b[i]+" ERR="+s);
	    }
	    if(!(Math.abs(s)<=eps))
	    {
		throw new RuntimeException("Error s="+s+" for n="+n);
	    }
	}

	// try transponsed, acually not transponsed in java
	if(diag)
	{
	    System.err.println("---solve with job=1");
	}
	System.arraycopy(rightpart,0,b,0,n);
	System.arraycopy(rightpart,0,b1d,0,n);
	dgesl(a,n,ipvt,b,1);
	dgesl_m1(a1d,n,nlda,ipvt1d,b1d,1);
	checkMatrs2(n,b,b1d);

	for(int i=0;i<n;i++)
	{
	    double s=0;
	    for(int j=0;j<n;j++)
	    {
		s+=matr[i][j]*b[j];
	    }
	    s-=rightpart[i];
	    if(diag)
	    {
		System.err.println("x["+i+"]="+b[i]+" ERR="+s);
	    }
	    if(!(Math.abs(s)<=eps))
	    {
		throw new RuntimeException("Error s="+s+" for n="+n);
	    }
	}

	if(diag)
	{
	    System.err.println("====================== END Testing linear system N="+n);
	}
    }








  /*
    dgefa factors a double precision matrix by gaussian elimination.

    dgefa is usually called by dgeco, but it can be called
    directly with a saving in time if  rcond  is not needed.
    (time for dgeco) = (1 + 9/n)*(time for dgefa) .

    on entry

    a       double precision[n][n]
    the matrix to be factored.

    n       integer
    the order of the matrix  a .

    on return

    a       an upper triangular matrix and the multipliers
    which were used to obtain it.
    the factorization can be written  a = l*u  where
    l  is a product of permutation and unit lower
    triangular matrices and  u  is upper triangular.

    ipvt    integer[n]
    an integer vector of pivot indices.

    info    integer
    = 0  normal value.
    = k  if  u[k-1][k-1] .eq. 0.0 .  this is not an error
    condition for this subroutine, but it does
    indicate that dgesl or dgedi will divide by zero
    if called.  use  rcond  in dgeco for a reliable
    indication of singularity.

    linpack. this version dated 08/14/78.
    cleve moler, university of new mexico, argonne national lab.

    functions

    blas daxpy,dscal,idamax
  */

  public static int dgefa( double a[][],int n, int ipvt[])
  {
    if(n==0) return 0;
    double[] col_k, col_j;
    double t;
    int j,k,kp1,l,nm1;
    int info;

    // gaussian elimination with partial pivoting

    info = 0;
    nm1 = n - 1;
    if (nm1 >=  0) {
      for (k = 0; k < nm1; k++) {
	col_k = a[k];
	kp1 = k + 1;

	// find l = pivot index

	l = idamax(n-k,col_k,k,1) + k;
	ipvt[k] = l;

	// zero pivot implies this column already triangularized

	if (col_k[l] != 0) {

	  // interchange if necessary

	  if (l != k) {
	    t = col_k[l];
	    col_k[l] = col_k[k];
	    col_k[k] = t;
	  }

	  // compute multipliers

	  t = -1.0/col_k[k];
	  dscal(n-(kp1),t,col_k,kp1,1);

	  // row elimination with column indexing

	  for (j = kp1; j < n; j++) {
	    col_j = a[j];
	    t = col_j[l];
	    if (l != k) {
	      col_j[l] = col_j[k];
	      col_j[k] = t;
	    }
	    daxpy(n-(kp1),t,col_k,kp1,1,col_j,kp1,1);
	  }
	}
	else {
	  info = k+1;
	}
      }
    }

    ipvt[n-1] = n-1;
    if (a[(n-1)][(n-1)] == 0) info = n;

    return info;
  }



  /**
   * dgesl solves the double precision system
   * a * x = b  or  trans(a) * x = b
   * using the factors computed by dgeco or dgefa.
   *
   * on entry
   *
   * a       double precision[n][n]
   * the output from dgeco or dgefa.
   *
   *
   * n       integer
   * the order of the matrix  a .
   *
   * ipvt    integer[n]
   * the pivot vector from dgeco or dgefa.
   *
   * b       double precision[n]
   * the right hand side vector.
   *
   * job     integer
   * = 0         to solve  a*x = b ,
   * = nonzero   to solve  trans(a)*x = b  where
   * trans(a)  is the transpose.
   *
   * on return
   *
   * b       the solution vector  x .
   *
   * error condition
   *
   * a division by zero will occur if the input factor contains a
   * zero on the diagonal.  technically this indicates singularity
   * but it is often caused by improper arguments or improper
   * setting of lda .  it will not occur if the subroutines are
   * called correctly and if dgeco has set rcond .gt. 0.0
   * or dgefa has set info .eq. 0 .
   *
   * to compute  inverse(a) * c  where  c  is a matrix
   * with  p  columns
   * dgeco(a,lda,n,ipvt,rcond,z)
   * if (!rcond is too small){
   * for (j=0,j<p,j++)
   * dgesl(a,lda,n,ipvt,c[j][0],0);
   * }
   *
   * linpack. this version dated 08/14/78 .
   * cleve moler, university of new mexico, argonne national lab.
   *
   * functions
   *
   * blas daxpy,ddot


   *  here job=0 a[j][i]*r[j]=b[i]
   *  here job=1 a[i][j]*r[j]=b[i]
   **/

  public static void dgesl( double a[][],int n, int ipvt[], double b[], int job)
  {
    if(n==0) return;
    double t;
    int k,kb,l,nm1,kp1;

    nm1 = n - 1;
    if (job == 0) {

      // job = 0 , solve  a * x = b.  first solve  l*y = b

      if (nm1 >= 1) {
	for (k = 0; k < nm1; k++) {
	  l = ipvt[k];
	  t = b[l];
	  if (l != k){
	    b[l] = b[k];
	    b[k] = t;
	  }
	  kp1 = k + 1;
	  daxpy(n-(kp1),t,a[k],kp1,1,b,kp1,1);
	}
      }

      // now solve  u*x = y

      for (kb = 0; kb < n; kb++) {
	k = n - (kb + 1);
	b[k] /= a[k][k];
	t = -b[k];
	daxpy(k,t,a[k],0,1,b,0,1);
      }
    }
    else {

      // job = nonzero, solve  trans(a) * x = b.  first solve  trans(u)*y = b

      for (k = 0; k < n; k++) {
	t = ddot(k,a[k],0,1,b,0,1);
	b[k] = (b[k] - t)/a[k][k];
      }

      // now solve trans(l)*x = y

      if (nm1 >= 1) {
	for (kb = 0; kb < nm1; kb++) {
	  k = n - (kb+2);
	  kp1 = k + 1;
	  b[k] += ddot(n-(kp1),a[k],kp1,1,b,kp1,1);
	  l = ipvt[k];
	  if (l != k) {
	    t = b[l];
	    b[l] = b[k];
	    b[k] = t;
	  }
	}
      }
    }
  }



  /**
   * constant times a vector plus a vector.
   * jack dongarra, linpack, 3/11/78.
   **/

   private static void daxpy( int n, double da, double dx[], int dx_off, int incx,
			      double dy[], int dy_off, int incy)
  {
    int i,ix,iy;

    if ((n > 0) && (da != 0)) {
      if (incx != 1 || incy != 1) {

	// code for unequal increments or equal increments not equal to 1

	ix = 0;
	iy = 0;
	if (incx < 0) ix = (-n+1)*incx;
	if (incy < 0) iy = (-n+1)*incy;
	for (i = 0;i < n; i++) {
	  dy[iy +dy_off] += da*dx[ix +dx_off];
	  ix += incx;
	  iy += incy;
	}
	return;
      } else {

	// code for both increments equal to 1

	for (i=0; i < n; i++)
	  dy[i +dy_off] += da*dx[i +dx_off];
      }
    }
  }



  /**
   * forms the dot product of two vectors.
   * jack dongarra, linpack, 3/11/78.
   **/

    private static double ddot( int n, double dx[], int dx_off, int incx, double dy[],
				int dy_off, int incy)
  {
    double dtemp;
    int i,ix,iy;

    dtemp = 0;

    if (n > 0) {

      if (incx != 1 || incy != 1) {

	// code for unequal increments or equal increments not equal to 1

	ix = 0;
	iy = 0;
	if (incx < 0) ix = (-n+1)*incx;
	if (incy < 0) iy = (-n+1)*incy;
	for (i = 0;i < n; i++) {
	  dtemp += dx[ix +dx_off]*dy[iy +dy_off];
	  ix += incx;
	  iy += incy;
	}
      } else {

	// code for both increments equal to 1

	for (i=0;i < n; i++)
	  dtemp += dx[i +dx_off]*dy[i +dy_off];
      }
    }
    return(dtemp);
  }

  /**
   * scales a vector by a constant.
   * jack dongarra, linpack, 3/11/78.
   **/

  private static void dscal( int n, double da, double dx[], int dx_off, int incx)
  {
    int i,nincx;

    if (n > 0) {
      if (incx != 1) {

	// code for increment not equal to 1

	nincx = n*incx;
	for (i = 0; i < nincx; i += incx)
	  dx[i +dx_off] *= da;
      } else {

	// code for increment equal to 1

	for (i = 0; i < n; i++)
	  dx[i +dx_off] *= da;
      }
    }
  }

  /**
   * finds the index of element having max. absolute value.
   * jack dongarra, linpack, 3/11/78.
   **/

  private static int idamax( int n, double dx[], int dx_off, int incx)
  {
    double dmax, dtemp;
    int i, ix, itemp=0;

    if (n < 1) {
      itemp = -1;
    } else if (n ==1) {
      itemp = 0;
    } else if (incx != 1) {

      // code for increment not equal to 1

      dmax = (dx[dx_off] < 0.0) ? -dx[dx_off]: dx[dx_off];
      ix = 1 + incx;
      for (i = 0; i < n; i++) {
	dtemp = (dx[ix + dx_off] < 0.0) ? -dx[ix + dx_off]: dx[ix + dx_off];
	if (dtemp > dmax)  {
	  itemp = i;
	  dmax = dtemp;
	}
	ix += incx;
      }
    } else {

      // code for increment equal to 1

      itemp = 0;
      dmax = (dx[dx_off] < 0.0)? -dx[dx_off] : dx[dx_off];
      for (i = 0; i < n; i++) {
	dtemp = (dx[i + dx_off] < 0.0) ? -dx[i+dx_off]: dx[i+dx_off];
	if (dtemp > dmax) {
	  itemp = i;
	  dmax = dtemp;
	}
      }
    }
    return (itemp);
  }


    /** A crude way to invert a matrix.
     */
    public static double [][] getInvertedMatrix(final double [][] matr)
    {
	return getInvertedMatrix(matr.length,matr);
    }
    /** A crude way to invert a matrix.
     */
    public static double [][] getInvertedMatrix(final int n,final double [][] matr)
    {
	final double [] m=new double [n*n];
	for(int i=0,o=0;i<n;i++,o+=n)
	{
	    System.arraycopy(matr[i],0,m,o,n);
	}
	final int [] ipvt=new int [n];
	final int info=com.polytechnik.utils.Linsystems.dgefa_m1(m,n,n,ipvt);
	final double [][] minv=new double [n][n];
	if(info!=0)
	{
	    for(double [] l:minv) java.util.Arrays.fill(l,Double.NaN);
	    return minv;
	}
	for(int i=0;i<n;i++)
	{
	    minv[i][i]=1;
	    com.polytechnik.utils.Linsystems.dgesl_m1(m,n,n,ipvt,minv[i],0);
	}
	return minv;
    }
    /** A crude way to invert a matrix.
     *  Returned matrix has a leading dimension n.
     */
    public static double [] getInvertedMatrix(final int n,final int nlda,final double [] matr)
    {
	final double [] m=new double [nlda*n];
	System.arraycopy(matr,0,m,0,n*nlda);

	final int [] ipvt=new int [n];
	final int info=com.polytechnik.utils.Linsystems.dgefa_m1(m,n,nlda,ipvt);
	final double [] minv=new double [n*n];
	if(info!=0)
	{
	    java.util.Arrays.fill(minv,Double.NaN);
	    return minv;
	}
	for(int i=n;--i>=0;)
	{
	    minv[i]=1;
	    com.polytechnik.utils.Linsystems.dgesl_m1(m,n,nlda,ipvt,minv,0);
	    if(i>0)
	    for(int j=0,o=i*n;j<n;++j,++o)
	    {
		minv[o]=minv[j];
		minv[j]=0;
	    }
	}
	return minv;
    }




  /* The same as dgesl, but store matix in 1D array.
   *  here job=0 a[j*nlda+i]*r[j]=b[i]
   *  here job=1 a[i*nlda+j]*r[j]=b[i]
   */
  public static void dgesl_m1(final double a[],final int n,final int nlda,final int ipvt[],final double b[],final int job)
  {

    final int nm1 = n - 1;
    if (job == 0) {

      // job = 0 , solve  a * x = b.  first solve  l*y = b

      for (int k = 0, ok=0; k < nm1; ok+=nlda,k++) {
	final int l = ipvt[k];
	final double t = b[l];
	if (l != k){
	  b[l] = b[k];
	  b[k] = t;
	}
	final int kp1 = k + 1;
	daxpy(n-kp1,t,a,ok+kp1,1,b,kp1,1);
      }

      // now solve  u*x = y

      for (int kb = 0; kb < n; kb++) {
	final int k = n - (kb + 1), ok=k*nlda;
	b[k] /= a[ok+k];
	daxpy(k,-b[k],a,ok,1,b,0,1);
      }
    }
    else {

      // job = nonzero, solve  trans(a) * x = b.  first solve  trans(u)*y = b

      for (int k = 0, ok=0; k < n; ok+=nlda,k++) {
	b[k] = (b[k] - ddot(k,a,ok,1,b,0,1))/a[ok+k];
      }

      // now solve trans(l)*x = y

      for (int kb = 0; kb < nm1; kb++) {
	final int k = n - (kb+2), ok=k*nlda, kp1 = k + 1;
	b[k] += ddot(n-kp1,a,ok+kp1,1,b,kp1,1);
	final int l = ipvt[k];
	if (l != k) {
	  final double t = b[l];
	  b[l] = b[k];
	  b[k] = t;
	}
      }
    }
  }



  /** The same as dgefa, but store matrix in 1D array. */
  public static int dgefa_m1(final double a[],final int n,final int nlda,final int ipvt[])
  {
    int info=0;

    // gaussian elimination with partial pivoting

    final int nm1 = n - 1;
    for (int k = 0,ok=0; k < nm1; ok+=nlda,k++) {
      final int ok_k=ok+k,kp1 = k + 1,ok_kp1=ok+kp1;

      // find l = pivot index

      final int l = ipvt[k] = idamax(n-k,a,ok_k,1) + k,ok_l=ok+l;

      // zero pivot implies this column already triangularized
      final double a_kl=a[ok_l];
      if (a_kl != 0) {

        // interchange if necessary

        if (l != k) {
	  a[ok_l] = a[ok_k];
	  a[ok_k]=a_kl;
        }

        // compute multipliers

        dscal(n-kp1,-1.0/a[ok_k],a,ok_kp1,1);

        // row elimination with column indexing

        for (int j = kp1,oj=kp1*nlda; j < n; oj+=nlda,j++) {
	  final double t = a[oj+l];
	  if (l != k) {
	    a[oj+l] = a[oj+k];
	    a[oj+k] = t;
	  }
	  daxpy(n-kp1,t,a,ok_kp1,1,a,oj+kp1,1);
	}
      } else {
	info = k+1;
      }
    }

    if(n>0) {
      ipvt[n-1] = n-1;
      if (!(a[(n-1)*(nlda+1)] != 0)) info = n;
    }

    return info;
  }

private static void checkMatrs2(final int n,final double [] r,final double [] r1d)
{
    for(int i=0;i<n;i++)
    {
	if(r[i]!=r1d[i])
	{
	    throw new RuntimeException("r["+i+"]="+r[i]+" r1d["+i+"]="+r1d[i]);
	}
    }
}
private static void checkMatrs(final int n,final int nlda,
			       final int info,final int info1d,
			       final double [][] a,final double [] a1d,
			       final int [] ipvt,final int [] ipvt1d)
{
    if(info!=info1d)
    {
	throw new RuntimeException("info="+info+" info1d="+info1d);
    }
    for(int i=0;i<n;i++)
    {
	if(ipvt[i]!=ipvt1d[i])
	{
	    throw new RuntimeException("ipvt["+i+"]="+ipvt[i]+" ipvt1d["+i+"]="+ipvt1d[i]);
	}
    }
    for(int i=0;i<n;i++)
    {
	for(int j=0;j<n;j++)
	{
	    if(a[i][j]!=a1d[i*nlda+j])
	    {
		throw new RuntimeException("a["+i+"]["+j+"]="+a[i][j]+
					   " a1d["+(i*nlda+j)+"]="+a1d[i*nlda+j]);
	    }
	}
    }
}

private static void checkInverted(final int n_tests,final java.util.Random r,final double eps)
{
    final int nmax=20;
    for(int t=0;t<n_tests;t++)
    {
	final int n=r.nextInt(nmax),nlda=n+r.nextInt(5);
	final double [] m0=new double [n*nlda];
	final double [][] mm0=new double [n][n];
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++) m0[k*nlda+l]=mm0[k][l]=r.nextDouble();
	    m0[k*nlda+k]=mm0[k][k]=(2+mm0[k][k]);
	}
	final double [] m0_inv=getInvertedMatrix(n,nlda,m0);
	final double [][] mm0_inv=getInvertedMatrix(n,mm0);
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		if(!(m0_inv[k*n+l]==mm0_inv[k][l]))
		{
		    throw new RuntimeException("mismatch: k="+k+" l="+l+
					       " array="+java.util.Arrays.toString(m0_inv));
		}
	    }
	}
	for(int k=0;k<n;k++)
	{
	    for(int l=0;l<n;l++)
	    {
		double sm=0,smm=0;
		for(int m=0;m<n;m++)
		{
		    sm+=m0[k*nlda+m]*m0_inv[m*n+l];
		    smm+=mm0[k][m]*mm0_inv[m][l];
		}
		UnitTests.checkDoubleRel(sm,"sm",smm,"smm",0);
		UnitTests.checkDoubleRel(sm,"sm",k==l?1:0,k==l?"1":"0",eps);
	    }
	}
    }
    System.err.println("checkInverted: OK n_tests="+n_tests+" eps="+eps);
}




    static void test_linsys(final String txt,final int n,final int ntests,final java.util.Random r)
    {
	final double eps=1e-8;
	final double [] matr=new double [n*n],smatr=new double [n*n],rd=new double [n],rt=new double [n],rs=new double [n];
	final int [] ipvt=new int [n];
	int nd=0,nt=0;
	for(int t=0;t<ntests;t++)
	{
	    for(int k=0;k<n;k++)
	    {
		for(int l=0;l<=k;l++)
		{
		    matr[k+l*n]=matr[l+k*n]=1-2*r.nextDouble();
		}
		rs[k]=rd[k]=rt[k]=1-2*r.nextDouble();
	    }
	    System.arraycopy(matr,0,smatr,0,matr.length);
	    final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr,n,n,ipvt);
	    if(pinfo!=0) throw new RuntimeException("pinfo="+pinfo);
	    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,rd,0);
	    com.polytechnik.utils.Linsystems.dgesl_m1(matr,n,n,ipvt,rt,1);
	    /*System.err.println("smatr="+java.util.Arrays.toString(smatr)+
			       "\nrs="+java.util.Arrays.toString(rs)+
			       "\n d="+java.util.Arrays.toString(rd)+
			       "\n t="+java.util.Arrays.toString(rt));*/
	    double sd=0,st=0;
	    for(int k=0;k<n;k++)
	    {
		double rsd=0,rst=0;
		for(int l=0;l<n;l++)
		{
		    rsd+=smatr[k*n+l]*rd[l];
		    rst+=smatr[k*n+l]*rt[l];
		}
		rsd-=rs[k];
		rst-=rs[k];
		UnitTests.checkDoubleRel(rsd,"rsd",0,"0",eps);
		UnitTests.checkDoubleRel(rst,"rst",0,"0",eps);
		sd+=Math.abs(rsd);
		st+=Math.abs(rst);
	    }
	    if(sd<st)
	    {
		nd++;
	    }
	    else
	    {
		nt++;
	    }
	}
	System.err.println("test_linsys:n="+n+" nd="+nd+" nt="+nt+" ntests="+ntests);
    }




}
