package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */

/**
 *  Calculates f/kt , (f/kt)' , (f/kt)''.
 *  @version $Id: PolynomialsRatio.java,v 1.2 2015/05/08 22:59:16 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PolynomialsRatio implements DerivativesCalculateable {
    final OrthogonalPolynomialsABasis M;
    final double [] f,kt;
    public PolynomialsRatio(final double [] kt,final double [] f,final OrthogonalPolynomialsABasis M)
    {
	this.M=M;
	this.kt=kt;
	this.f=f;
    }
    public double getF(final double x)
    {
	return (M.fx(f,x)/M.fx(kt,x));
    }
    public double getDF(final double x)
    {
	final double Ktx=M.fx(kt,x);
	return (M.Dfx(f,x)/Ktx-(M.fx(f,x)/Ktx)*(M.Dfx(kt,x)/Ktx));
    }
    public double getD2F(final double x)
    {
	final double Ktx=M.fx(kt,x),Kfx=M.fx(f,x),
	    DKtx=M.Dfx(kt,x),DKfx=M.Dfx(f,x),
	    D2Ktx=M.D2fx(kt,x),D2Kfx=M.D2fx(f,x),rt=DKtx/Ktx;
	final double t=(D2Kfx/Ktx)-2*(DKfx/Ktx)*rt-(Kfx/Ktx)*(D2Ktx/Ktx-2*rt*rt);
	return t;
    }
}
