package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */

/**
 *  Calculates sqrt(SNOM)/DENOM , (sqrt(SNOM)/DENOM)' , (sqrt(SNOM)/DENOM)''.
 *  @version $Id: PolynomialsSQDRatio.java,v 1.1 2017/04/04 22:46:15 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PolynomialsSQDRatio implements DerivativesCalculateable {
    final OrthogonalPolynomialsABasis M;
    final double [] SNOM,DENOM;
    public PolynomialsSQDRatio(final double [] DENOM,final double [] SNOM,final OrthogonalPolynomialsABasis M)
    {
	this.M=M;
	this.DENOM=DENOM;
	this.SNOM=SNOM;
    }
    public double getF(final double x)
    {
	return Math.sqrt(M.fx(SNOM,x))/M.fx(DENOM,x);
    }
    public double getDF(final double x)
    {
	final double a=M.fx(SNOM,x),
	    sqa=Math.sqrt(a),
	    da=M.Dfx(SNOM,x),
	    b=M.fx(DENOM,x),
	    db=M.Dfx(DENOM,x),
	    db_divb=db/b,
	    F=(sqa/b);
	return (0.5*da/sqa)/b-F*db_divb;
    }
    public double getD2F(final double x)
    {
	final double a=M.fx(SNOM,x),
	    sqa=Math.sqrt(a),
	    da=M.Dfx(SNOM,x),
	    d2a=M.D2fx(SNOM,x),
	    b=M.fx(DENOM,x),
	    db=M.Dfx(DENOM,x),
	    d2b=M.D2fx(DENOM,x),
	    db_divb=db/b,
	    da_diva=da/a,
	    F=(sqa/b),
	    DF=(0.5*da/sqa)/b-F*db_divb;
	return DF*(0.5*da_diva-db_divb)
	    +F*(0.5*(d2a/a-Math.pow(da_diva,2))-(d2b/b-Math.pow(db_divb,2)));
    }
}
