package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2015
 * This file is under GPL version 3.
 *
 */

/**
 *  Calculates NOM/sqrt(SDENOM) , (NOM/sqrt(SDENOM))' , (NOM/sqrt(SDENOM))''.
 *  @version $Id: PolynomialsSQRatio.java,v 1.3 2017/04/03 19:28:46 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class PolynomialsSQRatio implements DerivativesCalculateable {
    final OrthogonalPolynomialsABasis M;
    final double [] SDENOM,NOM;
    public PolynomialsSQRatio(final double [] SDENOM,final double [] NOM,final OrthogonalPolynomialsABasis M)
    {
	this.M=M;
	this.SDENOM=SDENOM;
	this.NOM=NOM;
    }
    public double getF(final double x)
    {
	return (M.fx(NOM,x)/Math.sqrt(M.fx(SDENOM,x)));
    }
    public double getDF(final double x)
    {
	final double a=M.fx(NOM,x),
	    da=M.Dfx(NOM,x),
	    b=M.fx(SDENOM,x),
	    sqb=Math.sqrt(b),
	    db=M.Dfx(SDENOM,x),
	    db_divb=db/b,
	    F=(a/sqb);
	return da/sqb-0.5*F*db_divb;
    }
    public double getD2F(final double x)
    {
	final double a=M.fx(NOM,x),
	    da=M.Dfx(NOM,x),
	    d2a=M.D2fx(NOM,x),
	    b=M.fx(SDENOM,x),
	    sqb=Math.sqrt(b),
	    db=M.Dfx(SDENOM,x),
	    d2b=M.D2fx(SDENOM,x),
	    db_divb=db/b,
	    F=(a/sqb),
	    DF=da/sqb-0.5*F*db_divb;
	return d2a/sqb-(2*0.5)*db_divb*DF
	    -0.5*F*(d2b/b-0.5*Math.pow(db_divb,2));
    }
}
