package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Sort arrays simple utils using insertion sort.
 *  Use this instead of java.util.Arrays for exactly specified behavior.
 *  @version $Id: SortArrays.java,v 1.21 2013/11/03 19:05:58 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class SortArrays {

public static void sortArrayWithLoad(final int n,final double [] tosort,final double [] load)
    {sortArrayWithLoads(n,tosort,load,null,true);}
public static void sortArrayWith2Load(final int n,final double [] tosort,final double [] load1,final double [] load2)
    {sortArrayWithLoads(n,tosort,load1,load2,true);}
public static void sortArrayWithLoadDesc(final int n,final double [] tosort,final double [] load)
    {sortArrayWithLoads(n,tosort,load,null,false);}
public static void sortArrayWith2LoadDesc(final int n,final double [] tosort,final double [] load1,final double [] load2)
    {sortArrayWithLoads(n,tosort,load1,load2,false);}


static void sortArrayWithLoads(final int n,final double [] tosort,final double [] load1,final double [] load2,final boolean flag_asc)
{
    // insertion sort by tosort with extra load
    for(int i=0;i<n;i++)
    {
	for(int j=i;j>0 && (flag_asc?tosort[j-1]>tosort[j]:tosort[j-1]<tosort[j]) ; --j)
	{
	    final double val=tosort[j-1];
	    tosort[j-1]=tosort[j];
	    tosort[j]=val;
	    if(load1!=null)
	    { // load1
		final double val2=load1[j-1];
		load1[j-1]=load1[j];
		load1[j]=val2;
	    }
	    if(load2!=null)
	    { // load2
		final double val2=load2[j-1];
		load2[j-1]=load2[j];
		load2[j]=val2;
	    }
	}
    }
}



/** The same as
sortArrayIndexes(compare_values.length,new IntPTRCompareDouble(compare_values,false)).
*/
public static int [] sortArrayIndexes(final double [] compare_values)
{
    final int n=compare_values.length;
    final int [] tosort=new int [n];
    // insertion sort of tosort indexes using compare_values as levels to compare
    for(int i=0;i<n;i++)
    {
	tosort[i]=i;
	for(int j=i;j>0 && compare_values[tosort[j-1]]>compare_values[tosort[j]] ; j--)
	{
	    final int v=tosort[j-1];
	    tosort[j-1]=tosort[j];
	    tosort[j]=v;
	}
    }
    return tosort;
}

/** Sort Array indexes.
 * @param n The number of elements to sort.
 * @param c Comparator.
 * @return index array, the (l,m) elements of which are sorted according to c.compare(index[l],index[m]).
 */
public static int [] sortArrayIndexes(final int n,final IntPTRCompare c)
{
    final int [] tosort=new int [n];
    // insertion sort of tosort indexes using a comparator.
    for(int i=0;i<n;i++)
    {
	for(int j=tosort[i]=i;j>0 && c.compare(tosort[j-1],tosort[j])>0 ; j--)
	{
	    final int v=tosort[j-1];
	    tosort[j-1]=tosort[j];
	    tosort[j]=v;
	}
    }
    return tosort;
}


static void test_ordered(final int n,final double [] d,final boolean flag_asc)
{
    double prev=flag_asc?Double.NEGATIVE_INFINITY:Double.POSITIVE_INFINITY;
    for(int k=0;k<n;k++)
    {
	final double dc=d[k];
	if(flag_asc?dc<prev:dc>prev) throw new RuntimeException();
	prev=dc;
    }
}
private static void test_load_index(final int n,final double [] d0,final double [] d_sorted,final double [] index)
{
    final int [] int_index=new int [n];
    for(int k=0;k<n;k++)
    {
	int_index[k]=(int)index[k];
    }
    test_load_index(n,d0,d_sorted,int_index);
}
static void test_load_index(final int n,final double [] d0,final double [] d_sorted,final int [] index)
{
  for(int k=0;k<n;k++)
  {
      final int in=index[k];
      if(!(d_sorted[k]==d0[in]))
	  throw new RuntimeException("d_sorted="+java.util.Arrays.toString(d_sorted)+"\n"+
				     "d0="+java.util.Arrays.toString(d0)+"\n"+
				     "index="+java.util.Arrays.toString(index));
  }
}

private static void test_sortArray(final java.util.Random r,final double eps,final int n_tests)
{
  for(int n=0;n<100;n++)
  for(int t=0;t<n_tests;t++)
  {
      final double [] d=new double [n],l1=new double [n],l2=new double [n];
      for(int k=0;k<n;k++)
      {
	  d[k]=r.nextInt(50);
	  l1[k]=k;
	  l2[k]=100+r.nextInt(50);
      }
      {
	  final double [] dd=d.clone(),ll1=l1.clone();
	  sortArrayWithLoad(n,dd,ll1);
	  test_ordered(n,dd,true);
	  test_load_index(n,d,dd,ll1);
      }
      {
	  final double [] dd=d.clone(),ll1=l1.clone(),ll2=l2.clone();
	  sortArrayWith2Load(n,dd,ll1,ll2);
	  test_ordered(n,dd,true);
	  test_load_index(n,d,dd,ll1);
	  test_load_index(n,l2,ll2,ll1);
      }
      {
	  final double [] dd=d.clone(),ll1=l1.clone();
	  sortArrayWithLoadDesc(n,dd,ll1);
	  test_ordered(n,dd,false);
	  test_load_index(n,d,dd,ll1);
      }
      {
	  final double [] dd=d.clone(),ll1=l1.clone(),ll2=l2.clone();
	  sortArrayWith2LoadDesc(n,dd,ll1,ll2);
	  test_ordered(n,dd,false);
	  test_load_index(n,d,dd,ll1);
	  test_load_index(n,l2,ll2,ll1);
      }
  }
  System.err.println("SortArraystest_sortArray: OK with n_tests="+n_tests+" eps="+eps);
}

private static void test_sortArrayIndexes(final java.util.Random r,final double eps,final int n_tests)
{
    for(int n=0;n<100;n++)
    for(int t=0;t<n_tests;t++)
    {
	final double [] d=new double [n],ll=new double [n];
	for(int k=0;k<n;k++)
	{
	    d[k]=100+r.nextInt(50);
	    ll[k]=k;
	}
	final IntPTRCompare c=new IntPTRCompare(){
		public int compare(int p1,int p2)
		{
		    if(d[p1]<d[p2]) return -1;
		    if(d[p1]>d[p2]) return 1;
		    return 0;
		}
	    };
	final double [] dx=d.clone();
	SortArrays.sortArrayWithLoad(n,dx,ll);
	final int [] i=SortArrays.sortArrayIndexes(d);
	final int [] j=SortArrays.sortArrayIndexes(n,c);
	final int [] jj=SortArrays.sortArrayIndexes(d.length,new IntPTRCompareDouble(d,false));
	final int [] jjr=SortArrays.sortArrayIndexes(d.length,new IntPTRCompareDouble(d,true));
	double prev=Double.NEGATIVE_INFINITY;
	final double [] dd=new double [n],ddr=new double [n];
	for(int k=0;k<n;k++)
	{
	    final double dc=d[i[k]];
	    if(dx[k]!=d[i[k]] || ll[k]!=i[k]) throw new RuntimeException();
	    dd[k]=dc;
	    ddr[k]=d[jjr[k]];
	    if(i[k]!=j[k]||i[k]!=jj[k]) throw new RuntimeException("i="+java.util.Arrays.toString(i)+"\nj="+java.util.Arrays.toString(j)+"\nd="+java.util.Arrays.toString(d)+"\njj="+java.util.Arrays.toString(jj));
	    if(false) System.err.println("n="+n+" d[i["+k+"]]="+dc+" i[k]="+i[k]);
	    if(dc<prev) throw new RuntimeException();
	    //if(dc==prev) System.err.println("test="+t+" n="+n+"\tk="+k+" dc="+dc+" prev="+prev);
	    prev=dc;
	}
	test_ordered(n,dd,true);
	test_load_index(n,d,dd,i);
	test_ordered(n,ddr,false);
	test_load_index(n,d,ddr,jjr);
    }
    System.err.println("SortArraystest_sortArrayIndexes: OK with n_tests="+n_tests+" eps="+eps);
}


public static void main(String [] args)
{
    final java.util.Random r=new java.util.Random(662);
    final double eps=1e-7;
    final int n_tests=100000;

    System.err.println("SortArrays:main(): starting ALL tests; ver=$Id: SortArrays.java,v 1.21 2013/11/03 19:05:58 mal Exp $");
    final long t0=System.currentTimeMillis();
    test_sortArrayIndexes(r,eps,n_tests);
    test_sortArray(r,eps,n_tests);
    final long t1=System.currentTimeMillis();
    System.err.println("SortArrays:main(): ALL tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: SortArrays.java,v 1.21 2013/11/03 19:05:58 mal Exp $");
}

}
