package com.polytechnik.utils;
/*
 * @(#)QSortAlgorithm.java	1.3   29 Feb 1996 James Gosling
 *
 * Copyright (c) 1994-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL or COMMERCIAL purposes and
 * without fee is hereby granted.
 * Please refer to the file http://java.sun.com/copy_trademarks.html
 * for further important copyright and trademark information and to
 * http://java.sun.com/licensing.html for further important
 * licensing information for the Java (tm) Technology.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  SUN
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

/**
 * A quick sort demonstration algorithm
 * SortAlgorithm.java
 *
 * @author James Gosling
 * @author Kevin A. Smith
 * @version 	@(#)QSortAlgorithm.java	1.3, 29 Feb 1996
 */
public abstract class TMPQSortAlgorithm
{
    private TMPQSortAlgorithm()
    {
    }

   /** This is a generic version of C.A.R Hoare's Quick Sort
    * algorithm.  This will handle arrays that are already
    * sorted, and arrays with duplicate keys.<BR>
    *
    * If you think of a one dimensional array as going from
    * the lowest index on the left to the highest index on the right
    * then the parameters to this function are lowest index or
    * left and highest index or right.  The first time you call
    * this function it will be with the parameters 0, a.length - 1.
    *
    * @param a       an integer array
    * @param lo0     left boundary of array partition
    * @param hi0     right boundary of array partition
    */

   public static void doubleQuickSort(double[] a, int lo0, int hi0)
   {
      int lo = lo0;
      int hi = hi0;
      double mid,t;

      // pause for redraw
      // pause(lo, hi);
      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
            while( ( lo < hi0 ) && ( a[lo] < mid ) )
               ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
            while( ( hi > lo0 ) && ( a[hi] > mid ) )
               --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               t=a[lo];
               a[lo]=a[hi];
               a[hi]=t;
               // pause
               // pause();

               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            doubleQuickSort( a, lo0, hi );

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            doubleQuickSort( a, lo, hi0 );

      }
   }


   public static void doubleQuickSortLongLoad(final double[] a,
					      final int lo0,
					      final int hi0,
					      final boolean flag_descending,
					      final long [] load)
   {
      int lo = lo0;
      int hi = hi0;
      double mid,t;
      long il;

      // pause for redraw
      // pause(lo, hi);
      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
	     while( ( lo < hi0 ) && (flag_descending?a[lo] > mid : a[lo] < mid ) )
               ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
	     while( ( hi > lo0 ) && (flag_descending?a[hi] < mid : a[hi] > mid ) )
               --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               t=a[lo];
               a[lo]=a[hi];
               a[hi]=t;
	       il=load[lo];
	       load[lo]=load[hi];
	       load[hi]=il;
               // pause
               // pause();

               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
	     doubleQuickSortLongLoad( a, lo0, hi, flag_descending, load);

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            doubleQuickSortLongLoad( a, lo, hi0, flag_descending, load);

      }
   }



   public static void longQuickSort(long[] a, int lo0, int hi0)
   {
      int lo = lo0;
      int hi = hi0;
      long mid,t;

      // pause for redraw
      // pause(lo, hi);
      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
            while( ( lo < hi0 ) && ( a[lo] < mid ) )
               ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
            while( ( hi > lo0 ) && ( a[hi] > mid ) )
               --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               t=a[lo];
               a[lo]=a[hi];
               a[hi]=t;
               // pause
               // pause();

               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            longQuickSort( a, lo0, hi );

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            longQuickSort( a, lo, hi0 );

      }
   }


   public static void intQuickSort(int[] a, int lo0, int hi0)
   {
      int lo = lo0;
      int hi = hi0;
      int mid,t;

      // pause for redraw
      // pause(lo, hi);
      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
            while( ( lo < hi0 ) && ( a[lo] < mid ) )
               ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
            while( ( hi > lo0 ) && ( a[hi] > mid ) )
               --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               t=a[lo];
               a[lo]=a[hi];
               a[hi]=t;
               // pause
               // pause();

               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            intQuickSort( a, lo0, hi );

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            intQuickSort( a, lo, hi0 );

      }
   }


public static int [] sortArrayIndexes(final int n,final  IntPTRCompare c)
{
    final int [] a=new int [n];
    for(int k=0;k<a.length;k++) a[k]=k;
    intQuickSortPointers(a,0,a.length-1,c);
    return a;
}
private static void intQuickSortPointers(int[] a, int lo0, int hi0, IntPTRCompare c)
   {
      int lo = lo0;
      int hi = hi0;
      int mid,t;

      // pause for redraw
      // pause(lo, hi);
      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
            while( ( lo < hi0 ) && c.compare( a[lo] , mid ) < 0 )
               ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
            while( ( hi > lo0 ) && c.compare( a[hi] , mid ) > 0 )
               --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               t=a[lo];
               a[lo]=a[hi];
               a[hi]=t;
               // pause
               // pause();

               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            intQuickSortPointers( a, lo0, hi ,c);

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            intQuickSortPointers( a, lo, hi0 ,c);

      }
   }




public static  <T> void objectQuickSortPointers(T[] a, int lo0, int hi0, java.util.Comparator<T> c)
   {
      int lo = lo0;
      int hi = hi0;
      T mid,t;

      // pause for redraw
      // pause(lo, hi);
      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
            while( ( lo < hi0 ) && c.compare( a[lo] , mid ) < 0 )
               ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
            while( ( hi > lo0 ) && c.compare( a[hi] , mid ) > 0 )
               --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               t=a[lo];
               a[lo]=a[hi];
               a[hi]=t;
               // pause
               // pause();

               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            objectQuickSortPointers( a, lo0, hi ,c);

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            objectQuickSortPointers( a, lo, hi0 ,c);

      }
   }



   public static void sort(double[] a)
   {
      doubleQuickSort(a, 0, a.length - 1);
   }




private static void test_sortArrayIndexes(final java.util.Random r,final double eps,final int n_tests)
{
    for(int n=0;n<100;n++)
    for(int t=0;t<n_tests;t++)
    {
	final double [] d=new double [n];
	for(int k=0;k<n;k++)
	{
	    d[k]=100+r.nextInt(50);
	}

	final int [] jqs=TMPQSortAlgorithm.sortArrayIndexes(n,new IntPTRCompareDouble(d,false)),
	    jrqs=TMPQSortAlgorithm.sortArrayIndexes(d.length,new IntPTRCompareDouble(d,true));
	final double [] ddqs=new double [n],ddrqs=new double [n];
	for(int k=0;k<n;k++)
	{
	    ddqs[k]=d[jqs[k]];
	    ddrqs[k]=d[jrqs[k]];
	}
	SortArrays.test_ordered(n,ddqs,true);
	SortArrays.test_load_index(n,d,ddqs,jqs);
	SortArrays.test_ordered(n,ddrqs,false);
	SortArrays.test_load_index(n,d,ddrqs,jrqs);
    }
    System.err.println("TMPQSortAlgorithm:test_sortArrayIndexes: OK with n_tests="+n_tests+" eps="+eps);
}

public static void main(String [] args)
{
    final java.util.Random r=new java.util.Random(662);
    final double eps=1e-7;
    final int n_tests=100000;

    System.err.println("TMPQSortAlgorithm:main(): starting ALL tests; ver=$Id: TMPQSortAlgorithm.java,v 1.14 2013/11/03 11:32:07 mal Exp $");
    final long t0=System.currentTimeMillis();
    test_sortArrayIndexes(r,eps,n_tests);
    final long t1=System.currentTimeMillis();
    System.err.println("TMPQSortAlgorithm:main(): ALL tests OK done in "+(t1-t0)+" milliseconds. ver=$Id: TMPQSortAlgorithm.java,v 1.14 2013/11/03 11:32:07 mal Exp $");
}

}
