package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Basis of orthogonal polynomials.
 *  @version $Id: TestBasisPolynomials.java,v 1.37 2017/09/08 10:13:21 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public abstract class TestBasisPolynomials {

abstract double [] getMonomialsMoments(final int n);
abstract OrthogonalPolynomialsABasis getOrtPol();
abstract double [][] getExact();

final BasisPolynomials B;
final java.util.Random r;
final double eps;
int nmax;
final String txt;

TestBasisPolynomials(final String txt,final BasisPolynomials B,final java.util.Random r,final double eps)
{
    this.txt=txt;
    this.r=r;
    this.eps=eps;
    this.B=B;
}

void test_ExactValues(final double eps)
{
    final double [][] P=getExact();
    if(P==null) return;
    double [] x=null,y=null;
    for(int k=0;k<P.length;k++)
    {
	final double [] res=new double [k+1],p=P[k];
	B.setNext(k,res,x,y);
	UnitTests.checkDoubleArrayRel(res,"res",p,"p",eps);
	y=x;
	x=res;
    }
    System.err.println(txt+":test_ExactValues: test OK with nmax="+P.length+" eps="+eps);
}


void test_setQlQmExpansionCoefs(final int n_tests)
{
    final double [][] P=new double [2*nmax+1][];
    for(int k=0;k<P.length;k++)
    {
	final double [] pk=P[k]=new double [k+1];
	B.setNext(k,pk,k<1?null:P[k-1],k<2?null:P[k-2]);
    }

    for(int t=0;t<n_tests;t++)
    {

    for(int l=0;l<=nmax;l++)
    for(int m=0;m<=nmax;m++)
    {
	final double [] res=new double [l+m+1];
	for(int k=0;k<res.length;k++){res[k]=r.nextDouble();}
	B.setQlQmExpansionCoefs(l,m,res);
	final double [] plpm=Polynomials.getMult2Pol(P[l],P[m]);
	final double [] Pplpm=B.convertToBasisFromPolynomial_Test(plpm.length-1,plpm);
	UnitTests.checkDoubleArrayRel(Pplpm,"direct TlTm conv",res,"setQlQmExpansionCoefs",eps);
    }

    }

    System.err.println(txt+":test_setQlQmExpansionCoefs(): test OK with nmax="+nmax+" eps="+eps);
}



void printPolynomials(final int nmax)
{
    System.err.println("========="+txt+": polynomials");
    double [] rn1=null,rn2=null;
    double [] ra=new double [nmax+1],rb=new double [nmax+1];
    java.util.Arrays.fill(ra,Double.NaN);
    java.util.Arrays.fill(rb,Double.NaN);
    for(int n=0;n<=nmax;n++)
    {
	final double [] rn=new double [n+1];
	B.setNext(n,rn,rn1,rn2);
	System.err.print("n="+n);
	for(int j=0;j<=n;j++)
	{
	    System.err.print("\t"+rn[j]);
	}
	System.err.println();
	rn2=rn1;
	rn1=rn;

	// check array re-use
	B.setNext(n,rb,ra,rb);
	final double [] x=rb;rb=ra;ra=x;
	UnitTests.checkDoubleArrayRelFirstindex(rn.length,rn,"rn",x,"x",0);
    }
}

void printPolynomialsType2(final int nmax)
{
    System.err.println("========="+txt+": Type 2 polynomials");
    final double [] pmom=getMonomialsMoments(nmax+1);
    if(pmom==null)
    {
	System.err.println("========="+txt+": Type 2 polynomials pmom=null, skipping");
	return;
    }

    double [] ra=new double [nmax+1],rb=new double [nmax+1];
    java.util.Arrays.fill(ra,Double.NaN);
    java.util.Arrays.fill(rb,Double.NaN);
    for(int n=0;n<=nmax;n++)
    {
	final double [] L=new double [n+2];L[n+1]=1;
	final double [] t2=Polynomials.getSecondKindOrthogonalPolynomialFromFirstKind(B.convertToPolynomialFromBasis_Test(n+1,L),pmom);
	System.err.print("n="+n);
	for(int j=0;j<t2.length;j++)
	{
	    System.err.print("\t"+t2[j]);
	}
	System.err.println();

	B.setNextType2(n,rb,ra,rb);
	final double [] x=rb;rb=ra;ra=x;
	final double [] t2x=new double [n+1];
	System.arraycopy(x,0,t2x,0,n+1);
	UnitTests.checkDoubleArrayRel(t2,"t2",t2x,"t2x",eps);
    }
}

void test_Sum(final int n_tests)
{
  for(int t=0;t<n_tests;t++)
  {
    for(int nn=0;nn<=nmax;nn++)
    {
	final double [] coefs=new double [nn+1];
	for(int k=coefs.length;--k>=0;) coefs[k]=1-2*r.nextDouble();
	double [] l=null,ra=new double [nn+1],rb=new double [nn+1],res=new double [nn+1];
	for(int n=0;n<=nn;n++)
	{
	    B.setNext(n,rb,ra,rb);
	    l=rb;rb=ra;ra=l;
	    TMatr.sumP1(res,l,n+1,coefs[n]);
	}
	final double [] sum_test=B.convertToPolynomialFromBasis_Test(nn,coefs);
	UnitTests.checkDoubleArrayRel(sum_test,"sum_test",res,"res",eps);
	UnitTests.checkDoubleArrayRel(coefs,"coefs",B.convertToBasisFromPolynomial_Test(nn,sum_test),"sum_test",eps);

	// differentiation
	UnitTests.checkDoubleArrayRel(B.convertToPolynomialFromBasis_Test(nn-1,B.getDifferentiated(nn,coefs)),"DBasis",
				      Polynomials.getPolynomialDifferentiated(B.convertToPolynomialFromBasis_Test(nn,coefs)),"DP",eps);
	UnitTests.checkDoubleArrayRel(B.getDifferentiated(nn,coefs),"DBasis",
				      B.convertToBasisFromPolynomial_Test(nn-1,Polynomials.getPolynomialDifferentiated(B.convertToPolynomialFromBasis_Test(nn,coefs))),"DP",eps);


	// integration
	UnitTests.checkDoubleArrayRel(B.convertToPolynomialFromBasis_Test(nn+1,B.getIntegrated(nn,coefs)),"IBasis",
				      Polynomials.getPolynomialIntegrated(B.convertToPolynomialFromBasis_Test(nn,coefs)),"IP",eps);
	UnitTests.checkDoubleArrayRel1(B.getIntegrated(nn,coefs),"IBasis",
				      B.convertToBasisFromPolynomial_Test(nn+1,Polynomials.getPolynomialIntegrated(B.convertToPolynomialFromBasis_Test(nn,coefs))),"IBasis",eps);


	final double FE=1e-3;
	for(int k=0;k<50;k++)
	{
	    final double x=r.nextDouble();
	    if(!(Math.abs(Polynomials.getD0NoScale(l,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD0NoScale(l,x),"Ln",B.calculate(nn,x),"calculate",eps);
	    if(!(Math.abs(Polynomials.getD0NoScale(res,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD0NoScale(res,x),"D0sum",B.calculateSumD0(nn,coefs,x),"calculateSumD0",eps);
	    if(!(Math.abs(Polynomials.getD1NoScale(res,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD1NoScale(res,x),"D1sum L",B.calculateSumD1(nn,coefs,x),"calculateSumD1",eps);
	    if(!(Math.abs(Polynomials.getD2NoScale(res,x))<=FE))
		UnitTests.checkDoubleRel(Polynomials.getD2NoScale(res,x),"D2sum L",B.calculateSumD2(nn,coefs,x),"calculateSumD2",eps);
	}
    }
  }
  System.err.println(txt+":test_Sum(): test OK with nmax="+nmax+" eps="+eps);
}

void matchmul(final double [] p,final double [] d,final double [] q,final double [] r)
{
    final double [] mul=new double [d.length+q.length-1];
    B.setMult2Pol(mul,d,d.length-1,q,q.length-1,new double [mul.length]);
    TMatr.sumP1(mul,r,r.length,1);
    UnitTests.checkDoubleArrayRel(p,"p",mul,"q x d +r",eps);
}
void test_setaxbP(final int n_tests)
{
    for(int t=0;t<n_tests;t++)
    {
	final double a=1-2*r.nextDouble(),b=1-2*r.nextDouble();
	final double [] bx=B.convertToBasisFromPolynomial_Test(1,new double []{b,a});
	final int n=r.nextInt(nmax+1);
	final double [] p=new double [n+1];
	for(int i=p.length;--i>=0;) p[i]=0.3*(1-2*r.nextDouble());
	if(true)
	{
	    final double [] res=new double [n+2];
	    for(int i=res.length;--i>=0;) res[i]=r.nextDouble();
	    B.setaxbP(n,res,p,a,b);
	    matchmul(res,p,bx,new double [] {0});

	    final double [] pp=B.convertToPolynomialFromBasis_Test(n,p);
	    final double [] pres=Polynomials.getMult2Pol(pp,new double []{b,a});
	    final double [] cpres=B.convertToBasisFromPolynomial_Test(pres.length-1,pres);
	    UnitTests.checkDoubleArrayRel(res,"res",cpres,"cpres",eps);
	}

	if(true)
	{
	    // check with the same array
	    final double [] res=new double [n+2],px=new double [n+2];
	    B.setaxbP(n,res,p,a,b);
	    for(int k=p.length;--k>=0;) px[k]=p[k];
	    px[p.length]=r.nextDouble();
	    B.setaxbP(n,px,px,a,b);
	    UnitTests.checkDoubleArrayRel(res,"res",px,"px",eps*0);
	}

	final double [] moms=new double [n+1];
	for(int k=0;k<moms.length;k++)
	{
	    moms[k]=r.nextDouble();
	}
	final double [] Txmom=new double [n];
	for(int k=0;k<n;k++)
	{
	    final double [] m=new double [k+1],mm=new double [k+2];
	    m[k]=1;
	    B.setaxbP(k,mm,m,1,0);
	    Txmom[k]=TMatr.mulVV(k+2,mm,moms);
	}
	final double [] xmom=new double [n];
	B.setXQnMomentsFromQnMoments(n,xmom,moms);
	UnitTests.checkDoubleArrayRel(xmom,"xmom",Txmom,"Txmom",eps);
    }
    System.err.println(txt+":test_setaxbP(): test OK with nmax="+nmax+" eps="+eps);
}

void test_setMult2Pol(final int n_tests)
{
    final int nc=4;
    for(int t=0;t<n_tests;t++)
    {
	final int n1,n2;
	if(t<=n_tests/100) n1=n2=nmax<=0?0:r.nextInt(nmax);
	else {n1=nmax<=0?0:r.nextInt(21);n2=nmax<=0?0:r.nextInt(nmax);}
	final double [] p1=new double [1+n1],p2=new double [1+n2];
	for(int i=p1.length;--i>=0;) p1[i]=r.nextDouble()/(i<nc?1:Polynomials.fact(i-nc));
	for(int i=p2.length;--i>=0;) p2[i]=r.nextDouble()/(i<nc?1:Polynomials.fact(i-nc));
	final double [] res=new double [1+n1+n2],tmp=new double [1+n1+n2];
	for(int i=res.length;--i>=0;) {res[i]=r.nextDouble();tmp[i]=r.nextDouble();}

	B.setMult2Pol(res,p1,n1,p2,n2,tmp);

	final double [] mtest=B.convertToBasisFromPolynomial_Test(n1+n2,Polynomials.getMult2Pol(B.convertToPolynomialFromBasis_Test(n1,p1),B.convertToPolynomialFromBasis_Test(n2,p2)));
	UnitTests.checkDoubleArrayRel1(res,"res",mtest,"mtest",eps);
    }
    System.err.println(txt+":test_setMult2Pol: OK with n_tests="+n_tests+" nmax="+nmax+" eps="+eps);
}


void test_sdiv1(final int n_tests)
{

    for(int t=0;t<n_tests;t++)
    {
	final int n=r.nextInt(nmax+1);
	final double [] p=new double [n+1];
	for(int i=p.length;--i>=0;) p[i]=0.3*(1-2*r.nextDouble());
	p[p.length-1]=(0.5+r.nextDouble())*(r.nextInt(2)<1?1:-1);
	p[0]+=0.1+r.nextDouble();
	if(true)
	{
	    final double x0=1-2*r.nextDouble();
	    final double [] Bxmx0=B.convertToBasisFromPolynomial_Test(1,new double []{-x0,1});
	    final double [] res=new double [n];
	    for(int i=res.length;--i>=0;) res[i]=r.nextDouble();
	    final double remainder=B.sdiv1(n,res,p,x0);
	    final double [] Bremainder=B.convertToBasisFromPolynomial_Test(0,new double []{remainder});

	    matchmul(p,Bxmx0,res,Bremainder);
	    matchmul(p,res,Bxmx0,Bremainder);

	    final double [] pp=B.convertToPolynomialFromBasis_Test(n,p);
	    final double [] pres=new double [n];
	    final double premainder=Polynomials.sdiv1(pres,pp,pp.length,x0);
	    final double [] cpres=B.convertToBasisFromPolynomial_Test(n-1,pres);
	    UnitTests.checkDoubleArrayRel(res,"res",cpres,"cpres",eps);
	    UnitTests.checkDoubleRel(remainder,"remainder",premainder,"premainder",eps);
	}

	if(true)
	{
	    // check with the same array
	    final double x0=1-2*r.nextDouble();
	    final double [] res=new double [n],px=p.clone();
	    final double premainder=B.sdiv1(n,res,p,x0);
	    final double remainder=B.sdiv1(n,px,px,x0);
	    UnitTests.checkDoubleArrayRelFirstindex(n,res,"res",px,"px",eps*0);
	    UnitTests.checkDoubleRel(remainder,"remainder",premainder,"premainder",eps*0);
	}
    }
    System.err.println(txt+":test_sdiv1(): test OK with nmax="+nmax+" eps="+eps);
}

void test_sdiv(final int n_tests)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] p=new double [1+(nmax<=0?0:this.r.nextInt(nmax))];
	for(int i=p.length;--i>=0;) p[i]=this.r.nextDouble();
	final double [] d=new double [1+this.r.nextInt(p.length)];
	for(int i=d.length;--i>=0;) d[i]=this.r.nextDouble();
	if(false)
	{
	    System.err.println("d="+java.util.Arrays.toString(d));
	    System.err.println("p="+java.util.Arrays.toString(p));
	}
	// make the leading coefficient >1, to avoid instability in tests.
	d[d.length-1]=2+r.nextDouble();

	final double [] q=new double [p.length-d.length+1],r=new double [d.length-1];
	for(int i=q.length;--i>=0;) q[i]=this.r.nextDouble();
	for(int i=r.length;--i>=0;) r[i]=this.r.nextDouble();

	B.sdiv(q,r,p,p.length-1,d,d.length-1);
	matchmul(p,d,q,r);
    }
    System.err.println(txt+":test_sdiv(): test OK with nmax="+nmax+" eps="+eps);
}



void test_setNewtonBinomialLikeCoefs(final int n_tests)
{
    for(int t=0;t<n_tests;t++)
    {
	final int n=r.nextInt(nmax+1);

	double [] ra=new double [n+1],rb=new double [n+1];
	java.util.Arrays.fill(ra,Double.NaN);
	java.util.Arrays.fill(rb,Double.NaN);
   	for(int k=0;k<=n;k++)
	{
	    B.setNext(k,rb,ra,rb);
	    final double [] l=rb;rb=ra;ra=l;

	    final double [] p=new double [k+1];
	    System.arraycopy(l,0,p,0,k+1);
	    final double x0=r.nextDouble(),a=r.nextDouble();
	    final double [] psbasis=B.convertToBasisFromPolynomial_Test(k,Polynomials.getPolynomialShifted(Polynomials.getPolynomialScaled(p,a),x0));

	    final double [] b=new double [k+1],q=new double [k];
	    for(int m=b.length;--m>=0;)b[m]=r.nextDouble();
	    for(int m=q.length;--m>=0;)q[m]=r.nextDouble();
	    for(int m=0;m<b.length;m++) B.setNewtonBinomialLikeCoefs(m,b,q,a,x0);
	    UnitTests.checkDoubleArrayRel1(b,"b",psbasis,"psbasis",eps);
	}
    }
    System.err.println(txt+":test_setNewtonBinomialLikeCoefs: test OK with nmax="+nmax+" eps="+eps);
}

void test_MomentsCalculation(final int n_tests,final int n,final int sample_max)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] x=new double [12+r.nextInt(sample_max)],w=new double [x.length];
	final int nelsinsample=x.length-r.nextInt(4);
	for(int k=0;k<x.length;k++)
	{
	    x[k]=2*r.nextDouble()-1;
	    w[k]=r.nextDouble();
	}
	final double [] m=B.calculateMomentsFromSample(n,x,w,nelsinsample),mm=new double [m.length];
	for(int j=0;j<m.length;j++)
	{
	    for(int k=0;k<nelsinsample;k++)
	   {
	       mm[j]+=w[k]*B.calculate(j,x[k]);
	   }
	}
	UnitTests.checkDoubleArrayRel(m,"m",mm,"mm",eps);
    }
    System.err.println(txt+":test_MomentsCalculation: test OK with n="+n+" eps="+eps);
}

void test_ConfederateMatrix(final int n_tests,final int n)
{
    for(int t=0;t<n_tests;t++)
    {
	final double [] pol=new double [n+1];
	for(int k=pol.length;--k>=0;) pol[k]=(1-2*r.nextDouble())/(k+1);
	pol[n]+=3.2;

	final double [] root_re=new double [n],root_im=new double [n];
	final int nroots=new PolynomialRootJenkinsTraubORIG(n).findRoots(n,pol,root_re,root_im);
	if(nroots!=n) throw new RuntimeException("nroots="+nroots+" n="+n);
	SortArrays.sortArrayWith2Load(nroots,TMatr.sumAB(nroots,root_re,root_im),root_im,root_re);

	final double [] bpol=B.convertToBasisFromPolynomial_Test(n,pol);
	final double [] rootb_re=new double [n],rootb_im=new double [n];
	final int nrootsb=new PolynomialRootsConfederateMatrixABasis.PolynomialRootsConfederateMatrixPBasis(B).findRoots(n,bpol,rootb_re,rootb_im);
	if(nrootsb!=n) throw new RuntimeException("nrootsb="+nrootsb+" n="+n+" bpol="+java.util.Arrays.toString(bpol));
	SortArrays.sortArrayWith2Load(nrootsb,TMatr.sumAB(nrootsb,rootb_re,rootb_im),rootb_im,rootb_re);
	UnitTests.checkDoubleArrayRel1(rootb_re,"rootb_re",root_re,"root_re",eps);
	UnitTests.checkDoubleArrayRel1(rootb_im,"rootb_im",root_im,"root_im",eps);
    }
    System.err.println(txt+":test_ConfederateMatrix: test OK with n="+n+" eps="+eps);
}


void printExtra(final int nm){}
void test_Extra(final int n_tests){}

void runTests(final String id)
{
    System.err.println(txt+"runTests(): starting tests; id="+id);
    System.err.println("ver=$Id: TestBasisPolynomials.java,v 1.37 2017/09/08 10:13:21 mal Exp $");
    final long t0=System.currentTimeMillis();
    final java.util.Random r=new java.util.Random(-79091);
    final int nm=15;
    printPolynomials(nm);
    printPolynomialsType2(nm);
    printExtra(nm);
    final OrthogonalPolynomialsABasis o=getOrtPol();
    if(o!=null)
    {
	for(int k=1,ne=25;k<=ne;k++)
	{
	    final double [] moments=new double [2*k];
	    moments[0]=1;
	    OrthogonalPolynomialsABasis.printQuadratures(txt,k,moments,o,k==ne);
	}
    }
    test_ExactValues(1e-14);

    final int n_tests=10000;

    for(int nmax=0;nmax<=7;nmax++)
    {
	this.nmax=nmax;
	for(int i=0;i<111;i++)
	{
	    test_Sum(n_tests); // must match almost identically
	    test_setaxbP(n_tests);
	    test_setQlQmExpansionCoefs(n_tests);
	    test_sdiv1(n_tests);
	    test_sdiv(n_tests);
	    test_setMult2Pol(n_tests);
	    test_setNewtonBinomialLikeCoefs(n_tests);
	    test_MomentsCalculation(n_tests,nmax,200);
	    test_ConfederateMatrix(n_tests,nmax<=7?nmax:7);
	    test_Extra(n_tests);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println("TestBasisPolynomials::main(): tests OK done in "+(t1-t0)+" milliseconds. id="+id);
    System.err.println("ver=$Id: TestBasisPolynomials.java,v 1.37 2017/09/08 10:13:21 mal Exp $");
}

}
