package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2011
 * This file is under GPL version 3.
 *
 */

/** Test OrthogonalPolynomialsABasis in monomials.
 *  @version $Id: TestMonomials.java,v 1.143 2017/08/26 11:28:05 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */

class TestMonomials {
    final OrthogonalPolynomialsABasis O;
    final String name;
    final double eps;
    final java.util.Random r;
    final double x_min,x_max;
    TestMonomials(final String name,
		  final OrthogonalPolynomialsABasis O,
		  final double eps,
		  final java.util.Random r,
		  final double x_min,
		  final double x_max)
    {
	this.name=name;
	this.O=O;
	this.eps=eps;
	this.r=r;
	this.x_min=x_min;
	this.x_max=x_max;
    }

    double [] convertToMonomials(final double [] p)
    {
	return O.B.convertToPolynomialFromBasis_Test(p.length-1,p);
    }

    /** Multiply (in monomials basis) nodes by (x-x0) or (x0-x).
     *  Similar method OrthogonalPolynomialsABasis.getRadauMultiplied does this
     *  in some basis, and this method does this in monomials basis.
     */
    double [] getRadauMultiplied(final double [] nodes){
	final double [] tmp1=new double []{1};tmp1[0]/=O.fx(tmp1,0);
	final double x0=O.getRadauRoot();
	final double [] tmp=O.getRadauMultiplied(tmp1);
	if(O.Dfx(tmp,x0)>0)
	{
	    return Polynomials.getMult2Pol(nodes,new double [] {-x0,1});
	}
	else
	{
	    return Polynomials.getMult2Pol(nodes,new double [] {x0,-1});
	}
    }


static double [] getPolynomialRandomMoments(final int n,final double x_min,final double x_max,final int size,final java.util.Random r)
{
    final double [] mom=new double [n];
    for(int k=size;--k>=0;)
    {
	final double x=x_min+(x_max-x_min)*r.nextDouble();
	//System.err.println("x="+x);
	double s=1;
	for(int m=0;m<mom.length;m++) {mom[m]+=s;s*=x;}
    }
    return mom;
}
double [] getBasisRandomMoments(final int n,final double x_min,final double x_max,final int size,final java.util.Random r)
{
    final double [] mom=new double [n];
    final double [][] f=new double [mom.length][];
    for(int m=0;m<mom.length;m++)
    {
	f[m]=new double [m+1];
	f[m][m]=1;
    }
    for(int k=size;--k>=0;)
    {
	final double x=x_min+(x_max-x_min)*r.nextDouble();
	//System.err.println("x="+x);
	for(int m=0;m<mom.length;m++) {mom[m]+=O.fx(f[m],x);}
    }
    return mom;
}

double [] mom_p,mom_b;
void setData(final int tsize,final int n_moments)
{
    mom_p=getPolynomialRandomMoments(n_moments,this.x_min,this.x_max,tsize,this.r);
    mom_b=getBasisMomentsConvertedFromMonomials(mom_p);
}

double [][] getXWDiscrete(final int n)
{
    final double [] x=new double [n],w=new double [n];
    final double delta=(x_max-x_min)/(n+1);
    for(int k=0;k<n;k++)
    {
	x[k]=x_min+k*delta+0.9*delta*r.nextDouble();
	w[k]=3.1*(r.nextDouble()+r.nextDouble());
    }
    sortNodes(x,w);
    //System.err.println("getXWDiscrete: x="+java.util.Arrays.toString(x));
    return new double [][]{x,w};
}
double [] getBmomsFromDiscrete(final int nmoms,final double [] x,final double [] w)
{
    final double [] bmoms=new double [nmoms];
    for(int k=bmoms.length;--k>=0;)
    {
	final double [] tmp=new double [k+1];
	tmp[k]=1;
	double s=0;
	for(int l=0;l<x.length;l++)
	{
	    s+=w[l]*O.fx(tmp,x[l]);
	}
	bmoms[k]=s;
    }
    return bmoms;
}
/** The returned [][] array contains 3 [] arrays: values,
 *  x nodes on left argument, xnodes on right argument.
 *  Uses generalized eigenvalues expansion.
 *  The method is used to test discrete measures.
 */
double [][] getD2QuadraturesFromMoments_EV(final int n,final double [] Lmoments,final double [] Rmoments)
{
    final double [] QQL=O.getQQMatr(n,Lmoments),QQR=O.getQQMatr(n,Rmoments);
    final double [] ev=O.EV.getGEV_Generalized(n,QQL,n,QQR,n,true);
    final double [] xQQL=O.getQQMatr(n,O.getXFMomentsFromMoments(Lmoments)),xQQR=O.getQQMatr(n,O.getXFMomentsFromMoments(Rmoments));
    final double [] values=new double [n],xl=new double [n],xr=new double [n];
    for(int k=0;k<n;k++)
    {
	final double lam=values[k]=ev[k];
	final int o=n+k*n;
	xl[k]=TMatr.getAMatrB(n,ev,o,ev,o,xQQL,n)/lam;
	xr[k]=TMatr.getAMatrB(n,ev,o,ev,o,xQQR,n);
    }
    return new double [][]{values,xl,xr};
}


double [] getBasisMomentsConvertedFromMonomials(final double [] monomial_moments)
{
    final int n=monomial_moments.length;
    final double [] basis_moments=new double [n];
    for(int k=0;k<n;k++)
    {
	final double [] mo=new double [k+1];
	mo[k]=1;
	basis_moments[k]=TMatr.mulVV(k+1,convertToMonomials(mo),monomial_moments);
    }
    return basis_moments;
}

static void sortNodes(final double [] x,final double [] w)
{
    SortArrays.sortArrayWithLoad(x.length,x,w);
}


double [][] test_NodesPolynomial(final double [] nodes,final double [] mom,final int maxdegree)
{
    final int n=nodes.length-1;
    final double [] xs=new double [n],root_im=new double [n];
    final int nroots=new com.polytechnik.utils.PolynomialRootJenkinsTraubORIG(n).findRoots(n,nodes,xs,root_im);
    if(nroots!=n) throw new RuntimeException("n="+n+" nroots="+nroots);

    final double [] ws=new double [n],ws2=new double [n],tmp=new double [n];
    for(int k=0;k<n;k++)
    {
	UnitTests.checkDoubleRel(root_im[k],"root_im[k]",0,"0",eps);
	Polynomials.sdiv1(tmp,nodes,nodes.length,xs[k]);
	final double t0=Polynomials.getD0NoScale(tmp,xs[k]);
	for(int m=tmp.length;--m>=0;) tmp[m]/=t0;
	ws[k]=TMatr.mulVV(tmp.length,tmp,mom);
	ws2[k]=TMatr.mulVV(2*tmp.length-1,Polynomials.getMult2Pol(tmp,tmp),mom);
    }
    UnitTests.checkDoubleRel(TMatr.sumElements(ws),"sum(ws)",mom[0],"mom[0]",eps);

    final double [] pt=new double [maxdegree+1];
    for(int k=pt.length;--k>=0;) pt[k]=r.nextDouble();
    pt[0]+=0.6+r.nextDouble();
    double i=0;
    for(int k=0;k<xs.length;k++)
    {
	i+=ws[k]*Polynomials.getD0NoScale(pt,xs[k]);
    }
    final double ia=TMatr.mulVV(pt.length,pt,mom);
    UnitTests.checkDoubleRel(i,"i",ia,"ia",eps);

    System.err.println(name+":test_NodesPolynomial(): test OK with n="+n+" maxdegree="+maxdegree+" eps="+eps);

    return new double [][]{xs,ws,ws2};
}

void test_QuadratureBasis(final double [] x,final double [] w,final double [] moments_b,final int maxdegree,final double eps)
{
    final double [] pt=new double [maxdegree+1];
    for(int k=pt.length;--k>=0;) pt[k]=r.nextDouble();
    double i=0;
    for(int k=0;k<x.length;k++)
    {
	i+=w[k]*O.fx(pt,x[k]);
    }
    final double ia=TMatr.mulVV(pt.length,pt,moments_b);
    UnitTests.checkDoubleRel(i,"i",ia,"ia",eps);

    System.err.println(name+":test_QuadratureBasis(): test OK with n_points="+x.length+" maxdegree="+maxdegree+" eps="+eps);
}

/** Out of 0 .. 2*n moments (n=p.length-1) used to construct p
 * only 0 .. 2*n-1 match. The last one is used
 * for normalization of p[n] and cannot be recovered.
 */
static double [] getMomentsFromPolynomial(final double [][] p)
{
    final int n=p.length;
    if(n<=0) throw new RuntimeException();
    final double [] mom=new double [2*n-1];
    // Obtain all moments from p. Use Gauss quadrature integration.
    final double [] xs=getXNodes(p[n-1]),ws=getWeightsChristoffel(xs,p);
    for(int k=mom.length;--k>=0;)
    {
	// monomial
	final double [] t=new double [k+1];t[k]=1;
	// integrate
	double s=0;
	for(int l=0;l<xs.length;l++) s+=ws[l]*Polynomials.getD0NoScale(t,xs[l]);
	mom[k]=s;
    }
    return mom;
}

static double [] getPolynomial(final int n,final double [] imom)
{
    final double [] rp=new double [n+1];
    for(int k=n;--k>=0;){rp[k]=-imom[n+k];}
    final double [] qcoefs=Polynomials.getLinsysSol(n,imom,0,rp,0);
    System.arraycopy(qcoefs,0,rp,0,qcoefs.length);
    rp[qcoefs.length]=1;
    // normalize
    final double norm=Math.sqrt(TMatr.mulVV(2*rp.length-1,Polynomials.getMult2Pol(rp,rp),imom));
    for(int l=rp.length;--l>=0;){rp[l]/=norm;}
    return rp;
}

static double [][] getOrthogonalPolynomialsFirstKind(final int n, final double [] mom)
{
    final double [][] p=new double [n][];
    for(int k=0;k<n;k++)
    {
	final double [] pk=p[k]=getPolynomial(k,mom);
    }
    return p;
}
/** Like Polynomials.getSecondKindOrthogonalPolynomialFromFirstKind,
 *  but scaled to orthonormal basis.
 */
static double [] getSecondKindOrthogonalPolynomialFromFirstKindAndMoments(final double [] p,final double [] mom)
{
    final double [] q=Polynomials.getSecondKindOrthogonalPolynomialFromFirstKind(p,mom);
    // normalize to compensate extra sqrt(<1>) divided in getSecondKindOrthogonalPolynomialFromFirstKind.
    final double normtocompensate=Math.sqrt(mom[0]);
    for(int l=q.length;--l>=0;){q[l]*=normtocompensate;}
    return q;
}
static double [][] getSecondKindOrthogonalPolynomialsFromFirstKindAndMoments(final double [][] p, final double [] mom)
{
    final int n=p.length-1;
    final double [][] q=new double [n][];
    for(int k=0;k<n;k++)
    {
	q[k]=getSecondKindOrthogonalPolynomialFromFirstKindAndMoments(p[k+1],mom);
    }
    return q;
}
static double [] getXNodes(final double [] nodes)
{
    final int n=nodes.length-1;
    final double [] xs=new double [n],root_im=new double [n];
    final int nroots=new com.polytechnik.utils.PolynomialRootJenkinsTraubORIG(n).findRoots(n,nodes,xs,root_im);
    if(nroots!=n) throw new RuntimeException("nroots="+nroots+" n="+n);
    return xs;
}
static double [] getWeightsChristoffel(final double [] x,final double [][] p)
{
    final int n=x.length;
    if(n<=0) throw new RuntimeException();
    else if(n<=1)  return new double []{1/(p[0][0]*p[0][0])};
    final double [] ws=new double [n];
    final double f=(p[n][n]/p[n-1][n-1]);
    for(int k=0;k<n;k++)
    {
	final double xx=x[k];
	ws[k]=f/((Polynomials.getD0NoScale(p[n-1],xx)*Polynomials.getD1NoScale(p[n],xx)));
    }
    return ws;
}
static double [] getWeigthsSDiv(final double [] nodes,final double [] x,final double [][] p)
{
    final int n=nodes.length-1;
    final double [] ws=new double [x.length],tmp=new double [n];
    final double [] quot=new double [1];
    // note, the integral dM =1/p[0][0]^2
    for(int k=0;k<x.length;k++)
    {
	final double xx=x[k];
	Polynomials.sdiv1(tmp,nodes,nodes.length,xx);
	final double t0=Polynomials.getD0NoScale(tmp,xx);
	//System.err.println("f("+xx+"_"+k+"="+Polynomials.getD0NoScale(nodes,xx)+" tmp.length="+tmp.length);
	for(int m=tmp.length;--m>=0;) tmp[m]/=t0;
	for(int m=tmp.length;--m>=0;)
	{
	    if(p[m].length!=m+1) throw new RuntimeException("m="+m);
	    Polynomials.sdiv_stable(quot,tmp,tmp,m+1,p[m],m+1);
	    //System.err.println("k="+k+" m="+m+" quot="+quot[0]+" tmp[0]="+tmp[0]+" "+(1/quot[0]));
	}
	ws[k]=(quot[0]/p[0][0]);
	//ws[k]=tmp[0];
    }
    return ws;
}
static double [] getWeightsByIntegration(final double [] nodes,final double [] x,final double [] moms)
{
    final double [] w=new double [x.length],tmp=new double [nodes.length-1];
    for(int k=0;k<x.length;k++)
    {
	Polynomials.sdiv1(tmp,nodes,nodes.length,x[k]);
	final double t0=Polynomials.getD0NoScale(tmp,x[k]);
	for(int m=tmp.length;--m>=0;) tmp[m]/=t0;
	w[k]=TMatr.mulVV(tmp.length,tmp,moms);
    }
    return w;
}


double [] getRadauMomentsFromMoments(final double [] mom)
{
    final double [] rmom=new double [mom.length-1];
    for(int k=0;k<rmom.length;k++)
    {
	final double [] p=new double [k+1];
	p[k]=1;
	rmom[k]=TMatr.mulVV(k+2,getRadauMultiplied(p),mom);
    }
    return rmom;
}
final double getX0()
{
    final double [] radau=getRadauMultiplied(new double []{1});
    final double x0=-radau[0]/radau[1];
    return x0;
}
double [][] getRadauQuadraturesFromPolynomials(final int n,final double [][] p)
{
    if(n+1>=p.length) throw new RuntimeException("n="+n+" p.length="+p.length);
    final int nn=n+1;
    final double [] matr=new double [nn*nn];
    final double [] radau=getRadauMultiplied(new double []{1}),quot=new double [1];
    final double x0=getX0();
    //System.err.println("x0="+x0);
    for(int k=0;k<nn;k++)
    {
	for(int l=0;l<=k;l++)
	{
	    final double [] mul=Polynomials.getMult2Pol(radau,p[l]);
	    quot[0]=0;
	    for(int j=l+1;j>=k;j--)
	    {
		Polynomials.sdiv_stable(quot,mul,mul,j+1,p[j],j+1);
	    }
	    matr[k+nn*l]=matr[l+nn*k]=quot[0]*1;
	}
    }

    final double [] matr2=new double [nn*nn];
    final int [] ipvt=new int [n];
    final double [][] qp=new double [nn][],qq=new double [nn][];
    for(int k=0;k<nn;k++)
    {
	System.arraycopy(matr,0,matr2,0,k*nn);
	final double [] rq=qq[k]=new double [k+1];
	rq[k]=1;
	for(int l=0;l<k;l++) rq[l]=-matr[k*nn+l];
	final int pinfo=com.polytechnik.utils.Linsystems.dgefa_m1(matr2,k,nn,ipvt);
	if(pinfo!=0) java.util.Arrays.fill(rq,Double.NaN);
	else com.polytechnik.utils.Linsystems.dgesl_m1(matr2,k,nn,ipvt,rq,1);
	// normalize
	final double s=Math.sqrt(TMatr.getAMatrA(k+1,rq,matr,nn));
	for(int l=0;l<=k;l++) rq[l]/=s;

	final double [] rp=qp[k]=new double [k+1];
	for(int l=0;l<=k;l++)
	{
	    TMatr.sumP1(rp,p[l],rq[l]);
	}
	//System.err.println("qp["+k+"]="+java.util.Arrays.toString(rp));
    }

    final double [] rnodes=qp[n],xr=getXNodes(rnodes),wr=getWeigthsSDiv(rnodes,xr,qp);
    final double [] xs=new double [n+1],ws=new double [n+1];
    for(int k=0;k<n;k++)
    {
	xs[k]=xr[k];
	ws[k]=wr[k]/Polynomials.getD0NoScale(radau,xs[k]);
    }
    xs[n]=x0;
    final double [] tmp=new double [rnodes.length];
    System.arraycopy(rnodes,0,tmp,0,rnodes.length);
    final double t0=Polynomials.getD0NoScale(tmp,x0);
    for(int m=tmp.length;--m>=0;) tmp[m]/=t0;
    quot[0]=0;
    for(int k=tmp.length;--k>=0;)
    {
	Polynomials.sdiv_stable(quot,tmp,tmp,k+1,p[k],k+1);
    }
    ws[n]=(quot[0]/p[0][0]);

    return new double [][] {xs,ws,getRadauMultiplied(rnodes)};
}

static double [][] getAB(final double [][] p)
{
    final int n=p.length,ne=n-1;
    if(n<=0) return new double [][]{{},{}};
    final double [] a=new double [n],b=new double [ne];
    final double [] tmp=new double [n];
    a[0]=1/p[0][0];
    for(int k=0;k<ne;k++)
    {
	final double [] pk=p[k],pk1=p[k+1];
	Polynomials.setxP(tmp,pk,k+1);
	a[k+1]=(tmp[k+1]/pk1[k+1]);
	b[k]=(tmp[k]-a[k+1]*pk1[k])/pk[k];
    }
    //b[ne]=Double.NaN;
    // a & b are now generated
    return new double [][]{a,b};
}
static double [][] getOrthogonalPolynomialsFromAB(final int kind,final double [] a,final double [] b)
{
    if(kind>=a.length) return new double [][]{};
    // generate p - kind-type polynomials.
    final double [][] p=new double [a.length-kind][];
    p[0]=new double []{1/a[kind]};
    for(int k=1;k<p.length;k++)
    {
	final double [] pk=p[k]=new double [k+1],pk1=p[k-1];
	final double ak=a[k+kind],bk1=b[k+kind-1];
	// save p_{k} to p[k];
	Polynomials.setxP(pk,pk1,k);
	pk[k]/=ak;
	for(int l=k;--l>=0;) pk[l]=(pk[l]-bk1*pk1[l])/ak;
	if(k>1) TMatr.sumP1(pk,p[k-2],-a[k+kind-1]/ak);
    }
    return p;
}

static double [][] getSecondKindOrthogonalPolynomialsFromFirstKind(final double [][] p)
{
    final double [][] ab=getAB(p);
    return getSecondKindOrthogonalPolynomialsFromAB(ab[0],ab[1]);
}
static double [][] getSecondKindOrthogonalPolynomialsFromAB(final double [] a,final double [] b)
{
    return getOrthogonalPolynomialsFromAB(1,a,b);
}
static double [][] getOrthogonalPolynomialsFirstKindFromAB(final double [] a,final double [] b)
{
    return getOrthogonalPolynomialsFromAB(0,a,b);
}

void test_RegularNodes(final int neq)
{
    final double const_aver=mom_p[0];
    // regular nodes
    final double [][] p=getOrthogonalPolynomialsFirstKind(neq+1,mom_p);
    //System.err.println("mom_p="+java.util.Arrays.toString(mom_p));
    final double [][] b=O.getOrthogonalPolynomialsFirstKind(neq+1,ReadWriteUtils.shrinkArray(2*(neq+1)-1,mom_b));
    //System.err.println("mom_b="+java.util.Arrays.toString(mom_b));
    for(int k=0;k<p.length;k++)
    {
	UnitTests.checkDoubleArrayRel(p[k],"p["+k+"]",convertToMonomials(b[k]),"conv(b["+k+"])",eps);
	final double [] bp=O.getSingleOrthogonalPolynomial(k,ReadWriteUtils.shrinkArray(2*k,mom_b));
	UnitTests.checkDoubleArrayRel(bp,"bp",TMatr.getVectorScaled(b[k],1/b[k][b[k].length-1]),"b[k]",eps);
    }
    final double [] nodes=p[neq];
    final double [][] ndp=test_NodesPolynomial(nodes,mom_p,2*neq-1);
    final double [] xt=ndp[0],wt=ndp[1];
    sortNodes(xt,wt);
    if(true)
    {
	final double [][] nr=O.getQuadraturesFromPolynomials(neq,b);
	final double [] xm=nr[0],wm=nr[1],bnodes=nr[2];
	UnitTests.checkDoubleRel(TMatr.sumElements(wm),"sum(wm)",mom_p[0],"mom_p[0]",eps);
	sortNodes(xm,wm);
	UnitTests.checkDoubleArrayRel(xt,"xt",xm,"xm",eps);
	UnitTests.checkDoubleArrayRel(wt,"wt",wm,"wm",eps);

	final double [][] ne=O.getQuadraturesFromMoments_EV(neq,ReadWriteUtils.shrinkArray(2*neq,mom_b));
	final double [] xe=ne[0],we=ne[1];
	UnitTests.checkDoubleRel(TMatr.sumElements(we),"sum(we)",mom_p[0],"mom_p[0]",eps);
	sortNodes(xe,we);
	UnitTests.checkDoubleArrayRel(xe,"xe",xm,"xm",eps);
	UnitTests.checkDoubleArrayRel(we,"we",wm,"wm",eps);
	test_QuadratureBasis(xe,we,mom_b,2*neq-1,eps);

	final double [][] nd=O.getQuadraturesFromMoments_Direct(neq,ReadWriteUtils.shrinkArray(2*neq,mom_b));
	final double [] xd=nd[0],wd=nd[1];
	UnitTests.checkDoubleRel(TMatr.sumElements(wd),"sum(wd)",mom_p[0],"mom_p[0]",eps);
	sortNodes(xd,wd);
	UnitTests.checkDoubleArrayRel(xd,"xd",xm,"xm",eps);
	UnitTests.checkDoubleArrayRel(wd,"wd",wm,"wm",eps);
	test_QuadratureBasis(xd,wd,mom_b,2*neq-1,eps);


	final double [] root_re=new double [xm.length],root_im=new double [xm.length];
	final int nbroots=O.getPolynomialRootsFinderInBasis().findRoots(bnodes.length-1,bnodes,root_re,root_im);
	if(nbroots!=xm.length) throw new RuntimeException("xm.length="+xm.length+" nbroots="+nbroots);
	sortNodes(root_re,root_im);
	UnitTests.checkDoubleArrayRel(root_re,"root_re",xm,"xm",eps);
	UnitTests.checkDoubleArrayRel(root_im,"root_im",new double [xm.length],"0",eps);
    }

    final double [] xs=getXNodes(nodes);
    sortNodes(xs,null);
    final double [] ws=getWeightsChristoffel(xs,p);
    UnitTests.checkDoubleArrayRel(xt,"xt",xs,"xs",eps);
    UnitTests.checkDoubleArrayRel(wt,"wt",ws,"ws",eps);

    final double [] ws2=getWeigthsSDiv(nodes,xs,p);
    UnitTests.checkDoubleArrayRel(ws,"ws",ws2,"ws2",eps);

    //System.err.println("mom_p="+java.util.Arrays.toString(mom_p));
    final double [] bnodes=b[neq];
    final double [] bnx=bnodes.clone();
    final double [][] ab=O.getAB(b);
    final double [] ax=ab[0].clone(),bx=ab[1].clone();
    final double [] xb=O.getXNodesFromAB(neq,ab[0],ab[1]),xbp=O.getXNodes(bnodes);
    sortNodes(xb,null);
    sortNodes(xbp,null);
    UnitTests.checkDoubleArrayRel(bnodes,"bnodes",bnx,"bnx",eps);
    UnitTests.checkDoubleArrayRel(ab[0],"ab[0]",ax,"ax",eps);
    UnitTests.checkDoubleArrayRel(ab[1],"ab[1]",bx,"bx",eps);
    UnitTests.checkDoubleArrayRel(xb,"xb",xbp,"xbp",eps);

    final double [] xbx=O.getXNodesFromAB(neq,ab[0],ab[1]);
    sortNodes(xbx,null);
    UnitTests.checkDoubleArrayRel(xs,"xs",xb,"xb",eps);
    UnitTests.checkDoubleArrayRel(xbx,"xbx",xb,"xb",eps*0);
    final double bb00=Math.pow(O.fx(b[0],0),2);
    UnitTests.checkDoubleRel(1/bb00,"1/bb00",const_aver,"const_aver",eps);
    UnitTests.checkDoubleRel(TMatr.sumElements(ws),"sum(ws)",1/bb00,"1/bb00",eps);
    UnitTests.checkDoubleArrayRel(ws,"ws",O.getWeights(bnodes,xb,b),"wb",eps);
    UnitTests.checkDoubleArrayRel(ws,"ws",O.getWeightsByIntegration(bnodes,xb,ReadWriteUtils.shrinkArray(bnodes.length-1,mom_b)),"intwb",eps);
    UnitTests.checkDoubleArrayRel(ws,"ws",O.getWeightsChristoffel(xb,b),"Christoffel",eps);


    final double [][] xwab=O.getQuadraturesFromAB_EV(neq,ReadWriteUtils.shrinkArray(neq+1,ab[0]),ReadWriteUtils.shrinkArray(neq,ab[1]));
    final double [] xab=xwab[0],wab=xwab[1];
    sortNodes(xab,wab);
    UnitTests.checkDoubleArrayRel(xab,"xab",xs,"xs",eps);
    UnitTests.checkDoubleArrayRel(wab,"wab",ws,"ws",eps);

    // one less a & b element available
    final double [][] abq=O.getABFromQuadrature(xab,wab);
    UnitTests.checkDoubleArrayRelFirstindex(abq[0].length,abq[0],"abq[0]",ab[0],"ab[0]",eps);
    UnitTests.checkDoubleArrayRelFirstindex(abq[1].length,abq[1],"abq[1]",ab[1],"ab[1]",eps);
    /* 1 less element to check
    final double [][] qabq=O.getQuadraturesFromAB_EV(neq,abq[0],abq[1]);
    sortNodes(qabq[0],qabq[1]);
    UnitTests.checkDoubleArrayRel(qabq[0],"qabq[0]",xs,"xs",eps);
    UnitTests.checkDoubleArrayRel(qabq[1],"qabq[1]",ws,"ws",eps);
    */


    System.err.println(name+":test_RegularNodes(): test OK"+" neq="+neq+" eps="+eps);
}


void test_WExtremal(final int n)
{
    if(O.B==null) return;

    final double [][] nr=O.getQuadraturesFromPolynomials(n,O.getOrthogonalPolynomialsFirstKind(n+1,mom_b));
    final double [] x=nr[0],w=nr[1],nodes=nr[2];
    final double [] QQ=O.getQQMatr(n,mom_b);

    final double [] psi=new double [nodes.length-1];
    for(int k=0;k<x.length;k++)
    {
	final double xk=x[k];
	O.sdiv1(psi,nodes,xk);
	final double fp=O.fx(psi,xk),fdp=O.Dfx(psi,xk);
	final double df=fdp/fp,wk=TMatr.getAMatrB(psi.length,psi,0,psi.length,psi,0,QQ,n)/(fp*fp);
	final double [] dpsi=O.B.getDifferentiated(psi.length-1,psi);
	final double ddm=TMatr.getAMatrB(psi.length,psi,0,dpsi.length,dpsi,0,QQ,n)/(fp*fp),dd=ddm-wk*df;
	System.err.println("test_WExtremal: x["+k+"]="+x[k]+" wk="+wk+" ddm="+ddm+" df="+df+" dd="+dd);
	UnitTests.checkDoubleRel(wk,"wk",w[k],"w[k]",eps);
	UnitTests.checkDoubleRel(dd,"dd",0,"0",eps);
    }

    System.err.println(name+":test_WExtremal(): test OK"+" neq="+n+" eps="+eps);
}

void test_RadauNodes(final int neq)
{
    // Radau nodes
    final double [][] pradau=getOrthogonalPolynomialsFirstKind(neq+1,getRadauMomentsFromMoments(mom_p));
    final double [][] nd=test_NodesPolynomial(getRadauMultiplied(pradau[neq]),mom_p,2*neq);
    final double [] xt=nd[0],wt=nd[1];
    sortNodes(xt,wt);

    final double [][] nc=getRadauQuadraturesFromPolynomials(neq,getOrthogonalPolynomialsFirstKind(neq+2,mom_p));
    final double [] xr=nc[0],wr=nc[1];
    UnitTests.checkDoubleRel(TMatr.sumElements(wr),"sum(wr)",mom_p[0],"mom_p[0]",eps);
    sortNodes(xr,wr);
    UnitTests.checkDoubleArrayRel(xr,"xr",xt,"xt",eps);
    UnitTests.checkDoubleArrayRel(wr,"wr",wt,"wt",eps);
    UnitTests.checkDoubleRel(TMatr.sumElements(wr),"sum(wr)",mom_p[0],"mom_p[0]",eps);

    final double [][] nb=O.getRadauQuadraturesFromPolynomials(neq,O.getOrthogonalPolynomialsFirstKind(neq+2,ReadWriteUtils.shrinkArray(2*(neq+2)-1,mom_b)));
    final double [] xb=nb[0],wb=nb[1],bnodes=nb[2];
    UnitTests.checkDoubleRel(TMatr.sumElements(wb),"sum(wb)",mom_p[0],"mom_p[0]",eps);
    sortNodes(xb,wb);
    UnitTests.checkDoubleArrayRel(xr,"xr",xb,"xb",eps);
    UnitTests.checkDoubleArrayRel(wr,"wr",wb,"wb",eps);
    UnitTests.checkDoubleRel(TMatr.sumElements(wb),"sum(wb)",mom_p[0],"mom_p[0]",eps);


    final double [][] ne=O.getRadauQuadraturesFromMoments_EV(neq,ReadWriteUtils.shrinkArray(2*neq+1,mom_b));
    final double [] xe=ne[0],we=ne[1],bnodese=ne[2];
    UnitTests.checkDoubleRel(TMatr.sumElements(we),"sum(we)",mom_p[0],"mom_p[0]",eps);
    sortNodes(xe,we);
    UnitTests.checkDoubleArrayRel(xe,"xe",xr,"xr",eps);
    UnitTests.checkDoubleArrayRel(we,"we",wr,"wr",eps);

    for(double [] no:new double [][]{bnodes,bnodese})
    {
	final double [] root_re=new double [xb.length],root_im=new double [xb.length];
	final int nbroots=O.getPolynomialRootsFinderInBasis().findRoots(no.length-1,no,root_re,root_im);
	if(nbroots!=xb.length) throw new RuntimeException("xb.length="+xb.length+" nbroots="+nbroots);
	sortNodes(root_re,root_im);
	UnitTests.checkDoubleArrayRel(root_re,"root_re",xb,"xb",eps);
	UnitTests.checkDoubleArrayRel(root_re,"root_re",xe,"xe",eps);
	UnitTests.checkDoubleArrayRel(root_im,"root_im",new double [xb.length],"0",eps);
    }

    System.err.println(name+":test_RadauNodes(): test OK"+" neq="+neq+" eps="+eps);
}

void test_Type2(final int neq)
{
    final double [][] p=getOrthogonalPolynomialsFirstKind(neq+1,mom_p);
    final double [][] qC=getSecondKindOrthogonalPolynomialsFromFirstKindAndMoments(p,mom_p),
	qP=getSecondKindOrthogonalPolynomialsFromFirstKind(p);
    final double [][] qB=O.getSecondKindOrthogonalPolynomials(O.getOrthogonalPolynomialsFirstKind(neq+1,mom_b));
    for(int k=0;k<qC.length;k++)
    {
	UnitTests.checkDoubleArrayRel(qC[k],"qC["+k+"]",qP[k],"qP["+k+"]",eps);
	UnitTests.checkDoubleArrayRel(qC[k],"qC["+k+"]",convertToMonomials(qB[k]),"conv(qB["+k+"])",eps);
    }

    System.err.println(name+":test_Type2(): test OK"+" neq="+neq+" eps="+eps);
}
void test_TypeK(final int neq)
{
    double [][] p=getOrthogonalPolynomialsFirstKind(neq+1,mom_p);
    final double [][] ab=O.getAB(O.getOrthogonalPolynomialsFirstKind(neq+1,mom_b));
    double [] ra=ab[0],rb=ab[1];
    for(int kind=1,n=ab[0].length;kind<ab[0].length;kind++,n--)
    {
	final double [] mom=getMomentsFromPolynomial(p);
	p=getSecondKindOrthogonalPolynomialsFromFirstKindAndMoments(p,mom);
	final double [][] b=O.getOrthogonalPolynomialsFromAB(kind,ab[0],ab[1]);
	ra=TMatr.remove0Element(ra);
	rb=TMatr.remove0Element(rb);
	final double [][] rpb=O.getOrthogonalPolynomialsFirstKindFromAB(ra,rb);
	for(int k=0;k<b.length;k++)
	{
	    UnitTests.checkDoubleArrayRel(p[k],"p["+k+"]",convertToMonomials(b[k]),"conv(b["+k+"])",eps);
	    UnitTests.checkDoubleArrayRel(rpb[k],"rpb["+k+"]",b[k],"b["+k+"]",eps);
	}
    }
    System.err.println(name+":test_TypeK(): test OK"+" neq="+neq+" eps="+eps);
}
void test_Polynomials(final int neq)
{
    final double [][] p=getOrthogonalPolynomialsFirstKind(neq,mom_p);
    // out of 2*neq-1 moments returned by getMomentsFromPolynomial
    // only 2*neq-2 match.
    if(p.length>1)UnitTests.checkDoubleArrayRelFirstindex(2*neq-2,mom_p,"mom_p",getMomentsFromPolynomial(p),"mom_p(p)",eps);
    final double [][] pb=O.getOrthogonalPolynomialsFirstKind(neq,ReadWriteUtils.shrinkArray(2*neq-1,mom_b));
    if(pb.length>1)
    {
	final double [] mom_bb=O.getMomentsFromPolynomial(pb);
	UnitTests.checkDoubleArrayRelFirstindex(mom_bb.length,mom_bb,"mom_b(pb)",mom_b,"mom_b",eps);
	final double [][] ab=O.getAB(pb);
	final double [] momab_b=O.getMomentsFrom_AB(ab[0],ab[1]);
	UnitTests.checkDoubleArrayRel(mom_bb,"mom_b(pb)",momab_b,"momab_b",eps);
    }

    for(int k=0;k<pb.length;k++)
    {
	UnitTests.checkDoubleArrayRel(p[k],"p["+k+"]",convertToMonomials(pb[k]),"conv(pb["+k+"])",eps);
    }
    final double [][] qp=getSecondKindOrthogonalPolynomialsFromFirstKind(p),
	qpc=getSecondKindOrthogonalPolynomialsFromFirstKindAndMoments(p,mom_p),
	qpb=O.getSecondKindOrthogonalPolynomials(pb);
    for(int k=0;k<qpb.length;k++)
    {
	UnitTests.checkDoubleArrayRel(qp[k],"qp["+k+"]",convertToMonomials(qpb[k]),"conv(qpb["+k+"])",eps);
	UnitTests.checkDoubleArrayRel(qpc[k],"qpc["+k+"]",convertToMonomials(qpb[k]),"conv(qpb["+k+"])",eps);
    }
    System.err.println(name+":test_Polynomials(): test OK, neq="+neq+" eps="+eps);
}
void test_PolynomialsGrammSchmidt(final int neq)
{
  for(int no=0;no<=2;no++)
  {
     final int nm=neq+no;
     final double [][] pb=O.getOrthogonalized(neq,O.getQQMatr(nm,ReadWriteUtils.shrinkArray(2*nm-1,mom_b)),nm);
     // test with QQ leading dimension!=neq
     final double [][] pb_grammschmidt=O.getGrammSchmidtOrthogonalized(neq,O.getQQMatr(neq+2,mom_b),neq+2);
     if(pb.length!=pb_grammschmidt.length) throw new RuntimeException("pb.length="+pb.length+" pb_grammschmidt.length="+pb_grammschmidt.length);

    for(int k=0;k<pb.length;k++)
    {
	final double [] pk=pb[k],pgsk=pb_grammschmidt[k].clone();
	if(Math.signum(pgsk[pgsk.length-1])!=Math.signum(pk[pk.length-1]))
	{
	    for(int j=pgsk.length;--j>=0;) pgsk[j]=-pgsk[j];
	}
	UnitTests.checkDoubleArrayRel(pk,"pk",pgsk,"pgsk",eps);
    }
  }
  System.err.println(name+":test_PolynomialsGrammSchmidt(): test OK, neq="+neq+" eps="+eps);
}
void test_AB(final int neq)
{
    final double [][] p=getOrthogonalPolynomialsFirstKind(neq,mom_p),
	b=O.getOrthogonalPolynomialsFirstKind(neq,ReadWriteUtils.shrinkArray(2*neq-1,mom_b));
    for(int k=0;k<p.length;k++)
    {
	UnitTests.checkDoubleArrayRel(p[k],"p["+k+"]",convertToMonomials(b[k]),"conv(b["+k+"])",eps);
    }
    final double [][] ab_p=getAB(p),ab_b=O.getAB(b);
    UnitTests.checkDoubleArrayRel(ab_p[0],"ab_p[0]",ab_b[0],"ab_b[0]",eps);
    UnitTests.checkDoubleArrayRel(ab_p[1],"ab_p[1]",ab_b[1],"ab_b[1]",eps);
    //System.err.println("a="+java.util.Arrays.toString(ab_b[0])+" b="+java.util.Arrays.toString(ab_b[1]));
    if(true)
    {
	final double [][] vp=getOrthogonalPolynomialsFirstKindFromAB(ab_p[0],ab_p[1]),
	    vb=O.getOrthogonalPolynomialsFirstKindFromAB(ab_b[0],ab_b[1]);
	for(int k=0;k<p.length;k++)
	{
	    UnitTests.checkDoubleArrayRel(p[k],"p["+k+"]",vp[k],"vp[k]",eps);
	    UnitTests.checkDoubleArrayRel(b[k],"b["+k+"]",vb[k],"vb[k]",eps);
	}
    }
    if(true)
    {
	final double [][] vp=getSecondKindOrthogonalPolynomialsFromAB(ab_p[0],ab_p[1]),
	    vc=getSecondKindOrthogonalPolynomialsFromFirstKindAndMoments(p,mom_p),
	    vb=O.getSecondKindOrthogonalPolynomialsFromAB(ab_b[0],ab_b[1]);
	for(int k=0;k<vp.length;k++)
	{
	    UnitTests.checkDoubleArrayRel(vp[k],"vp["+k+"]",convertToMonomials(vb[k]),"conv(vb["+k+"])",eps);
	    UnitTests.checkDoubleArrayRel(vc[k],"vc["+k+"]",convertToMonomials(vb[k]),"conv(vb["+k+"])",eps);
	}
	final double [][] ab2_p=getAB(vp),ab2_b=O.getAB(vb);
	UnitTests.checkDoubleArrayRel(ab2_p[0],"ab2_p[0]",ab2_b[0],"ab2_b[0]",eps);
	UnitTests.checkDoubleArrayRel(ab2_p[1],"ab2_p[1]",ab2_b[1],"ab2_b[1]",eps);

	UnitTests.checkDoubleArrayRel(ab2_b[0],"ab2_b[0]",TMatr.remove0Element(ab_b[0]),"rem0(ab_b[0])",eps);
	UnitTests.checkDoubleArrayRel(ab2_b[1],"ab2_b[1]",TMatr.remove0Element(ab_b[1]),"rem0(ab_b[1])",eps);
    }

    System.err.println(name+":test_AB(): test OK neq="+neq+" eps="+eps);
}

void test_getQxFromAB(final int neq)
{
    final double [][] p=getOrthogonalPolynomialsFirstKind(neq,mom_p),
	b=O.getOrthogonalPolynomialsFirstKind(neq,ReadWriteUtils.shrinkArray(2*neq-1,mom_b));
    final double [][] _ab=O.getAB(b);
    for(int k=0;k<p.length;k++)
    {
	final double [] Pb=getBasisRandomMoments(k+1,this.x_min,this.x_max,2*p.length+2,this.r);
	final double [] A=ReadWriteUtils.shrinkArray(k+1,_ab[0]),B=ReadWriteUtils.shrinkArray(k,_ab[1]);
	for(int t=0;t<10000;t++)
	{
	    final double x=-r.nextDouble();
	    final double bx=O.fx(b[k],x),px=Polynomials.getD0NoScale(p[k],x),
		bx_ab=O.getQxFromAB(k,A,B,x);
	    UnitTests.checkDoubleRel(bx,"bx",px,"px",eps);
	    UnitTests.checkDoubleRel(bx,"bx",bx_ab,"bx_ab",eps);
	    final double f=O.getSUMQxFromAB(Pb,x,A,B);
	    double sf=0;
	    for(int j=0;j<Pb.length;j++){sf+=Pb[j]*O.getQxFromAB(j,A,B,x);}
	    UnitTests.checkDoubleRel(f,"f",sf,"sf",eps);
	}
    }

    System.err.println(name+":test_getQxFromAB(): test OK neq="+neq+" eps="+eps);
}

void test_getABRadauModified(final int neq)
{
    if(neq<=1) return;
    // test Radau AB
    for(int n=neq,nmax=neq+2;n<=nmax;n++)
    {
	final double [][] rr=getOrthogonalPolynomialsFirstKind(nmax-1,getRadauMomentsFromMoments(mom_p));

	final double [][] bb=O.getOrthogonalPolynomialsFirstKind(nmax,mom_b);
	final double [][] ab_bb=O.getAB(bb);
	final double [][] abr=O.getABRadauModified(n,ab_bb[0],ab_bb[1]);
	final double [][] bradau=O.getOrthogonalPolynomialsFirstKindFromAB(abr[0],abr[1]);
	final double [] rn=getRadauMultiplied(rr[n-2]),bn=convertToMonomials(bradau[n-1]);
	final double f=rn[rn.length-1]/bn[bn.length-1];for(int k=0;k<bn.length;k++) bn[k]*=f;
	UnitTests.checkDoubleArrayRel(rn,"rn",bn,"bn",eps);
	for(int k=0;k<bradau.length;k++)
	{
	    if(k<bradau.length-1)
	    {
		UnitTests.checkDoubleArrayRel(bradau[k],"bradau[k]",bb[k],"bb[k]",eps);
	    }
	    else
	    {
		final double [] brad=O.getRadauPolynomials(bb)[k-1],bmulrad=O.getRadauMultiplied(brad);
		final double ff=bradau[k][k]/bmulrad[k];
		for(int j=0;j<bmulrad.length;j++) bmulrad[j]*=ff;
		UnitTests.checkDoubleArrayRel(bradau[k],"bradau[k]",bmulrad,"bmulrad",eps);
	    }
	    //System.err.println("k="+k+" fx("+getX0()+")="+O.fx(bradau[k],getX0()));
	}
	final int nq=bradau[bradau.length-1].length-1;
	System.err.println("x0="+getX0()+" n="+n+" neq="+neq+" F_end="+O.fx(bradau[n-1],getX0())+" nq="+nq+" (abr[0].length-1)="+(abr[0].length-1));
	//UnitTests.checkDoubleArrayRel(rr[n-1],"rr[n-1]",convertToMonomials(bradau[n-1]),"conv(bradau[n-1])",eps);

	final double [][] qd_ab=O.getQuadraturesFromAB_EV(nq,abr[0],abr[1]),
	    qd_pol=O.getRadauQuadraturesFromPolynomials(nq-1,bb);
	sortNodes(qd_ab[0],qd_ab[1]);
	sortNodes(qd_pol[0],qd_pol[1]);
	UnitTests.checkDoubleArrayRel(qd_ab[0],"qd_ab[0]",qd_pol[0],"qd_pol[0]",eps);
	UnitTests.checkDoubleArrayRel(qd_ab[1],"qd_ab[1]",qd_pol[1],"qd_pol[1]",eps);
    }
    System.err.println(name+":test_getABRadauModified(): test OK neq="+neq+" eps="+eps);
}

void test_RadauMoments_directly_Bbasis(final double [] bnodes,final double [] rbnodes){}
void test_RadauMoments(final int neq)
{
    final double [] rp=getRadauMomentsFromMoments(mom_p);
    final double [] rb=O.getRadauMomentsFromMoments(mom_b);
    final double [] rb_fromrp=getBasisMomentsConvertedFromMonomials(rp);
    // may be rather unstable
    if(rp.length<=10) UnitTests.checkDoubleArrayRel(rb,"rb",rb_fromrp,"rb_fromrp",eps);
    final double [][] brad=O.getRadauPolynomials(O.getOrthogonalPolynomialsFirstKind(neq+1,mom_b));
    final double [][] bradc=O.getOrthogonalPolynomialsFirstKind(neq,rb);
    for(int k=0;k<brad.length;k++)
    {
	UnitTests.checkDoubleArrayRel(brad[k],"brad[k]",bradc[k],"bradc[k]",eps);
    }
    test_RadauMoments_directly_Bbasis(mom_b,rb);
    System.err.println(name+":test_RadauMoments(): test OK neq="+neq+" eps="+eps);
}

void test_WeightsAndNodes(final int neq)
{
    final double [][] p=getOrthogonalPolynomialsFirstKind(neq+1,mom_p);
    final double [][] pb=O.getOrthogonalPolynomialsFirstKind(neq+1,mom_b);
    final double [] pnodes=p[neq],pbnodes=pb[neq];
    final double [][] pab=getAB(p),ab=O.getAB(pb);
    UnitTests.checkDoubleArrayRel(ab[0],"ab[0]",pab[0],"pab[0]",eps);
    UnitTests.checkDoubleArrayRel(ab[1],"ab[1]",pab[1],"pab[1]",eps);

    final double [] xp=getXNodes(pnodes),xb=O.getXNodesFromAB(neq,ab[0],ab[1]),xbx=O.getXNodesFromAB(neq,ab[0],ab[1]),xbp=O.getXNodes(pbnodes);
    sortNodes(xp,null);
    sortNodes(xb,null);
    sortNodes(xbx,null);
    sortNodes(xbp,null);
    UnitTests.checkDoubleArrayRel(xp,"xp",xb,"xb",eps);
    UnitTests.checkDoubleArrayRel(xbx,"xbx",xb,"xb",eps*0);
    UnitTests.checkDoubleArrayRel(xbp,"xbp",xb,"xb",eps);

    final double [] wp=getWeigthsSDiv(pnodes,xp,p),
	wpin=getWeightsByIntegration(pnodes,xp,mom_p),
	wb=O.getWeights(pbnodes,xb,pb),
	wbin=O.getWeightsByIntegration(pbnodes,xb,ReadWriteUtils.shrinkArray(pbnodes.length-1,mom_b));
    UnitTests.checkDoubleArrayRel(wp,"wp",wb,"wb",eps);
    UnitTests.checkDoubleArrayRel(wpin,"wpin",wb,"wb",eps);
    UnitTests.checkDoubleArrayRel(wbin,"wbin",wb,"wb",eps);

    System.err.println(name+":test_WeightsAndNodes(): test OK neq="+neq+" eps="+eps);
}

void test_PsiAver(final int n)
{
    final int nmoms=2*n-1,tsize=1000+n*n*10;
    final double [] ml=getBasisRandomMoments(nmoms,this.x_min,this.x_max,tsize,this.r);
    final double [] Rev=O.EV.getGEV_Generalized(n,O.getQQMatr(n,ml),n,O.getQQMatr(n,mom_b),n,true),psi=new double [n];
    for(int k=0;k<n;k++)
    {
	System.arraycopy(Rev,n+k*n,psi,0,n);
	final double [] psi2=O.getPQMultiplied(psi,psi);
	final double s1=TMatr.mulVV(psi.length,psi,ml),w1=TMatr.mulVV(psi.length,psi,mom_b),
	    s2=TMatr.mulVV(psi2.length,psi2,ml),w2=TMatr.mulVV(psi2.length,psi2,mom_b);
	UnitTests.checkDoubleRel(w2,"w2",1,"1",eps);
	UnitTests.checkDoubleRel(s2/w2,"s2/w2",Rev[k],"Rev[k]",eps);
	UnitTests.checkDoubleRel(s1/w1,"s1/w1",Rev[k],"Rev[k]",eps);
    }

    System.err.println(name+":test_PsiAver(): test OK n="+n+" eps="+eps);
}
void test_DistributionSkewness()
{
    final Skewness sk=Skewness.getSkewness(mom_b[0],mom_b[1],mom_b[2],mom_b[3],O),
	sk2=Skewness.getSkewness2(mom_b[0],mom_b[1],mom_b[2],mom_b[3],O),
	psk=Skewness.getSkewnessFromMonomialMoments(mom_p[0],mom_p[1],mom_p[2],mom_p[3]),
	psk2=Skewness.getSkewnessFromMonomialMoments2(mom_p[0],mom_p[1],mom_p[2],mom_p[3]);
    Skewness.matchSkewness(sk,sk2,eps);
    Skewness.matchSkewness(sk,psk,eps);
    Skewness.matchSkewness(sk,psk2,eps);
    final double [] Xmom_b=O.getXFMomentsFromMoments(mom_b);
    final Skewness sk3=Skewness.getGSkewness(mom_b[0],mom_b[1],mom_b[2],Xmom_b[0],Xmom_b[1],Xmom_b[2],O);
    Skewness.matchSkewness(sk,sk3,eps);

    // Test explicit skewness expression
    UnitTests.checkDoubleRel(sk.sk,"sk.sk",(2*sk.xa-sk.x1-sk.x2)/(sk.x1-sk.x2),"(2*sk.xa-sk.x1-sk.x2)/(sk.x1-sk.x2)",eps);
    UnitTests.checkDoubleRel(sk.sk,"sk.sk",(sk.w1-sk.w2)/(sk.w1+sk.w2),"(sk.w1-sk.w2)/(sk.w1+sk.w2)",eps);
    final ProbabilityCorrelation pc=
	new ProbabilityCorrelation(mom_p[0],mom_p[1],mom_p[2],
				   mom_p[1],mom_p[2],mom_p[3],
				   mom_p[1],mom_p[2],mom_p[3]);
    UnitTests.checkDoubleRel(sk.sk,"sk.sk",pc.skewnessLike_p,"pc.skewnessLike_p",eps);
    UnitTests.checkDoubleRel(sk.sk,"sk.sk",pc.skewnessLike_i,"pc.skewnessLike_i",eps);
    UnitTests.checkDoubleRel(1,"1",pc.corr,"pc.corr",eps);


    final double [] momI_p=getPolynomialRandomMoments(3,this.x_min,this.x_max,100,this.r),
	momI_b=getBasisMomentsConvertedFromMonomials(momI_p),
	momP_p=getPolynomialRandomMoments(3,this.x_min,this.x_max,100,this.r),
	momP_b=getBasisMomentsConvertedFromMonomials(momP_p);
    final ProbabilityCorrelation c=new ProbabilityCorrelation(mom_p[0],mom_p[1],mom_p[2],
							      momP_p[0],momP_p[1],momP_p[2],
							      momI_p[0],momI_p[1],momI_p[2]),
	bc=new ProbabilityCorrelation(mom_b[0],mom_b[1],mom_b[2],
				      momP_b[0],momP_b[1],momP_b[2],
				      momI_b[0],momI_b[1],momI_b[2],
				      O,eps);
    ProbabilityCorrelation.matchProbabilityCorrelation(c,bc,eps);

    final Skewness gP=Skewness.getGSkewnessFromMonomialMoments(mom_p[0],mom_p[1],mom_p[2],momP_p[0],momP_p[1],momP_p[2]),
	bgP=Skewness.getGSkewness(mom_b[0],mom_b[1],mom_b[2],momP_b[0],momP_b[1],momP_b[2],O);
    Skewness.matchSkewness(gP,bgP,eps);

    System.err.println(name+":test_DistributionSkewness(): test OK eps="+eps);
}

void test_Discrete(final int n)
{
    final double [][] xw=getXWDiscrete(n);
    final double [] x=xw[0],w=xw[1];
    final double [] bmoms=getBmomsFromDiscrete(2*n,x,w);

    if(true)
    {
	final double [][] xwq=O.getQuadraturesFromMoments_EV(n,ReadWriteUtils.shrinkArray(2*n,bmoms));
	final double [] xq=xwq[0],wq=xwq[1];
	UnitTests.checkDoubleArrayRel(x,"x",xq,"xq",eps);
	UnitTests.checkDoubleArrayRel(w,"w",wq,"wq",eps);
    }

    final double [] w0=new double [n],wc=w.clone(),xc=x.clone();
    java.util.Arrays.fill(w0,1);
    sortNodes(wc,xc);

    final double [][] q2=getD2QuadraturesFromMoments_EV(n,bmoms,getBmomsFromDiscrete(2*n,x,w0));
    final double [] wq=q2[0],xv=q2[1],xt=q2[2];

    UnitTests.checkDoubleArrayRel(wq,"wq",wc,"wc",eps);
    UnitTests.checkDoubleArrayRel(xv,"xv",xt,"xt",eps);
    UnitTests.checkDoubleArrayRel(xc,"xc",xt,"xt",eps);

    if(true)
    {
	final double [] QQbv=O.getQQMatr(n+1,getBmomsFromDiscrete(2*n+1,x,w)),QQbt=O.getQQMatr(n+1,mom_b);
	final double [] ev=O.EV.getGEV_Generalized(n+1,QQbv,n+1,QQbt,n+1,true);
	final double [] psi=new double [n+1],xre=new double [n],xim=new double [n];
	System.arraycopy(ev,n+1+0,psi,0,n+1);
	final int nr=O.getPolynomialRootsFinderInBasis().findRoots(n,psi,xre,xim);
	if(nr!=n) throw new RuntimeException("nr="+nr+" n="+n);
	sortNodes(xre,xim);
	UnitTests.checkDoubleArrayRel(x,"x",xre,"xre",eps);
	UnitTests.checkDoubleArrayRel(new double [n],"0",xim,"xim",eps);
    }

    System.err.println(name+":test_Discrete(): test OK neq="+n+" eps="+eps);
}

void test_ECOND(final int n)
{
    final double [] momt=mom_b;
    setData(200+30*momt.length,momt.length);
    final double [] momv=mom_b;
    test_ONEECOND(n,momv,momt,"ORD");
    test_ONEECOND(n,O.getRadauMomentsFromMoments(momv),O.getRadauMomentsFromMoments(momt),"RAD");
}

/** Print the results of
Carlos F. Borges,Numer. Math. 67: 271-288 (1994),
On a class of Gauss-like quadrature rules.
x1 = 0.791263; x2 = 0.237959; x3 = 0.427455; x4 = 0.887481
Q1f = PI*(0.079050 f(x1) + 0.204235 f(x2) + 0.176469 f(x3) + 0.040245 f(x4)).
Q2f = 0.082275 f(x1) + 0.507998 f(x2) + 0.866157 f(x3) + 0.543569 f(x4).
The results match exactly.
*/
void test_Borges(final int neq)
{
    final int [] multiindex=new int []{neq,neq};
    final int N=multiindex[0]+multiindex[1];
    final int nmoms=N+neq;
    final double [] pmom1=new double [nmoms],pmom2=new double [nmoms];
    for(int k=0;k<nmoms;k++)
    {
	final boolean e=k%2==0;
	// measure sqrt(1-x^2)dx
	// Gradstein Ryzik, 1963, 3.248.{2,3}, page 308 and integration by parts.
	pmom1[k]=e?Math.PI*Polynomials.fact2(2+k-1)/Polynomials.fact2(2+k)/(k+1):0;
	// measure (1+x)dx
	pmom2[k]=e?2.0/(k+1):2.0/(k+2);
    }
    final double [][] measures_b=new double [][]{
	getBasisMomentsConvertedFromMonomials(pmom1),
	getBasisMomentsConvertedFromMonomials(pmom2)
    };
    final double [][] quad=O.getQuadraturesForMultipleOrthogonalPolynomial(multiindex,measures_b);
    final double [] xe=quad[0],imxe=quad[1];
    System.err.println("test_Borges:N="+N+" multiindex="+java.util.Arrays.toString(multiindex));
    System.err.println("test_Borges:xe="+java.util.Arrays.toString(xe));
    System.err.println("test_Borges:imxe="+java.util.Arrays.toString(imxe));
    final int off_s=2;
    for(int k=0;k<multiindex.length;k++)
    {
	System.err.println("test_Borges:W["+k+"]="+java.util.Arrays.toString(quad[off_s+k]));
	test_QuadratureBasis(xe,quad[off_s+k],measures_b[k],N-1+multiindex[k],eps);
    }
    System.err.println(name+":test_Borges(): test OK"+" neq="+neq+" eps="+eps);
}

/** Test multiple orthogonality on the same interval.
 * Because the measures not always form Chebyshev system
 * the unit test often fails.
 */
void test_MultipleOrthogonality(final int neq)
{
    final int [] multiindex=new int [neq];
    int N=0;
    for(int k=0;k<multiindex.length;k++)
    {
	multiindex[k]=r.nextInt(5);
	N+=multiindex[k];
    }
    System.err.println("N="+N+" multiindex="+java.util.Arrays.toString(multiindex)+" x_min="+x_min+" x_max="+x_max);
    final double [][] measures_b=new double [multiindex.length][];
    final int tsize=N*N+1001;
    for(int k=0;k<multiindex.length;k++)
    {
	if(true)
	{
	    //measures_b[k]=getBasisRandomMoments(N+multiindex[k],this.x_min,this.x_max,tsize,this.r);
	    measures_b[k]=getBasisMomentsConvertedFromMonomials(getPolynomialRandomMoments(N+multiindex[k],this.x_min,this.x_max,tsize,this.r));
	}
	else
	{
	    final double [][] xw=getXWDiscrete(2*N+100);
	    final double [] x=xw[0],w=xw[1];
	    measures_b[k]=getBmomsFromDiscrete(N+multiindex[k],x,w);
	}
    }

    final double [][] quad=O.getQuadraturesForMultipleOrthogonalPolynomial(multiindex,measures_b);
    final double [] xe=quad[0],imxe=quad[1];
    for(int k=0;k<imxe.length;k++)
    {
	if(!(imxe[k]==0))
	{
	    System.err.println("N="+N+" xe="+java.util.Arrays.toString(xe)+"\nimxe="+java.util.Arrays.toString(imxe)+" skipping as im part is not 0");
	    return;
	}
    }
    final int off_s=2;
    final double epsS=1e-3; // rather large, the quadratures are often not that good.
    System.err.println("N="+N+" xe="+java.util.Arrays.toString(xe)+"\nim="+java.util.Arrays.toString(quad[1]));
    for(int k=0;k<multiindex.length;k++)
    {
	System.err.println("measure#"+k+" weights="+java.util.Arrays.toString(quad[off_s+k]));
	test_QuadratureBasis(xe,quad[off_s+k],measures_b[k],N-1+multiindex[k],epsS);
    }

    System.err.println(name+":test_MultipleOrthogonality(): test OK"+" neq="+neq+" eps="+eps+" epsS="+epsS);
}


/** Test multiple orthogonality on the disjoin intervals.
 */
void test_MultipleOrthogonality_Disjoin(final int neq)
{
    final int [] multiindex=new int [neq];
    int N=0;
    final double [] intervals=new double [multiindex.length+1];
    intervals[0]=this.x_min;
    final double delta=(this.x_max-this.x_min)/multiindex.length;
    for(int k=0;k<multiindex.length;k++)
    {
	intervals[k+1]=(k+1)*delta;
	multiindex[k]=1+r.nextInt(3);
	N+=multiindex[k];
    }
    System.err.println("N="+N+" multiindex="+java.util.Arrays.toString(multiindex)+" x_min="+x_min+" x_max="+x_max);
    final double [][] measures_b=new double [multiindex.length][];
    final int tsize=N*N+1001;
    for(int k=0;k<multiindex.length;k++)
    {
	measures_b[k]=getBasisRandomMoments(N+multiindex[k],intervals[k],intervals[k+1],tsize,this.r);
    }

    final double [][] quad=O.getQuadraturesForMultipleOrthogonalPolynomial(multiindex,measures_b);
    final double [] xe=quad[0],imxe=quad[1];
    final int off_s=2;

    System.err.println("N="+N+" xe="+java.util.Arrays.toString(xe)+"\nim="+java.util.Arrays.toString(quad[1]));
    final double epsS=1e-3; // rather large, the quadratures are often not that good.
    for(int k=0;k<multiindex.length;k++)
    {
	int nintk=0;
	for(int j=0;j<xe.length;j++)
	{
	    if(intervals[k]<=xe[j] && xe[j]<=intervals[k+1])
	    {
		nintk++;
	    }
	}
	// the number of roots on k-th interval must be equal to multiindex.
	if(nintk!=multiindex[k])
	{
	    throw new RuntimeException("N="+N+" k="+k+" nintk="+k+" multiindex["+k+"]="+multiindex[k]);
	}
	// often not perfect, because support intervals are disjoin
	System.err.println("interval#"+k+" ["+intervals[k]+";"+intervals[k+1]+"] weights="+java.util.Arrays.toString(quad[off_s+k]));
	// many points are outside of the support of k-th measure
	// weights are ofthen negative & unstable for large number of intervals.
	if(false)
	{
	test_QuadratureBasis(xe,quad[off_s+k],measures_b[k],N-1+multiindex[k],epsS);
	}
    }

    System.err.println(name+":test_MultipleOrthogonality_Disjoin(): test OK"+" neq="+neq+" eps="+eps+" epsS="+epsS);
}




void test_Kronrod(final int n,final double [] moms,final boolean flag_print)
{
    final double [][] quad=O.getKronrodQuadratures(n,ReadWriteUtils.shrinkArray(3*n+2,moms));
    if(flag_print)
    {
	System.err.println("test_Kronrod: n="+n+" n_points="+(quad[0].length+quad[3].length));
	System.err.println("test_Kronrod:G:x="+java.util.Arrays.toString(quad[0]));
	System.err.println("test_Kronrod:G:im(x)="+java.util.Arrays.toString(quad[1]));
	System.err.println("test_Kronrod:G:w="+java.util.Arrays.toString(quad[2]));
	System.err.println("test_Kronrod:K:x="+java.util.Arrays.toString(quad[3]));
	System.err.println("test_Kronrod:K:im(x)="+java.util.Arrays.toString(quad[4]));
	System.err.println("test_Kronrod:K:w="+java.util.Arrays.toString(quad[5]));
    }

    final double [] xx=new double [quad[0].length+quad[3].length],ww=new double [xx.length];
    System.arraycopy(quad[0],0,xx,0,quad[0].length);
    System.arraycopy(quad[3],0,xx,quad[0].length,quad[3].length);
    System.arraycopy(quad[2],0,ww,0,quad[0].length);
    System.arraycopy(quad[5],0,ww,quad[0].length,quad[3].length);

    test_QuadratureBasis(xx,ww,moms,3*n+1,eps);

    System.err.println(name+":test_Kronrod(): test OK"+" n="+n+" eps="+eps);
}


void test_Kronrod_random(final int n)
{
    final int tsize=1000+n*n*10;
    final double [] mom_b=getBasisRandomMoments(3*n+2,this.x_min,this.x_max,tsize,this.r);
    test_Kronrod(n,mom_b,true);
}


void test_ONEECOND(final int n,final double [] momv,final double [] momt,final String txt)
{

    final double [] QQbv=O.getQQMatr(n,momv),QQbt=O.getQQMatr(n,momt);
    final double [] ev=O.EV.getGEV_Generalized(n,QQbv,n,QQbt,n,true);

    final double [] psi=new double [n];
    final double [] xre=new double [n-1],xim=new double [n-1],x0=new double [n-1];
    for(int k=0;k<n;k++)
    {
	final double lam=ev[k];
	System.arraycopy(ev,n+k*n,psi,0,n);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(n,psi,QQbv,n),"<psi|QQbv|psi>",lam,"lam",eps);
	UnitTests.checkDoubleRel(TMatr.getAMatrA(n,psi,QQbt,n),"<psi|QQbt|psi>",1,"1",eps);
	final double apsi_v=TMatr.mulVV(psi.length,psi,momv),apsi_t=TMatr.mulVV(psi.length,psi,momt);

	final int nr=O.getPolynomialRootsFinderInBasis().findRoots(n-1,psi,xre,xim);
	sortNodes(xre,xim);
	System.err.println("k="+k+" lam="+lam+" xre="+java.util.Arrays.toString(xre)+" xim="+java.util.Arrays.toString(xim)+"\tapsi_v="+apsi_v+" apsi_t="+apsi_t);
	if(nr!=(n-1)) throw new RuntimeException("nr="+nr+" n="+n);
	if(n>=2 && (k==0||k==(n-1)) )
	{
	    /*
	    final double [] xmom=new double [momt.length];
	    final int sign=k<=0?1:-1;
	    for(int j=0;j<momt.length;j++)
	    {
		xmom[j]=sign*(momv[j]-ev[k]*momt[j]);
	    }
	    final double [][] qa=O.getQuadraturesFromMoments_EV(n-1,xmom);
	    sortNodes(qa[0],qa[1]);
	    final double [] x_qa=qa[0],w_qa=qa[1];
	    System.err.println("k="+k+" x_qa="+java.util.Arrays.toString(x_qa)+" w_qa="+java.util.Arrays.toString(w_qa));
	    UnitTests.checkDoubleArrayRel(xre,"xre",x_qa,"x_qa",eps);
	    */
	    final double [] psd=new double [psi.length-1];
	    for(int m=0;m<xre.length;m++)
	    {
		final double rem=O.sdiv1(psd,psi,xre[m]);
		final double bt=TMatr.getAMatrB(psi.length,psi,0,psd.length,psd,0,QQbt,n),
		    bv_lam=TMatr.getAMatrB(psi.length,psi,0,psd.length,psd,0,QQbv,n)/lam;
		System.err.println("bt="+bt+" bv/lam="+(bv_lam)+" rem="+rem);
		UnitTests.checkDoubleRel(bt,"bt",bv_lam,"bv_lam",eps);
		//UnitTests.checkDoubleRel(rem,"rem",0,"0",eps);
	    }

	    if(xre[0]<=x_min||xre[n-2]>=x_max){
		System.err.println("test_ONEECOND:"+txt+": At least ONE root outside of the interval: xre[0]="+xre[0]+
				   "<x_min="+x_min+" xre["+(n-2)+"]="+xre[n-2]+">x_max="+x_max);
	    }
	    if(n>2&&xre[0]<=x_min&&xre[n-2]>=x_max){
		System.err.println("test_ONEECOND:"+txt+": At TWO roots outside of the interval: xre[0]="+xre[0]+
				   "<x_min="+x_min+" xre["+(n-2)+"]="+xre[n-2]+">x_max="+x_max);
	    }
	}
	if(k==0||k==(n-1))
	{
	    UnitTests.checkDoubleArrayRel(xim,"xim",x0,"0",eps);
	}
    }
    System.err.println(name+":test_ONEECOND():"+txt+": test OK neq="+n+" eps="+eps);
}

int getMaxNeqCut(final int neq){return neq<=4?neq:4;}

void test_Calculations(final int tsize,final int neq)
{
    System.err.println();
    System.err.println("------------------------------------------------------------------------");
    System.err.println(name+":test_Calculations(): test with N points="+tsize+
		       " neq="+neq+" eps="+eps);

    setData(tsize,2*neq+5);
    test_RegularNodes(neq);
    test_WExtremal(neq);
    test_RadauNodes(neq);
    test_Type2(neq);
    test_TypeK(neq);
    test_Polynomials(neq);
    test_PolynomialsGrammSchmidt(neq);
    test_AB(neq);
    test_getQxFromAB(neq);
    test_getABRadauModified(neq);
    test_RadauMoments(neq);
    test_WeightsAndNodes(neq);
    if(mom_b.length>=4) test_DistributionSkewness();
    test_Discrete(neq);
    test_PsiAver(neq);
    test_Kronrod_random(neq);
    test_Borges(getMaxNeqCut(neq));
    test_MultipleOrthogonality_Disjoin(getMaxNeqCut(neq));
    if(false) test_MultipleOrthogonality(neq); // may fail, no guarantee of roots.
    test_ECOND(neq); // must be called the last, clobber moments

    System.err.println(name+":test_Calculations(): test OK with N points="+tsize+
		       " neq="+neq+" eps="+eps);
}
void runTests(final int neq_max)
{
    System.err.println(name+":TestMonomials:starting tests; ver=$Id: TestMonomials.java,v 1.143 2017/08/26 11:28:05 mal Exp $");
    final long t0=System.currentTimeMillis();
    for(int neq=1;neq<=neq_max;neq++)
    {
	for(int i=0;i<118;i++)
	{
	    test_Calculations(2210+r.nextInt(2210),neq);
	}
    }
    final long t1=System.currentTimeMillis();
    System.err.println(name+":TestMonomials: tests OK done in "+(t1-t0)+" milliseconds. neq_max="+neq_max+" ver=$Id: TestMonomials.java,v 1.143 2017/08/26 11:28:05 mal Exp $");
}
} // END OF class TestMonomials
