package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2010
 * This file is under GPL version 3.
 *
 */

/** A crude and simple trial & error + bissesction way to finnd two close to x0 roots.
 *  @version $Id: TwoCloseRootsFinder.java,v 1.16 2015/07/06 06:47:54 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class TwoCloseRootsFinder {


public static class RootsData {
    public static final int TYPE_MIN=101,TYPE_MAX=102,TYPE_UNKNOWN=103;
    public final double x0,x1,x2;
    public final double F0,F1,F2;
    public final double dF0,dF1,dF2;
    public final double d2F0,d2F1,d2F2;
    public final int x1_type,x2_type;
    public final DerivativesCalculateable D;
    public RootsData(final double x0,
		     final double x1,
		     final double x2,
		     final double F0,
		     final double F1,
		     final double F2,
		     final double dF0,
		     final double dF1,
		     final double dF2,
		     final double d2F0,
		     final double d2F1,
		     final double d2F2,
		     final int x1_type,
		     final int x2_type,
		     final DerivativesCalculateable D)
    {
	this.x0=x0;
	this.x1=x1;
	this.x2=x2;
	this.F0=F0;
	this.F1=F1;
	this.F2=F2;
	this.dF0=dF0;
	this.dF1=dF1;
	this.dF2=dF2;
	this.d2F0=d2F0;
	this.d2F1=d2F1;
	this.d2F2=d2F2;
	this.x1_type=x1_type;
	this.x2_type=x2_type;
	this.D=D;
    }
} // END OF class RootsData


public static RootsData findRootsData(final double x0,final double xend,final int ntries,final DerivativesCalculateable D)
{
    final double F0=D.getF(x0),dF0=D.getDF(x0),d2F0=D.getD2F(x0);
    int dFsigntocheck=dF0>0?1:-1;
    int d2Fsigntocheck=dFsigntocheck*(xend<x0?1:-1);
    double x1=Double.NaN,F1=Double.NaN,dF1=Double.NaN,d2F1=Double.NaN;
    double x2=Double.NaN,F2=Double.NaN,dF2=Double.NaN,d2F2=Double.NaN;
    int x1_type=RootsData.TYPE_UNKNOWN,x2_type=RootsData.TYPE_UNKNOWN;
    final double d=(xend-x0)/ntries;
    double xprev=x0;int n=1;
    for(int j=1;j<=ntries;j++)
    {
	final double x=x0+d*j,dF=D.getDF(x);
	if(dFsigntocheck>0?dF<0:dF>0)
	{
	    final double d2F=D.getD2F(x);
	    if(d2Fsigntocheck>0?d2F>0:d2F<0)
	    {
		final double xm=new BissectionRootFinder(){public double getF(final double xbp){return D.getDF(xbp);}}.doBissection(xprev,x);
		//System.err.println("findRootsData: n="+n+" found xm="+xm+" for j="+j+" out of "+ntries);
		if(n==1)
		{
		    x1=xm;F1=D.getF(xm);dF1=D.getDF(xm);d2F1=D.getD2F(xm);
		    x1_type=d2Fsigntocheck>0?RootsData.TYPE_MIN:RootsData.TYPE_MAX;
		}
		else if(n==2)
		{
		    x2=xm;F2=D.getF(xm);dF2=D.getDF(xm);d2F2=D.getD2F(xm);
		    x2_type=d2Fsigntocheck>0?RootsData.TYPE_MIN:RootsData.TYPE_MAX;
		}
		if(++n>2) break;
		dFsigntocheck=-dFsigntocheck;
		d2Fsigntocheck=-d2Fsigntocheck;
		xprev=x;
	    }
	}
	else xprev=x;
    }
    return new RootsData(x0,x1,x2,F0,F1,F2,dF0,dF1,dF2,d2F0,d2F1,d2F2,x1_type,x2_type,D);
}


}
