package com.polytechnik.utils;
/*
 * (C) Vladislav Malyshkin 2014
 * This file is under GPL version 3.
 *
 */

/** Value Correlation of p and r.
 *  @version $Id: ValueCorrelation.java,v 1.8 2017/10/07 10:35:09 mal Exp $
 *  @author Vladislav Malyshkin mal@gromco.com
 */
public class ValueCorrelation {
public final double w_pL_rL,w_pL_rH,w_pH_rL,w_pH_rH,corr;
public final Skewness skp,skr;

/** Value Correlation (from monomials moments).
 */
public ValueCorrelation(final double c0,final double pr,
			final double p1,final double p2,final double p3,
			final double r1,final double r2,final double r3)
{
    this(c0,pr,Skewness.getSkewnessFromMonomialMoments(c0,p1,p2,p3),Skewness.getSkewnessFromMonomialMoments(c0,r1,r2,r3));
}

public ValueCorrelation(final double c0,final double pr,
			final Skewness skp,final Skewness skr)
{
    this.skp=skp;
    this.skr=skr;
    final double pa=skp.xa*c0,p12=skp.x1-skp.x2,r12=skr.x1-skr.x2;
    w_pL_rL=((pr-pa*skr.x2)/r12-skp.x2*skr.w1)/p12;
    w_pL_rH=-((pr-pa*skr.x1)/r12+skp.x2*skr.w2)/p12;
    w_pH_rL=-((pr-pa*skr.x2)/r12-skp.x1*skr.w1)/p12;
    w_pH_rH=((pr-pa*skr.x1)/r12+skp.x1*skr.w2)/p12;
    corr=(w_pL_rL+w_pH_rH-w_pL_rH-w_pH_rL)/c0;
    if(false)
    {
	final double eps=1e-5;
	UnitTests.checkDoubleRel(w_pL_rL+w_pL_rH,"w_pL_rL+w_pL_rH",skp.w1,"skp.w1",eps);
	UnitTests.checkDoubleRel(w_pH_rL+w_pH_rH,"w_pH_rL+w_pH_rH",skp.w2,"skp.w2",eps);
	UnitTests.checkDoubleRel(w_pL_rL+w_pH_rL,"w_pL_rL+w_pH_rL",skr.w1,"skr.w1",eps);
	UnitTests.checkDoubleRel(w_pL_rH+w_pH_rH,"w_pL_rH+w_pH_rH",skr.w2,"skr.w2",eps);
	final double p1=skp.x1,p2=skp.x2,wp=skp.w1+skp.w2,
	r1=skr.x1,r2=skr.x2,wr=skr.w1+skr.w2,ra=wr*skr.xa,pr12=(p1-p2)*(r1-r2);
	UnitTests.checkDoubleRel(wp,"wp",c0,"c0",eps);
	UnitTests.checkDoubleRel(wr,"wr",c0,"c0",eps);
	UnitTests.checkDoubleRel(w_pL_rL,"w_pL_rL",(pr-p2*ra-r2*pa+p2*r2*wr)/pr12,"w_pL_rL",eps);
	UnitTests.checkDoubleRel(w_pL_rH,"w_pL_rH",-(pr-p2*ra-r1*pa+p2*r1*wr)/pr12,"w_pL_rH",eps);
	UnitTests.checkDoubleRel(w_pH_rL,"w_pH_rL",-(pr-p1*ra-r2*pa+p1*r2*wr)/pr12,"w_pH_rL",eps);
	UnitTests.checkDoubleRel(w_pH_rH,"w_pH_rH",(pr-p1*ra-r1*pa+p1*r1*wr)/pr12,"w_pH_rH",eps);
	System.err.println("ValueCorrelation: match OK");
    }
}

}
